@@ -4,13 +4,10 @@ set_help_text findcheat \
 {Cheat finder version 0.5
 
 Welcome to the openMSX cheat finder. Please visit
-  http://forum.openmsx.org/viewtopic.php?t=32
+  http://forum.vampier.net/viewtopic.php?t=32 and
+  http://www.youtube.com/watch?v=F11ltfkCtKo
 for a quick tutorial
 
-Credits:
-  Copyright 2005 Wouter Vermaelen all rights reserved
-  Copyright 2005 Patrick van Arkel all rights reserved
-
 Usage:
   findcheat [-start] [-max n] [expression]
      -start     :  restart search, discard previously found addresses
@@ -19,16 +16,34 @@ Usage:
 
 Examples:
   findcheat 42                 search for specific value
-  findcheat bigger             search for values that have become bigger
-  findcheat new == (2 * old)   search for values that have doubled
-  findcheat new == (old - 1)   search for values that have decreased by 1
-  findcheat                    repeat the previous found addresses
-  findcheat -start new < 10    restart and search for values less then 10
+  findcheat bigger             search for increased values
+  findcheat new == (2 * old)   search for doubled values
+  findcheat new == (old - 1)   search for values decreased by 1
+  findcheat                    repeat the results from the previous operation
+  findcheat -start new < 10    restart and search for values less than 10
   findcheat -max 40 smaller    search for smaller values, show max 40 results
-}
+  findcheat -start addr>0xe000 && addr<0xefff search in defined memory locations
+  }
 
 namespace eval cheat_finder {
 
+set_tabcompletion_proc findcheat [namespace code tab_cheat_type]
+
+proc tab_cheat_type { args } {
+	set result [list]
+	lappend result "-start"
+	lappend result "bigger"
+	lappend result "smaller"
+	lappend result "more"
+	lappend result "less"
+	lappend result "notequal"
+	lappend result "equal"
+	lappend result "loe"	
+	lappend result "moe"	
+	lappend result "-max"
+	return $result
+}
+
 #set maximum to display cheats
 variable max_num_results 15
 variable mem
