@@ -58,16 +58,16 @@ proc redraw_osd_icons { icon } {
 		set widget2 osd_icons.${icon}_on
 		set fade_delay $current_fade_delay_non_active($icon)
 	}
-	osd configure $widget2 -alpha 0 -fadeTarget 0
+	osd configure $widget2 -fadeCurrent 0 -fadeTarget 0
 
 	if {$fade_delay == 0} {
 		# no fading yet
-		osd configure $widget -alpha 255 -fadeTarget 255
+		osd configure $widget -fadeCurrent 1 -fadeTarget 1
 	} else {
 		set diff [expr [openmsx_info realtime] - $last_change($icon)]
 		if {$diff < $fade_delay} {
 			# no fading yet
-			osd configure $widget -alpha 255 -fadeTarget 255
+			osd configure $widget -fadeCurrent 1 -fadeTarget 1
 			catch {
 				after cancel $fade_id($icon)
 			}
@@ -292,8 +292,8 @@ set icon_list [list "led_power" "led_caps" "led_kana" "led_pause" "led_turbo" "l
 osd create rectangle osd_icons -scaled true -alpha 0 -z 1
 foreach icon $icon_list {
 	variable last_change
-	osd create rectangle osd_icons.${icon}_on  -alpha 0 -fadeTarget 0 -fadePeriod 5.0
-	osd create rectangle osd_icons.${icon}_off -alpha 0 -fadeTarget 0 -fadePeriod 5.0
+	osd create rectangle osd_icons.${icon}_on  -fadeCurrent 0 -fadeTarget 0 -fadePeriod 5.0
+	osd create rectangle osd_icons.${icon}_off -fadeCurrent 0 -fadeTarget 0 -fadePeriod 5.0
 	trace add variable ::$icon "write unset" load_icons::trace_icon_status
 	set last_change($icon) [openmsx_info realtime]
 }
@@ -305,7 +305,12 @@ namespace export load_icons
 namespace import load_icons::*
 
 # Restore settings from previous session
-user_setting create string osd_leds_set "Name of the OSD icon set" "set1"
+# default is set1, but if only scale_factor 1 is supported, use handheld
+if {[lindex [lindex [openmsx_info setting scale_factor] 2] 1] == 1} {
+	user_setting create string osd_leds_set "Name of the OSD icon set" "handheld"
+} else {
+	user_setting create string osd_leds_set "Name of the OSD icon set" "set1"
+}
 user_setting create string osd_leds_pos "Position of the OSD icons" "default"
 set load_icons::current_osd_leds_set $osd_leds_set
 set load_icons::current_osd_leds_pos $osd_leds_pos
