@@ -1,7 +1,9 @@
-// $Id: Autofire.cc 5993 2007-01-12 22:12:12Z m9710797 $
+// $Id: Autofire.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "Autofire.hh"
 #include "IntegerSetting.hh"
+#include <algorithm>
+#include <cassert>
 
 using std::string;
 
@@ -9,20 +11,13 @@ namespace openmsx {
 
 Autofire::Autofire(CommandController& commandController,
                    unsigned newMinInts, unsigned newMaxInts, const string& name)
-	: min_ints(newMinInts)
-	, max_ints(newMaxInts)
+	: min_ints(std::max(newMinInts, 1u))
+	, max_ints(std::max(newMaxInts, min_ints + 1))
 	, speedSetting(new IntegerSetting(commandController, name,
 	               "controls the speed of this autofire circuit", 0, 0, 100))
 	, clock(EmuTime::zero)
 {
-	if (min_ints < 1) {
-		min_ints = 1;
-	}
-	if (max_ints <= min_ints) {
-		max_ints = min_ints + 1;
-	}
 	setClock();
-
 	speedSetting->attach(*this);
 }
 
@@ -45,7 +40,7 @@ void Autofire::update(const Setting& setting)
 	setClock();
 }
 
-bool Autofire::getSignal(const EmuTime& time)
+bool Autofire::getSignal(EmuTime::param time)
 {
 	return speedSetting->getValue() == 0 ?
 		false : clock.getTicksTill(time) & 1;
