@@ -1,16 +1,16 @@
-// $Id: CLIOption.cc 5954 2006-12-22 08:10:58Z mthuurne $
+// $Id: CLIOption.cc 7731 2008-04-03 19:58:10Z m9710797 $
 
 #include "CLIOption.hh"
 #include "MSXException.hh"
 
 using std::string;
-using std::list;
+using std::deque;
 
 namespace openmsx {
 
 // class CLIOption
 
-string CLIOption::getArgument(const string& option, list<string>& cmdLine) const
+string CLIOption::getArgument(const string& option, deque<string>& cmdLine) const
 {
 	if (cmdLine.empty()) {
 		throw FatalError("Missing argument for option \"" + option + "\"");
@@ -20,7 +20,7 @@ string CLIOption::getArgument(const string& option, list<string>& cmdLine) const
 	return argument;
 }
 
-string CLIOption::peekArgument(const list<string>& cmdLine) const
+string CLIOption::peekArgument(const deque<string>& cmdLine) const
 {
 	return cmdLine.empty() ? "" : cmdLine.front();
 }
