@@ -1,4 +1,4 @@
-// $Id: CartridgeSlotManager.cc 9393 2009-03-18 19:24:48Z m9710797 $
+// $Id: CartridgeSlotManager.cc 10747 2009-11-04 19:21:39Z m9710797 $
 
 #include "CartridgeSlotManager.hh"
 #include "MSXMotherBoard.hh"
@@ -13,6 +13,7 @@
 #include "StringOp.hh"
 #include "openmsx.hh"
 #include "CliComm.hh"
+#include "unreachable.hh"
 #include <cassert>
 
 using std::string;
@@ -131,7 +132,7 @@ void CartridgeSlotManager::createExternalSlot(int ps, int ss)
 			return;
 		}
 	}
-	assert(false);
+	UNREACHABLE;
 }
 
 
@@ -143,7 +144,7 @@ int CartridgeSlotManager::getSlot(int ps, int ss) const
 			return slot;
 		}
 	}
-	assert(false); // was not an external slot
+	UNREACHABLE; // was not an external slot
 	return 0; // avoid warning
 }
 
@@ -248,7 +249,7 @@ int CartridgeSlotManager::useExternalSlot(
 			return slot;
 		}
 	}
-	assert(false); // ps-ss is not an external slot
+	UNREACHABLE; // ps-ss is not an external slot
 	return 0; // avoid warning
 }
 
@@ -275,7 +276,7 @@ bool CartridgeSlotManager::isExternalSlot(int ps, int ss, bool convert) const
 CartCmd::CartCmd(CartridgeSlotManager& manager_, MSXMotherBoard& motherBoard,
                  const string& commandName)
 	: RecordedCommand(motherBoard.getCommandController(),
-	                  motherBoard.getMSXEventDistributor(),
+	                  motherBoard.getStateChangeDistributor(),
 	                  motherBoard.getScheduler(),
 	                  commandName)
 	, manager(manager_)
@@ -308,10 +309,11 @@ string CartCmd::execute(const vector<string>& tokens, EmuTime::param /*time*/)
 	if (tokens.size() == 1) {
 		// query name of cartridge
 		const HardwareConfig* extConf = getExtensionConfig(cartname);
-		TclObject object(getCommandController().getInterpreter());
+		Interpreter& interpreter = getInterpreter();
+		TclObject object(interpreter);
 		object.addListElement(cartname + ':');
 		object.addListElement(extConf ? extConf->getName() : "");
-		TclObject options(getCommandController().getInterpreter());
+		TclObject options(interpreter);
 		if (!extConf) {
 			options.addListElement("empty");
 		}
@@ -322,7 +324,8 @@ string CartCmd::execute(const vector<string>& tokens, EmuTime::param /*time*/)
 	} else if ( (tokens[1] == "eject") || (tokens[1] == "-eject") ) {
 		// remove cartridge (or extension)
 		if (tokens[1] == "-eject") {
-			result += "Warning: use of '-eject' is deprecated, instead use the 'eject' subcommand";
+			result += "Warning: use of '-eject' is deprecated, "
+			          "instead use the 'eject' subcommand";
 		}
 		if (const HardwareConfig* extConf = getExtensionConfig(cartname)) {
 			try {
@@ -333,7 +336,6 @@ string CartCmd::execute(const vector<string>& tokens, EmuTime::param /*time*/)
 				                       e.getMessage());
 			}
 		}
-
 	} else {
 		// insert cartridge
 		string slotname = (cartname.size() == 5)
