@@ -1,4 +1,4 @@
-// $Id: CliExtension.cc 5746 2006-10-05 16:57:09Z m9710797 $
+// $Id: CliExtension.cc 7731 2008-04-03 19:58:10Z m9710797 $
 
 #include "CliExtension.hh"
 #include "CommandLineParser.hh"
@@ -16,7 +16,7 @@ CliExtension::CliExtension(CommandLineParser& cmdLineParser_)
 	cmdLineParser.registerOption("-ext", *this, 7);
 }
 
-bool CliExtension::parseOption(const string& option, std::list<string>& cmdLine)
+bool CliExtension::parseOption(const string& option, std::deque<string>& cmdLine)
 {
 	try {
 		string extensionName = getArgument(option, cmdLine);
