@@ -1,8 +1,8 @@
-// $Id: CommandLineParser.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: CommandLineParser.cc 11268 2010-02-15 17:41:47Z mfeingol $
 
 #include "CommandLineParser.hh"
 #include "CLIOption.hh"
-#include "CommandController.hh"
+#include "GlobalCommandController.hh"
 #include "SettingsConfig.hh"
 #include "File.hh"
 #include "FileContext.hh"
@@ -11,11 +11,11 @@
 #include "Version.hh"
 #include "MSXRomCLI.hh"
 #include "CliExtension.hh"
+#include "CliConnection.hh"
 #include "CassettePlayerCLI.hh"
 #include "DiskImageCLI.hh"
 #include "HDImageCLI.hh"
 #include "CDImageCLI.hh"
-#include "MSXEventRecorderReplayerCLI.hh"
 #include "ConfigException.hh"
 #include "FileException.hh"
 #include "EnumSetting.hh"
@@ -23,10 +23,17 @@
 #include "XMLElement.hh"
 #include "XMLException.hh"
 #include "HostCPU.hh"
+#include "StringOp.hh"
+#include "utf8_checked.hh"
 #include "GLUtil.hh"
 #include "Reactor.hh"
 #include "build-info.hh"
 #include "components.hh"
+
+#if COMPONENT_LASERDISC
+#include "LaserdiscPlayerCLI.hh"
+#endif
+
 #include <cassert>
 #include <iostream>
 #include <cstdio>
@@ -39,6 +46,10 @@ using std::set;
 using std::string;
 using std::vector;
 
+#ifdef _WIN32
+using namespace utf8;
+#endif
+
 namespace openmsx {
 
 class HelpOption : public CLIOption
@@ -141,7 +152,7 @@ private:
 
 CommandLineParser::CommandLineParser(Reactor& reactor_)
 	: reactor(reactor_)
-	, settingsConfig(reactor.getCommandController().getSettingsConfig())
+	, settingsConfig(reactor.getGlobalCommandController().getSettingsConfig())
 	, output(reactor.getCliComm())
 	, helpOption(new HelpOption(*this))
 	, versionOption(new VersionOption(*this))
@@ -157,10 +168,12 @@ CommandLineParser::CommandLineParser(Reactor& reactor_)
 	, msxRomCLI(new MSXRomCLI(*this))
 	, cliExtension(new CliExtension(*this))
 	, cassettePlayerCLI(new CassettePlayerCLI(*this))
+#if COMPONENT_LASERDISC
+	, laserdiscPlayerCLI(new LaserdiscPlayerCLI(*this))
+#endif
 	, diskImageCLI(new DiskImageCLI(*this))
 	, hdImageCLI(new HDImageCLI(*this))
 	, cdImageCLI(new CDImageCLI(*this))
-	, eventRecorderReplayerCLI(new MSXEventRecorderReplayerCLI(*this))
 	, parseStatus(UNPARSED)
 {
 	haveConfig = false;
@@ -174,12 +187,12 @@ CommandLineParser::CommandLineParser(Reactor& reactor_)
 	registerOption("--version",   *versionOption, 1, 1);
 	registerOption("-control",    *controlOption, 1, 1);
 	registerOption("-script",     *scriptOption, 1, 1);
-	#ifdef ASM_X86
+	#if ASM_X86
 	registerOption("-nommx",      *noMMXOption, 1, 1);
 	registerOption("-nosse",      *noSSEOption, 1, 1);
 	registerOption("-nosse2",     *noSSE2Option, 1, 1);
 	#endif
-	#ifdef COMPONENT_GL
+	#if COMPONENT_GL
 	registerOption("-nopbo",      *noPBOOption, 1, 1);
 	#endif
 	registerOption("-testconfig", *testConfigOption, 1, 1);
@@ -230,6 +243,7 @@ void CommandLineParser::registerFileTypes()
 		fileExtMap["xsa"] = "diskimage";
 		fileExtMap["wav"] = "cassetteimage";
 		fileExtMap["cas"] = "cassetteimage";
+		fileExtMap["ogv"] = "laserdiscimage";
 		for (map<string, string>::const_iterator j = fileExtMap.begin();
 		     j != fileExtMap.end(); ++j) {
 			FileClassMap::const_iterator i = fileClassMap.find(j->second);
@@ -263,7 +277,7 @@ bool CommandLineParser::parseFileName(const string& arg, deque<string>& cmdLine)
 	string originalName(arg);
 	try {
 		UserFileContext context;
-		File file(context.resolve(getCommandController(), arg));
+		File file(context.resolve(getGlobalCommandController(), arg));
 		originalName = file.getOriginalName();
 	} catch (FileException&) {
 		// ignore
@@ -330,7 +344,6 @@ void CommandLineParser::parse(int argc, char** argv)
 				// load default config file in case the user didn't specify one
 				const string& machine =
 					reactor.getMachineSetting().getValueString();
-				output.printInfo("Using default machine: " + machine);
 				try {
 					reactor.switchMachine(machine);
 				} catch (MSXException& e) {
@@ -412,9 +425,9 @@ MSXMotherBoard* CommandLineParser::getMotherBoard() const
 	return reactor.getMotherBoard();
 }
 
-CommandController& CommandLineParser::getCommandController() const
+GlobalCommandController& CommandLineParser::getGlobalCommandController() const
 {
-	return reactor.getCommandController();
+	return reactor.getGlobalCommandController();
 }
 
 
@@ -427,7 +440,36 @@ ControlOption::ControlOption(CommandLineParser& parser_)
 
 bool ControlOption::parseOption(const string& option, deque<string>& cmdLine)
 {
-	parser.reactor.getGlobalCliComm().startInput(getArgument(option, cmdLine));
+	string fullType = getArgument(option, cmdLine);
+	string type, arguments;
+	StringOp::splitOnFirst(fullType, ":", type, arguments);
+
+	CommandController& controller = parser.getGlobalCommandController();
+	EventDistributor& distributor = parser.reactor.getEventDistributor();
+	GlobalCliComm& cliComm        = parser.reactor.getGlobalCliComm();
+	std::auto_ptr<CliConnection> connection;
+	if (type == "stdio") {
+		connection.reset(new StdioConnection(
+			controller, distributor));
+#ifdef _WIN32
+	} else if (type == "pipe") {
+		OSVERSIONINFO info;
+		info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+		GetVersionEx(&info);
+		if (info.dwPlatformId == VER_PLATFORM_WIN32_NT) {
+			connection.reset(new PipeConnection(
+				controller, distributor, arguments));
+		} else {
+			throw FatalError("Pipes are not supported on this "
+			                 "version of Windows");
+		}
+#endif
+	} else {
+		throw FatalError("Unknown control type: '"  + type + "'");
+	}
+	cliComm.addConnection(connection);
+	cliComm.setXMLOutput();
+
 	parser.parseStatus = CommandLineParser::CONTROL;
 	return true;
 }
@@ -609,7 +651,6 @@ bool MachineOption::parseOption(const string& option, deque<string>& cmdLine)
 		throw FatalError("Only one machine option allowed");
 	}
 	string machine(getArgument(option, cmdLine));
-	parser.output.printInfo("Using specified machine: " + machine);
 	try {
 		parser.reactor.switchMachine(machine);
 	} catch (MSXException& e) {
@@ -716,7 +757,7 @@ const string& NoSSE2Option::optionHelp() const
 bool NoPBOOption::parseOption(const string& /*option*/,
                               deque<string>& /*cmdLine*/)
 {
-	#ifdef COMPONENT_GL
+	#if COMPONENT_GL
 	cout << "Disabling PBO" << endl;
 	PixelBuffers::enabled = false;
 	#endif
