@@ -1,4 +1,4 @@
-// $Id: DebugDevice.cc 8527 2009-01-29 17:06:52Z mfeingol $
+// $Id: DebugDevice.cc 10107 2009-06-30 18:26:06Z m9710797 $
 
 #include "DebugDevice.hh"
 #include "Clock.hh"
@@ -17,19 +17,24 @@ namespace openmsx {
 DebugDevice::DebugDevice(MSXMotherBoard& motherBoard, const XMLElement& config)
 	: MSXDevice(motherBoard, config)
 {
-	mode = OFF;
-	modeParameter = 0;
 	string outputFile = config.getChildData("filename", "stdout");
 	fileNameSetting.reset(new FilenameSetting(
 		motherBoard.getCommandController(), "debugoutput",
 		"name of the file the debugdevice outputs to", outputFile));
 	openOutput(fileNameSetting->getValueString());
+	reset(EmuTime::dummy());
 }
 
 DebugDevice::~DebugDevice()
 {
 }
 
+void DebugDevice::reset(EmuTime::param /*time*/)
+{
+	mode = OFF;
+	modeParameter = 0;
+}
+
 void DebugDevice::writeIO(word port, byte value, EmuTime::param time)
 {
 	string newName = fileNameSetting->getValueString();
