@@ -1,4 +1,4 @@
-// $Id: DeviceFactory.cc 9371 2009-03-13 19:28:37Z m9710797 $
+// $Id: DeviceFactory.cc 10418 2009-08-27 12:28:00Z mthuurne $
 
 #include "DeviceFactory.hh"
 #include "XMLElement.hh"
@@ -54,8 +54,12 @@
 #include "MSXMapperIO.hh"
 #include "VDPIODelay.hh"
 #include "MSXMotherBoard.hh"
-#include "EmuTime.hh"
 #include "MSXException.hh"
+#include "components.hh"
+
+#if COMPONENT_LASERDISC
+#include "PioneerLDControl.hh"
+#endif
 
 namespace openmsx {
 
@@ -157,6 +161,12 @@ std::auto_ptr<MSXDevice> DeviceFactory::create(
 		result.reset(new V9990(motherBoard, conf));
 	} else if (type == "ADVram") {
 		result.reset(new ADVram(motherBoard, conf));
+	} else if (type == "PioneerLDControl") {
+#if COMPONENT_LASERDISC
+		result.reset(new PioneerLDControl(motherBoard, conf));
+#else
+		throw MSXException("Laserdisc component not compiled in");
+#endif
 	} else if (type == "Nowind") {
 		result.reset(new NowindInterface(motherBoard, conf));
 	} else if (type == "Mirror") {
@@ -202,18 +212,17 @@ std::auto_ptr<MSXMapperIO> DeviceFactory::createMapperIO(
 		MSXMotherBoard& motherBoard)
 {
 	static XMLElement config(createConfig("MapperIO", "MapperIO"));
-	std::auto_ptr<MSXMapperIO> result(
-		new MSXMapperIO(motherBoard, config, EmuTime::zero));
+	std::auto_ptr<MSXMapperIO> result(new MSXMapperIO(motherBoard, config));
 	result->init(*motherBoard.getMachineConfig());
 	return result;
 }
 
 std::auto_ptr<VDPIODelay> DeviceFactory::createVDPIODelay(
-		MSXMotherBoard& motherBoard)
+		MSXMotherBoard& motherBoard, MSXCPUInterface& cpuInterface)
 {
 	static XMLElement config(createConfig("VDPIODelay", "VDPIODelay"));
 	std::auto_ptr<VDPIODelay> result(
-		new VDPIODelay(motherBoard, config, EmuTime::zero));
+		new VDPIODelay(motherBoard, config, cpuInterface));
 	result->init(*motherBoard.getMachineConfig());
 	return result;
 }
