@@ -1,4 +1,4 @@
-// $Id: DynamicClock.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: DynamicClock.cc 10141 2009-07-02 19:01:40Z m9710797 $
 
 #include "DynamicClock.hh"
 #include "serialize.hh"
@@ -11,7 +11,7 @@ void DynamicClock::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("lastTick", lastTick);
 	unsigned freq = getFreq();
 	ar.serialize("freq", freq);
-	setFreq(freq);
+	if (ar.isLoader()) setFreq(freq);
 }
 INSTANTIATE_SERIALIZE_METHODS(DynamicClock);
 
