@@ -1,7 +1,7 @@
-// $Id: EmptyPatch.cc 4296 2005-04-23 12:47:44Z m9710797 $
+// $Id: EmptyPatch.cc 8193 2008-09-02 18:15:39Z m9710797 $
 
 #include "EmptyPatch.hh"
-#include <string.h>
+#include <cstring>
 #include <cassert>
 
 namespace openmsx {
@@ -39,5 +39,10 @@ unsigned EmptyPatch::getSize() const
 	return size;
 }
 
+void EmptyPatch::getFilenames(std::vector<Filename>& /*result*/) const
+{
+	// nothing
+}
+
 } // namespace openmsx
 
