@@ -1,4 +1,4 @@
-// $Id: GlobalSettings.cc 9358 2009-03-09 20:02:53Z manuelbi $
+// $Id: GlobalSettings.cc 11438 2010-05-19 21:57:51Z manuelbi $
 
 #include "GlobalSettings.hh"
 #include "IntegerSetting.hh"
@@ -6,12 +6,14 @@
 #include "StringSetting.hh"
 #include "SettingsConfig.hh"
 #include "CommandController.hh"
+#include "GlobalCommandController.hh"
 #include "ThrottleManager.hh"
 #include "EnumSetting.hh"
+#include "build-info.hh"
 
 namespace openmsx {
 
-GlobalSettings::GlobalSettings(CommandController& commandController_)
+GlobalSettings::GlobalSettings(GlobalCommandController& commandController_)
 	: commandController(commandController_)
 {
 	speedSetting.reset(new IntegerSetting(commandController, "speed",
@@ -24,35 +26,24 @@ GlobalSettings::GlobalSettings(CommandController& commandController_)
 	autoSaveSetting.reset(new BooleanSetting(commandController,
 	        "save_settings_on_exit",
 	        "automatically save settings when openMSX exits", true));
-	consoleSetting.reset(new BooleanSetting(commandController, "console",
-	        "turns console display on/off", false, Setting::DONT_SAVE));
 	userDirSetting.reset(new StringSetting(commandController,
 	        "user_directories", "list of user directories", ""));
 	pauseOnLostFocusSetting.reset(new BooleanSetting(commandController, "pause_on_lost_focus",
 	       "pause emulation when the openMSX window loses focus", false));
 	umrCallBackSetting.reset(new StringSetting(commandController,
 	        "umr_callback", "Tcl proc to call when an UMR is detected", ""));
-	EnumSetting<SyncMode>::Map syncDirAsDSKMap;
-	syncDirAsDSKMap["read_only"] = SYNC_READONLY;
-	syncDirAsDSKMap["cached_write"] = SYNC_CACHEDWRITE;
-	syncDirAsDSKMap["nodelete"] = SYNC_NODELETE;
-	syncDirAsDSKMap["full"] = SYNC_FULL;
-	syncDirAsDSKSetting.reset(new EnumSetting<SyncMode>(commandController,
-		"DirAsDSKmode", "type of syncronisation between host directory and dir-as-dsk diskimage",
-		SYNC_FULL, syncDirAsDSKMap));
-	EnumSetting<bool>::Map bootsectorMap;
-	bootsectorMap["DOS1"] = false;
-	bootsectorMap["DOS2"] = true;
-	bootSectorSetting.reset(new EnumSetting<bool>(commandController,
-		"bootsector", "boot sector type for dir-as-dsk",
-		true, bootsectorMap));
-	EnumSetting<ResampleType>::Map resampleMap;
-	resampleMap["hq"]   = RESAMPLE_HQ;
-	resampleMap["fast"] = RESAMPLE_LQ;
-	resampleMap["blip"] = RESAMPLE_BLIP;
-	resampleSetting.reset(new EnumSetting<ResampleType>(commandController,
-		"resampler", "Resample algorithm",
-		RESAMPLE_BLIP, resampleMap));
+	EnumSetting<Resample::ResampleType>::Map resampleMap;
+	resampleMap["hq"]   = Resample::RESAMPLE_HQ;
+	resampleMap["fast"] = Resample::RESAMPLE_LQ;
+	resampleMap["blip"] = Resample::RESAMPLE_BLIP;
+	resampleSetting.reset(new EnumSetting<Resample::ResampleType>(
+		commandController, "resampler", "Resample algorithm",
+#if PLATFORM_DINGUX
+		Resample::RESAMPLE_LQ,
+#else
+		Resample::RESAMPLE_BLIP,
+#endif
+		resampleMap));
 
 	throttleManager.reset(new ThrottleManager(commandController));
 
@@ -86,11 +77,6 @@ BooleanSetting& GlobalSettings::getAutoSaveSetting()
 	return *autoSaveSetting.get();
 }
 
-BooleanSetting& GlobalSettings::getConsoleSetting()
-{
-	return *consoleSetting.get();
-}
-
 ThrottleManager& GlobalSettings::getThrottleManager()
 {
 	return *throttleManager.get();
@@ -111,17 +97,7 @@ BooleanSetting& GlobalSettings::getPauseOnLostFocusSetting()
 	return *pauseOnLostFocusSetting.get();
 }
 
-EnumSetting<GlobalSettings::SyncMode>& GlobalSettings::getSyncDirAsDSKSetting()
-{
-	return *syncDirAsDSKSetting.get();
-}
-
-EnumSetting<bool>& GlobalSettings::getBootSectorSetting()
-{
-	return *bootSectorSetting.get();
-}
-
-EnumSetting<GlobalSettings::ResampleType>& GlobalSettings::getResampleSetting()
+EnumSetting<Resample::ResampleType>& GlobalSettings::getResampleSetting()
 {
 	return *resampleSetting.get();
 }
