@@ -1,9 +1,9 @@
-// $Id: LedStatus.cc 10039 2009-06-22 16:25:53Z m9710797 $
+// $Id: LedStatus.cc 10163 2009-07-04 17:15:17Z mthuurne $
 
 #include "LedStatus.hh"
-#include "MSXMotherBoard.hh"
-#include "EventDistributor.hh"
-#include "CliComm.hh"
+#include "Reactor.hh"
+#include "AlarmEvent.hh"
+#include "MSXCliComm.hh"
 #include "BooleanSetting.hh"
 #include "ReadOnlySetting.hh"
 #include "Timer.hh"
@@ -18,18 +18,19 @@ static std::string getLedName(LedStatus::Led led)
 	return names[led];
 }
 
-LedStatus::LedStatus(MSXMotherBoard& motherBoard_)
-	: motherBoard(motherBoard_)
+LedStatus::LedStatus(
+		EventDistributor& eventDistributor,
+		CommandController& commandController,
+		MSXCliComm& msxCliComm_)
+	: msxCliComm(msxCliComm_)
+	, alarm(new AlarmEvent(eventDistributor, *this, OPENMSX_THROTTLE_LED_EVENT))
 {
-	motherBoard.getEventDistributor().registerEventListener(
-		OPENMSX_THROTTLE_LED_EVENT, *this);
-
 	lastTime = Timer::getTime();
 	for (int i = 0; i < NUM_LEDS; ++i) {
 		ledValue[i] = false;
 		std::string name = getLedName(static_cast<Led>(i));
 		ledStatus[i].reset(new ReadOnlySetting<BooleanSetting>(
-			motherBoard.getCommandController(),
+			commandController,
 			"led_" + name,
 			"Current status for LED: " + name,
 			ledValue[i]));
@@ -38,9 +39,6 @@ LedStatus::LedStatus(MSXMotherBoard& motherBoard_)
 
 LedStatus::~LedStatus()
 {
-	prepareDelete();
-	motherBoard.getEventDistributor().unregisterEventListener(
-		OPENMSX_THROTTLE_LED_EVENT, *this);
 }
 
 void LedStatus::setLed(Led led, bool status)
@@ -60,8 +58,8 @@ void LedStatus::setLed(Led led, bool status)
 		handleEvent(led);
 	} else {
 		// schedule to handle it later, if we didn't plan to do so already
-		if (!pending()) {
-			schedule(10000 - diff);
+		if (!alarm->pending()) {
+			alarm->schedule(10000 - diff);
 		}
 	}
 }
@@ -72,20 +70,11 @@ void LedStatus::handleEvent(Led led)
 
 	static const std::string ON  = "on";
 	static const std::string OFF = "off";
-	motherBoard.getMSXCliComm().update(
+	msxCliComm.update(
 		CliComm::LED, getLedName(led),
 		ledValue[led] ? ON : OFF);
 }
 
-bool LedStatus::alarm()
-{
-	// Runs in timer thread.
-	// Schedule event so that the main thread can do the real work
-	motherBoard.getEventDistributor().distributeEvent(
-		new SimpleEvent<OPENMSX_THROTTLE_LED_EVENT>());
-	return false; // don't repeat
-}
-
 bool LedStatus::signalEvent(shared_ptr<const Event> /*event*/)
 {
 	// Runs in main thread.
