@@ -1,4 +1,4 @@
-// $Id: MSXDevice.cc 9804 2009-05-12 16:19:07Z m9710797 $
+// $Id: MSXDevice.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "MSXDevice.hh"
 #include "XMLElement.hh"
@@ -11,6 +11,7 @@
 #include "StringOp.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
+#include "unreachable.hh"
 #include <set>
 #include <algorithm>
 #include <cassert>
@@ -46,7 +47,7 @@ void MSXDevice::initName(const string& name)
 	if (motherBoard.findDevice(deviceName)) {
 		unsigned n = 0;
 		do {
-			deviceName = name + " (" + StringOp::toString(++n) + ")";
+			deviceName = StringOp::Builder() << name << " (" << ++n << ')';
 		} while (motherBoard.findDevice(deviceName));
 	}
 }
@@ -424,7 +425,7 @@ byte MSXDevice::peekMem(word address, EmuTime::param /*time*/) const
 void MSXDevice::globalWrite(word /*address*/, byte /*value*/,
                             EmuTime::param /*time*/)
 {
-	assert(false);
+	UNREACHABLE;
 }
 
 byte* MSXDevice::getWriteCacheLine(word /*start*/) const
