@@ -1,4 +1,4 @@
-// $Id: MSXDeviceSwitch.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: MSXDeviceSwitch.cc 10392 2009-08-18 19:56:02Z m9710797 $
 
 #include "MSXDeviceSwitch.hh"
 #include "MSXSwitchedDevice.hh"
@@ -35,8 +35,8 @@ void MSXDeviceSwitch::registerDevice(byte id, MSXSwitchedDevice* device)
 {
 	if (devices[id]) {
 		// TODO implement multiplexing
-		throw MSXException("Already have a switched device with id " +
-		                   StringOp::toString(int(id)));
+		throw MSXException(StringOp::Builder() <<
+			"Already have a switched device with id " << int(id));
 	}
 	devices[id] = device;
 	if (count == 0) {
