@@ -1,18 +1,19 @@
-// $Id: MSXMatsushita.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: MSXMatsushita.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 #include "MSXMatsushita.hh"
 #include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
 #include "FirmwareSwitch.hh"
 #include "SRAM.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
-const byte ID = 0x08;
+static const byte ID = 0x08;
 
 MSXMatsushita::MSXMatsushita(MSXMotherBoard& motherBoard,
-                             const XMLElement& config, const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+                             const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, MSXSwitchedDevice(motherBoard, ID)
 	, firmwareSwitch(
 	     new FirmwareSwitch(motherBoard.getCommandController(), config))
@@ -20,22 +21,22 @@ MSXMatsushita::MSXMatsushita(MSXMotherBoard& motherBoard,
 {
 	// TODO find out what ports 0x41 0x45 0x46 are used for
 
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
 MSXMatsushita::~MSXMatsushita()
 {
 }
 
-void MSXMatsushita::reset(const EmuTime& /*time*/)
+void MSXMatsushita::reset(EmuTime::param /*time*/)
 {
 	address = 0; // TODO check this
 }
 
-byte MSXMatsushita::readIO(word port, const EmuTime& time)
+byte MSXMatsushita::readSwitchedIO(word port, EmuTime::param time)
 {
 	// TODO: Port 7 and 8 can be read as well.
-	byte result = peekIO(port, time);
+	byte result = peekSwitchedIO(port, time);
 	switch (port & 0x0F) {
 	case 3:
 		pattern = (pattern << 2) | (pattern >> 6);
@@ -47,7 +48,7 @@ byte MSXMatsushita::readIO(word port, const EmuTime& time)
 	return result;
 }
 
-byte MSXMatsushita::peekIO(word port, const EmuTime& /*time*/) const
+byte MSXMatsushita::peekSwitchedIO(word port, EmuTime::param /*time*/) const
 {
 	byte result;
 	switch (port & 0x0F) {
@@ -74,7 +75,7 @@ byte MSXMatsushita::peekIO(word port, const EmuTime& /*time*/) const
 	return result;
 }
 
-void MSXMatsushita::writeIO(word port, byte value, const EmuTime& /*time*/)
+void MSXMatsushita::writeSwitchedIO(word port, byte value, EmuTime::param /*time*/)
 {
 	switch (port & 0x0F) {
 	case 1:
@@ -111,4 +112,19 @@ void MSXMatsushita::writeIO(word port, byte value, const EmuTime& /*time*/)
 	}
 }
 
+template<typename Archive>
+void MSXMatsushita::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	// no need to serialize MSXSwitchedDevice base class
+
+	ar.serialize("SRAM", *sram);
+	ar.serialize("address", address);
+	ar.serialize("color1", color1);
+	ar.serialize("color2", color2);
+	ar.serialize("pattern", pattern);
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXMatsushita);
+REGISTER_MSXDEVICE(MSXMatsushita, "Matsushita");
+
 } // namespace openmsx
