@@ -1,20 +1,21 @@
-// $Id: MSXMotherBoard.cc 10048 2009-06-23 17:01:29Z m9710797 $
+// $Id: MSXMotherBoard.cc 11434 2010-05-15 16:38:13Z m9710797 $
 
 #include "MSXMotherBoard.hh"
 #include "Reactor.hh"
 #include "MSXDevice.hh"
+#include "ReverseManager.hh"
 #include "HardwareConfig.hh"
 #include "XMLElement.hh"
 #include "MSXCliComm.hh"
 #include "GlobalCliComm.hh"
 #include "MSXCommandController.hh"
 #include "Scheduler.hh"
+#include "Schedulable.hh"
 #include "CartridgeSlotManager.hh"
 #include "EventDistributor.hh"
 #include "Debugger.hh"
 #include "MSXMixer.hh"
 #include "PluggingController.hh"
-#include "DummyDevice.hh"
 #include "MSXCPUInterface.hh"
 #include "MSXCPU.hh"
 #include "PanasonicMemory.hh"
@@ -24,8 +25,8 @@
 #include "RenShaTurbo.hh"
 #include "LedStatus.hh"
 #include "MSXEventDistributor.hh"
+#include "StateChangeDistributor.hh"
 #include "EventDelay.hh"
-#include "EventTranslator.hh"
 #include "RealTime.hh"
 #include "DeviceFactory.hh"
 #include "BooleanSetting.hh"
@@ -40,9 +41,11 @@
 #include "StringOp.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
+#include "unreachable.hh"
 #include "ref.hh"
 #include <cassert>
 #include <map>
+#include <vector>
 #include <iostream>
 
 using std::set;
@@ -61,19 +64,22 @@ class ExtCmd;
 class RemoveExtCmd;
 class MachineNameInfo;
 class DeviceInfo;
+class FastForwardHelper;
 
 class MSXMotherBoardImpl : private Observer<Setting>, private noncopyable
 {
 public:
-	MSXMotherBoardImpl(MSXMotherBoard& self, Reactor& reactor);
+	MSXMotherBoardImpl(MSXMotherBoard& self,
+	                   Reactor& reactor, FilePool& filePool);
 	virtual ~MSXMotherBoardImpl();
 
 	const string& getMachineID();
 	const string& getMachineName() const;
 
 	bool execute();
-	void exitCPULoopSync();
+	void fastForward(EmuTime::param time);
 	void exitCPULoopAsync();
+	void exitCPULoopSync();
 	void pause();
 	void unpause();
 	void powerUp();
@@ -86,7 +92,9 @@ public:
 	void setMachineConfig(HardwareConfig* machineConfig);
 	bool isTurboR() const;
 	void loadMachine(const string& machine);
-	const MSXMotherBoard::Extensions& getExtensions() const;
+
+	typedef std::vector<HardwareConfig*> Extensions;
+	const Extensions& getExtensions() const;
 	HardwareConfig* findExtension(const string& extensionName);
 	string loadExtension(const string& extensionName);
 	string insertExtension(const std::string& name,
@@ -94,18 +102,15 @@ public:
 	void removeExtension(const HardwareConfig& extension);
 
 	CliComm& getMSXCliComm();
-	CliComm* getMSXCliCommIfAvailable();
 	MSXEventDistributor& getMSXEventDistributor();
+	StateChangeDistributor& getStateChangeDistributor();
 	MSXCommandController& getMSXCommandController();
 	Scheduler& getScheduler();
 	CartridgeSlotManager& getSlotManager();
-	EventDelay& getEventDelay();
-	EventTranslator& getEventTranslator();
 	RealTime& getRealTime();
 	Debugger& getDebugger();
 	MSXMixer& getMSXMixer();
 	PluggingController& getPluggingController();
-	DummyDevice& getDummyDevice();
 	MSXCPU& getCPU();
 	MSXCPUInterface& getCPUInterface();
 	PanasonicMemory& getPanasonicMemory();
@@ -113,13 +118,9 @@ public:
 	CassettePortInterface& getCassettePort();
 	RenShaTurbo& getRenShaTurbo();
 	LedStatus& getLedStatus();
+	ReverseManager& getReverseManager();
 	Reactor& getReactor();
-	EventDistributor& getEventDistributor();
-	Display& getDisplay();
-	DiskManipulator& getDiskManipulator();
 	FilePool& getFilePool();
-	GlobalSettings& getGlobalSettings();
-	GlobalCliComm& getGlobalCliComm();
 	CommandController& getCommandController();
 	InfoCommand& getMachineInfoCommand();
 	EmuTime::param getCurrentTime();
@@ -135,6 +136,9 @@ public:
 	string getUserName(const string& hwName);
 	void freeUserName(const string& hwName, const string& userName);
 
+	unsigned getReRecordCount() const;
+	void setReRecordCount(unsigned count);
+
 	template<typename Archive>
 	void serialize(Archive& ar, unsigned version);
 
@@ -148,6 +152,7 @@ private:
 	MSXMotherBoard& self;
 
 	Reactor& reactor;
+	FilePool& filePool;
 	string machineID;
 	string machineName;
 
@@ -169,22 +174,20 @@ private:
 	auto_ptr<HardwareConfig> machineConfig2;
 	HardwareConfig* machineConfig;
 
-	MSXMotherBoard::Extensions extensions;
+	Extensions extensions;
 
 	// order of auto_ptr's is important!
 	auto_ptr<AddRemoveUpdate> addRemoveUpdate;
 	auto_ptr<MSXCliComm> msxCliComm;
 	auto_ptr<MSXEventDistributor> msxEventDistributor;
+	auto_ptr<StateChangeDistributor> stateChangeDistributor;
 	auto_ptr<MSXCommandController> msxCommandController;
 	auto_ptr<Scheduler> scheduler;
-	auto_ptr<CartridgeSlotManager> slotManager;
 	auto_ptr<EventDelay> eventDelay;
-	auto_ptr<EventTranslator> eventTranslator;
 	auto_ptr<RealTime> realTime;
 	auto_ptr<Debugger> debugger;
 	auto_ptr<MSXMixer> msxMixer;
 	auto_ptr<PluggingController> pluggingController;
-	auto_ptr<DummyDevice> dummyDevice;
 	auto_ptr<MSXCPU> msxCpu;
 	auto_ptr<MSXCPUInterface> msxCpuInterface;
 	auto_ptr<PanasonicMemory> panasonicMemory;
@@ -193,6 +196,8 @@ private:
 	auto_ptr<RenShaTurbo> renShaTurbo;
 	auto_ptr<LedStatus> ledStatus;
 
+	auto_ptr<CartridgeSlotManager> slotManager;
+	auto_ptr<ReverseManager> reverseManager;
 	auto_ptr<ResetCmd>     resetCommand;
 	auto_ptr<LoadMachineCmd> loadMachineCommand;
 	auto_ptr<ListExtCmd>   listExtCommand;
@@ -202,8 +207,12 @@ private:
 	auto_ptr<DeviceInfo>   deviceInfo;
 	friend class DeviceInfo;
 
+	auto_ptr<FastForwardHelper> fastForwardHelper;
+
 	BooleanSetting& powerSetting;
 
+	unsigned reRecordCount;
+
 	bool powered;
 	bool active;
 };
@@ -298,18 +307,46 @@ private:
 	MSXMotherBoardImpl& motherBoard;
 };
 
+class FastForwardHelper : private Schedulable
+{
+public:
+	FastForwardHelper(MSXMotherBoardImpl& msxMotherBoardImpl);
+	void setTarget(EmuTime::param targetTime);
+private:
+	virtual void executeUntil(EmuTime::param time, int userData);
+	virtual const std::string& schedName() const;
+	MSXMotherBoardImpl& motherBoard;
+};
 
-MSXMotherBoardImpl::MSXMotherBoardImpl(MSXMotherBoard& self_, Reactor& reactor_)
+static unsigned machineIDCounter = 0;
+
+MSXMotherBoardImpl::MSXMotherBoardImpl(
+		MSXMotherBoard& self_, Reactor& reactor_, FilePool& filePool_)
 	: self(self_)
 	, reactor(reactor_)
+	, filePool(filePool_)
+	, machineID(StringOp::Builder() << "machine" << ++machineIDCounter)
 	, mapperIOCounter(0)
 	, machineConfig(NULL)
-	, powerSetting(getGlobalSettings().getPowerSetting())
+	, msxEventDistributor(new MSXEventDistributor())
+	, stateChangeDistributor(new StateChangeDistributor())
+	, msxCommandController(new MSXCommandController(
+		reactor.getGlobalCommandController(), reactor,
+		self, *msxEventDistributor, machineID))
+	, scheduler(new Scheduler())
+	, msxMixer(new MSXMixer(
+		reactor.getMixer(), *scheduler, *msxCommandController,
+		reactor.getGlobalSettings()))
+	, fastForwardHelper(new FastForwardHelper(*this))
+	, powerSetting(reactor.getGlobalSettings().getPowerSetting())
+	, reRecordCount(0)
 	, powered(false)
 	, active(false)
 {
 	self.pimple.reset(this);
 
+	slotManager.reset(new CartridgeSlotManager(self));
+	reverseManager.reset(new ReverseManager(self));
 	resetCommand.reset(new ResetCmd(*this));
 	loadMachineCommand.reset(new LoadMachineCmd(*this));
 	listExtCommand.reset(new ListExtCmd(*this));
@@ -318,9 +355,16 @@ MSXMotherBoardImpl::MSXMotherBoardImpl(MSXMotherBoard& self_, Reactor& reactor_)
 	machineNameInfo.reset(new MachineNameInfo(*this));
 	deviceInfo.reset(new DeviceInfo(*this));
 
-	getMSXMixer().mute(); // powered down
-	getRealTime(); // make sure it's instantiated
-	getEventTranslator();
+	msxMixer->mute(); // powered down
+	// TODO: Initialization of this field cannot be done much earlier because
+	//       EventDelay creates a setting, calling getMSXCliComm()
+	//       on MSXMotherBoard, so "pimple" has to be set up already.
+	eventDelay.reset(new EventDelay(
+		*scheduler, *msxCommandController,
+		reactor.getEventDistributor(), *msxEventDistributor,
+		*reverseManager));
+	realTime.reset(new RealTime(
+		self, reactor.getGlobalSettings(), *eventDelay));
 	powerSetting.attach(*this);
 
 	addRemoveUpdate.reset(new AddRemoveUpdate(*this));
@@ -347,7 +391,7 @@ void MSXMotherBoardImpl::deleteMachine()
 			std::cerr << "Internal error: failed to remove "
 			             "extension while deleting a machine: "
 			          << e.getMessage() << std::endl;
-			assert(false);
+			UNREACHABLE;
 		}
 	}
 
@@ -357,10 +401,6 @@ void MSXMotherBoardImpl::deleteMachine()
 
 const string& MSXMotherBoardImpl::getMachineID()
 {
-	if (machineID.empty()) {
-		static unsigned counter = 0;
-		machineID = "machine" + StringOp::toString(++counter);
-	}
 	return machineID;
 }
 
@@ -453,7 +493,7 @@ string MSXMotherBoardImpl::insertExtension(
 
 HardwareConfig* MSXMotherBoardImpl::findExtension(const string& extensionName)
 {
-	for (MSXMotherBoard::Extensions::const_iterator it = extensions.begin();
+	for (Extensions::const_iterator it = extensions.begin();
 	     it != extensions.end(); ++it) {
 		if ((*it)->getName() == extensionName) {
 			return *it;
@@ -462,7 +502,7 @@ HardwareConfig* MSXMotherBoardImpl::findExtension(const string& extensionName)
 	return NULL;
 }
 
-const MSXMotherBoard::Extensions& MSXMotherBoardImpl::getExtensions() const
+const MSXMotherBoardImpl::Extensions& MSXMotherBoardImpl::getExtensions() const
 {
 	return extensions;
 }
@@ -470,7 +510,7 @@ const MSXMotherBoard::Extensions& MSXMotherBoardImpl::getExtensions() const
 void MSXMotherBoardImpl::removeExtension(const HardwareConfig& extension)
 {
 	extension.testRemove();
-	MSXMotherBoard::Extensions::iterator it =
+	Extensions::iterator it =
 		find(extensions.begin(), extensions.end(), &extension);
 	assert(it != extensions.end());
 	self.getMSXCliComm().update(CliComm::EXTENSION, extension.getName(), "remove");
@@ -481,73 +521,38 @@ void MSXMotherBoardImpl::removeExtension(const HardwareConfig& extension)
 CliComm& MSXMotherBoardImpl::getMSXCliComm()
 {
 	if (!msxCliComm.get()) {
-		msxCliComm.reset(new MSXCliComm(self, getGlobalCliComm()));
+		msxCliComm.reset(new MSXCliComm(self, reactor.getGlobalCliComm()));
 	}
 	return *msxCliComm;
 }
 
-CliComm* MSXMotherBoardImpl::getMSXCliCommIfAvailable()
-{
-	return msxCliComm.get();
-}
-
 MSXEventDistributor& MSXMotherBoardImpl::getMSXEventDistributor()
 {
-	if (!msxEventDistributor.get()) {
-		msxEventDistributor.reset(new MSXEventDistributor());
-	}
 	return *msxEventDistributor;
 }
 
+StateChangeDistributor& MSXMotherBoardImpl::getStateChangeDistributor()
+{
+	return *stateChangeDistributor;
+}
+
 MSXCommandController& MSXMotherBoardImpl::getMSXCommandController()
 {
-	if (!msxCommandController.get()) {
-		msxCommandController.reset(new MSXCommandController(
-			reactor.getGlobalCommandController(), self));
-	}
 	return *msxCommandController;
 }
 
 Scheduler& MSXMotherBoardImpl::getScheduler()
 {
-	if (!scheduler.get()) {
-		scheduler.reset(new Scheduler());
-	}
 	return *scheduler;
 }
 
 CartridgeSlotManager& MSXMotherBoardImpl::getSlotManager()
 {
-	if (!slotManager.get()) {
-		slotManager.reset(new CartridgeSlotManager(self));
-	}
 	return *slotManager;
 }
 
-EventDelay& MSXMotherBoardImpl::getEventDelay()
-{
-	if (!eventDelay.get()) {
-		eventDelay.reset(new EventDelay(
-			getScheduler(), getCommandController(),
-			getMSXEventDistributor()));
-	}
-	return *eventDelay;
-}
-
-EventTranslator& MSXMotherBoardImpl::getEventTranslator()
-{
-	if (!eventTranslator.get()) {
-		eventTranslator.reset(new EventTranslator(
-			getEventDistributor(), getEventDelay()));
-	}
-	return *eventTranslator;
-}
-
 RealTime& MSXMotherBoardImpl::getRealTime()
 {
-	if (!realTime.get()) {
-		realTime.reset(new RealTime(self));
-	}
 	return *realTime;
 }
 
@@ -561,10 +566,6 @@ Debugger& MSXMotherBoardImpl::getDebugger()
 
 MSXMixer& MSXMotherBoardImpl::getMSXMixer()
 {
-	if (!msxMixer.get()) {
-		msxMixer.reset(new MSXMixer(reactor.getMixer(), getScheduler(),
-		                            getMSXCommandController()));
-	}
 	return *msxMixer;
 }
 
@@ -576,14 +577,6 @@ PluggingController& MSXMotherBoardImpl::getPluggingController()
 	return *pluggingController;
 }
 
-DummyDevice& MSXMotherBoardImpl::getDummyDevice()
-{
-	if (!dummyDevice.get()) {
-		dummyDevice = DeviceFactory::createDummyDevice(self);
-	}
-	return *dummyDevice;
-}
-
 MSXCPU& MSXMotherBoardImpl::getCPU()
 {
 	// because CPU needs to know if we're emulating turbor or not
@@ -638,7 +631,7 @@ RenShaTurbo& MSXMotherBoardImpl::getRenShaTurbo()
 	if (!renShaTurbo.get()) {
 		assert(getMachineConfig());
 		renShaTurbo.reset(new RenShaTurbo(
-			getCommandController(),
+			*msxCommandController,
 			getMachineConfig()->getConfig()));
 	}
 	return *renShaTurbo;
@@ -647,59 +640,43 @@ RenShaTurbo& MSXMotherBoardImpl::getRenShaTurbo()
 LedStatus& MSXMotherBoardImpl::getLedStatus()
 {
 	if (!ledStatus.get()) {
-		ledStatus.reset(new LedStatus(self));
+		getMSXCliComm(); // force init, to be on the safe side
+		ledStatus.reset(new LedStatus(
+			reactor.getEventDistributor(),
+			*msxCommandController,
+			*msxCliComm));
 	}
 	return *ledStatus;
 }
 
+ReverseManager& MSXMotherBoardImpl::getReverseManager()
+{
+	return *reverseManager;
+}
+
 Reactor& MSXMotherBoardImpl::getReactor()
 {
 	return reactor;
 }
 
-EventDistributor& MSXMotherBoardImpl::getEventDistributor()
-{
-	return reactor.getEventDistributor();
-}
-
-Display& MSXMotherBoardImpl::getDisplay()
-{
-	return reactor.getDisplay();
-}
-
-DiskManipulator& MSXMotherBoardImpl::getDiskManipulator()
-{
-	return reactor.getDiskManipulator();
-}
-
-GlobalSettings& MSXMotherBoardImpl::getGlobalSettings()
-{
-	return reactor.getGlobalSettings();
-}
-
-GlobalCliComm& MSXMotherBoardImpl::getGlobalCliComm()
-{
-	return reactor.getGlobalCliComm();
-}
-
 FilePool& MSXMotherBoardImpl::getFilePool()
 {
-	return reactor.getFilePool();
+	return filePool;
 }
 
 CommandController& MSXMotherBoardImpl::getCommandController()
 {
-	return getMSXCommandController();
+	return *msxCommandController;
 }
 
 InfoCommand& MSXMotherBoardImpl::getMachineInfoCommand()
 {
-	return getMSXCommandController().getMachineInfoCommand();
+	return msxCommandController->getMachineInfoCommand();
 }
 
 EmuTime::param MSXMotherBoardImpl::getCurrentTime()
 {
-	return getScheduler().getCurrentTime();
+	return scheduler->getCurrentTime();
 }
 
 bool MSXMotherBoardImpl::execute()
@@ -713,12 +690,27 @@ bool MSXMotherBoardImpl::execute()
 	return true;
 }
 
+void MSXMotherBoardImpl::fastForward(EmuTime::param time)
+{
+	if (time <= getCurrentTime()) return;
+
+	realTime->disable();
+	msxMixer->mute();
+	fastForwardHelper->setTarget(time);
+	while (time > getCurrentTime()) {
+		// note: this can run (slightly) past the requested time
+		execute();
+	}
+	realTime->enable();
+	msxMixer->unmute();
+}
+
 void MSXMotherBoardImpl::pause()
 {
 	if (getMachineConfig()) {
 		getCPU().setPaused(true);
 	}
-	getMSXMixer().mute();
+	msxMixer->mute();
 }
 
 void MSXMotherBoardImpl::unpause()
@@ -726,7 +718,7 @@ void MSXMotherBoardImpl::unpause()
 	if (getMachineConfig()) {
 		getCPU().setPaused(false);
 	}
-	getMSXMixer().unmute();
+	msxMixer->unmute();
 }
 
 void MSXMotherBoardImpl::addDevice(MSXDevice& device)
@@ -756,8 +748,8 @@ void MSXMotherBoardImpl::doReset()
 	getCPU().doReset(time);
 	// let everyone know we're booting, note that the fact that this is
 	// done after the reset call to the devices is arbitrary here
-	getEventDistributor().distributeEvent(
-		new SimpleEvent<OPENMSX_BOOT_EVENT>());
+	reactor.getEventDistributor().distributeEvent(
+		new SimpleEvent(OPENMSX_BOOT_EVENT));
 }
 
 byte MSXMotherBoardImpl::readIRQVector()
@@ -793,11 +785,11 @@ void MSXMotherBoardImpl::powerUp()
 		(*it)->powerUp(time);
 	}
 	getCPU().doReset(time);
-	getMSXMixer().unmute();
+	msxMixer->unmute();
 	// let everyone know we're booting, note that the fact that this is
 	// done after the reset call to the devices is arbitrary here
-	getEventDistributor().distributeEvent(
-		new SimpleEvent<OPENMSX_BOOT_EVENT>());
+	reactor.getEventDistributor().distributeEvent(
+		new SimpleEvent(OPENMSX_BOOT_EVENT));
 }
 
 void MSXMotherBoardImpl::powerDown()
@@ -812,7 +804,7 @@ void MSXMotherBoardImpl::powerDown()
 	powerSetting.changeValue(false);
 	getLedStatus().setLed(LedStatus::POWER, false);
 
-	getMSXMixer().mute();
+	msxMixer->mute();
 
 	EmuTime::param time = getCurrentTime();
 	for (Devices::iterator it = availableDevices.begin();
@@ -825,11 +817,11 @@ void MSXMotherBoardImpl::activate(bool active_)
 {
 	active = active_;
 	MSXEventDistributor::EventPtr event = active
-		? MSXEventDistributor::EventPtr(new SimpleEvent<OPENMSX_MACHINE_ACTIVATED>())
-		: MSXEventDistributor::EventPtr(new SimpleEvent<OPENMSX_MACHINE_DEACTIVATED>());
-	getMSXEventDistributor().distributeEvent(event, getScheduler().getCurrentTime());
+		? MSXEventDistributor::EventPtr(new SimpleEvent(OPENMSX_MACHINE_ACTIVATED))
+		: MSXEventDistributor::EventPtr(new SimpleEvent(OPENMSX_MACHINE_DEACTIVATED));
+	msxEventDistributor->distributeEvent(event, scheduler->getCurrentTime());
 	if (active) {
-		getRealTime().resync();
+		realTime->resync();
 	}
 }
 bool MSXMotherBoardImpl::isActive() const
@@ -837,13 +829,6 @@ bool MSXMotherBoardImpl::isActive() const
 	return active;
 }
 
-void MSXMotherBoardImpl::exitCPULoopSync()
-{
-	if (getMachineConfig()) {
-		getCPU().exitCPULoopSync();
-	}
-}
-
 void MSXMotherBoardImpl::exitCPULoopAsync()
 {
 	if (getMachineConfig()) {
@@ -851,6 +836,11 @@ void MSXMotherBoardImpl::exitCPULoopAsync()
 	}
 }
 
+void MSXMotherBoardImpl::exitCPULoopSync()
+{
+	getCPU().exitCPULoopSync();
+}
+
 // Observer<Setting>
 void MSXMotherBoardImpl::update(const Setting& setting)
 {
@@ -861,7 +851,7 @@ void MSXMotherBoardImpl::update(const Setting& setting)
 			powerDown();
 		}
 	} else {
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -927,7 +917,7 @@ string MSXMotherBoardImpl::getUserName(const string& hwName)
 	unsigned n = 0;
 	string userName;
 	do {
-		userName = "untitled" + StringOp::toString(++n);
+		userName = StringOp::Builder() << "untitled" << ++n;
 	} while (s.find(userName) != s.end());
 	s.insert(userName);
 	return userName;
@@ -941,19 +931,30 @@ void MSXMotherBoardImpl::freeUserName(const string& hwName,
 	s.erase(userName);
 }
 
+unsigned MSXMotherBoardImpl::getReRecordCount() const
+{
+	return reRecordCount;
+}
+
+void MSXMotherBoardImpl::setReRecordCount(unsigned count)
+{
+	assert(count >= reRecordCount); // can only increase
+	reRecordCount = count;
+}
+
 
 // AddRemoveUpdate
 
 AddRemoveUpdate::AddRemoveUpdate(MSXMotherBoardImpl& motherBoard_)
 	: motherBoard(motherBoard_)
 {
-	motherBoard.getGlobalCliComm().update(
+	motherBoard.getReactor().getGlobalCliComm().update(
 		CliComm::HARDWARE, motherBoard.getMachineID(), "add");
 }
 
 AddRemoveUpdate::~AddRemoveUpdate()
 {
-	motherBoard.getGlobalCliComm().update(
+	motherBoard.getReactor().getGlobalCliComm().update(
 		CliComm::HARDWARE, motherBoard.getMachineID(), "remove");
 }
 
@@ -961,7 +962,7 @@ AddRemoveUpdate::~AddRemoveUpdate()
 // ResetCmd
 ResetCmd::ResetCmd(MSXMotherBoardImpl& motherBoard_)
 	: RecordedCommand(motherBoard_.getCommandController(),
-	                  motherBoard_.getMSXEventDistributor(),
+	                  motherBoard_.getStateChangeDistributor(),
 	                  motherBoard_.getScheduler(),
 	                  "reset")
 	, motherBoard(motherBoard_)
@@ -1023,8 +1024,8 @@ ListExtCmd::ListExtCmd(MSXMotherBoardImpl& motherBoard_)
 void ListExtCmd::execute(const vector<TclObject*>& /*tokens*/,
                          TclObject& result)
 {
-	const MSXMotherBoard::Extensions& extensions = motherBoard.getExtensions();
-	for (MSXMotherBoard::Extensions::const_iterator it = extensions.begin();
+	const MSXMotherBoardImpl::Extensions& extensions = motherBoard.getExtensions();
+	for (MSXMotherBoardImpl::Extensions::const_iterator it = extensions.begin();
 	     it != extensions.end(); ++it) {
 		result.addListElement((*it)->getName());
 	}
@@ -1039,7 +1040,7 @@ string ListExtCmd::help(const vector<string>& /*tokens*/) const
 // ExtCmd
 ExtCmd::ExtCmd(MSXMotherBoardImpl& motherBoard_)
 	: RecordedCommand(motherBoard_.getCommandController(),
-	                  motherBoard_.getMSXEventDistributor(),
+	                  motherBoard_.getStateChangeDistributor(),
 	                  motherBoard_.getScheduler(),
 	                  "ext")
 	, motherBoard(motherBoard_)
@@ -1074,7 +1075,7 @@ void ExtCmd::tabCompletion(vector<string>& tokens) const
 // RemoveExtCmd
 RemoveExtCmd::RemoveExtCmd(MSXMotherBoardImpl& motherBoard_)
 	: RecordedCommand(motherBoard_.getCommandController(),
-	                  motherBoard_.getMSXEventDistributor(),
+	                  motherBoard_.getStateChangeDistributor(),
 	                  motherBoard_.getScheduler(),
 	                  "remove_extension")
 	, motherBoard(motherBoard_)
@@ -1108,7 +1109,7 @@ void RemoveExtCmd::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
 		set<string> names;
-		for (MSXMotherBoard::Extensions::const_iterator it =
+		for (MSXMotherBoardImpl::Extensions::const_iterator it =
 		         motherBoard.getExtensions().begin();
 		     it != motherBoard.getExtensions().end(); ++it) {
 			names.insert((*it)->getName());
@@ -1191,27 +1192,57 @@ void DeviceInfo::tabCompletion(vector<string>& tokens) const
 	}
 }
 
+
+// FastForwardHelper
+
+FastForwardHelper::FastForwardHelper(MSXMotherBoardImpl& motherBoard_)
+	: Schedulable(motherBoard_.getScheduler())
+	, motherBoard(motherBoard_)
+{
+}
+
+void FastForwardHelper::setTarget(EmuTime::param targetTime)
+{
+	setSyncPoint(targetTime);
+}
+
+void FastForwardHelper::executeUntil(EmuTime::param /*time*/, int /*userData*/)
+{
+	motherBoard.exitCPULoopSync();
+}
+
+const string& FastForwardHelper::schedName() const
+{
+	static const string NAME = "FastForwardHelper";
+	return NAME;
+}
+
+
 // serialize
+// version 1: initial version
+// version 2: added reRecordCount
 template<typename Archive>
-void MSXMotherBoardImpl::serialize(Archive& ar, unsigned /*version*/)
+void MSXMotherBoardImpl::serialize(Archive& ar, unsigned version)
 {
 	// don't serialize:
 	//    machineID, userNames, availableDevices, addRemoveUpdate,
 	//    sharedStuffMap, msxCliComm, msxEventDistributor,
-	//    msxCommandController, slotManager, eventDelay, eventTranslator,
-	//    debugger, msxMixer, dummyDevice, panasonicMemory, renShaTurbo,
+	//    msxCommandController, slotManager, eventDelay,
+	//    debugger, msxMixer, panasonicMemory, renShaTurbo,
 	//    ledStatus
 
 	// Scheduler must come early so that devices can query current time
-	ar.serialize("scheduler", getScheduler());
+	ar.serialize("scheduler", *scheduler);
 	// MSXMixer has already set syncpoints, those are invalid now
 	// the following call will fix this
-	getMSXMixer().reschedule();
+	if (ar.isLoader()) {
+		msxMixer->reschedule();
+	}
 
 	ar.serialize("name", machineName);
-	ar.serialize("config", machineConfig2, ref(self));
+	ar.serializeWithID("config", machineConfig2, ref(self));
 	assert(getMachineConfig() == machineConfig2.get());
-	ar.serialize("extensions", extensions, ref(self));
+	ar.serializeWithID("extensions", extensions, ref(self));
 
 	if (mapperIO.get()) {
 		ar.serialize("mapperIO", *mapperIO);
@@ -1230,12 +1261,15 @@ void MSXMotherBoardImpl::serialize(Archive& ar, unsigned /*version*/)
 	if (CassettePort* port = dynamic_cast<CassettePort*>(&getCassettePort())) {
 		ar.serialize("cassetteport", *port);
 	}
+	if (ar.versionAtLeast(version, 2)) {
+		ar.serialize("reRecordCount", reRecordCount);
+	}
 
 	if (ar.isLoader()) {
 		if (powerSetting.getValue()) {
 			powered = true;
 			getLedStatus().setLed(LedStatus::POWER, true);
-			getMSXMixer().unmute();
+			msxMixer->unmute();
 		}
 	}
 }
@@ -1243,9 +1277,9 @@ void MSXMotherBoardImpl::serialize(Archive& ar, unsigned /*version*/)
 
 // MSXMotherBoard
 
-MSXMotherBoard::MSXMotherBoard(Reactor& reactor)
+MSXMotherBoard::MSXMotherBoard(Reactor& reactor, FilePool& filePool)
 {
-	new MSXMotherBoardImpl(*this, reactor);
+	new MSXMotherBoardImpl(*this, reactor, filePool);
 }
 MSXMotherBoard::~MSXMotherBoard()
 {
@@ -1254,17 +1288,13 @@ const string& MSXMotherBoard::getMachineID()
 {
 	return pimple->getMachineID();
 }
-const string& MSXMotherBoard::getMachineName() const
-{
-	return pimple->getMachineName();
-}
 bool MSXMotherBoard::execute()
 {
 	return pimple->execute();
 }
-void MSXMotherBoard::exitCPULoopSync()
+void MSXMotherBoard::fastForward(EmuTime::param time)
 {
-	pimple->exitCPULoopSync();
+	return pimple->fastForward(time);
 }
 void MSXMotherBoard::exitCPULoopAsync()
 {
@@ -1310,10 +1340,6 @@ void MSXMotherBoard::loadMachine(const string& machine)
 {
 	pimple->loadMachine(machine);
 }
-const MSXMotherBoard::Extensions& MSXMotherBoard::getExtensions() const
-{
-	return pimple->getExtensions();
-}
 HardwareConfig* MSXMotherBoard::findExtension(const string& extensionName)
 {
 	return pimple->findExtension(extensionName);
@@ -1336,10 +1362,6 @@ CliComm& MSXMotherBoard::getMSXCliComm()
 	// note: return-type is CliComm instead of MSXCliComm
 	return pimple->getMSXCliComm();
 }
-CliComm* MSXMotherBoard::getMSXCliCommIfAvailable()
-{
-	return pimple->getMSXCliCommIfAvailable();
-}
 MSXCommandController& MSXMotherBoard::getMSXCommandController()
 {
 	return pimple->getMSXCommandController();
@@ -1352,18 +1374,14 @@ MSXEventDistributor& MSXMotherBoard::getMSXEventDistributor()
 {
 	return pimple->getMSXEventDistributor();
 }
+StateChangeDistributor& MSXMotherBoard::getStateChangeDistributor()
+{
+	return pimple->getStateChangeDistributor();
+}
 CartridgeSlotManager& MSXMotherBoard::getSlotManager()
 {
 	return pimple->getSlotManager();
 }
-EventDelay& MSXMotherBoard::getEventDelay()
-{
-	return pimple->getEventDelay();
-}
-EventTranslator& MSXMotherBoard::getEventTranslator()
-{
-	return pimple->getEventTranslator();
-}
 RealTime& MSXMotherBoard::getRealTime()
 {
 	return pimple->getRealTime();
@@ -1380,10 +1398,6 @@ PluggingController& MSXMotherBoard::getPluggingController()
 {
 	return pimple->getPluggingController();
 }
-DummyDevice& MSXMotherBoard::getDummyDevice()
-{
-	return pimple->getDummyDevice();
-}
 MSXCPU& MSXMotherBoard::getCPU()
 {
 	return pimple->getCPU();
@@ -1412,35 +1426,18 @@ LedStatus& MSXMotherBoard::getLedStatus()
 {
 	return pimple->getLedStatus();
 }
+ReverseManager& MSXMotherBoard::getReverseManager()
+{
+	return pimple->getReverseManager();
+}
 Reactor& MSXMotherBoard::getReactor()
 {
 	return pimple->getReactor();
 }
-EventDistributor& MSXMotherBoard::getEventDistributor()
-{
-	return pimple->getEventDistributor();
-}
-Display& MSXMotherBoard::getDisplay()
-{
-	return pimple->getDisplay();
-}
-DiskManipulator& MSXMotherBoard::getDiskManipulator()
-{
-	return pimple->getDiskManipulator();
-}
 FilePool& MSXMotherBoard::getFilePool()
 {
 	return pimple->getFilePool();
 }
-GlobalSettings& MSXMotherBoard::getGlobalSettings()
-{
-	return pimple->getGlobalSettings();
-}
-CliComm& MSXMotherBoard::getGlobalCliComm()
-{
-	// note: return-type is CliComm instead of GlobalCliComm
-	return pimple->getGlobalCliComm();
-}
 CommandController& MSXMotherBoard::getCommandController()
 {
 	return pimple->getCommandController();
@@ -1486,6 +1483,14 @@ void MSXMotherBoard::freeUserName(const string& hwName,
 {
 	pimple->freeUserName(hwName, userName);
 }
+unsigned MSXMotherBoard::getReRecordCount() const
+{
+	return pimple->getReRecordCount();
+}
+void MSXMotherBoard::setReRecordCount(unsigned count)
+{
+	pimple->setReRecordCount(count);
+}
 
 template<typename Archive>
 void MSXMotherBoard::serialize(Archive& ar, unsigned version)
