@@ -1,11 +1,11 @@
-// $Id: MSXPrinterPort.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: MSXPrinterPort.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "MSXPrinterPort.hh"
 #include "DummyPrinterPortDevice.hh"
 #include "MSXMotherBoard.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
-#include <cassert>
+#include "unreachable.hh"
 
 using std::string;
 
@@ -53,7 +53,7 @@ void MSXPrinterPort::writeIO(word port, byte value, EmuTime::param time)
 		writeData(value, time);
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
