@@ -1,11 +1,11 @@
-// $Id: MSXRTC.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: MSXRTC.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "MSXRTC.hh"
 #include "SRAM.hh"
 #include "RP5C01.hh"
 #include "MSXMotherBoard.hh"
 #include "serialize.hh"
-#include <cassert>
+#include "unreachable.hh"
 
 namespace openmsx {
 
@@ -15,7 +15,7 @@ MSXRTC::MSXRTC(MSXMotherBoard& motherBoard, const XMLElement& config)
 	, rp5c01(new RP5C01(motherBoard.getCommandController(), *sram,
 	                    getCurrentTime()))
 {
-	registerLatch = 0; // TODO verify on real hardware
+	reset(getCurrentTime());
 }
 
 MSXRTC::~MSXRTC()
@@ -24,6 +24,10 @@ MSXRTC::~MSXRTC()
 
 void MSXRTC::reset(EmuTime::param time)
 {
+	// TODO verify on real hardware .. how?
+	//  - registerLatch set to zero or some other value?
+	//  - only on power-up or also on reset?
+	registerLatch = 0;
 	rp5c01->reset(time);
 }
 
@@ -47,7 +51,7 @@ void MSXRTC::writeIO(word port, byte value, EmuTime::param time)
 		rp5c01->writePort(registerLatch, value & 0x0F, time);
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
