@@ -1,4 +1,4 @@
-// $Id: MSXTurboRPause.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: MSXTurboRPause.cc 10107 2009-06-30 18:26:06Z m9710797 $
 
 #include "MSXTurboRPause.hh"
 #include "LedStatus.hh"
@@ -24,20 +24,21 @@ MSXTurboRPause::MSXTurboRPause(MSXMotherBoard& motherBoard,
 
 MSXTurboRPause::~MSXTurboRPause()
 {
+	powerDown(EmuTime::dummy());
 	pauseSetting->detach(*this);
 }
 
+void MSXTurboRPause::powerDown(EmuTime::param dummy)
+{
+	writeIO(0, 0, dummy); // send LED OFF events (if needed)
+}
+
 void MSXTurboRPause::reset(EmuTime::param dummy)
 {
 	pauseSetting->changeValue(false);
 	writeIO(0, 0, dummy);
 }
 
-void MSXTurboRPause::powerDown(EmuTime::param dummy)
-{
-	writeIO(0, 0, dummy);
-}
-
 byte MSXTurboRPause::readIO(word port, EmuTime::param time)
 {
 	return peekIO(port, time);
