@@ -1,8 +1,9 @@
-// $Id: Pluggable.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: Pluggable.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "Pluggable.hh"
 #include "PlugException.hh"
 #include "Connector.hh"
+#include "unreachable.hh"
 #include <cassert>
 
 using std::string;
@@ -38,7 +39,11 @@ void Pluggable::plug(Connector& newConnector, EmuTime::param time)
 
 void Pluggable::unplug(EmuTime::param time)
 {
-	unplugHelper(time);
+	try {
+		unplugHelper(time);
+	} catch (MSXException&) {
+		UNREACHABLE;
+	}
 	setConnector(NULL);
 }
 
