@@ -1,7 +1,7 @@
-// $Id: Printer.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: Printer.cc 11035 2009-12-28 22:52:11Z mthuurne $
 
 #include "Printer.hh"
-#include "ScreenShotSaver.hh"
+#include "PNG.hh"
 #include "FileOperations.hh"
 #include "IntegerSetting.hh"
 #include "MSXMotherBoard.hh"
@@ -1703,7 +1703,7 @@ string Paper::save() const
 	for (unsigned y = 0; y < sizeY; ++y) {
 		rowPointers[y] = &buf[sizeX * y];
 	}
-	ScreenShotSaver::saveGrayscale(sizeX, sizeY, rowPointers, filename);
+	PNG::saveGrayscale(sizeX, sizeY, rowPointers, filename);
 	return filename;
 }
 
