@@ -1,4 +1,4 @@
-// $Id: RP5C01.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: RP5C01.cc 8552 2009-02-03 10:03:34Z m9710797 $
 
 #include "RP5C01.hh"
 #include "EnumSetting.hh"
@@ -196,7 +196,7 @@ void RP5C01::updateTimeRegs(EmuTime::param time)
 {
 	if (modeSetting->getValue() == EMUTIME) {
 		// sync with EmuTime, perfect emulation
-		uint64 elapsed = reference.getTicksTill(time);
+		unsigned elapsed = unsigned(reference.getTicksTill(time));
 		reference.advance(time);
 
 		// in test mode increase sec/min/.. at a rate of 16384Hz
