@@ -1,16 +1,19 @@
-// $Id: Reactor.cc 9443 2009-03-31 19:19:47Z m9710797 $
+// $Id: Reactor.cc 11036 2009-12-28 22:52:50Z m9710797 $
 
 #include "Reactor.hh"
 #include "CommandLineParser.hh"
 #include "EventDistributor.hh"
 #include "GlobalCommandController.hh"
-#include "CommandConsole.hh"
 #include "InputEventGenerator.hh"
 #include "InputEvents.hh"
+#include "DiskFactory.hh"
 #include "DiskManipulator.hh"
+#include "DiskChanger.hh"
 #include "FilePool.hh"
 #include "UserSettings.hh"
+#include "RomDatabase.hh"
 #include "MSXMotherBoard.hh"
+#include "StateChangeDistributor.hh"
 #include "Command.hh"
 #include "CommandException.hh"
 #include "GlobalCliComm.hh"
@@ -36,6 +39,7 @@
 #include "openmsx.hh"
 #include "checked_cast.hh"
 #include "statp.hh"
+#include "unreachable.hh"
 #include <cassert>
 #include <memory>
 #include <iostream>
@@ -49,11 +53,11 @@ namespace openmsx {
 class QuitCommand : public SimpleCommand
 {
 public:
-	QuitCommand(CommandController& commandController, Reactor& reactor);
+	QuitCommand(CommandController& commandController, EventDistributor& distributor);
 	virtual string execute(const vector<string>& tokens);
 	virtual string help(const vector<string>& tokens) const;
 private:
-	Reactor& reactor;
+	EventDistributor& distributor;
 };
 
 class MachineCommand : public SimpleCommand
@@ -143,28 +147,6 @@ private:
 	Reactor& reactor;
 };
 
-class StoreMachineMemCommand : public SimpleCommand
-{
-public:
-	StoreMachineMemCommand(CommandController& commandController, Reactor& reactor);
-	virtual string execute(const vector<string>& tokens);
-	virtual string help(const vector<string>& tokens) const;
-	virtual void tabCompletion(vector<string>& tokens) const;
-private:
-	Reactor& reactor;
-};
-
-class RestoreMachineMemCommand : public SimpleCommand
-{
-public:
-	RestoreMachineMemCommand(CommandController& commandController, Reactor& reactor);
-	virtual string execute(const vector<string>& tokens);
-	virtual string help(const vector<string>& tokens) const;
-	virtual void tabCompletion(vector<string>& tokens) const;
-private:
-	Reactor& reactor;
-};
-
 class PollEventGenerator : private Alarm
 {
 public:
@@ -201,20 +183,32 @@ private:
 
 Reactor::Reactor()
 	: mbSem(1)
+	, eventDistributor(new EventDistributor(*this))
+	, globalCliComm(new GlobalCliComm())
+	, globalCommandController(new GlobalCommandController(
+		*eventDistributor, *globalCliComm, *this))
+	, globalSettings(new GlobalSettings(*globalCommandController))
+	, inputEventGenerator(new InputEventGenerator(
+		*globalCommandController, *eventDistributor))
+	, mixer(new Mixer(*globalCommandController))
+	, diskFactory(new DiskFactory(*globalCommandController))
+	, diskManipulator(new DiskManipulator(*globalCommandController, *this))
+	, virtualDrive(new DiskChanger("virtual_drive", *globalCommandController,
+	                               *diskFactory, *diskManipulator, true))
+	, filePool(new FilePool(globalCommandController->getSettingsConfig()))
 	, pauseSetting(getGlobalSettings().getPauseSetting())
 	, pauseOnLostFocusSetting(getGlobalSettings().getPauseOnLostFocusSetting())
-	, userSettings(new UserSettings(getCommandController()))
-	, quitCommand(new QuitCommand(getCommandController(), *this))
-	, machineCommand(new MachineCommand(getCommandController(), *this))
-	, testMachineCommand(new TestMachineCommand(getCommandController(), *this))
-	, createMachineCommand(new CreateMachineCommand(getCommandController(), *this))
-	, deleteMachineCommand(new DeleteMachineCommand(getCommandController(), *this))
-	, listMachinesCommand(new ListMachinesCommand(getCommandController(), *this))
-	, activateMachineCommand(new ActivateMachineCommand(getCommandController(), *this))
-	, storeMachineCommand(new StoreMachineCommand(getCommandController(), *this))
-	, restoreMachineCommand(new RestoreMachineCommand(getCommandController(), *this))
-	, storeMachineMemCommand(new StoreMachineMemCommand(getCommandController(), *this))
-	, restoreMachineMemCommand(new RestoreMachineMemCommand(getCommandController(), *this))
+	, userSettings(new UserSettings(*globalCommandController))
+	, softwareDatabase(new RomDatabase(*globalCommandController, *globalCliComm))
+	, quitCommand(new QuitCommand(*globalCommandController, *eventDistributor))
+	, machineCommand(new MachineCommand(*globalCommandController, *this))
+	, testMachineCommand(new TestMachineCommand(*globalCommandController, *this))
+	, createMachineCommand(new CreateMachineCommand(*globalCommandController, *this))
+	, deleteMachineCommand(new DeleteMachineCommand(*globalCommandController, *this))
+	, listMachinesCommand(new ListMachinesCommand(*globalCommandController, *this))
+	, activateMachineCommand(new ActivateMachineCommand(*globalCommandController, *this))
+	, storeMachineCommand(new StoreMachineCommand(*globalCommandController, *this))
+	, restoreMachineCommand(new RestoreMachineCommand(*globalCommandController, *this))
 	, aviRecordCommand(new AviRecorder(*this))
 	, extensionInfo(new ConfigInfo(getOpenMSXInfoCommand(), "extensions"))
 	, machineInfo  (new ConfigInfo(getOpenMSXInfoCommand(), "machines"))
@@ -227,109 +221,73 @@ Reactor::Reactor()
 
 	pauseSetting.attach(*this);
 
-	getEventDistributor().registerEventListener(
-		OPENMSX_QUIT_EVENT, *this);
-	getEventDistributor().registerEventListener(
-		OPENMSX_FOCUS_EVENT, *this);
+	eventDistributor->registerEventListener(OPENMSX_QUIT_EVENT, *this);
+	eventDistributor->registerEventListener(OPENMSX_FOCUS_EVENT, *this);
+	eventDistributor->registerEventListener(OPENMSX_DELETE_BOARDS, *this);
 }
 
 Reactor::~Reactor()
 {
 	deleteBoard(activeBoard);
 
-	getEventDistributor().unregisterEventListener(
-		OPENMSX_QUIT_EVENT, *this);
-	getEventDistributor().unregisterEventListener(
-		OPENMSX_FOCUS_EVENT, *this);
+	eventDistributor->unregisterEventListener(OPENMSX_QUIT_EVENT, *this);
+	eventDistributor->unregisterEventListener(OPENMSX_FOCUS_EVENT, *this);
+	eventDistributor->unregisterEventListener(OPENMSX_DELETE_BOARDS, *this);
 
 	pauseSetting.detach(*this);
 }
 
 EventDistributor& Reactor::getEventDistributor()
 {
-	if (!eventDistributor.get()) {
-		eventDistributor.reset(new EventDistributor(*this));
-	}
 	return *eventDistributor;
 }
 
 GlobalCommandController& Reactor::getGlobalCommandController()
 {
-	if (!globalCommandController.get()) {
-		globalCommandController.reset(new GlobalCommandController(
-			getEventDistributor(), *this));
-	}
 	return *globalCommandController;
 }
 
 GlobalCliComm& Reactor::getGlobalCliComm()
 {
-	if (!globalCliComm.get()) {
-		globalCliComm.reset(new GlobalCliComm(
-			getGlobalCommandController(), getEventDistributor()));
-	}
 	return *globalCliComm;
 }
 
 CliComm& Reactor::getCliComm()
 {
-	return getGlobalCliComm();
+	return *globalCliComm;
 }
 
 InputEventGenerator& Reactor::getInputEventGenerator()
 {
-	if (!inputEventGenerator.get()) {
-		inputEventGenerator.reset(new InputEventGenerator(
-			getCommandController(), getEventDistributor()));
-	}
 	return *inputEventGenerator;
 }
 
 Display& Reactor::getDisplay()
 {
-	if (!display.get()) {
-		display.reset(new Display(*this));
-		display->createVideoSystem();
-	}
+	assert(display.get());
 	return *display;
 }
 
+RomDatabase& Reactor::getSoftwareDatabase()
+{
+	return *softwareDatabase;
+}
+
 Mixer& Reactor::getMixer()
 {
-	if (!mixer.get()) {
-		mixer.reset(new Mixer(getCommandController()));
-	}
 	return *mixer;
 }
 
-CommandConsole& Reactor::getCommandConsole()
+DiskFactory& Reactor::getDiskFactory()
 {
-	if (!commandConsole.get()) {
-		commandConsole.reset(new CommandConsole(
-			getCommandController(), getEventDistributor(),
-			getDisplay()));
-	}
-	return *commandConsole;
+	return *diskFactory;
 }
 
 DiskManipulator& Reactor::getDiskManipulator()
 {
-	if (!diskManipulator.get()) {
-		diskManipulator.reset(new DiskManipulator(
-			getCommandController()));
-	}
 	return *diskManipulator;
 }
 
-FilePool& Reactor::getFilePool()
-{
-	if (!filePool.get()) {
-		filePool.reset(new FilePool(
-			getCommandController().getSettingsConfig()));
-	}
-	return *filePool;
-}
-
 EnumSetting<int>& Reactor::getMachineSetting()
 {
 	return *machineSetting;
@@ -337,17 +295,17 @@ EnumSetting<int>& Reactor::getMachineSetting()
 
 GlobalSettings& Reactor::getGlobalSettings()
 {
-	return getCommandController().getGlobalSettings();
+	return *globalSettings;
 }
 
 CommandController& Reactor::getCommandController()
 {
-	return getGlobalCommandController();
+	return *globalCommandController;
 }
 
 InfoCommand& Reactor::getOpenMSXInfoCommand()
 {
-	return getGlobalCommandController().getOpenMSXInfoCommand();
+	return globalCommandController->getOpenMSXInfoCommand();
 }
 
 void Reactor::getHwConfigs(const string& type, set<string>& result)
@@ -384,7 +342,7 @@ void Reactor::createMachineSetting()
 	machines["C-BIOS_MSX2+"] = 0; // default machine
 
 	machineSetting.reset(new EnumSetting<int>(
-		getCommandController(), "default_machine",
+		*globalCommandController, "default_machine",
 		"default machine (takes effect next time openMSX is started)",
 		0, machines));
 }
@@ -419,8 +377,29 @@ Reactor::Board Reactor::getMachine(const string& machineID) const
 	throw CommandException("No machine with ID: " + machineID);
 }
 
+Reactor::Board Reactor::createEmptyMotherBoard()
+{
+	return Board(new MSXMotherBoard(*this, *filePool));
+}
+
+void Reactor::replaceActiveBoard(Board newBoard)
+{
+	boards.push_back(newBoard);
+	Board oldBoard = activeBoard;
+	switchBoard(newBoard);
+	deleteBoard(oldBoard);
+}
+
 void Reactor::switchMachine(const string& machine)
 {
+	if (!display.get()) {
+		display.reset(new Display(*this));
+		// TODO: Currently it is not possible to move this call into the
+		//       constructor of Display because the call to createVideoSystem()
+		//       indirectly calls Reactor.getDisplay().
+		display->createVideoSystem();
+	}
+
 	// create+load new machine
 	// switch to new machine
 	// delete old active machine
@@ -428,7 +407,7 @@ void Reactor::switchMachine(const string& machine)
 	assert(Thread::isMainThread());
 	// Note: loadMachine can throw an exception and in that case the
 	//       motherboard must be considered as not created at all.
-	Board newBoard(new MSXMotherBoard(*this));
+	Board newBoard = createEmptyMotherBoard();
 	newBoard->loadMachine(machine);
 	boards.push_back(newBoard);
 
@@ -454,9 +433,9 @@ void Reactor::switchBoard(Board newBoard)
 		ScopedLock lock(mbSem);
 		activeBoard = newBoard;
 	}
-	getEventDistributor().distributeEvent(
-		new SimpleEvent<OPENMSX_MACHINE_LOADED_EVENT>());
-	getGlobalCliComm().update(CliComm::HARDWARE, getMachineID(), "select");
+	eventDistributor->distributeEvent(
+		new SimpleEvent(OPENMSX_MACHINE_LOADED_EVENT));
+	globalCliComm->update(CliComm::HARDWARE, getMachineID(), "select");
 	if (activeBoard.get()) {
 		activeBoard->activate(true);
 	}
@@ -479,6 +458,8 @@ void Reactor::deleteBoard(Board board)
 	// board. Instead remember this board and delete it at a safe moment
 	// in time.
 	garbageBoards.push_back(board);
+	eventDistributor->distributeEvent(
+		new SimpleEvent(OPENMSX_DELETE_BOARDS));
 }
 
 void Reactor::enterMainLoop()
@@ -496,9 +477,7 @@ void Reactor::enterMainLoop()
 
 void Reactor::run(CommandLineParser& parser)
 {
-	GlobalCommandController& commandController = getGlobalCommandController();
-	getDiskManipulator(); // make sure it gets instantiated
-	                      // (also on machines without disk drive)
+	GlobalCommandController& commandController = *globalCommandController;
 
 	PRT_DEBUG("Reactor::run Trying to execute init.tcl...");
 
@@ -542,11 +521,11 @@ void Reactor::run(CommandLineParser& parser)
 	}
 
 	PRT_DEBUG("Reactor::run Instantiate poll event generator...");
-	PollEventGenerator pollEventGenerator(getEventDistributor());
+	PollEventGenerator pollEventGenerator(*eventDistributor);
 
 	while (running) {
-		garbageBoards.clear(); // see deleteBoard()
-		getEventDistributor().deliverEvents();
+		eventDistributor->deliverEvents();
+		assert(garbageBoards.empty());
 		MSXMotherBoard* motherboard = activeBoard.get();
 		bool blocked = (blockedCounter > 0) || !motherboard;
 		if (!blocked) blocked = !motherboard->execute();
@@ -557,7 +536,7 @@ void Reactor::run(CommandLineParser& parser)
 			// to also use a sleep/poll loop, with even shorter
 			// sleep periods as we use here. Maybe in future
 			// SDL implementations this will be improved.
-			getEventDistributor().sleep(100 * 1000);
+			eventDistributor->sleep(100 * 1000);
 		}
 	}
 }
@@ -566,7 +545,7 @@ void Reactor::unpause()
 {
 	if (paused) {
 		paused = false;
-		getGlobalCliComm().update(CliComm::STATUS, "paused", "false");
+		globalCliComm->update(CliComm::STATUS, "paused", "false");
 		unblock();
 	}
 }
@@ -575,7 +554,7 @@ void Reactor::pause()
 {
 	if (!paused) {
 		paused = true;
-		getGlobalCliComm().update(CliComm::STATUS, "paused", "true");
+		globalCliComm->update(CliComm::STATUS, "paused", "true");
 		block();
 	}
 }
@@ -584,14 +563,14 @@ void Reactor::block()
 {
 	++blockedCounter;
 	enterMainLoop();
-	getMixer().mute();
+	mixer->mute();
 }
 
 void Reactor::unblock()
 {
 	--blockedCounter;
 	assert(blockedCounter >= 0);
-	getMixer().unmute();
+	mixer->unmute();
 }
 
 
@@ -612,7 +591,8 @@ bool Reactor::signalEvent(shared_ptr<const Event> event)
 {
 	EventType type = event->getType();
 	if (type == OPENMSX_QUIT_EVENT) {
-		getCommandController().executeCommand("exit");
+		enterMainLoop();
+		running = false;
 	} else if (type == OPENMSX_FOCUS_EVENT) {
 		if (!pauseOnLostFocusSetting.getValue()) return true;
 		const FocusEvent& focusEvent = checked_cast<const FocusEvent&>(*event);
@@ -625,27 +605,28 @@ bool Reactor::signalEvent(shared_ptr<const Event> event)
 			// lost focus
 			pause();
 		}
+	} else if (type == OPENMSX_DELETE_BOARDS) {
+		assert(!garbageBoards.empty());
+		garbageBoards.erase(garbageBoards.begin());
 	} else {
-		assert(false); // we didn't subscribe to this event...
+		UNREACHABLE; // we didn't subscribe to this event...
 	}
 	return true;
 }
 
 
 // class QuitCommand
-// TODO: Unify QuitCommand and OPENMSX_QUIT_EVENT.
 
 QuitCommand::QuitCommand(CommandController& commandController,
-                         Reactor& reactor_)
+                         EventDistributor& distributor_)
 	: SimpleCommand(commandController, "exit")
-	, reactor(reactor_)
+	, distributor(distributor_)
 {
 }
 
 string QuitCommand::execute(const vector<string>& /*tokens*/)
 {
-	reactor.enterMainLoop();
-	reactor.running = false;
+	distributor.distributeEvent(new QuitEvent());
 	return "";
 }
 
@@ -710,7 +691,7 @@ string TestMachineCommand::execute(const vector<string>& tokens)
 		throw SyntaxError();
 	}
 	try {
-		MSXMotherBoard mb(reactor);
+		MSXMotherBoard mb(reactor, *reactor.filePool);
 		mb.loadMachine(tokens[1]);
 		return ""; // success
 	} catch (MSXException& e) {
@@ -747,7 +728,7 @@ string CreateMachineCommand::execute(const vector<string>& tokens)
 	if (tokens.size() != 1) {
 		throw SyntaxError();
 	}
-	Reactor::Board newBoard(new MSXMotherBoard(reactor));
+	Reactor::Board newBoard = reactor.createEmptyMotherBoard();
 	reactor.boards.push_back(newBoard);
 	return newBoard->getMachineID();
 }
@@ -925,7 +906,7 @@ RestoreMachineCommand::RestoreMachineCommand(
 
 string RestoreMachineCommand::execute(const vector<string>& tokens)
 {
-	Reactor::Board newBoard(new MSXMotherBoard(reactor));
+	Reactor::Board newBoard = reactor.createEmptyMotherBoard();
 
 	string filename;
 	string machineID;
@@ -969,6 +950,12 @@ string RestoreMachineCommand::execute(const vector<string>& tokens)
 	} catch (MSXException& e) {
 		throw CommandException("Cannot load state: " + e.getMessage());
 	}
+
+	// Savestate also contains stuff like the keyboard state at the moment
+	// the snapshot was created (this is required for reverse/replay). But
+	// now we want the MSX to see the actual host keyboard state.
+	newBoard->getStateChangeDistributor().stopReplay(newBoard->getCurrentTime());
+
 	reactor.boards.push_back(newBoard);
 	return newBoard->getMachineID();
 }
@@ -991,72 +978,6 @@ void RestoreMachineCommand::tabCompletion(vector<string>& tokens) const
 }
 
 
-// class StoreMachineMemCommand   TODO temp code
-
-StoreMachineMemCommand::StoreMachineMemCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: SimpleCommand(commandController, "store_machine_mem")
-	, reactor(reactor_)
-{
-}
-
-string StoreMachineMemCommand::execute(const vector<string>& tokens)
-{
-	if (tokens.size() != 2) {
-		throw SyntaxError();
-	}
-	Reactor::Board board = reactor.getMachine(tokens[1]);
-
-	reactor.snapshot.clear();
-	MemOutputArchive out(reactor.snapshot);
-	out.serialize("machine", *board);
-	return StringOp::toString(reactor.snapshot.size()); // size in bytes
-}
-
-string StoreMachineMemCommand::help(const vector<string>& /*tokens*/) const
-{
-	return "This is an experimental command. "
-	       "It will change in the next release.";
-}
-
-void StoreMachineMemCommand::tabCompletion(vector<string>& tokens) const
-{
-	set<string> ids;
-	reactor.getMachineIDs(ids);
-	completeString(tokens, ids);
-}
-
-
-// class RestoreMachineMemCommand   TODO temp code
-
-RestoreMachineMemCommand::RestoreMachineMemCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: SimpleCommand(commandController, "restore_machine_mem")
-	, reactor(reactor_)
-{
-}
-
-string RestoreMachineMemCommand::execute(const vector<string>& /*tokens*/)
-{
-	Reactor::Board newBoard(new MSXMotherBoard(reactor));
-	MemInputArchive in(reactor.snapshot);
-	in.serialize("machine", *newBoard);
-	reactor.boards.push_back(newBoard);
-	return newBoard->getMachineID();
-}
-
-string RestoreMachineMemCommand::help(const vector<string>& /*tokens*/) const
-{
-	return "This is an experimental command. "
-	       "It will change in the next release.";
-}
-
-void RestoreMachineMemCommand::tabCompletion(vector<string>& /*tokens*/) const
-{
-	// TODO
-}
-
-
 // class PollEventGenerator
 
 PollEventGenerator::PollEventGenerator(EventDistributor& eventDistributor_)
@@ -1072,7 +993,7 @@ PollEventGenerator::~PollEventGenerator()
 
 bool PollEventGenerator::alarm()
 {
-	eventDistributor.distributeEvent(new SimpleEvent<OPENMSX_POLL_EVENT>());
+	eventDistributor.distributeEvent(new SimpleEvent(OPENMSX_POLL_EVENT));
 	return true; // reschedule
 }
 
