@@ -1,4 +1,4 @@
-// $Id: RealTime.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: RealTime.cc 10824 2009-11-28 17:47:58Z m9710797 $
 
 #include "RealTime.hh"
 #include "Timer.hh"
@@ -8,6 +8,7 @@
 #include "FinishFrameEvent.hh"
 #include "GlobalSettings.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
 #include "ThrottleManager.hh"
@@ -20,14 +21,19 @@ const double             SYNC_INTERVAL = 0.08;  // s
 const long long          MAX_LAG       = 200000; // us
 const unsigned long long ALLOWED_LAG   =  20000; // us
 
-RealTime::RealTime(MSXMotherBoard& motherBoard_)
+RealTime::RealTime(
+		MSXMotherBoard& motherBoard_, GlobalSettings& globalSettings,
+		EventDelay& eventDelay_)
 	: Schedulable(motherBoard_.getScheduler())
 	, motherBoard(motherBoard_)
-	, throttleManager(motherBoard.getGlobalSettings().getThrottleManager())
-	, speedSetting   (motherBoard.getGlobalSettings().getSpeedSetting())
-	, pauseSetting   (motherBoard.getGlobalSettings().getPauseSetting())
-	, powerSetting   (motherBoard.getGlobalSettings().getPowerSetting())
+	, eventDistributor(motherBoard.getReactor().getEventDistributor())
+	, eventDelay(eventDelay_)
+	, throttleManager(globalSettings.getThrottleManager())
+	, speedSetting   (globalSettings.getSpeedSetting())
+	, pauseSetting   (globalSettings.getPauseSetting())
+	, powerSetting   (globalSettings.getPowerSetting())
 	, emuTime(EmuTime::zero)
+	, enabled(true)
 {
 	speedSetting.attach(*this);
 	throttleManager.attach(*this);
@@ -36,14 +42,12 @@ RealTime::RealTime(MSXMotherBoard& motherBoard_)
 
 	resync();
 
-	EventDistributor& eventDistributor = motherBoard.getEventDistributor();
 	eventDistributor.registerEventListener(OPENMSX_FINISH_FRAME_EVENT, *this);
 	eventDistributor.registerEventListener(OPENMSX_FRAME_DRAWN_EVENT,  *this);
 }
 
 RealTime::~RealTime()
 {
-	EventDistributor& eventDistributor = motherBoard.getEventDistributor();
 	eventDistributor.unregisterEventListener(OPENMSX_FRAME_DRAWN_EVENT,  *this);
 	eventDistributor.unregisterEventListener(OPENMSX_FINISH_FRAME_EVENT, *this);
 
@@ -112,7 +116,7 @@ void RealTime::internalSync(EmuTime::param time, bool allowSleep)
 		}
 	}
 	if (allowSleep) {
-		motherBoard.getEventDelay().sync(time);
+		eventDelay.sync(time);
 	}
 
 	emuTime = time;
@@ -132,7 +136,7 @@ const std::string& RealTime::schedName() const
 
 bool RealTime::signalEvent(shared_ptr<const Event> event)
 {
-	if (!motherBoard.isActive()) {
+	if (!motherBoard.isActive() || !enabled) {
 		// these are global events, only the active machine should
 		// synchronize with real time
 		return true;
@@ -163,6 +167,8 @@ void RealTime::update(const ThrottleManager& /*throttleManager*/)
 
 void RealTime::resync()
 {
+	if (!enabled) return;
+
 	idealRealTime = Timer::getTime();
 	sleepAdjust = 0.0;
 	removeSyncPoint();
@@ -170,4 +176,16 @@ void RealTime::resync()
 	setSyncPoint(emuTime + getEmuDuration(SYNC_INTERVAL));
 }
 
+void RealTime::enable()
+{
+	enabled = true;
+	resync();
+}
+
+void RealTime::disable()
+{
+	enabled = false;
+	removeSyncPoint();
+}
+
 } // namespace openmsx
