@@ -1,4 +1,4 @@
-// $Id: Scheduler.cc 8452 2009-01-08 22:38:26Z m9710797 $
+// $Id: Scheduler.cc 10139 2009-07-02 19:01:01Z m9710797 $
 
 #include "Scheduler.hh"
 #include "Schedulable.hh"
@@ -194,7 +194,7 @@ void SynchronizationPoint::serialize(Archive& ar, unsigned /*version*/)
 	// Schedulable has a collection of SynchronizationPoints, all with the
 	// same Schedulable. So there's no need to serialize 'device'.
 	//Schedulable* device;
-	ar.serializeNoID("time", timeStamp);
+	ar.serialize("time", timeStamp);
 	ar.serialize("type", userData);
 }
 INSTANTIATE_SERIALIZE_METHODS(SynchronizationPoint);
