@@ -1,4 +1,4 @@
-// $Id: CassettePlayer.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: CassettePlayer.cc 11341 2010-03-20 20:06:21Z m9710797 $
 
 // TODO:
 // - improve consistency when a reset occurs: tape is removed when you were
@@ -28,7 +28,6 @@
 #include "CassettePort.hh"
 #include "CommandController.hh"
 #include "RecordedCommand.hh"
-#include "GlobalSettings.hh"
 #include "XMLElement.hh"
 #include "FileContext.hh"
 #include "WavImage.hh"
@@ -43,6 +42,7 @@
 #include "DynamicClock.hh"
 #include "Clock.hh"
 #include "StringOp.hh"
+#include "unreachable.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -65,7 +65,7 @@ class TapeCommand : public RecordedCommand
 {
 public:
 	TapeCommand(CommandController& commandController,
-	            MSXEventDistributor& msxEventDistributor,
+	            StateChangeDistributor& stateChangeDistributor,
 	            Scheduler& scheduler,
 	            CassettePlayer& cassettePlayer);
 	virtual string execute(const vector<string>& tokens, EmuTime::param time);
@@ -79,11 +79,13 @@ private:
 CassettePlayer::CassettePlayer(
 		CommandController& commandController_,
 		MSXMixer& mixer, Scheduler& scheduler,
-		MSXEventDistributor& msxEventDistributor,
+		StateChangeDistributor& stateChangeDistributor,
 		EventDistributor& eventDistributor_,
-		CliComm& cliComm_)
+		CliComm& cliComm_,
+		EnumSetting<ResampleType>& resampleSetting,
+		ThrottleManager& throttleManager)
 	: SoundDevice(mixer, getName(), getDescription(), 1)
-	, Resample(commandController_.getGlobalSettings(), 1)
+	, Resample(resampleSetting)
 	, Schedulable(scheduler)
 	, tapePos(EmuTime::zero)
 	, prevSyncTime(EmuTime::zero)
@@ -91,10 +93,9 @@ CassettePlayer::CassettePlayer(
 	, commandController(commandController_)
 	, cliComm(cliComm_)
 	, eventDistributor(eventDistributor_)
-	, tapeCommand(new TapeCommand(commandController, msxEventDistributor,
+	, tapeCommand(new TapeCommand(commandController, stateChangeDistributor,
 	                              scheduler, *this))
-	, loadingIndicator(new LoadingIndicator(
-	       commandController.getGlobalSettings().getThrottleManager()))
+	, loadingIndicator(new LoadingIndicator(throttleManager))
 	, autoRunSetting(new BooleanSetting(commandController,
 		"autoruncassettes", "automatically try to run cassettes", false))
 	, sampcnt(0)
@@ -149,7 +150,7 @@ void CassettePlayer::autoRun()
 			loadingInstruction = "CLOAD\\\\rRUN";
 			break;
 		default:
-			assert(false); // Shouldn't be possible
+			UNREACHABLE; // Shouldn't be possible
 	}
 	string var = "::auto_run_cas_counter";
 	string command =
@@ -178,8 +179,7 @@ string CassettePlayer::getStateString() const
 		case RECORD: return "record";
 		case STOP:   return "stop";
 	}
-	assert(false);
-	return "";
+	UNREACHABLE; return "";
 }
 
 bool CassettePlayer::isRolling() const
@@ -232,7 +232,7 @@ void CassettePlayer::checkInvariants() const
 		assert(!playImage.get());
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -257,7 +257,7 @@ void CassettePlayer::setState(State newState, const Filename& newImage,
 		recordImage.reset();
 		if (empty) {
 			// delete the created WAV file, as it is useless
-			FileOperations::unlink(getImageName().getResolved());
+			FileOperations::unlink(getImageName().getResolved()); // ignore errors
 			setImageName(Filename());
 		}
 	}
@@ -364,7 +364,7 @@ void CassettePlayer::rewind(EmuTime::param time)
 void CassettePlayer::recordTape(const Filename& filename, EmuTime::param time)
 {
 	removeTape(time); // flush (possible) previous recording
-	recordImage.reset(new WavWriter(filename, 1, 8, RECORD_FREQ));
+	recordImage.reset(new Wav8Writer(filename, 1, RECORD_FREQ));
 	tapePos = EmuTime::zero;
 	setState(RECORD, filename, time);
 }
@@ -492,9 +492,14 @@ void CassettePlayer::fillBuf(size_t length, double x)
 
 void CassettePlayer::flushOutput()
 {
-	recordImage->write8mono(buf, unsigned(sampcnt));
-	sampcnt = 0;
-	recordImage->flush(); // update wav header
+	try {
+		recordImage->write(buf, 1, unsigned(sampcnt));
+		sampcnt = 0;
+		recordImage->flush(); // update wav header
+	} catch (MSXException& e) {
+		cliComm.printWarning(
+			"Failed to write to tape: " + e.getMessage());
+	}
 }
 
 
@@ -522,6 +527,7 @@ void CassettePlayer::plugHelper(Connector& connector, EmuTime::param time)
 
 void CassettePlayer::unplugHelper(EmuTime::param time)
 {
+	// note: may not throw exceptions
 	setState(STOP, getImageName(), time); // keep current image
 }
 
@@ -532,7 +538,7 @@ void CassettePlayer::setOutputRate(unsigned newOutputRate)
 	unsigned inputRate = playImage.get() ? playImage->getFrequency()
 	                                     : outputRate;
 	setInputRate(inputRate);
-	setResampleRatio(inputRate, outputRate);
+	setResampleRatio(inputRate, outputRate, isStereo());
 }
 
 void CassettePlayer::generateChannels(int** buffers, unsigned num)
@@ -611,10 +617,10 @@ void CassettePlayer::executeUntil(EmuTime::param time, int userData)
 // class TapeCommand
 
 TapeCommand::TapeCommand(CommandController& commandController,
-                         MSXEventDistributor& msxEventDistributor,
+                         StateChangeDistributor& stateChangeDistributor,
                          Scheduler& scheduler,
                          CassettePlayer& cassettePlayer_)
-	: RecordedCommand(commandController, msxEventDistributor,
+	: RecordedCommand(commandController, stateChangeDistributor,
 	                  scheduler, "cassetteplayer")
 	, cassettePlayer(cassettePlayer_)
 {
@@ -622,31 +628,46 @@ TapeCommand::TapeCommand(CommandController& commandController,
 
 string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 {
-	string result;
+	StringOp::Builder result;
 	if (tokens.size() == 1) {
+		Interpreter& interpreter = getInterpreter();
 		// Returning Tcl lists here, similar to the disk commands in
 		// DiskChanger
-		TclObject tmp(getCommandController().getInterpreter());
+		TclObject tmp(interpreter);
 		tmp.addListElement(getName() + ':');
 		tmp.addListElement(cassettePlayer.getImageName().getResolved());
 
-		TclObject options(getCommandController().getInterpreter());
+		TclObject options(interpreter);
 		options.addListElement(cassettePlayer.getStateString());
 		tmp.addListElement(options);
-		result += tmp.getString();
+		result << tmp.getString();
 
 	} else if (tokens[1] == "new") {
+		string directory = "taperecordings";
+		string extension = ".wav";
 		string filename = (tokens.size() == 3)
 		                ? tokens[2]
 		                : FileOperations::getNextNumberedFileName(
-		                         "taperecordings", "openmsx", ".wav");
+		                         directory, "openmsx", extension);
+
+		if (FileOperations::getExtension(filename).empty()) {
+			// no extension given, append standard one
+			filename += extension;
+		}
+
+		if (FileOperations::getBaseName(filename).empty()) {
+			// no dir given, use standard dir
+			filename = FileOperations::getUserOpenMSXDir() + "/" +
+				directory + "/" + filename;
+		}
+
 		cassettePlayer.recordTape(Filename(filename), time);
-		result += "Created new cassette image file: " + filename
-		       + ", inserted it and set recording mode.";
+		result << "Created new cassette image file: " << filename
+		       << ", inserted it and set recording mode.";
 
 	} else if (tokens[1] == "insert" && tokens.size() == 3) {
 		try {
-			result += "Changing tape";
+			result << "Changing tape";
 			Filename filename(tokens[2], getCommandController());
 			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
@@ -656,10 +677,10 @@ string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 	} else if (tokens[1] == "motorcontrol" && tokens.size() == 3) {
 		if (tokens[2] == "on") {
 			cassettePlayer.setMotorControl(true, time);
-			result += "Motor control enabled.";
+			result << "Motor control enabled.";
 		} else if (tokens[2] == "off") {
 			cassettePlayer.setMotorControl(false, time);
-			result += "Motor control disabled.";
+			result << "Motor control disabled.";
 		} else {
 			throw SyntaxError();
 		}
@@ -668,16 +689,16 @@ string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 		throw SyntaxError();
 
 	} else if (tokens[1] == "motorcontrol") {
-		result += "Motor control is ";
-		result += cassettePlayer.motorControl ? "on" : "off";
+		result << "Motor control is "
+		       << (cassettePlayer.motorControl ? "on" : "off");
 
 	} else if (tokens[1] == "record") {
-			result += "TODO: implement this... (sorry)";
+			result << "TODO: implement this... (sorry)";
 
 	} else if (tokens[1] == "play") {
 		if (cassettePlayer.getState() == CassettePlayer::RECORD) {
 			try {
-				result += "Play mode set, rewinding tape.";
+				result << "Play mode set, rewinding tape.";
 				cassettePlayer.playTape(
 					cassettePlayer.getImageName(), time);
 			} catch (MSXException& e) {
@@ -687,17 +708,17 @@ string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 			throw CommandException("No tape inserted or tape at end!");
 		} else {
 			// PLAY mode
-			result += "Already in play mode.";
+			result << "Already in play mode.";
 		}
 
 	} else if (tokens[1] == "eject") {
-		result += "Tape ejected";
+		result << "Tape ejected";
 		cassettePlayer.removeTape(time);
 
 	} else if (tokens[1] == "rewind") {
 		if (cassettePlayer.getState() == CassettePlayer::RECORD) {
 			try {
-				result += "First stopping recording... ";
+				result << "First stopping recording... ";
 				cassettePlayer.playTape(
 					cassettePlayer.getImageName(), time);
 			} catch (MSXException& e) {
@@ -705,17 +726,17 @@ string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 			}
 		}
 		cassettePlayer.rewind(time);
-		result += "Tape rewound";
+		result << "Tape rewound";
 
 	} else if (tokens[1] == "getpos") {
-		result += StringOp::toString(cassettePlayer.getTapePos(time));
+		result << cassettePlayer.getTapePos(time);
 
 	} else if (tokens[1] == "getlength") {
-		result += StringOp::toString(cassettePlayer.getTapeLength(time));
+		result << cassettePlayer.getTapeLength(time);
 
 	} else {
 		try {
-			result += "Changing tape";
+			result << "Changing tape";
 			Filename filename(tokens[1], getCommandController());
 			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
@@ -847,10 +868,6 @@ void CassettePlayer::serialize(Archive& ar, unsigned /*version*/)
 		// buf, sampcnt
 		flushOutput();
 	}
-	if (state == RECORD) {
-		// TODO we don't support savestates in RECORD mode yet
-		setState(STOP, getImageName(), getCurrentTime());
-	}
 
 	ar.serialize("casImage", casImage);
 	if (ar.isLoader()) {
@@ -875,6 +892,10 @@ void CassettePlayer::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("motorControl", motorControl);
 
 	if (ar.isLoader()) {
+		if (state == RECORD) {
+			// TODO we don't support savestates in RECORD mode yet
+			setState(STOP, getImageName(), getCurrentTime());
+		}
 		updateLoadingState(getCurrentTime());
 	}
 }
