@@ -1,14 +1,16 @@
-// $Id: CassettePort.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: CassettePort.cc 11231 2010-02-03 17:51:48Z m9710797 $
 
 #include "CassettePort.hh"
 #include "CassetteDevice.hh"
 #include "CassettePlayer.hh"
 #include "components.hh"
-#ifdef COMPONENT_JACK
-#include "CassetteJack.hh"
+#if COMPONENT_LASERDISC
+#include "LaserdiscPlayer.hh"
 #endif
 #include "DummyCassetteDevice.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "PluggingController.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
@@ -44,6 +46,12 @@ bool DummyCassettePort::lastOut() const
 {
 	return false; // not relevant
 }
+#if COMPONENT_LASERDISC
+void DummyCassettePort::setLaserdiscPlayer(LaserdiscPlayer* /* laserdisc */)
+{
+	// do nothing
+}
+#endif
 
 
 // CassettePort
@@ -52,35 +60,36 @@ CassettePort::CassettePort(MSXMotherBoard& motherBoard_)
 	: Connector(motherBoard_.getPluggingController(), "cassetteport",
 	            auto_ptr<Pluggable>(new DummyCassetteDevice()))
 	, motherBoard(motherBoard_)
+#if COMPONENT_LASERDISC
+	, laserdiscPlayer(NULL)
+#endif
 	, lastOutput(false)
+	, motorControl(false)
 {
 	cassettePlayer.reset(new CassettePlayer(
 		motherBoard.getCommandController(),
 		motherBoard.getMSXMixer(),
 		motherBoard.getScheduler(),
-		motherBoard.getMSXEventDistributor(),
-		motherBoard.getEventDistributor(),
-		motherBoard.getMSXCliComm()));
+		motherBoard.getStateChangeDistributor(),
+		motherBoard.getReactor().getEventDistributor(),
+		motherBoard.getMSXCliComm(),
+		motherBoard.getReactor().getGlobalSettings().getResampleSetting(),
+		motherBoard.getReactor().getGlobalSettings().getThrottleManager()
+		));
 	getPluggingController().registerPluggable(cassettePlayer.get());
-#ifdef COMPONENT_JACK
-	cassetteJack.reset(new CassetteJack(motherBoard.getScheduler()));
-	getPluggingController().registerPluggable(cassetteJack.get());
-#endif
 }
 
 CassettePort::~CassettePort()
 {
 	unplug(motherBoard.getCurrentTime());
 	getPluggingController().unregisterPluggable(cassettePlayer.get());
-#ifdef COMPONENT_JACK
-	getPluggingController().unregisterPluggable(cassetteJack.get());
-#endif
 }
 
 
 void CassettePort::setMotor(bool status, EmuTime::param time)
 {
 	// TODO make 'click' sound
+	motorControl = status;
 	getPluggedCasDev().setMotor(status, time);
 }
 
@@ -101,11 +110,26 @@ bool CassettePort::cassetteIn(EmuTime::param time)
 	// All analog filtering is ignored for now
 	//   only important component is DC-removal
 	//   we just assume sample has no DC component
-	short sample = getPluggedCasDev().readSample(time); // read 1 sample
+	short sample;
+#if COMPONENT_LASERDISC
+	if (!motorControl && laserdiscPlayer != NULL) {
+		sample = laserdiscPlayer->readSample(time);
+	} else
+#endif
+	{
+		sample = getPluggedCasDev().readSample(time); // read 1 sample
+	}
 	bool result = (sample >= 0); // comparator
 	return result;
 }
 
+#if COMPONENT_LASERDISC
+void CassettePort::setLaserdiscPlayer(LaserdiscPlayer *laserdiscPlayer_)
+{
+	laserdiscPlayer = laserdiscPlayer_;
+}
+#endif
+
 void CassettePort::unplug(EmuTime::param time)
 {
 	Connector::unplug(time);
