@@ -1,8 +1,11 @@
-// $Id: Command.cc 5830 2006-11-02 19:44:55Z m9710797 $
+// $Id: Command.cc 10710 2009-10-25 16:56:05Z mthuurne $
 
 #include "Command.hh"
 #include "CommandController.hh"
+#include "GlobalCommandController.hh"
+#include "MSXCommandController.hh"
 #include "TclObject.hh"
+#include "checked_cast.hh"
 
 using std::vector;
 using std::string;
@@ -28,16 +31,47 @@ CommandCompleter::~CommandCompleter()
 	}
 }
 
+// TODO: getCommandController(), getGlobalCommandController() and
+//       getInterpreter() occur both here and in Setting.
+
 CommandController& CommandCompleter::getCommandController() const
 {
 	return commandController;
 }
 
+GlobalCommandController& CommandCompleter::getGlobalCommandController() const
+{
+	GlobalCommandController* globalCommandController =
+		dynamic_cast<GlobalCommandController*>(&commandController);
+	if (globalCommandController) {
+		return *globalCommandController;
+	} else {
+		return checked_cast<MSXCommandController*>(&commandController)
+			->getGlobalCommandController();
+	}
+}
+
+Interpreter& CommandCompleter::getInterpreter() const
+{
+	return getGlobalCommandController().getInterpreter();
+}
+
 
 // class Command
 
 Command::Command(CommandController& commandController, const string& name)
 	: CommandCompleter(commandController, name)
+{
+	init();
+}
+
+Command::Command(CommandController& commandController, const char* name)
+	: CommandCompleter(commandController, name)
+{
+	init();
+}
+
+void Command::init()
 {
 	if (!getName().empty()) {
 		getCommandController().registerCommand(*this, getName());
@@ -65,6 +99,12 @@ SimpleCommand::SimpleCommand(CommandController& commandController,
 {
 }
 
+SimpleCommand::SimpleCommand(CommandController& commandController,
+                             const char* name)
+	: Command(commandController, name)
+{
+}
+
 SimpleCommand::~SimpleCommand()
 {
 }
