@@ -1,4 +1,4 @@
-// $Id: GlobalCommandController.cc 9618 2009-04-23 22:05:07Z manuelbi $
+// $Id: GlobalCommandController.cc 10720 2009-10-27 00:24:52Z mthuurne $
 
 #include "GlobalCommandController.hh"
 #include "Command.hh"
@@ -7,19 +7,19 @@
 #include "ProxySetting.hh"
 #include "InfoTopic.hh"
 #include "LocalFileReference.hh"
-#include "openmsx.hh"
-#include "CliComm.hh"
+#include "GlobalCliComm.hh"
+#include "CliConnection.hh"
 #include "HotKey.hh"
 #include "Interpreter.hh"
 #include "InfoCommand.hh"
 #include "CommandException.hh"
 #include "SettingsConfig.hh"
 #include "SettingsManager.hh"
-#include "GlobalSettings.hh"
 #include "RomInfoTopic.hh"
 #include "TclObject.hh"
 #include "Version.hh"
 #include "ScopedAssign.hh"
+#include "checked_cast.hh"
 #include "openmsx.hh"
 #include <cassert>
 #include <cstdlib>
@@ -53,6 +53,19 @@ private:
 	GlobalCommandController& controller;
 };
 
+class UpdateCmd : public SimpleCommand
+{
+public:
+	UpdateCmd(CommandController& commandController,
+	          GlobalCliComm& cliComm);
+	virtual string execute(const vector<string>& tokens);
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
+private:
+	CliConnection& getConnection();
+	GlobalCliComm& cliComm;
+};
+
 class VersionInfo : public InfoTopic
 {
 public:
@@ -64,18 +77,46 @@ public:
 
 
 GlobalCommandController::GlobalCommandController(
-	EventDistributor& eventDistributor_, Reactor& reactor_)
-	: cliComm(NULL)
+	EventDistributor& eventDistributor,
+	GlobalCliComm& cliComm_, Reactor& reactor_)
+	: cliComm(cliComm_)
 	, connection(NULL)
-	, eventDistributor(eventDistributor_)
 	, reactor(reactor_)
-	, openMSXInfoCommand(new InfoCommand(*this, "openmsx_info", &reactor))
+	, interpreter(new Interpreter(eventDistributor))
+	, openMSXInfoCommand(new InfoCommand(*this, "openmsx_info"))
+	, hotKey(new HotKey(*this, eventDistributor))
 	, helpCmd(new HelpCmd(*this))
 	, tabCompletionCmd(new TabCompletionCmd(*this))
 	, proxyCmd(new ProxyCmd(*this, reactor))
 	, versionInfo(new VersionInfo(getOpenMSXInfoCommand()))
 	, romInfoTopic(new RomInfoTopic(getOpenMSXInfoCommand()))
 {
+	// For backwards compatibility:
+	//  In the past we had an openMSX command 'update'. This was a mistake
+	//  because it overlaps with the native Tcl command with the same name.
+	//  We renamed 'update' to 'openmsx_update'. And installed a wrapper
+	//  around 'update' that either forwards to the native Tcl command or
+	//  to the 'openmsx_update' command.
+	//  In future openMSX versions this wrapper will be removed.
+	interpreter->execute("rename update __tcl_update");
+	interpreter->execute(
+		"proc update { args } {\n"
+		"    if {$args == \"\"} {\n"
+		"        __tcl_update\n"
+		"    } elseif {$args == \"idletasks\"} {\n"
+		"        __tcl_update idletasks\n"
+		"    } else {\n"
+		"        puts stderr \"Warning: the openMSX \\'update\\' command "
+		                      "overlapped with a native Tcl command "
+		                      "and has been renamed to \\'openmsx_update\\'. "
+		                      "In future openMSX releases this forwarder "
+		                      "will stop working, so please change your "
+		                      "scripts to use the \\'openmsx_update\\' "
+		                      "command instead of \\'update\\'.\"\n"
+		"        eval \"openmsx_update $args\"\n"
+		"    }\n"
+		"}\n");
+	updateCmd.reset(new UpdateCmd(*this, cliComm));
 }
 
 GlobalCommandController::~GlobalCommandController()
@@ -85,9 +126,9 @@ GlobalCommandController::~GlobalCommandController()
 	// TODO find a cleaner way to do this
 	romInfoTopic.reset();
 	versionInfo.reset();
+	updateCmd.reset();
 	tabCompletionCmd.reset();
 	helpCmd.reset();
-	globalSettings.reset();
 	settingsConfig.reset();
 	hotKey.reset();
 	openMSXInfoCommand.reset();
@@ -133,7 +174,7 @@ void GlobalCommandController::registerProxySetting(Setting& setting)
 	ProxySettings::iterator it = findProxySetting(name);
 	if (it == proxySettings.end()) {
 		// first occurrence
-		ProxySetting* proxy = new ProxySetting(*this, name);
+		ProxySetting* proxy = new ProxySetting(*this, reactor, name);
 		proxySettings.push_back(std::make_pair(proxy, 1));
 		getSettingsConfig().getSettingsManager().registerSetting(*proxy, name);
 		getInterpreter().registerSetting(*proxy, name);
@@ -159,18 +200,7 @@ void GlobalCommandController::unregisterProxySetting(Setting& setting)
 	}
 }
 
-void GlobalCommandController::setCliComm(CliComm* cliComm_)
-{
-	cliComm = cliComm_;
-}
-
 CliComm& GlobalCommandController::getCliComm()
-{
-	assert(cliComm);
-	return *cliComm;
-}
-
-CliComm* GlobalCommandController::getCliCommIfAvailable()
 {
 	return cliComm;
 }
@@ -182,53 +212,29 @@ CliConnection* GlobalCommandController::getConnection() const
 
 Interpreter& GlobalCommandController::getInterpreter()
 {
-	if (!interpreter.get()) {
-		interpreter.reset(new Interpreter(eventDistributor));
-	}
 	return *interpreter;
 }
 
-Reactor& GlobalCommandController::getReactor() const
-{
-	return reactor;
-}
-
 InfoCommand& GlobalCommandController::getOpenMSXInfoCommand()
 {
 	return *openMSXInfoCommand;
 }
 
-HotKey& GlobalCommandController::getHotKey()
-{
-	if (!hotKey.get()) {
-		hotKey.reset(new HotKey(*this, eventDistributor));
-	}
-	return *hotKey;
-}
-
 SettingsConfig& GlobalCommandController::getSettingsConfig()
 {
 	if (!settingsConfig.get()) {
-		settingsConfig.reset(new SettingsConfig(*this, getHotKey()));
+		settingsConfig.reset(new SettingsConfig(*this, *hotKey));
 	}
 	return *settingsConfig;
 }
 
-GlobalSettings& GlobalCommandController::getGlobalSettings()
-{
-	if (!globalSettings.get()) {
-		globalSettings.reset(new GlobalSettings(*this));
-	}
-	return *globalSettings;
-}
-
 void GlobalCommandController::registerCommand(
 	Command& command, const string& str)
 {
 	assert(commands.find(str) == commands.end());
 
 	commands[str] = &command;
-	getInterpreter().registerCommand(str, command);
+	interpreter->registerCommand(str, command);
 }
 
 void GlobalCommandController::unregisterCommand(
@@ -237,7 +243,7 @@ void GlobalCommandController::unregisterCommand(
 	assert(commands.find(str) != commands.end());
 	assert(commands.find(str)->second == &command);
 
-	getInterpreter().unregisterCommand(str, command);
+	interpreter->unregisterCommand(str, command);
 	commands.erase(str);
 }
 
@@ -261,13 +267,13 @@ void GlobalCommandController::registerSetting(Setting& setting)
 {
 	const string& name = setting.getName();
 	getSettingsConfig().getSettingsManager().registerSetting(setting, name);
-	getInterpreter().registerSetting(setting, name);
+	interpreter->registerSetting(setting, name);
 }
 
 void GlobalCommandController::unregisterSetting(Setting& setting)
 {
 	const string& name = setting.getName();
-	getInterpreter().unregisterSetting(setting, name);
+	interpreter->unregisterSetting(setting, name);
 	getSettingsConfig().getSettingsManager().unregisterSetting(setting, name);
 }
 
@@ -279,7 +285,7 @@ Setting* GlobalCommandController::findSetting(const std::string& name)
 void GlobalCommandController::changeSetting(
 	const std::string& name, const string& value)
 {
-	getInterpreter().setVariable(name, value);
+	interpreter->setVariable(name, value);
 }
 
 void GlobalCommandController::changeSetting(Setting& setting, const string& value)
@@ -287,11 +293,6 @@ void GlobalCommandController::changeSetting(Setting& setting, const string& valu
 	changeSetting(setting.getName(), value);
 }
 
-string GlobalCommandController::makeUniqueSettingName(const string& name)
-{
-	return getSettingsConfig().getSettingsManager().makeUnique(name);
-}
-
 bool GlobalCommandController::hasCommand(const string& command) const
 {
 	return commands.find(command) != commands.end();
@@ -437,27 +438,27 @@ string GlobalCommandController::join(
 
 bool GlobalCommandController::isComplete(const string& command)
 {
-	return getInterpreter().isComplete(command);
+	return interpreter->isComplete(command);
 }
 
 string GlobalCommandController::executeCommand(
 	const string& cmd, CliConnection* connection_)
 {
 	ScopedAssign<CliConnection*> sa(connection, connection_);
-	return getInterpreter().execute(cmd);
+	return interpreter->execute(cmd);
 }
 
 void GlobalCommandController::splitList(
 	const string& list, vector<string>& result)
 {
-	getInterpreter().splitList(list, result);
+	interpreter->splitList(list, result);
 }
 
 void GlobalCommandController::source(const string& script)
 {
 	try {
 		LocalFileReference file(script);
-		getInterpreter().executeFile(file.getFilename());
+		interpreter->executeFile(file.getFilename());
 	} catch (CommandException& e) {
 		getCliComm().printWarning(
 			 "While executing init.tcl: " + e.getMessage());
@@ -515,14 +516,14 @@ void GlobalCommandController::tabCompletion(vector<string>& tokens)
 	if (tokens.size() == 1) {
 		// build a list of all command strings
 		set<string> cmds;
-		getInterpreter().getCommandNames(cmds);
+		interpreter->getCommandNames(cmds);
 		Completer::completeString(tokens, cmds);
 	} else {
 		CompleterMap::const_iterator it = commandCompleters.find(tokens.front());
 		if (it != commandCompleters.end()) {
 			it->second->tabCompletion(tokens);
 		} else {
-			TclObject command(getInterpreter());
+			TclObject command(*interpreter);
 			command.addListElement("openmsx::tabcompletion");
 			command.addListElements(tokens.begin(), tokens.end());
 			try {
@@ -542,7 +543,7 @@ void GlobalCommandController::tabCompletion(vector<string>& tokens)
 				set<string> completions(split.begin(), split.end());
 				Completer::completeString(tokens, completions, sensitive);
 			} catch (CommandException& e) {
-				cliComm->printWarning(
+				cliComm.printWarning(
 					"Error while executing tab-completion "
 					"proc: " + e.getMessage());
 			}
@@ -645,6 +646,78 @@ string TabCompletionCmd::help(const vector<string>& /*tokens*/) const
 }
 
 
+// class UpdateCmd
+
+UpdateCmd::UpdateCmd(CommandController& commandController,
+                     GlobalCliComm& cliComm_)
+	: SimpleCommand(commandController, "openmsx_update")
+	, cliComm(cliComm_)
+{
+}
+
+static GlobalCliComm::UpdateType getType(const string& name)
+{
+	const char* const* updateStr = GlobalCliComm::getUpdateStrs();
+	for (unsigned i = 0; i < CliComm::NUM_UPDATES; ++i) {
+		if (updateStr[i] == name) {
+			return static_cast<CliComm::UpdateType>(i);
+		}
+	}
+	throw CommandException("No such update type: " + name);
+}
+
+CliConnection& UpdateCmd::getConnection()
+{
+	GlobalCommandController* controller =
+		checked_cast<GlobalCommandController*>(&getCommandController());
+	CliConnection* connection = controller->getConnection();
+	if (!connection) {
+		throw CommandException("This command only makes sense when "
+		                       "it's used from an external application.");
+	}
+	return *connection;
+}
+
+string UpdateCmd::execute(const vector<string>& tokens)
+{
+	if (tokens.size() != 3) {
+		throw SyntaxError();
+	}
+	if (tokens[1] == "enable") {
+		getConnection().setUpdateEnable(getType(tokens[2]), true);
+	} else if (tokens[1] == "disable") {
+		getConnection().setUpdateEnable(getType(tokens[2]), false);
+	} else {
+		throw SyntaxError();
+	}
+	return "";
+}
+
+string UpdateCmd::help(const vector<string>& /*tokens*/) const
+{
+	static const string helpText = "Enable or disable update events for external applications. See doc/openmsx-control-xml.txt.";
+	return helpText;
+}
+
+void UpdateCmd::tabCompletion(vector<string>& tokens) const
+{
+	switch (tokens.size()) {
+	case 2: {
+		set<string> ops;
+		ops.insert("enable");
+		ops.insert("disable");
+		completeString(tokens, ops);
+		break;
+	}
+	case 3: {
+		const char* const* updateStr = GlobalCliComm::getUpdateStrs();
+		set<string> types(updateStr, updateStr + CliComm::NUM_UPDATES);
+		completeString(tokens, types);
+	}
+	}
+}
+
+
 // Version info
 
 VersionInfo::VersionInfo(InfoCommand& openMSXInfoCommand)
