@@ -1,4 +1,4 @@
-// $Id: InfoCommand.cc 5779 2006-10-14 16:17:32Z m9710797 $
+// $Id: InfoCommand.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "InfoCommand.hh"
 #include "InfoTopic.hh"
@@ -6,8 +6,8 @@
 #include "MSXCommandController.hh"
 #include "CliComm.hh"
 #include "CommandException.hh"
-#include "Reactor.hh"
 #include "MSXMotherBoard.hh"
+#include "unreachable.hh"
 #include <iostream>
 #include <cassert>
 
@@ -18,10 +18,8 @@ using std::vector;
 
 namespace openmsx {
 
-InfoCommand::InfoCommand(CommandController& commandController, const string& name,
-                         Reactor* reactor_)
+InfoCommand::InfoCommand(CommandController& commandController, const string& name)
 	: Command(commandController, name)
-	, reactor(reactor_)
 {
 }
 
@@ -36,7 +34,7 @@ void InfoCommand::registerTopic(InfoTopic& topic, const string& name)
 	if (infoTopics.find(name) != infoTopics.end()) {
 		std::cerr << "INTERNAL ERROR: already have a info topic with "
 		             "name " << name << std::endl;
-		assert(false);
+		UNREACHABLE;
 	}
 #endif
 	infoTopics[name] = &topic;
@@ -48,38 +46,14 @@ void InfoCommand::unregisterTopic(InfoTopic& topic, const string& name)
 	if (infoTopics.find(name) == infoTopics.end()) {
 		std::cerr << "INTERNAL ERROR: can't unregister topic with name "
 			"name " << name << ", not found!" << std::endl;
-		assert(false);
+		UNREACHABLE;
 	}
 	assert(infoTopics[name] == &topic);
 	infoTopics.erase(name);
 }
 
-bool InfoCommand::hasTopic(const std::string& name) const
-{
-	return infoTopics.find(name) != infoTopics.end();
-}
-
 // Command
 
-bool InfoCommand::executeBWC(const vector<TclObject*>& tokens,
-                             TclObject& result)
-{
-	if (!reactor) return false;
-	MSXMotherBoard* motherBoard = reactor->getMotherBoard();
-	if (!motherBoard) return false;
-	MSXCommandController& msxCommandController =
-		motherBoard->getMSXCommandController();
-	InfoCommand& machineInfoCommand =
-		msxCommandController.getMachineInfoCommand();
-	string topic = tokens[1]->getString();
-	if (!machineInfoCommand.hasTopic(topic)) return false;
-	msxCommandController.getCliComm().printWarning(
-		"'openmsx_info " + topic + "' is deprecated, "
-		"please use 'machine_info " + topic + "'.");
-	machineInfoCommand.execute(tokens, result);
-	return true;
-}
-
 void InfoCommand::execute(const vector<TclObject*>& tokens,
                           TclObject& result)
 {
@@ -98,8 +72,6 @@ void InfoCommand::execute(const vector<TclObject*>& tokens,
 		map<string, const InfoTopic*>::const_iterator it =
 			infoTopics.find(topic);
 		if (it == infoTopics.end()) {
-			// backwards compatibility: also try machine_info
-			if (executeBWC(tokens, result)) return;
 			throw CommandException("No info on: " + topic);
 		}
 		it->second->execute(tokens, result);
