@@ -1,4 +1,4 @@
-// $Id: TclObject.cc 8157 2008-08-12 17:58:55Z m9710797 $
+// $Id: TclObject.cc 9286 2009-02-28 18:15:49Z mfeingol $
 
 #include "TclObject.hh"
 #include "Interpreter.hh"
@@ -20,13 +20,13 @@ TclObject::TclObject(Tcl_Interp* interp_, Tcl_Obj* obj_)
 TclObject::TclObject(Tcl_Interp* interp_, const string& value)
 	: interp(interp_)
 {
-	init(Tcl_NewStringObj(value.data(), value.size()));
+	init(Tcl_NewStringObj(value.data(), int(value.size())));
 }
 
 TclObject::TclObject(const string& value)
 	: interp(0)
 {
-	init(Tcl_NewStringObj(value.data(), value.size()));
+	init(Tcl_NewStringObj(value.data(), int(value.size())));
 }
 
 TclObject::TclObject(Tcl_Interp* interp_)
@@ -110,7 +110,7 @@ void TclObject::throwException() const
 void TclObject::setString(const string& value)
 {
 	unshare();
-	Tcl_SetStringObj(obj, value.c_str(), value.length());
+	Tcl_SetStringObj(obj, value.c_str(), int(value.length()));
 }
 
 void TclObject::setInt(int value)
@@ -139,7 +139,7 @@ void TclObject::setBinary(byte* buf, unsigned length)
 
 void TclObject::addListElement(const string& element)
 {
-	addListElement(Tcl_NewStringObj(element.data(), element.size()));
+	addListElement(Tcl_NewStringObj(element.data(), int(element.size())));
 }
 
 void TclObject::addListElement(int value)
@@ -180,7 +180,7 @@ bool TclObject::getBoolean() const
 	if (Tcl_GetBooleanFromObj(interp, obj, &result) != TCL_OK) {
 		throwException();
 	}
-	return result;
+	return result != 0;
 }
 
 double TclObject::getDouble() const
@@ -230,19 +230,19 @@ bool TclObject::evalBool() const
 	if (Tcl_ExprBooleanObj(interp, obj, &result) != TCL_OK) {
 		throwException();
 	}
-	return result;
+	return result != 0;
 }
 
 void TclObject::checkExpression() const
 {
 	string tmp = getString();
-	parse(tmp.data(), tmp.size(), true);
+	parse(tmp.data(), int(tmp.size()), true);
 }
 
 void TclObject::checkCommand() const
 {
 	string tmp = getString();
-	parse(tmp.data(), tmp.size(), false);
+	parse(tmp.data(), int(tmp.size()), false);
 }
 
 void TclObject::parse(const char* str, int len, bool expression) const
