@@ -1,4 +1,4 @@
-// $Id: HardwareConfig.cc 8470 2009-01-12 17:28:10Z m9710797 $
+// $Id: HardwareConfig.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "HardwareConfig.hh"
 #include "XMLLoader.hh"
@@ -16,6 +16,7 @@
 #include "serialize_stl.hh"
 #include "StringOp.hh"
 #include "ref.hh"
+#include "unreachable.hh"
 #include <cassert>
 
 #include <iostream>
@@ -164,7 +165,7 @@ HardwareConfig::~HardwareConfig()
 		testRemove();
 	} catch (MSXException& e) {
 		std::cerr << e.getMessage() << std::endl;
-		assert(false);
+		UNREACHABLE;
 	}
 #endif
 	for (Devices::reverse_iterator it = devices.rbegin();
@@ -387,7 +388,7 @@ void HardwareConfig::setName(const string& proposedName)
 	} else {
 		unsigned n = 0;
 		do {
-			name = proposedName + " (" + StringOp::toString(++n) + ")";
+			name = StringOp::Builder() << proposedName << " (" << ++n << ')';
 		} while (motherBoard.findExtension(name));
 	}
 }
