@@ -1,4 +1,4 @@
-// $Id: SettingsConfig.cc 8377 2008-11-30 20:56:25Z manuelbi $
+// $Id: SettingsConfig.cc 9276 2009-02-28 06:58:00Z mfeingol $
 
 #include "SettingsConfig.hh"
 #include "SettingsManager.hh"
@@ -105,7 +105,7 @@ void SettingsConfig::saveSetting(const string& filename)
 	File file(name, File::TRUNCATE);
 	string data = "<!DOCTYPE settings SYSTEM 'settings.dtd'>\n" +
 	              xmlElement->dump();
-	file.write(data.c_str(), data.size());
+	file.write(data.c_str(), unsigned(data.size()));
 }
 
 void SettingsConfig::setSaveSettings(bool save)
