@@ -1,4 +1,4 @@
-// $Id: XMLElement.cc 8374 2008-11-29 20:30:41Z m9710797 $
+// $Id: XMLElement.cc 11144 2010-01-13 13:48:24Z m9710797 $
 
 #include "XMLElement.hh"
 #include "StringOp.hh"
@@ -210,6 +210,16 @@ int XMLElement::getChildDataAsInt(const string& name, int defaultValue) const
 	return child ? StringOp::stringToInt(child->getData()) : defaultValue;
 }
 
+void XMLElement::setChildData(const string& name, const string& value)
+{
+	XMLElement* child = findChild(name);
+	if (child) {
+		child->setData(value);
+	} else {
+		addChild(auto_ptr<XMLElement>(new XMLElement(name, value)));
+	}
+}
+
 void XMLElement::removeAllChildren()
 {
 	for (Children::const_iterator it = children.begin();
@@ -322,7 +332,7 @@ void XMLElement::dump(string& result, unsigned indentNum) const
 		if (data.empty()) {
 			result += "/>\n";
 		} else {
-			result += '>' + XMLEscape(data.str()) + "</" +
+			result += '>' + XMLEscape(data) + "</" +
 			          getName() + ">\n";
 		}
 	} else {
