@@ -1,8 +1,8 @@
-// $Id: CommandConsole.cc 9511 2009-04-09 18:34:13Z m9710797 $
+// $Id: CommandConsole.cc 11265 2010-02-13 21:16:27Z m9710797 $
 
 #include "CommandConsole.hh"
 #include "CommandException.hh"
-#include "CommandController.hh"
+#include "GlobalCommandController.hh"
 #include "Completer.hh"
 #include "Interpreter.hh"
 #include "Keys.hh"
@@ -38,13 +38,14 @@ const char* const PROMPT1 = "> ";
 const char* const PROMPT2 = "| ";
 
 CommandConsole::CommandConsole(
-		CommandController& commandController_,
+		GlobalCommandController& commandController_,
 		EventDistributor& eventDistributor_,
 		Display& display_)
 	: commandController(commandController_)
 	, eventDistributor(eventDistributor_)
 	, display(display_)
-	, consoleSetting(commandController.getGlobalSettings().getConsoleSetting())
+	, consoleSetting(new BooleanSetting(commandController, "console",
+		"turns console display on/off", false, Setting::DONT_SAVE))
 	, historySizeSetting(new IntegerSetting(commandController, "console_history_size",
 		"amount of commands kept in console history", 100, 0, 10000))
 	, removeDoublesSetting(new BooleanSetting(commandController,
@@ -72,11 +73,15 @@ CommandConsole::CommandConsole(
 	commandController.getInterpreter().setOutput(this);
 	eventDistributor.registerEventListener(
 		OPENMSX_KEY_DOWN_EVENT, *this, EventDistributor::CONSOLE);
+	// also listen to KEY_UP events, so that we can consume them
+	eventDistributor.registerEventListener(
+		OPENMSX_KEY_UP_EVENT, *this, EventDistributor::CONSOLE);
 }
 
 CommandConsole::~CommandConsole()
 {
 	eventDistributor.unregisterEventListener(OPENMSX_KEY_DOWN_EVENT, *this);
+	eventDistributor.unregisterEventListener(OPENMSX_KEY_UP_EVENT, *this);
 	commandController.getInterpreter().setOutput(NULL);
 	Completer::setOutput(NULL);
 }
@@ -124,6 +129,11 @@ void CommandConsole::loadHistory()
 	}
 }
 
+BooleanSetting& CommandConsole::getConsoleSetting()
+{
+	return *consoleSetting.get();
+}
+
 void CommandConsole::getCursorPosition(unsigned& xPosition, unsigned& yPosition) const
 {
 	xPosition = cursorPosition % getColumns();
@@ -154,28 +164,82 @@ string CommandConsole::getLine(unsigned line) const
 bool CommandConsole::signalEvent(shared_ptr<const Event> event)
 {
 	const KeyEvent& keyEvent = checked_cast<const KeyEvent&>(*event);
-	if ((event->getType() == OPENMSX_KEY_DOWN_EVENT) &&
-	    consoleSetting.getValue()) {
-		handleEvent(keyEvent);
-		display.repaintDelayed(40000); // 25fps
-		return false; // deny event to other listeners
+	if (consoleSetting->getValue()) {
+		if (event->getType() == OPENMSX_KEY_DOWN_EVENT) {
+			if (handleEvent(keyEvent)) {
+				// event was used
+				display.repaintDelayed(40000); // 25fps
+				return false; // deny event to other listeners
+			}
+		} else {
+			assert(event->getType() == OPENMSX_KEY_UP_EVENT);
+			// In SVN revision 11232 we tried to only consume keyup
+			// events that were meaningful for the console. Though
+			// SDL doesn't fill in the unicode field for keyup
+			// events, so this approach didn't work. So now we
+			// again consume all keyup events. In the future we
+			// could try to work around this limitation (e.g. by
+			// building a map of keydown events that were consumed
+			// and also consume the corresponding keyup event).
+			return false;
+		}
 	}
 	return true;
 }
 
-void CommandConsole::handleEvent(const KeyEvent& keyEvent)
+bool CommandConsole::handleEvent(const KeyEvent& keyEvent)
 {
 	Keys::KeyCode keyCode = keyEvent.getKeyCode();
-	switch (keyCode) {
-		case (Keys::K_PAGEUP | Keys::KM_SHIFT):
+	int key = keyCode &  Keys::K_MASK;
+	int mod = keyCode & ~Keys::K_MASK;
+	word chr = keyEvent.getUnicode();
+
+	bool used = true;
+	switch (mod) {
+	case Keys::KM_CTRL:
+		switch (key) {
+		case Keys::K_H:
+			backspace();
+			break;
+		case Keys::K_A:
+			cursorPosition = unsigned(prompt.size());
+			break;
+		case Keys::K_E:
+			cursorPosition = utf8::unchecked::size(lines[0]);
+			break;
+		case Keys::K_C:
+			clearCommand();
+			break;
+		default:
+			used = false;
+		}
+		break;
+	case Keys::KM_SHIFT:
+		switch (key) {
+		case Keys::K_PAGEUP:
 			scroll(max<int>(getRows() - 1, 1));
 			break;
+		case Keys::K_PAGEDOWN:
+			scroll(-max<int>(getRows() - 1, 1));
+			break;
+		default:
+			if (chr) {
+				normalKey(chr);
+			} else {
+				used = false;
+			}
+			break;
+		}
+		break;
+	case Keys::KM_MODE: // e.g. to type '1-9' on a N900
+	case Keys::KM_ALT:  // e.g. to type | [ ] on a azerty keyboard layout
+	// Don't add the META modifier here because that will break the Cmd+L
+	// hotkey on MacOSX to toggle the console.
+	case 0:
+		switch (key) {
 		case Keys::K_PAGEUP:
 			scroll(1);
 			break;
-		case (Keys::K_PAGEDOWN | Keys::KM_SHIFT):
-			scroll(-max<int>(getRows() - 1, 1));
-			break;
 		case Keys::K_PAGEDOWN:
 			scroll(-1);
 			break;
@@ -186,7 +250,6 @@ void CommandConsole::handleEvent(const KeyEvent& keyEvent)
 			nextCommand();
 			break;
 		case Keys::K_BACKSPACE:
-		case Keys::K_H | Keys::KM_CTRL:
 			backspace();
 			break;
 		case Keys::K_DELETE:
@@ -211,22 +274,34 @@ void CommandConsole::handleEvent(const KeyEvent& keyEvent)
 			}
 			break;
 		case Keys::K_HOME:
-		case Keys::K_A | Keys::KM_CTRL:
 			cursorPosition = unsigned(prompt.size());
 			break;
 		case Keys::K_END:
-		case Keys::K_E | Keys::KM_CTRL:
 			cursorPosition = utf8::unchecked::size(lines[0]);
 			break;
-		case Keys::K_C | Keys::KM_CTRL:
-			clearCommand();
-			break;
 		default:
-			// Treat as normal key if no modifiers except shift.
-			if (!(keyCode & ~(Keys::K_MASK | Keys::KM_SHIFT))) {
-				normalKey(keyEvent.getUnicode());
+			if (chr) {
+				normalKey(chr);
+			} else {
+				used = false;
 			}
+			break;
+		}
+		break;
+	default:
+		used = false;
 	}
+	if ((key == Keys::K_LSHIFT) || (key == Keys::K_RSHIFT) ||
+	    (key == Keys::K_LCTRL)  || (key == Keys::K_RCTRL)  ||
+	    (key == Keys::K_LALT)   || (key == Keys::K_RALT)   ||
+	    (key == Keys::K_LMETA)  || (key == Keys::K_RMETA)  ||
+	    (key == Keys::K_LSUPER) || (key == Keys::K_RSUPER)) {
+		// Pure-modifier key event, act as-if the console consumed this
+		// event. Without this check, pressing SHIFT or CTRL is passed
+		// to the MSX, where it possibly cancels a reverse-replay.
+		used = true;
+	}
+	return used;
 }
 
 void CommandConsole::setColumns(unsigned columns_)
@@ -428,7 +503,7 @@ void CommandConsole::delete_key()
 
 void CommandConsole::normalKey(word chr)
 {
-	if (!chr) return;
+	assert(chr);
 	resetScrollBack();
 	string::iterator pos = lines[0].begin();
 	utf8::unchecked::advance(pos, cursorPosition);
