@@ -1,4 +1,4 @@
-// $Id: OSDConsoleRenderer.cc 9506 2009-04-07 19:43:56Z m9710797 $
+// $Id: OSDConsoleRenderer.cc 11071 2009-12-30 11:57:08Z m9710797 $
 
 #include "OSDConsoleRenderer.hh"
 #include "CommandConsole.hh"
@@ -10,9 +10,6 @@
 #include "TTFFont.hh"
 #include "SDLImage.hh"
 #include "Display.hh"
-#include "Event.hh"
-#include "InputEvents.hh"
-#include "EventDistributor.hh"
 #include "InputEventGenerator.hh"
 #include "Timer.hh"
 #include "FileContext.hh"
@@ -20,11 +17,12 @@
 #include "Reactor.hh"
 #include "MSXException.hh"
 #include "openmsx.hh"
+#include "unreachable.hh"
 #include <algorithm>
 #include <cassert>
 
 #include "components.hh"
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 #include "GLImage.hh"
 #endif
 
@@ -55,17 +53,19 @@ private:
 // class OSDConsoleRenderer
 
 OSDConsoleRenderer::OSDConsoleRenderer(
-		Reactor& reactor_, unsigned screenW_, unsigned screenH_,
+		Reactor& reactor_, CommandConsole& console_,
+		unsigned screenW_, unsigned screenH_,
 		bool openGL_)
 	: Layer(COVER_NONE, Z_CONSOLE)
 	, reactor(reactor_)
-	, consoleSetting(reactor.getGlobalSettings().getConsoleSetting())
+	, console(console_)
+	, consoleSetting(console.getConsoleSetting())
 	, settingChecker(new OSDSettingChecker(*this))
 	, screenW(screenW_)
 	, screenH(screenH_)
 	, openGL(openGL_)
 {
-#ifndef COMPONENT_GL
+#if !COMPONENT_GL
 	assert(!openGL);
 #endif
 	destX = destY = destW = destH = 0; // avoid UMR
@@ -160,7 +160,6 @@ void OSDConsoleRenderer::adjustColRow()
 	unsigned consoleRows = std::min<unsigned>(
 		consoleRowsSetting->getValue(),
 		screenH / font->getHeight());
-	CommandConsole& console = reactor.getCommandConsole();
 	console.setColumns(consoleColumns);
 	console.setRows(consoleRows);
 }
@@ -172,7 +171,7 @@ void OSDConsoleRenderer::update(const Setting& setting)
 	} else if (&setting == fontSizeSetting.get()) {
 		loadFont(fontSetting->getValue());
 	} else {
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -186,13 +185,6 @@ void OSDConsoleRenderer::setActive(bool active_)
 	activeTime = Timer::getTime();
 
 	reactor.getInputEventGenerator().setKeyRepeat(active);
-	if (active) {
-		reactor.getEventDistributor().distributeEvent(
-			new ConsoleEvent(OPENMSX_CONSOLE_ON_EVENT));
-	} else {
-		reactor.getEventDistributor().distributeEvent(
-			new ConsoleEvent(OPENMSX_CONSOLE_OFF_EVENT));
-	}
 }
 
 byte OSDConsoleRenderer::getVisibility() const
@@ -224,7 +216,6 @@ bool OSDConsoleRenderer::updateConsoleRect()
 	adjustColRow();
 
 	unsigned x, y, w, h;
-	CommandConsole& console = reactor.getCommandConsole();
 	h = font->getHeight() * console.getRows();
 	w = (font->getWidth() * console.getColumns()) + CHAR_BORDER;
 
@@ -292,7 +283,7 @@ void OSDConsoleRenderer::loadBackground(const string& value)
 	if (!openGL) {
 		backgroundImage.reset(new SDLImage(filename, destW, destH));
 	}
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	else {
 		backgroundImage.reset(new GLImage(filename, destW, destH));
 	}
@@ -303,7 +294,7 @@ void OSDConsoleRenderer::drawText(OutputSurface& output, const string& text,
                                   int x, int y, byte alpha)
 {
 	if (text.empty()) return;
-	SDL_Surface* surf;
+	SDLSurfacePtr surf;
 	try {
 		surf = font->render(text, 255, 255, 255);
 	} catch (MSXException& e) {
@@ -319,7 +310,7 @@ void OSDConsoleRenderer::drawText(OutputSurface& output, const string& text,
 		SDLImage image(surf);
 		image.draw(output, x, y, alpha);
 	}
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	else {
 		GLImage image(surf);
 		image.draw(output, x, y, alpha);
@@ -348,7 +339,7 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 				backgroundImage.reset(new SDLImage(
 					destW, destH, CONSOLE_ALPHA));
 			}
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 			else {
 				backgroundImage.reset(new GLImage(
 					destW, destH, CONSOLE_ALPHA));
@@ -362,7 +353,6 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 		backgroundImage->draw(output, destX, destY, visibility);
 	}
 
-	CommandConsole& console = reactor.getCommandConsole();
 	int screenlines = destH / font->getHeight();
 	for (int loop = 0; loop < screenlines; ++loop) {
 		drawText(output,
@@ -417,7 +407,7 @@ void OSDSettingChecker::check(SettingImpl<FilenameSetting::Policy>& setting,
 	} else if (&setting == renderer.fontSetting.get()) {
 		renderer.loadFont(value);
 	} else {
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
