@@ -1,7 +1,8 @@
-// $Id: OSDRectangle.cc 9771 2009-05-07 15:14:07Z m9710797 $
+// $Id: OSDRectangle.cc 10418 2009-08-27 12:28:00Z mthuurne $
 
 #include "OSDRectangle.hh"
 #include "SDLImage.hh"
+#include "OutputSurface.hh"
 #include "CommandException.hh"
 #include "FileContext.hh"
 #include "FileOperations.hh"
@@ -9,7 +10,7 @@
 #include "components.hh"
 #include "Math.hh"
 #include <cassert>
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 #include "GLImage.hh"
 #endif
 
@@ -111,7 +112,7 @@ bool OSDRectangle::takeImageDimensions() const
 	       (relw == 0.0) && (relh == 0.0);
 }
 
-void OSDRectangle::getWidthHeight(const OutputSurface& output,
+void OSDRectangle::getWidthHeight(const OutputRectangle& output,
                                   double& width, double& height) const
 {
 	if (!imageName.empty() && image.get() && takeImageDimensions()) {
@@ -177,7 +178,7 @@ BaseImage* OSDRectangle::createSDL(OutputSurface& output)
 
 BaseImage* OSDRectangle::createGL(OutputSurface& output)
 {
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	return create<GLImage>(output);
 #else
 	(void)&output;
