@@ -1,15 +1,16 @@
-// $Id: OSDText.cc 9588 2009-04-18 17:51:37Z mthuurne $
+// $Id: OSDText.cc 11071 2009-12-30 11:57:08Z m9710797 $
 
 #include "OSDText.hh"
 #include "TTFFont.hh"
 #include "SDLImage.hh"
+#include "OutputSurface.hh"
 #include "CommandException.hh"
 #include "FileContext.hh"
 #include "FileOperations.hh"
 #include "TclObject.hh"
 #include "components.hh"
 #include <cassert>
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 #include "GLImage.hh"
 #endif
 
@@ -94,7 +95,7 @@ string OSDText::getType() const
 	return "text";
 }
 
-void OSDText::getWidthHeight(const OutputSurface& /*output*/,
+void OSDText::getWidthHeight(const OutputRectangle& /*output*/,
                              double& width, double& height) const
 {
 	if (image.get()) {
@@ -125,8 +126,8 @@ template <typename IMAGE> BaseImage* OSDText::create(OutputSurface& output)
 		}
 	}
 	try {
-		SDL_Surface* surface = font->render(
-			text, getRed(), getGreen(), getBlue());
+		SDLSurfacePtr surface(font->render(
+			text, getRed(), getGreen(), getBlue()));
 		return new IMAGE(surface);
 	} catch (MSXException& e) {
 		throw MSXException("Couldn't render text: " + e.getMessage());
@@ -140,7 +141,7 @@ BaseImage* OSDText::createSDL(OutputSurface& output)
 
 BaseImage* OSDText::createGL(OutputSurface& output)
 {
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	return create<GLImage>(output);
 #else
 	(void)&output;
