@@ -1,4 +1,4 @@
-// $Id: OSDTopWidget.cc 7749 2008-04-09 22:27:35Z mthuurne $
+// $Id: OSDTopWidget.cc 10348 2009-08-12 10:44:39Z m9710797 $
 
 #include "OSDTopWidget.hh"
 #include "OutputSurface.hh"
@@ -15,11 +15,11 @@ std::string OSDTopWidget::getType() const
 	return "top";
 }
 
-void OSDTopWidget::getWidthHeight(const OutputSurface& output,
+void OSDTopWidget::getWidthHeight(const OutputRectangle& output,
                                   double& width, double& height) const
 {
-	width  = output.getWidth();
-	height = output.getHeight();
+	width  = output.getOutputWidth();
+	height = output.getOutputHeight();
 }
 
 void OSDTopWidget::invalidateLocal()
