@@ -1,4 +1,4 @@
-// $Id: OSDWidget.cc 9821 2009-05-18 18:15:43Z m9710797 $
+// $Id: OSDWidget.cc 11174 2010-01-21 18:36:31Z m9710797 $
 
 #include "OSDWidget.hh"
 #include "OutputSurface.hh"
@@ -7,9 +7,9 @@
 #include "StringOp.hh"
 #include "GLUtil.hh"
 #include "openmsx.hh"
+#include "unreachable.hh"
 #include <SDL.h>
 #include <algorithm>
-#include <cassert>
 
 using std::string;
 using std::set;
@@ -73,7 +73,7 @@ SDLScopedClip::~SDLScopedClip()
 
 ////
 
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 
 class GLScopedClip
 {
@@ -163,13 +163,8 @@ OSDWidget* OSDWidget::findSubWidget(const string& name)
 	}
 	string first, last;
 	StringOp::splitOnFirst(name, ".", first, last);
-	for (SubWidgets::const_iterator it = subWidgets.begin();
-	     it != subWidgets.end(); ++it) {
-		if ((*it)->getName() == first) {
-			return (*it)->findSubWidget(last);
-		}
-	}
-	return NULL;
+	SubWidgetsMap::const_iterator it = subWidgetsMap.find(first);
+	return it == subWidgetsMap.end() ? NULL : it->second->findSubWidget(last);
 }
 
 const OSDWidget* OSDWidget::findSubWidget(const std::string& name) const
@@ -180,21 +175,27 @@ const OSDWidget* OSDWidget::findSubWidget(const std::string& name) const
 void OSDWidget::addWidget(std::auto_ptr<OSDWidget> widget)
 {
 	widget->setParent(this);
-	subWidgets.push_back(widget.release());
+	OSDWidget* widgetPtr = widget.release();
+	subWidgets.push_back(widgetPtr);
+	subWidgetsMap[widgetPtr->getName()] = widgetPtr;
 	resort();
 }
 
 void OSDWidget::deleteWidget(OSDWidget& widget)
 {
+	string widgetName = widget.getName();
 	for (SubWidgets::iterator it = subWidgets.begin();
 	     it != subWidgets.end(); ++it) {
 		if (*it == &widget) {
 			delete *it;
 			subWidgets.erase(it);
+			SubWidgetsMap::size_type existed =
+				subWidgetsMap.erase(widgetName);
+			assert(existed); (void)existed;
 			return;
 		}
 	}
-	assert(false);
+	UNREACHABLE;
 }
 
 struct AscendingZ {
@@ -217,11 +218,12 @@ void OSDWidget::getProperties(set<string>& result) const
 	result.insert("-rely");
 	result.insert("-scaled");
 	result.insert("-clip");
+	result.insert("-mousecoord");
 }
 
 void OSDWidget::setProperty(const string& name, const TclObject& value)
 {
-	if (name == "-type") {
+	if ((name == "-type") || (name == "-mousecoord")) {
 		throw CommandException("-type property is readonly");
 	} else if (name == "-x") {
 		x = value.getDouble();
@@ -252,6 +254,20 @@ void OSDWidget::setProperty(const string& name, const TclObject& value)
 	}
 }
 
+class DummyOutputRectangle : public OutputRectangle
+{
+public:
+	DummyOutputRectangle(unsigned width_, unsigned height_)
+		: width(width_), height(height_)
+	{
+	}
+	virtual unsigned getOutputWidth()  const { return width;  }
+	virtual unsigned getOutputHeight() const { return height; }
+private:
+	const unsigned width;
+	const unsigned height;
+};
+
 void OSDWidget::getProperty(const string& name, TclObject& result) const
 {
 	if (name == "-type") {
@@ -270,11 +286,21 @@ void OSDWidget::getProperty(const string& name, TclObject& result) const
 		result.setBoolean(scaled);
 	} else if (name == "-clip") {
 		result.setBoolean(clip);
+	} else if (name == "-mousecoord") {
+		double x, y;
+		getMouseCoord(x, y);
+		result.addListElement(x);
+		result.addListElement(y);
 	} else {
 		throw CommandException("No such property: " + name);
 	}
 }
 
+double OSDWidget::getRecursiveFadeValue() const
+{
+	return 1.0; // fully opaque
+}
+
 void OSDWidget::invalidateRecursive()
 {
 	invalidateLocal();
@@ -309,7 +335,7 @@ void OSDWidget::paintSDLRecursive(OutputSurface& output)
 void OSDWidget::paintGLRecursive (OutputSurface& output)
 {
 	(void)output;
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	paintGL(output);
 
 	std::auto_ptr<GLScopedClip> scopedClip;
@@ -326,10 +352,10 @@ void OSDWidget::paintGLRecursive (OutputSurface& output)
 #endif
 }
 
-int OSDWidget::getScaleFactor(const OutputSurface& output) const
+int OSDWidget::getScaleFactor(const OutputRectangle& output) const
 {
 	if (scaled) {
-		return output.getWidth() / 320;;
+		return output.getOutputWidth() / 320;;
 	} else if (getParent()) {
 		return getParent()->getScaleFactor(output);
 	} else {
@@ -337,7 +363,7 @@ int OSDWidget::getScaleFactor(const OutputSurface& output) const
 	}
 }
 
-void OSDWidget::transformXY(const OutputSurface& output,
+void OSDWidget::transformXY(const OutputRectangle& output,
                             double x, double y, double relx, double rely,
                             double& outx, double& outy) const
 {
@@ -352,7 +378,49 @@ void OSDWidget::transformXY(const OutputSurface& output,
 	}
 }
 
-void OSDWidget::getBoundingBox(const OutputSurface& output,
+void OSDWidget::transformReverse(
+	const OutputRectangle& output, double x, double y,
+	double& outx, double& outy) const
+{
+	if (const OSDWidget* parent = getParent()) {
+		parent->transformReverse(output, x, y, x, y);
+		double width, height;
+		parent->getWidthHeight(output, width, height);
+		int factor = getScaleFactor(output);
+		outx = x - (getRelX() * width ) - (getX() * factor);
+		outy = y - (getRelY() * height) - (getY() * factor);
+	} else {
+		outx = x;
+		outy = y;
+	}
+}
+
+void OSDWidget::getMouseCoord(double& outx, double& outy) const
+{
+	SDL_Surface* surface = SDL_GetVideoSurface();
+	if (!surface) {
+		throw CommandException(
+			"-can't get mouse coordinates: no window visible");
+	}
+	DummyOutputRectangle output(surface->w, surface->h);
+
+	int mouseX, mouseY;
+	SDL_GetMouseState(&mouseX, &mouseY);
+
+	transformReverse(output, mouseX, mouseY, outx, outy);
+
+	double width, height;
+	getWidthHeight(output, width, height);
+	if ((width == 0) || (height == 0)) {
+		throw CommandException(
+			"-can't get mouse coordinates: "
+			"widget has zero width or height");
+	}
+	outx /= width;
+	outy /= height;
+}
+
+void OSDWidget::getBoundingBox(const OutputRectangle& output,
                                int& x, int& y, int& w, int& h)
 {
 	double x1, y1, x2, y2;
