@@ -1,14 +1,14 @@
-// $Id: BreakPointBase.cc 7711 2008-03-26 17:47:02Z m9710797 $
+// $Id: BreakPointBase.cc 11371 2010-03-28 17:00:33Z m9710797 $
 
 #include "BreakPointBase.hh"
 #include "TclObject.hh"
 #include "CommandException.hh"
-#include "CliComm.hh"
+#include "GlobalCliComm.hh"
 #include "ScopedAssign.hh"
 
 namespace openmsx {
 
-BreakPointBase::BreakPointBase(CliComm& cliComm_,
+BreakPointBase::BreakPointBase(GlobalCliComm& cliComm_,
                                std::auto_ptr<TclObject> command_,
                                std::auto_ptr<TclObject> condition_)
 	: cliComm(cliComm_), command(command_), condition(condition_)
@@ -60,5 +60,10 @@ void BreakPointBase::checkAndExecute()
 	}
 }
 
+Tcl_Interp* BreakPointBase::getInterpreter() const
+{
+	return command->getInterpreter();
+}
+
 } // namespace openmsx
 
