@@ -1,4 +1,4 @@
-// $Id: DebugCondition.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: DebugCondition.cc 11371 2010-03-28 17:00:33Z m9710797 $
 
 #include "DebugCondition.hh"
 #include "TclObject.hh"
@@ -7,7 +7,7 @@ namespace openmsx {
 
 unsigned DebugCondition::lastId = 0;
 
-DebugCondition::DebugCondition(CliComm& cliComm,
+DebugCondition::DebugCondition(GlobalCliComm& cliComm,
                                std::auto_ptr<TclObject> command,
                                std::auto_ptr<TclObject> condition)
 	: BreakPointBase(cliComm, command, condition)
