@@ -1,4 +1,4 @@
-// $Id: MSXCPU.cc 9389 2009-03-18 18:11:38Z m9710797 $
+// $Id: MSXCPU.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
@@ -13,6 +13,7 @@
 #include "InfoTopic.hh"
 #include "TclObject.hh"
 #include "serialize.hh"
+#include "unreachable.hh"
 #include <cassert>
 
 using std::string;
@@ -106,7 +107,7 @@ void MSXCPU::setActiveCPU(CPUType cpu)
 			tmp = r800.get();
 			break;
 		default:
-			assert(false);
+			UNREACHABLE;
 			tmp = NULL; // prevent warning
 	}
 	if (tmp != activeCPU) {
@@ -195,6 +196,11 @@ void MSXCPU::lowerNMI()
 	}
 }
 
+bool MSXCPU::isM1Cycle(unsigned address) const
+{
+	return activeCPU->isM1Cycle(address);
+}
+
 bool MSXCPU::isR800Active()
 {
 	return activeCPU == r800.get();
@@ -307,9 +313,8 @@ byte MSXCPUDebuggable::read(unsigned address)
 	case 25: return regs.getR();
 	case 26: return regs.getIM();
 	case 27: return regs.getIFF1() + 2 * regs.getIFF2();
+	default: UNREACHABLE; return 0;
 	}
-	assert(false);
-	return 0;
 }
 
 void MSXCPUDebuggable::write(unsigned address, byte value)
@@ -350,8 +355,7 @@ void MSXCPUDebuggable::write(unsigned address, byte value)
 		regs.setIFF2((value & 0x02) != 0);
 		break;
 	default:
-		assert(false);
-		break;
+		UNREACHABLE;
 	}
 }
 
@@ -359,9 +363,9 @@ void MSXCPUDebuggable::write(unsigned address, byte value)
 template<typename Archive>
 void MSXCPU::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("z80",  *z80);
+	ar.serializeWithID("z80", *z80);
 	if (r800.get()) {
-		ar.serialize("r800", *r800);
+		ar.serializeWithID("r800", *r800);
 	}
 	ar.serializePointerID("activeCPU", activeCPU);
 	ar.serializePointerID("newCPU",    newCPU);
