@@ -1,4 +1,4 @@
-// $Id: MSXCPUInterface.cc 9422 2009-03-24 19:55:40Z m9710797 $
+// $Id: MSXCPUInterface.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "MSXCPUInterface.hh"
 #include "BreakPoint.hh"
@@ -26,6 +26,8 @@
 #include "serialize.hh"
 #include "StringOp.hh"
 #include "checked_cast.hh"
+#include <tcl.h>
+#include "unreachable.hh"
 #include <cstdio>
 #include <memory> // for auto_ptr
 #include <set>
@@ -162,14 +164,14 @@ MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard_)
 	        motherBoard_.getMachineInfoCommand(), *this, true))
 	, outputPortInfo(new IOInfo(
 	        motherBoard_.getMachineInfoCommand(), *this, false))
-	, dummyDevice(motherBoard_.getDummyDevice())
+	, dummyDevice(DeviceFactory::createDummyDevice(motherBoard_))
 	, msxcpu(motherBoard_.getCPU())
 	, cliComm(motherBoard_.getMSXCliComm())
 	, motherBoard(motherBoard_)
 {
 	for (int port = 0; port < 256; ++port) {
-		IO_In [port] = &dummyDevice;
-		IO_Out[port] = &dummyDevice;
+		IO_In [port] = dummyDevice.get();
+		IO_Out[port] = dummyDevice.get();
 	}
 	for (int primSlot = 0; primSlot < 4; ++primSlot) {
 		primarySlotState[primSlot] = 0;
@@ -178,12 +180,12 @@ MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard_)
 		subSlotRegister[primSlot] = 0;
 		for (int secSlot = 0; secSlot < 4; ++secSlot) {
 			for (int page = 0; page < 4; ++page) {
-				slotLayout[primSlot][secSlot][page] = &dummyDevice;
+				slotLayout[primSlot][secSlot][page] = dummyDevice.get();
 			}
 		}
 	}
 	for (int page = 0; page < 4; ++page) {
-		visibleDevices[page] = &dummyDevice;
+		visibleDevices[page] = dummyDevice.get();
 	}
 
 	// initially allow all regions to be cached
@@ -196,10 +198,10 @@ MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard_)
 
 	if (motherBoard.isTurboR()) {
 		// TODO also MSX2+ needs (slightly different) VDPIODelay
-		delayDevice = DeviceFactory::createVDPIODelay(motherBoard);
+		delayDevice = DeviceFactory::createVDPIODelay(motherBoard, *this);
 		for (int port = 0x98; port <= 0x9B; ++port) {
-			assert(IO_In [port] == &dummyDevice);
-			assert(IO_Out[port] == &dummyDevice);
+			assert(IO_In [port] == dummyDevice.get());
+			assert(IO_Out[port] == dummyDevice.get());
 			IO_In [port] = delayDevice.get();
 			IO_Out[port] = delayDevice.get();
 		}
@@ -227,8 +229,8 @@ MSXCPUInterface::~MSXCPUInterface()
 		for (int port = 0x98; port <= 0x9B; ++port) {
 			assert(IO_In [port] == delayDevice.get());
 			assert(IO_Out[port] == delayDevice.get());
-			IO_In [port] = &dummyDevice;
-			IO_Out[port] = &dummyDevice;
+			IO_In [port] = dummyDevice.get();
+			IO_Out[port] = dummyDevice.get();
 		}
 	}
 
@@ -236,22 +238,22 @@ MSXCPUInterface::~MSXCPUInterface()
 
 	#ifndef NDEBUG
 	for (int port = 0; port < 256; ++port) {
-		if (IO_In[port] != &dummyDevice) {
+		if (IO_In[port] != dummyDevice.get()) {
 			std::cout << "In-port " << port << " still registered "
 			          << IO_In[port]->getName() << std::endl;
-			assert(false);
+			UNREACHABLE;
 		}
-		if (IO_Out[port] != &dummyDevice) {
+		if (IO_Out[port] != dummyDevice.get()) {
 			std::cout << "Out-port " << port << " still registered "
 			          << IO_Out[port]->getName() << std::endl;
-			assert(false);
+			UNREACHABLE;
 		}
 	}
 	for (int primSlot = 0; primSlot < 4; ++primSlot) {
 		assert(!isExpanded(primSlot));
 		for (int secSlot = 0; secSlot < 4; ++secSlot) {
 			for (int page = 0; page < 4; ++page) {
-				assert(slotLayout[primSlot][secSlot][page] == &dummyDevice);
+				assert(slotLayout[primSlot][secSlot][page] == dummyDevice.get());
 			}
 		}
 	}
@@ -261,7 +263,7 @@ MSXCPUInterface::~MSXCPUInterface()
 void MSXCPUInterface::removeAllWatchPoints()
 {
 	while (!watchPoints.empty()) {
-		removeWatchPoint(*watchPoints.back());
+		removeWatchPoint(watchPoints.back());
 	}
 
 }
@@ -273,7 +275,7 @@ byte MSXCPUInterface::readMemSlow(word address, EmuTime::param time)
 		// execute read watches before actual read
 		if (readWatchSet[address >> CacheLine::BITS]
 		                [address &  CacheLine::LOW]) {
-			executeMemWatch(address, WatchPoint::READ_MEM);
+			executeMemWatch(WatchPoint::READ_MEM, address);
 		}
 	}
 	if (unlikely((address == 0xFFFF) && isExpanded(primarySlotState[3]))) {
@@ -304,7 +306,7 @@ void MSXCPUInterface::writeMemSlow(word address, byte value, EmuTime::param time
 		// execute write watches after actual write
 		if (writeWatchSet[address >> CacheLine::BITS]
 		                 [address &  CacheLine::LOW]) {
-			executeMemWatch(address, WatchPoint::WRITE_MEM);
+			executeMemWatch(WatchPoint::WRITE_MEM, address, value);
 		}
 	}
 }
@@ -313,7 +315,7 @@ void MSXCPUInterface::setExpanded(int ps)
 {
 	if (expanded[ps] == 0) {
 		for (int page = 0; page < 4; ++page) {
-			if (slotLayout[ps][0][page] != &dummyDevice) {
+			if (slotLayout[ps][0][page] != dummyDevice.get()) {
 				throw MSXException("Can't expand slot because "
 				                   "it's already in use.");
 			}
@@ -327,7 +329,7 @@ void MSXCPUInterface::testUnsetExpanded(
 {
 	// TODO handle multi-devices
 	std::set<MSXDevice*> allowed(alreadyRemoved.begin(), alreadyRemoved.end());
-	allowed.insert(&dummyDevice);
+	allowed.insert(dummyDevice.get());
 	assert(isExpanded(ps));
 	if (expanded[ps] == 1) {
 		for (int ss = 0; ss < 4; ++ss) {
@@ -350,7 +352,7 @@ void MSXCPUInterface::unsetExpanded(int ps)
 		vector<MSXDevice*> dummy;
 		testUnsetExpanded(ps, dummy);
 	} catch (...) {
-		assert(false);
+		UNREACHABLE;
 	}
 #endif
 	expanded[ps]--;
@@ -399,7 +401,7 @@ void MSXCPUInterface::register_IO(int port, bool isIn,
 	PRT_DEBUG(device->getName() << " registers " << (isIn ? "In" : "Out") <<
 	          "-port " << std::hex << port << std::dec);
 
-	if (devicePtr == &dummyDevice) {
+	if (devicePtr == dummyDevice.get()) {
 		// first, replace DummyDevice
 		devicePtr = device;
 	} else {
@@ -439,17 +441,16 @@ void MSXCPUInterface::unregister_IO(MSXDevice*& devicePtr, MSXDevice* device)
 	} else {
 		// remove last, put back DummyDevice
 		assert(devicePtr == device);
-		devicePtr = &dummyDevice;
+		devicePtr = dummyDevice.get();
 	}
 }
 
 
 static void reportMemOverlap(int ps, int ss, MSXDevice& dev1, MSXDevice& dev2)
 {
-	throw MSXException("Overlapping memory devices in slot " +
-	                   StringOp::toString(ps) + "." +
-	                   StringOp::toString(ss) + ": " +
-	                   dev1.getName() + " and " + dev2.getName() + ".");
+	throw MSXException(StringOp::Builder() <<
+		"Overlapping memory devices in slot " << ps + '.' << ss <<
+		": " << dev1.getName() << " and " << dev2.getName() + '.');
 }
 
 void MSXCPUInterface::registerSlot(
@@ -459,25 +460,24 @@ void MSXCPUInterface::registerSlot(
 	          std::dec << ps << " " << ss << " 0x" <<
 	          std::hex << base << "-0x" << (base + size - 1));
 	if (!isExpanded(ps) && (ss != 0)) {
-		throw MSXException(
-			"Slot " + StringOp::toString(ps) +
-			"."     + StringOp::toString(ss) +
+		throw MSXException(StringOp::Builder() <<
+			"Slot " << ps << '.' << ss <<
 			" does not exist because slot is not expanded.");
 	}
 	int page = base >> 14;
 	MSXDevice*& slot = slotLayout[ps][ss][page];
 	if (size == 0x4000) {
 		// full 16kb, directly register device (no multiplexer)
-		if (slot != &dummyDevice) {
+		if (slot != dummyDevice.get()) {
 			reportMemOverlap(ps, ss, *slot, device);
 		}
 		slot = &device;
 	} else {
 		// partial page
-		if (slot == &dummyDevice) {
+		if (slot == dummyDevice.get()) {
 			// first
-			MSXMultiMemDevice* multi = new MSXMultiMemDevice(
-			                            device.getMotherBoard());
+			MSXMultiMemDevice* multi =
+				new MSXMultiMemDevice(device.getMotherBoard(), *this);
 			multi->add(device, base, size);
 			slot = multi;
 		} else if (MSXMultiMemDevice* multi =
@@ -504,12 +504,12 @@ void MSXCPUInterface::unregisterSlot(
 		multi->remove(device, base, size);
 		if (multi->empty()) {
 			delete multi;
-			slot = &dummyDevice;
+			slot = dummyDevice.get();
 		}
 	} else {
 		// full 16kb range
 		assert(slot == &device);
-		slot = &dummyDevice;
+		slot = dummyDevice.get();
 	}
 	updateVisible(page);
 }
@@ -691,6 +691,11 @@ void MSXCPUInterface::writeSlottedMem(unsigned address, byte value,
 	}
 }
 
+DummyDevice& MSXCPUInterface::getDummyDevice()
+{
+	return *dummyDevice;
+}
+
 
 void MSXCPUInterface::insertBreakPoint(shared_ptr<BreakPoint> bp)
 {
@@ -750,33 +755,35 @@ void MSXCPUInterface::setWatchPoint(shared_ptr<WatchPoint> watchPoint)
 		updateMemWatch(type);
 		break;
 	default:
-		assert(false);
-		break;
+		UNREACHABLE; break;
 	}
 }
 
-void MSXCPUInterface::removeWatchPoint(WatchPoint& watchPoint)
+void MSXCPUInterface::removeWatchPoint(shared_ptr<WatchPoint> watchPoint)
 {
+	// Pass by shared_ptr to keep the object alive for the duration of
+	// this function, otherwise it gets deleted as soon as it's
+	// removed from the watchPoints collection.
 	for (WatchPoints::iterator it = watchPoints.begin();
 	     it != watchPoints.end(); ++it) {
-		if (it->get() == &watchPoint) {
-			WatchPoint::Type type = watchPoint.getType();
+		if (*it == watchPoint) {
+			// remove before calling updateMemWatch()
+			watchPoints.erase(it);
+			WatchPoint::Type type = watchPoint->getType();
 			switch (type) {
 			case WatchPoint::READ_IO:
-				unregisterIOWatch(watchPoint, IO_In);
+				unregisterIOWatch(*watchPoint, IO_In);
 				break;
 			case WatchPoint::WRITE_IO:
-				unregisterIOWatch(watchPoint, IO_Out);
+				unregisterIOWatch(*watchPoint, IO_Out);
 				break;
 			case WatchPoint::READ_MEM:
 			case WatchPoint::WRITE_MEM:
 				updateMemWatch(type);
 				break;
 			default:
-				assert(false);
-				break;
+				UNREACHABLE; break;
 			}
-			watchPoints.erase(it);
 			break;
 		}
 	}
@@ -812,22 +819,22 @@ const MSXCPUInterface::Conditions& MSXCPUInterface::getConditions()
 
 void MSXCPUInterface::registerIOWatch(WatchPoint& watchPoint, MSXDevice** devices)
 {
-	assert(dynamic_cast<MSXWatchIODevice*>(&watchPoint));
-	MSXWatchIODevice& ioWatch = static_cast<MSXWatchIODevice&>(watchPoint);
+	assert(dynamic_cast<WatchIO*>(&watchPoint));
+	WatchIO& ioWatch = static_cast<WatchIO&>(watchPoint);
 	unsigned beginPort = ioWatch.getBeginAddress();
 	unsigned endPort   = ioWatch.getEndAddress();
 	assert(beginPort <= endPort);
 	assert(endPort < 0x100);
 	for (unsigned port = beginPort; port <= endPort; ++port) {
-		ioWatch.getDevicePtr() = devices[port];
-		devices[port] = &ioWatch;
+		ioWatch.getDevice(port).getDevicePtr() = devices[port];
+		devices[port] = &ioWatch.getDevice(port);
 	}
 }
 
 void MSXCPUInterface::unregisterIOWatch(WatchPoint& watchPoint, MSXDevice** devices)
 {
-	const MSXWatchIODevice& ioWatch =
-		checked_cast<const MSXWatchIODevice&>(watchPoint);
+	assert(dynamic_cast<WatchIO*>(&watchPoint));
+	WatchIO& ioWatch = static_cast<WatchIO&>(watchPoint);
 	unsigned beginPort = ioWatch.getBeginAddress();
 	unsigned endPort   = ioWatch.getEndAddress();
 	assert(beginPort <= endPort);
@@ -836,7 +843,7 @@ void MSXCPUInterface::unregisterIOWatch(WatchPoint& watchPoint, MSXDevice** devi
 	for (unsigned port = beginPort; port <= endPort; ++port) {
 		// find pointer to watchpoint
 		MSXDevice** prev = &devices[port];
-		while (*prev != &ioWatch) {
+		while (*prev != &ioWatch.getDevice(port)) {
 			prev = &checked_cast<MSXWatchIODevice*>(*prev)->getDevicePtr();
 		}
 		// remove watchpoint from chain
@@ -879,8 +886,19 @@ void MSXCPUInterface::updateMemWatch(WatchPoint::Type type)
 	msxcpu.invalidateMemCache(0x0000, 0x10000);
 }
 
-void MSXCPUInterface::executeMemWatch(word address, WatchPoint::Type type)
+void MSXCPUInterface::executeMemWatch(WatchPoint::Type type,
+                                      unsigned address, unsigned value)
 {
+	assert(!watchPoints.empty());
+
+	Tcl_Interp* interp = watchPoints.front()->getInterpreter();
+	Tcl_SetVar(interp, "wp_last_address", StringOp::toString(address).c_str(),
+	           TCL_GLOBAL_ONLY);
+	if (value != ~0u) {
+		Tcl_SetVar(interp, "wp_last_value", StringOp::toString(value).c_str(),
+			   TCL_GLOBAL_ONLY);
+	}
+
 	WatchPoints wpCopy(watchPoints);
 	for (WatchPoints::const_iterator it = wpCopy.begin();
 	     it != wpCopy.end(); ++it) {
@@ -890,6 +908,9 @@ void MSXCPUInterface::executeMemWatch(word address, WatchPoint::Type type)
 			(*it)->checkAndExecute();
 		}
 	}
+
+	Tcl_UnsetVar(interp, "wp_last_address", TCL_GLOBAL_ONLY);
+	Tcl_UnsetVar(interp, "wp_last_value", TCL_GLOBAL_ONLY);
 }
 
 
@@ -903,7 +924,7 @@ void MSXCPUInterface::doBreak()
 	breakedSetting->setReadOnlyValue(true);
 	reactor.getCliComm().update(CliComm::STATUS, "cpu", "suspended");
 	reactor.getEventDistributor().distributeEvent(
-		new SimpleEvent<OPENMSX_BREAK_EVENT>());
+		new SimpleEvent(OPENMSX_BREAK_EVENT));
 }
 
 void MSXCPUInterface::doStep()
