@@ -1,8 +1,8 @@
-// $Id: MSXMultiDevice.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: MSXMultiDevice.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "MSXMultiDevice.hh"
 #include "XMLElement.hh"
-#include <cassert>
+#include "unreachable.hh"
 
 namespace openmsx {
 
@@ -19,17 +19,17 @@ MSXMultiDevice::MSXMultiDevice(MSXMotherBoard& motherboard)
 
 void MSXMultiDevice::reset(EmuTime::param /*time*/)
 {
-	assert(false);
+	UNREACHABLE;
 }
 
 void MSXMultiDevice::powerUp(EmuTime::param /*time*/)
 {
-	assert(false);
+	UNREACHABLE;
 }
 
 void MSXMultiDevice::powerDown(EmuTime::param /*time*/)
 {
-	assert(false);
+	UNREACHABLE;
 }
 
 } // namespace openmsx
