@@ -1,4 +1,4 @@
-// $Id: MSXMultiIODevice.cc 5038 2006-01-13 20:55:59Z m9710797 $
+// $Id: MSXMultiIODevice.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXMultiIODevice.hh"
 #include <algorithm>
@@ -43,7 +43,7 @@ std::string MSXMultiIODevice::getName() const
 	return name;
 }
 
-byte MSXMultiIODevice::readIO(word port, const EmuTime& time)
+byte MSXMultiIODevice::readIO(word port, EmuTime::param time)
 {
 	// conflict: return the result from the first device, call readIO()
 	//           also on all other devices, but discard result
@@ -56,14 +56,14 @@ byte MSXMultiIODevice::readIO(word port, const EmuTime& time)
 	return result;
 }
 
-byte MSXMultiIODevice::peekIO(word port, const EmuTime& time) const
+byte MSXMultiIODevice::peekIO(word port, EmuTime::param time) const
 {
 	// conflict: just peek first device
 	assert(!devices.empty());
 	return devices.front()->peekIO(port, time);
 }
 
-void MSXMultiIODevice::writeIO(word port, byte value, const EmuTime& time)
+void MSXMultiIODevice::writeIO(word port, byte value, EmuTime::param time)
 {
 	for (Devices::iterator it = devices.begin();
 	     it != devices.end(); ++it) {
