@@ -1,7 +1,8 @@
-// $Id: VDPIODelay.cc 8552 2009-02-03 10:03:34Z m9710797 $
+// $Id: VDPIODelay.cc 10184 2009-07-05 18:58:52Z mthuurne $
 
 #include "VDPIODelay.hh"
 #include "MSXCPU.hh"
+#include "MSXCPUInterface.hh"
 #include "MSXMotherBoard.hh"
 #include "DummyDevice.hh"
 #include "serialize.hh"
@@ -10,14 +11,14 @@
 namespace openmsx {
 
 VDPIODelay::VDPIODelay(MSXMotherBoard& motherboard, const XMLElement& config,
-                       EmuTime::param time)
+                       MSXCPUInterface& cpuInterface)
 	: MSXDevice(motherboard, config)
 	, cpu(motherboard.getCPU())
-	, lastTime(time)
+	, lastTime(EmuTime::zero)
 {
 	for (int port = 0x098; port <= 0x9B; ++port) {
-		getInDevicePtr (port) = &motherboard.getDummyDevice();
-		getOutDevicePtr(port) = &motherboard.getDummyDevice();
+		getInDevicePtr (port) = &cpuInterface.getDummyDevice();
+		getOutDevicePtr(port) = &cpuInterface.getDummyDevice();
 	}
 }
 
