@@ -1,4 +1,4 @@
-// $Id: WatchPoint.cc 8182 2008-08-28 18:05:52Z m9710797 $
+// $Id: WatchPoint.cc 11371 2010-03-28 17:00:33Z m9710797 $
 
 #include "WatchPoint.hh"
 #include "TclObject.hh"
@@ -8,7 +8,7 @@ namespace openmsx {
 
 unsigned WatchPoint::lastId = 0;
 
-WatchPoint::WatchPoint(CliComm& cliComm,
+WatchPoint::WatchPoint(GlobalCliComm& cliComm,
                        std::auto_ptr<TclObject> command,
                        std::auto_ptr<TclObject> condition,
                        Type type_, unsigned beginAddr_, unsigned endAddr_)
