@@ -1,10 +1,11 @@
-// $Id: Debugger.cc 9422 2009-03-24 19:55:40Z m9710797 $
+// $Id: Debugger.cc 11371 2010-03-28 17:00:33Z m9710797 $
 
 #include "Debugger.hh"
 #include "Debuggable.hh"
 #include "Probe.hh"
 #include "ProbeBreakPoint.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "MSXCPU.hh"
 #include "MSXCPUInterface.hh"
 #include "BreakPoint.hh"
@@ -15,6 +16,7 @@
 #include "CommandException.hh"
 #include "StringOp.hh"
 #include "shared_ptr.hh"
+#include "unreachable.hh"
 #include <cassert>
 #include <memory>
 
@@ -32,8 +34,8 @@ class DebugCmd : public RecordedCommand
 {
 public:
 	DebugCmd(CommandController& commandController,
-	         MSXEventDistributor& msxEventDistributor,
-	         Scheduler& scheduler, CliComm& cliComm,
+	         StateChangeDistributor& stateChangeDistributor,
+	         Scheduler& scheduler, GlobalCliComm& cliComm,
 	         Debugger& debugger);
 	virtual bool needRecord(const vector<TclObject*>& tokens) const;
 	virtual void execute(const vector<TclObject*>& tokens,
@@ -91,7 +93,7 @@ private:
 	void probeListBreakPoints(const vector<TclObject*>& tokens,
 	                          TclObject& result);
 
-	CliComm& cliComm;
+	GlobalCliComm& cliComm;
 	Debugger& debugger;
 };
 
@@ -99,9 +101,9 @@ private:
 Debugger::Debugger(MSXMotherBoard& motherBoard_)
 	: motherBoard(motherBoard_)
 	, debugCmd(new DebugCmd(motherBoard.getCommandController(),
-	                        motherBoard.getMSXEventDistributor(),
+	                        motherBoard.getStateChangeDistributor(),
 	                        motherBoard.getScheduler(),
-	                        motherBoard.getMSXCliComm(), *this))
+	                        motherBoard.getReactor().getGlobalCliComm(), *this))
 	, cpu(0)
 {
 }
@@ -256,10 +258,10 @@ static word getAddress(const vector<TclObject*>& tokens)
 }
 
 DebugCmd::DebugCmd(CommandController& commandController,
-                   MSXEventDistributor& msxEventDistributor,
-                   Scheduler& scheduler, CliComm& cliComm_,
+                   StateChangeDistributor& stateChangeDistributor,
+                   Scheduler& scheduler, GlobalCliComm& cliComm_,
                    Debugger& debugger_)
-	: RecordedCommand(commandController, msxEventDistributor,
+	: RecordedCommand(commandController, stateChangeDistributor,
 	                  scheduler, "debug")
 	, cliComm(cliComm_)
 	, debugger(debugger_)
@@ -463,7 +465,7 @@ void DebugCmd::setBreakPoint(const vector<TclObject*>& tokens,
 			throw CommandException("Too many arguments.");
 		}
 	}
-	result.setString("bp#" + StringOp::toString(bp->getId()));
+	result.setString(StringOp::Builder() << "bp#" << bp->getId());
 	debugger.motherBoard.getCPUInterface().insertBreakPoint(bp);
 }
 
@@ -518,7 +520,7 @@ void DebugCmd::listBreakPoints(const vector<TclObject*>& /*tokens*/,
 	     it != breakPoints.end(); ++it) {
 		const BreakPoint& bp = *it->second;
 		TclObject line(result.getInterpreter());
-		line.addListElement("bp#" + StringOp::toString(bp.getId()));
+		line.addListElement(StringOp::Builder() << "bp#" << bp.getId());
 		line.addListElement("0x" + StringOp::toHexString(bp.getAddress(), 4));
 		line.addListElement(bp.getCondition());
 		line.addListElement(bp.getCommand());
@@ -581,9 +583,9 @@ void DebugCmd::setWatchPoint(const vector<TclObject*>& tokens,
 		if (endAddr >= max) {
 			throw CommandException("Invalid address: out of range");
 		}
-		wp.reset(new MSXWatchIODevice(debugger.motherBoard, type,
-		                              beginAddr, endAddr,
-		                              command, condition));
+		wp.reset(new WatchIO(debugger.motherBoard, type,
+		                     beginAddr, endAddr,
+		                     command, condition));
 		break;
 	}
 	default:
@@ -593,7 +595,7 @@ void DebugCmd::setWatchPoint(const vector<TclObject*>& tokens,
 			throw CommandException("Too many arguments.");
 		}
 	}
-	result.setString("wp#" + StringOp::toString(wp->getId()));
+	result.setString(StringOp::Builder() << "wp#" << wp->getId());
 	debugger.motherBoard.getCPUInterface().setWatchPoint(wp);
 }
 
@@ -612,8 +614,8 @@ void DebugCmd::removeWatchPoint(const vector<TclObject*>& tokens,
 		unsigned id = StringOp::stringToInt(tmp.substr(3));
 		for (MSXCPUInterface::WatchPoints::const_iterator it =
 			watchPoints.begin(); it != watchPoints.end(); ++it) {
-			WatchPoint& wp = **it;
-			if (wp.getId() == id) {
+			shared_ptr<WatchPoint> wp = *it;
+			if (wp->getId() == id) {
 				interface.removeWatchPoint(wp);
 				return;
 			}
@@ -632,7 +634,7 @@ void DebugCmd::listWatchPoints(const vector<TclObject*>& /*tokens*/,
 	     it != watchPoints.end(); ++it) {
 		const WatchPoint& wp = **it;
 		TclObject line(result.getInterpreter());
-		line.addListElement("wp#" + StringOp::toString(wp.getId()));
+		line.addListElement(StringOp::Builder() << "wp#" << wp.getId());
 		string type;
 		switch (wp.getType()) {
 		case WatchPoint::READ_IO:
@@ -648,8 +650,7 @@ void DebugCmd::listWatchPoints(const vector<TclObject*>& /*tokens*/,
 			type = "write_mem";
 			break;
 		default:
-			assert(false);
-			break;
+			UNREACHABLE; break;
 		}
 		line.addListElement(type);
 		unsigned beginAddr = wp.getBeginAddress();
@@ -697,7 +698,7 @@ void DebugCmd::setCondition(const vector<TclObject*>& tokens,
 			throw CommandException("Too many arguments.");
 		}
 	}
-	result.setString("cond#" + StringOp::toString(dc->getId()));
+	result.setString(StringOp::Builder() << "cond#" << dc->getId());
 	debugger.motherBoard.getCPUInterface().setCondition(dc);
 }
 
@@ -736,7 +737,7 @@ void DebugCmd::listConditions(const vector<TclObject*>& /*tokens*/,
 	     it != conditions.end(); ++it) {
 		const DebugCondition& cond = **it;
 		TclObject line(result.getInterpreter());
-		line.addListElement("cond#" + StringOp::toString(cond.getId()));
+		line.addListElement(StringOp::Builder() << "cond#" << cond.getId());
 		line.addListElement(cond.getCondition());
 		line.addListElement(cond.getCommand());
 		res += line.getString() + '\n';
@@ -824,7 +825,7 @@ void DebugCmd::probeSetBreakPoint(const vector<TclObject*>& tokens,
 			throw CommandException("Too many arguments.");
 		}
 	}
-	result.setString("pp#" + StringOp::toString(bp->getId()));
+	result.setString(StringOp::Builder() << "pp#" << bp->getId());
 	debugger.insertProbeBreakPoint(bp);
 }
 void DebugCmd::probeRemoveBreakPoint(const vector<TclObject*>& tokens,
@@ -844,7 +845,7 @@ void DebugCmd::probeListBreakPoints(const vector<TclObject*>& /*tokens*/,
 	     it != debugger.probeBreakPoints.end(); ++it) {
 		const ProbeBreakPoint& bp = **it;
 		TclObject line(result.getInterpreter());
-		line.addListElement("pp#" + StringOp::toString(bp.getId()));
+		line.addListElement(StringOp::Builder() << "pp#" << bp.getId());
 		line.addListElement(bp.getProbe().getName());
 		line.addListElement(bp.getCondition());
 		line.addListElement(bp.getCommand());
@@ -967,9 +968,15 @@ string DebugCmd::help(const vector<string>& tokens) const
 		"memory location or IO port. Otherwise region must be a list of "
 		"two values (enclosed in braces) that specify a begin and end "
 		"point of a whole memory region or a range of IO ports.\n"
-		"  Examples:\n"
-		"    debug set_watchpoint write_io 0x99 {[reg A] == 0x81}\n"
-		"    debug set_watchpoint read_mem {0xfbe5 0xfbef}\n";
+		"During the execution of <cmd>, the following global Tcl "
+		"variables are set:\n"
+		"  ::wp_last_address   this is the actual address of the mem/io "
+		"read/write that triggered the watchpoint\n"
+		"  ::wp_last_value     this is the actual value that was written "
+		"by the mem/io write that triggered the watchpoint\n"
+		"Examples:\n"
+		"  debug set_watchpoint write_io 0x99 {[reg A] == 0x81}\n"
+		"  debug set_watchpoint read_mem {0xfbe5 0xfbef}\n";
 	static const string removeWatchPointHelp =
 		"debug remove_watchpoint <id>\n"
 		"  Remove the watchpoint with given ID again. You can use the "
@@ -1094,7 +1101,7 @@ set<string> DebugCmd::getBreakPointIdsAsStringSet() const
 	set<string> bpids;
 	for (BreakPoints::const_iterator it = breakPoints.begin();
 	     it != breakPoints.end(); ++it) {
-		bpids.insert("bp#" + StringOp::toString((*it->second).getId()));
+		bpids.insert(StringOp::Builder() << "bp#" << it->second->getId());
 	}
 	return bpids;
 }
@@ -1105,7 +1112,7 @@ set<string> DebugCmd::getWatchPointIdsAsStringSet() const
 	set<string> wpids;
 	for (MSXCPUInterface::WatchPoints::const_iterator it = watchPoints.begin();
 	     it != watchPoints.end(); ++it) {
-		wpids.insert("wp#" + StringOp::toString((*it)->getId()));
+		wpids.insert(StringOp::Builder() << "wp#" << (*it)->getId());
 	}
 	return wpids;
 }
@@ -1116,7 +1123,7 @@ set<string> DebugCmd::getConditionIdsAsStringSet() const
 	set<string> condids;
 	for (MSXCPUInterface::Conditions::const_iterator it = conditions.begin();
 	     it != conditions.end(); ++it) {
-		condids.insert("cond#" + StringOp::toString((*it)->getId()));
+		condids.insert(StringOp::Builder() << "cond#" << (*it)->getId());
 	}
 	return condids;
 }
