@@ -1,4 +1,4 @@
-// $Id: ProbeBreakPoint.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: ProbeBreakPoint.cc 11371 2010-03-28 17:00:33Z m9710797 $
 
 #include "ProbeBreakPoint.hh"
 #include "Probe.hh"
@@ -10,7 +10,7 @@ namespace openmsx {
 unsigned ProbeBreakPoint::lastId = 0;
 
 ProbeBreakPoint::ProbeBreakPoint(
-		CliComm& cliComm,
+		GlobalCliComm& cliComm,
 		std::auto_ptr<TclObject> command,
 		std::auto_ptr<TclObject> condition,
 		Debugger& debugger_,
