@@ -1,9 +1,9 @@
-// $Id: SimpleDebuggable.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: SimpleDebuggable.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include "Debugger.hh"
-#include <cassert>
+#include "unreachable.hh"
 
 namespace openmsx {
 
@@ -45,8 +45,7 @@ byte SimpleDebuggable::read(unsigned address)
 
 byte SimpleDebuggable::read(unsigned /*address*/, EmuTime::param /*time*/)
 {
-	assert(false);
-	return 0;
+	UNREACHABLE; return 0;
 }
 
 void SimpleDebuggable::write(unsigned address, byte value)
