@@ -1,16 +1,21 @@
-// $Id: AfterCommand.cc 9440 2009-03-30 16:55:41Z m9710797 $
+// $Id: AfterCommand.cc 11441 2010-05-24 20:18:08Z m9710797 $
 
 #include "AfterCommand.hh"
 #include "CommandController.hh"
 #include "CliComm.hh"
 #include "Schedulable.hh"
 #include "EventDistributor.hh"
+#include "MSXEventDistributor.hh"
+#include "InputEventFactory.hh"
 #include "Reactor.hh"
 #include "MSXMotherBoard.hh"
 #include "Alarm.hh"
 #include "EmuTime.hh"
 #include "CommandException.hh"
+#include "Interpreter.hh"
+#include "StringOp.hh"
 #include "openmsx.hh"
+#include "unreachable.hh"
 #include <algorithm>
 #include <cstdlib>
 #include <sstream>
@@ -26,18 +31,18 @@ class AfterCmd
 {
 public:
 	virtual ~AfterCmd();
-	const std::string& getCommand() const;
-	const std::string& getId() const;
-	virtual const std::string& getType() const = 0;
+	const string& getCommand() const;
+	const string& getId() const;
+	virtual string getType() const = 0;
 	void execute();
 protected:
 	AfterCmd(AfterCommand& afterCommand,
-		 const std::string& command);
+		 const string& command);
 	shared_ptr<AfterCmd> removeSelf();
 private:
 	AfterCommand& afterCommand;
-	std::string command;
-	std::string id;
+	string command;
+	string id;
 	static unsigned lastAfterId;
 };
 
@@ -49,11 +54,11 @@ public:
 protected:
 	AfterTimedCmd(Scheduler& scheduler,
 		      AfterCommand& afterCommand,
-		      const std::string& command, double time);
+		      const string& command, double time);
 private:
 	virtual void executeUntil(EmuTime::param time, int userData);
 	virtual void schedulerDeleted();
-	virtual const std::string& schedName() const;
+	virtual const string& schedName() const;
 
 	double time;
 };
@@ -63,8 +68,8 @@ class AfterTimeCmd : public AfterTimedCmd
 public:
 	AfterTimeCmd(Scheduler& scheduler,
 		     AfterCommand& afterCommand,
-		     const std::string& command, double time);
-	virtual const std::string& getType() const;
+		     const string& command, double time);
+	virtual string getType() const;
 };
 
 class AfterIdleCmd : public AfterTimedCmd
@@ -72,8 +77,8 @@ class AfterIdleCmd : public AfterTimedCmd
 public:
 	AfterIdleCmd(Scheduler& scheduler,
 		     AfterCommand& afterCommand,
-		     const std::string& command, double time);
-	virtual const std::string& getType() const;
+		     const string& command, double time);
+	virtual string getType() const;
 };
 
 template<EventType T>
@@ -81,11 +86,23 @@ class AfterEventCmd : public AfterCmd
 {
 public:
 	AfterEventCmd(AfterCommand& afterCommand,
-		      const std::string& type,
-		      const std::string& command);
-	virtual const std::string& getType() const;
+		      const string& type,
+		      const string& command);
+	virtual string getType() const;
 private:
-	const std::string type;
+	const string type;
+};
+
+class AfterMSXEventCmd : public AfterCmd
+{
+public:
+	AfterMSXEventCmd(AfterCommand& afterCommand,
+	                 AfterCommand::EventPtr event,
+	                 const string& command);
+	virtual string getType() const;
+	AfterCommand::EventPtr getEvent() const { return event; }
+private:
+	AfterCommand::EventPtr event;
 };
 
 class AfterRealTimeCmd : public AfterCmd, private Alarm
@@ -93,9 +110,9 @@ class AfterRealTimeCmd : public AfterCmd, private Alarm
 public:
 	AfterRealTimeCmd(AfterCommand& afterCommand,
 	                 EventDistributor& eventDistributor,
-	                 const std::string& command, double time);
+	                 const string& command, double time);
 	virtual ~AfterRealTimeCmd();
-	virtual const std::string& getType() const;
+	virtual string getType() const;
 	bool hasExpired() const;
 
 private:
@@ -112,6 +129,7 @@ AfterCommand::AfterCommand(Reactor& reactor_,
 	: SimpleCommand(commandController, "after")
 	, reactor(reactor_)
 	, eventDistributor(eventDistributor_)
+	, msxEvents(NULL)
 {
 	// TODO DETACHED <-> EMU types should be cleaned up
 	//      (moved to event iso listener?)
@@ -135,22 +153,32 @@ AfterCommand::AfterCommand(Reactor& reactor_,
 		OPENMSX_FINISH_FRAME_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_BREAK_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_QUIT_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_BOOT_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_MACHINE_LOADED_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_AFTER_REALTIME_EVENT, *this);
+
+	machineSwitch();
 }
 
 AfterCommand::~AfterCommand()
 {
+	if (msxEvents) {
+		msxEvents->unregisterEventListener(*this);
+	}
+
 	eventDistributor.unregisterEventListener(
 		OPENMSX_AFTER_REALTIME_EVENT, *this);
 	eventDistributor.unregisterEventListener(
 		OPENMSX_MACHINE_LOADED_EVENT, *this);
 	eventDistributor.unregisterEventListener(
 		OPENMSX_BOOT_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_QUIT_EVENT, *this);
 	eventDistributor.unregisterEventListener(
 		OPENMSX_BREAK_EVENT, *this);
 	eventDistributor.unregisterEventListener(
@@ -178,6 +206,7 @@ string AfterCommand::execute(const vector<string>& tokens)
 	if (tokens.size() < 2) {
 		throw CommandException("Missing argument");
 	}
+	int time;
 	if (tokens[1] == "time") {
 		return afterTime(tokens);
 	} else if (tokens[1] == "realtime") {
@@ -188,6 +217,8 @@ string AfterCommand::execute(const vector<string>& tokens)
 		return afterEvent<OPENMSX_FINISH_FRAME_EVENT>(tokens);
 	} else if (tokens[1] == "break") {
 		return afterEvent<OPENMSX_BREAK_EVENT>(tokens);
+	} else if (tokens[1] == "quit") {
+		return afterEvent<OPENMSX_QUIT_EVENT>(tokens);
 	} else if (tokens[1] == "boot") {
 		return afterEvent<OPENMSX_BOOT_EVENT>(tokens);
 	} else if (tokens[1] == "machine_switch") {
@@ -196,17 +227,23 @@ string AfterCommand::execute(const vector<string>& tokens)
 		return afterInfo(tokens);
 	} else if (tokens[1] == "cancel") {
 		return afterCancel(tokens);
+	} else if (StringOp::stringToInt(tokens[1], time)) {
+		return afterTclTime(time, tokens);
+	} else {
+		// try to interpret token as an event name
+		try {
+			EventPtr event(InputEventFactory::createInputEvent(tokens[1]));
+			return afterMSXEvent(event, tokens);
+		} catch (MSXException&) {
+			throw SyntaxError();
+		}
 	}
-	throw SyntaxError();
 }
 
 static double getTime(const string& str)
 {
-	// Use "strtod" instead of "strtof" because the latter doesn't exist in
-	// the libc of FreeBSD. It's not equivalent, but we don't need maximum
-	// precision or range checks, so it's close enough.
-	double time = strtod(str.c_str(), NULL);
-	if (time <= 0) {
+	double time = StringOp::stringToDouble(str);
+	if (time < 0) {
 		throw CommandException("Not a valid time specification");
 	}
 	return time;
@@ -240,6 +277,15 @@ string AfterCommand::afterRealTime(const vector<string>& tokens)
 	return cmd->getId();
 }
 
+string AfterCommand::afterTclTime(int ms, const vector<string>& tokens)
+{
+	string command = Interpreter::mergeList(tokens.begin() + 2, tokens.end());
+	shared_ptr<AfterCmd> cmd(new AfterRealTimeCmd(
+		*this, eventDistributor, command, ms / 1000.0));
+	afterCmds.push_back(cmd);
+	return cmd->getId();
+}
+
 template<EventType T>
 string AfterCommand::afterEvent(const vector<string>& tokens)
 {
@@ -251,6 +297,16 @@ string AfterCommand::afterEvent(const vector<string>& tokens)
 	return cmd->getId();
 }
 
+string AfterCommand::afterMSXEvent(EventPtr event, const vector<string>& tokens)
+{
+	if (tokens.size() != 3) {
+		throw SyntaxError();
+	}
+	shared_ptr<AfterCmd> cmd(new AfterMSXEventCmd(*this, event, tokens[2]));
+	afterCmds.push_back(cmd);
+	return cmd->getId();
+}
+
 string AfterCommand::afterIdle(const vector<string>& tokens)
 {
 	if (tokens.size() != 4) {
@@ -292,16 +348,28 @@ string AfterCommand::afterCancel(const vector<string>& tokens)
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	AfterCmds::iterator it = afterCmds.begin();
-	for (/**/; it != afterCmds.end(); ++it) {
-		if ((*it)->getId() == tokens[2]) {
-			break;
+	if (tokens.size() == 3) {
+		for (AfterCmds::iterator it = afterCmds.begin();
+		     it != afterCmds.end(); ++it) {
+			if ((*it)->getId() == tokens[2]) {
+				afterCmds.erase(it);
+				return "";
+			}
 		}
 	}
-	if (it == afterCmds.end()) {
-		throw CommandException("No delayed command with this id");
+	string command = Interpreter::mergeList(tokens.begin() + 2, tokens.end());
+	for (AfterCmds::iterator it = afterCmds.begin();
+	     it != afterCmds.end(); ++it) {
+		if ((*it)->getCommand() == command) {
+			afterCmds.erase(it);
+			// Tcl manual is not clear about this, but it seems
+			// there's only occurence of this command canceled.
+			// It's also not clear which of the (possibly) several
+			// matches is canceled.
+			return "";
+		}
 	}
-	afterCmds.erase(it);
+	// It's not an error if no match is found
 	return "";
 }
 
@@ -312,6 +380,8 @@ string AfterCommand::help(const vector<string>& /*tokens*/) const
 	       "after idle     <seconds> <command>  execute a command after some time being idle\n"
 	       "after frame <command>               execute a command after a new frame is drawn\n"
 	       "after break <command>               execute a command after a breakpoint is reached\n"
+	       "after boot <command>                execute a command after a (re)boot\n"
+	       "after machine_switch <command>      execute a command after a switch to a new machine\n"
 	       "after info                          list all postponed commands\n"
 	       "after cancel <id>                   cancel the postponed command with given id\n";
 }
@@ -334,6 +404,16 @@ void AfterCommand::tabCompletion(vector<string>& tokens) const
 	// TODO : make more complete
 }
 
+template<typename PRED> void AfterCommand::executeMatches(PRED pred)
+{
+	// predicate should return false on matches
+	AfterCmds::iterator it = partition(afterCmds.begin(), afterCmds.end(), pred);
+	AfterCmds tmp(it, afterCmds.end());
+	afterCmds.erase(it, afterCmds.end());
+	for (AfterCmds::iterator it = tmp.begin(); it != tmp.end(); ++it) {
+		(*it)->execute();
+	}
+}
 
 template<EventType T> struct AfterEventPred {
 	bool operator()(shared_ptr<AfterCmd> x) const {
@@ -342,14 +422,7 @@ template<EventType T> struct AfterEventPred {
 };
 template<EventType T> void AfterCommand::executeEvents()
 {
-	AfterCmds::iterator it = partition(afterCmds.begin(), afterCmds.end(),
-	                                   AfterEventPred<T>());
-	AfterCmds tmp(it, afterCmds.end());
-	afterCmds.erase(it, afterCmds.end());
-
-	for (AfterCmds::iterator it = tmp.begin(); it != tmp.end(); ++it) {
-		(*it)->execute();
-	}
+	executeMatches(AfterEventPred<T>());
 }
 
 struct AfterTimePred {
@@ -365,14 +438,7 @@ struct AfterTimePred {
 };
 void AfterCommand::executeRealTime()
 {
-	AfterCmds::iterator it = partition(afterCmds.begin(), afterCmds.end(),
-	                                   AfterTimePred());
-	AfterCmds tmp(it, afterCmds.end());
-	afterCmds.erase(it, afterCmds.end());
-
-	for (AfterCmds::iterator it = tmp.begin(); it != tmp.end(); ++it) {
-		(*it)->execute();
-	}
+	executeMatches(AfterTimePred());
 }
 
 bool AfterCommand::signalEvent(shared_ptr<const Event> event)
@@ -383,7 +449,10 @@ bool AfterCommand::signalEvent(shared_ptr<const Event> event)
 		executeEvents<OPENMSX_BREAK_EVENT>();
 	} else if (event->getType() == OPENMSX_BOOT_EVENT) {
 		executeEvents<OPENMSX_BOOT_EVENT>();
+	} else if (event->getType() == OPENMSX_QUIT_EVENT) {
+		executeEvents<OPENMSX_QUIT_EVENT>();
 	} else if (event->getType() == OPENMSX_MACHINE_LOADED_EVENT) {
+		machineSwitch();
 		executeEvents<OPENMSX_MACHINE_LOADED_EVENT>();
 	} else if (event->getType() == OPENMSX_AFTER_REALTIME_EVENT) {
 		executeRealTime();
@@ -399,6 +468,36 @@ bool AfterCommand::signalEvent(shared_ptr<const Event> event)
 	return true;
 }
 
+void AfterCommand::machineSwitch()
+{
+	if (msxEvents) {
+		msxEvents->unregisterEventListener(*this);
+	}
+	MSXMotherBoard* motherBoard = reactor.getMotherBoard();
+	msxEvents = motherBoard ? &motherBoard->getMSXEventDistributor() : NULL;
+	if (msxEvents) {
+		msxEvents->registerEventListener(*this);
+	}
+}
+
+
+struct AfterMSXEventPred {
+	AfterMSXEventPred(AfterCommand::EventPtr event_)
+		: event(event_) {}
+	bool operator()(shared_ptr<AfterCmd> x) const {
+		if (AfterMSXEventCmd* cmd = dynamic_cast<AfterMSXEventCmd*>(x.get())) {
+			if (*cmd->getEvent() == *event) return false;
+		}
+		return true;
+	}
+	AfterCommand::EventPtr event;
+};
+void AfterCommand::signalEvent(shared_ptr<const Event> event,
+                               EmuTime::param /*time*/)
+{
+	executeMatches(AfterMSXEventPred(event));
+}
+
 
 // class AfterCmd
 
@@ -446,8 +545,7 @@ shared_ptr<AfterCmd> AfterCmd::removeSelf()
 			return result;
 		}
 	}
-	assert(false);
-	return shared_ptr<AfterCmd>(0);
+	UNREACHABLE; return shared_ptr<AfterCmd>(0);
 }
 
 
@@ -505,10 +603,9 @@ AfterTimeCmd::AfterTimeCmd(
 {
 }
 
-const string& AfterTimeCmd::getType() const
+string AfterTimeCmd::getType() const
 {
-	static const string type("time");
-	return type;
+	return "time";
 }
 
 
@@ -522,10 +619,9 @@ AfterIdleCmd::AfterIdleCmd(
 {
 }
 
-const string& AfterIdleCmd::getType() const
+string AfterIdleCmd::getType() const
 {
-	static const string type("idle");
-	return type;
+	return "idle";
 }
 
 
@@ -540,17 +636,32 @@ AfterEventCmd<T>::AfterEventCmd(
 }
 
 template<EventType T>
-const string& AfterEventCmd<T>::getType() const
+string AfterEventCmd<T>::getType() const
 {
 	return type;
 }
 
 
+// AfterMSXEventCmd
+
+AfterMSXEventCmd::AfterMSXEventCmd(
+		AfterCommand& afterCommand, AfterCommand::EventPtr event_,
+		const string& command)
+	: AfterCmd(afterCommand, command)
+	, event(event_)
+{
+}
+
+string AfterMSXEventCmd::getType() const
+{
+	return event->toString();
+}
+
 // class AfterRealTimeCmd
 
 AfterRealTimeCmd::AfterRealTimeCmd(
 		AfterCommand& afterCommand, EventDistributor& eventDistributor_,
-		const std::string& command, double time)
+		const string& command, double time)
 	: AfterCmd(afterCommand, command)
 	, eventDistributor(eventDistributor_)
 	, expired(false)
@@ -563,10 +674,9 @@ AfterRealTimeCmd::~AfterRealTimeCmd()
 	prepareDelete();
 }
 
-const std::string& AfterRealTimeCmd::getType() const
+string AfterRealTimeCmd::getType() const
 {
-	static const string type("realtime");
-	return type;
+	return "realtime";
 }
 
 bool AfterRealTimeCmd::alarm()
@@ -575,7 +685,7 @@ bool AfterRealTimeCmd::alarm()
 	// command here
 	expired = true;
 	eventDistributor.distributeEvent(
-			new SimpleEvent<OPENMSX_AFTER_REALTIME_EVENT>());
+			new SimpleEvent(OPENMSX_AFTER_REALTIME_EVENT));
 	return false; // don't repeat alarm
 }
 
