@@ -1,4 +1,4 @@
-// $Id: CliServer.cc 8552 2009-02-03 10:03:34Z m9710797 $
+// $Id: CliServer.cc 10420 2009-08-27 13:22:09Z seanyoung $
 
 #include "CliServer.hh"
 #include "GlobalCliComm.hh"
@@ -135,7 +135,7 @@ void CliServer::createSocket()
 	if (!checkSocketDir(dir)) {
 		throw MSXException("Couldn't create socket directory.");
 	}
-	socketName = dir + "/socket." + StringOp::toString(getpid());
+	socketName = StringOp::Builder() << dir << "/socket." << int(getpid());
 
 #ifdef _WIN32
 	listenSock = socket(AF_INET, SOCK_STREAM, 0);
