@@ -1,8 +1,8 @@
-// $Id: HotKey.cc 8365 2008-11-25 19:26:40Z m9710797 $
+// $Id: HotKey.cc 10703 2009-10-25 15:19:13Z mthuurne $
 
 #include "HotKey.hh"
 #include "InputEventFactory.hh"
-#include "CommandController.hh"
+#include "GlobalCommandController.hh"
 #include "Command.hh"
 #include "CommandException.hh"
 #include "EventDistributor.hh"
@@ -10,7 +10,8 @@
 #include "InputEvents.hh"
 #include "XMLElement.hh"
 #include "SettingsConfig.hh"
-#include "Alarm.hh"
+#include "AlarmEvent.hh"
+#include "unreachable.hh"
 #include <cassert>
 
 using std::string;
@@ -52,24 +53,16 @@ private:
 	const bool defaultCmd;
 };
 
-class RepeatAlarm : public Alarm
-{
-public:
-	explicit RepeatAlarm(EventDistributor& eventDistributor);
-	virtual ~RepeatAlarm();
-private:
-	virtual bool alarm();
-	EventDistributor& eventDistributor;
-};
 
-
-HotKey::HotKey(CommandController& commandController_,
+HotKey::HotKey(GlobalCommandController& commandController_,
                EventDistributor& eventDistributor_)
 	: bindCmd         (new BindCmd  (commandController_, *this, false))
 	, unbindCmd       (new UnbindCmd(commandController_, *this, false))
 	, bindDefaultCmd  (new BindCmd  (commandController_, *this, true))
 	, unbindDefaultCmd(new UnbindCmd(commandController_, *this, true))
-	, repeatAlarm(new RepeatAlarm(eventDistributor_))
+	, repeatAlarm(new AlarmEvent(eventDistributor_, *this,
+	                             OPENMSX_REPEAT_HOTKEY,
+	                             EventDistributor::HOTKEY))
 	, commandController(commandController_)
 	, eventDistributor(eventDistributor_)
 {
@@ -89,13 +82,10 @@ HotKey::HotKey(CommandController& commandController_,
 		OPENMSX_JOY_BUTTON_UP_EVENT, *this, EventDistributor::HOTKEY);
 	eventDistributor.registerEventListener(
 		OPENMSX_FOCUS_EVENT, *this, EventDistributor::HOTKEY);
-	eventDistributor.registerEventListener(
-		OPENMSX_REPEAT_HOTKEY, *this, EventDistributor::HOTKEY);
 }
 
 HotKey::~HotKey()
 {
-	eventDistributor.unregisterEventListener(OPENMSX_REPEAT_HOTKEY, *this);
 	eventDistributor.unregisterEventListener(OPENMSX_FOCUS_EVENT, *this);
 	eventDistributor.unregisterEventListener(OPENMSX_JOY_BUTTON_UP_EVENT, *this);
 	eventDistributor.unregisterEventListener(OPENMSX_JOY_BUTTON_DOWN_EVENT, *this);
@@ -316,25 +306,6 @@ void HotKey::stopRepeat()
 }
 
 
-// class RepeatAlarm
-
-RepeatAlarm::RepeatAlarm(EventDistributor& eventDistributor_)
-	: eventDistributor(eventDistributor_)
-{
-}
-
-RepeatAlarm::~RepeatAlarm()
-{
-	prepareDelete();
-}
-
-bool RepeatAlarm::alarm()
-{
-	eventDistributor.distributeEvent(new SimpleEvent<OPENMSX_REPEAT_HOTKEY>());
-	return false; // repeat handled elsewhere
-}
-
-
 // class BindCmd
 
 static string getBindCmdName(bool defaultCmd)
@@ -364,7 +335,7 @@ string BindCmd::execute(const vector<string>& tokens)
 	string result;
 	switch (tokens.size()) {
 	case 0:
-		assert(false);
+		UNREACHABLE;
 	case 1:
 		// show all bounded keys
 		for (HotKey::BindMap::const_iterator it = cmdMap.begin();
