@@ -1,9 +1,10 @@
-// $Id: InputEvents.cc 7414 2007-12-12 22:12:56Z m9710797 $
+// $Id: InputEvents.cc 10747 2009-11-04 19:21:39Z m9710797 $
 
 #include "InputEvents.hh"
 #include "Keys.hh"
 #include "TclObject.hh"
 #include "StringOp.hh"
+#include "Timer.hh"
 #include "checked_cast.hh"
 #include <string>
 #include <cassert>
@@ -13,26 +14,24 @@ using std::vector;
 
 namespace openmsx {
 
-// class InputEvent
+// class TimedEvent
 
-InputEvent::InputEvent(EventType type)
+TimedEvent::TimedEvent(EventType type)
 	: Event(type)
+	, realtime(Timer::getTime())
 {
 }
 
-bool InputEvent::lessImpl(const Event& other) const
+unsigned long long TimedEvent::getRealTime() const
 {
-	const InputEvent* otherEvent = checked_cast<const InputEvent*>(&other);
-	return (getType() != otherEvent->getType())
-	     ? (getType() <  otherEvent->getType())
-	     : lessImpl(*otherEvent);
+	return realtime;
 }
 
 
 // class KeyEvent
 
 KeyEvent::KeyEvent(EventType type, Keys::KeyCode keyCode_, word unicode_)
-	: InputEvent(type), keyCode(keyCode_), unicode(unicode_)
+	: TimedEvent(type), keyCode(keyCode_), unicode(unicode_)
 {
 }
 
@@ -51,11 +50,12 @@ void KeyEvent::toStringImpl(TclObject& result) const
 	result.addListElement("keyb");
 	result.addListElement(Keys::getName(getKeyCode()));
 	if (getUnicode() != 0) {
-		result.addListElement("unicode" + StringOp::toString(getUnicode()));
+		result.addListElement(StringOp::Builder() <<
+			"unicode" << getUnicode());
 	}
 }
 
-bool KeyEvent::lessImpl(const InputEvent& other) const
+bool KeyEvent::lessImpl(const Event& other) const
 {
 	// note: don't compare unicode
 	const KeyEvent* otherKeyEvent = checked_cast<const KeyEvent*>(&other);
@@ -92,7 +92,7 @@ KeyDownEvent::KeyDownEvent(Keys::KeyCode keyCode, word unicode)
 // class MouseButtonEvent
 
 MouseButtonEvent::MouseButtonEvent(EventType type, unsigned button_)
-	: InputEvent(type), button(button_)
+	: TimedEvent(type), button(button_)
 {
 }
 
@@ -104,10 +104,10 @@ unsigned MouseButtonEvent::getButton() const
 void MouseButtonEvent::toStringHelper(TclObject& result) const
 {
 	result.addListElement("mouse");
-	result.addListElement("button" + StringOp::toString(getButton()));
+	result.addListElement(StringOp::Builder() << "button" << getButton());
 }
 
-bool MouseButtonEvent::lessImpl(const InputEvent& other) const
+bool MouseButtonEvent::lessImpl(const Event& other) const
 {
 	const MouseButtonEvent* otherMouseEvent =
 		checked_cast<const MouseButtonEvent*>(&other);
@@ -146,7 +146,7 @@ void MouseButtonDownEvent::toStringImpl(TclObject& result) const
 // class MouseMotionEvent
 
 MouseMotionEvent::MouseMotionEvent(int xrel_, int yrel_)
-	: InputEvent(OPENMSX_MOUSE_MOTION_EVENT), xrel(xrel_), yrel(yrel_)
+	: TimedEvent(OPENMSX_MOUSE_MOTION_EVENT), xrel(xrel_), yrel(yrel_)
 {
 }
 
@@ -168,7 +168,7 @@ void MouseMotionEvent::toStringImpl(TclObject& result) const
 	result.addListElement(getY());
 }
 
-bool MouseMotionEvent::lessImpl(const InputEvent& other) const
+bool MouseMotionEvent::lessImpl(const Event& other) const
 {
 	const MouseMotionEvent* otherMouseEvent =
 		checked_cast<const MouseMotionEvent*>(&other);
@@ -181,7 +181,7 @@ bool MouseMotionEvent::lessImpl(const InputEvent& other) const
 // class JoystickEvent
 
 JoystickEvent::JoystickEvent(EventType type, unsigned joystick_)
-	: InputEvent(type), joystick(joystick_)
+	: TimedEvent(type), joystick(joystick_)
 {
 }
 
@@ -192,10 +192,10 @@ unsigned JoystickEvent::getJoystick() const
 
 void JoystickEvent::toStringHelper(TclObject& result) const
 {
-	result.addListElement("joy" + StringOp::toString(getJoystick() + 1));
+	result.addListElement(StringOp::Builder() << "joy" << getJoystick() + 1);
 }
 
-bool JoystickEvent::lessImpl(const InputEvent& other) const
+bool JoystickEvent::lessImpl(const Event& other) const
 {
 	const JoystickEvent* otherJoystickEvent =
 		checked_cast<const JoystickEvent*>(&other);
@@ -221,7 +221,7 @@ unsigned JoystickButtonEvent::getButton() const
 void JoystickButtonEvent::toStringHelper(TclObject& result) const
 {
 	JoystickEvent::toStringHelper(result);
-	result.addListElement("button" + StringOp::toString(getButton()));
+	result.addListElement(StringOp::Builder() << "button" << getButton());
 }
 
 bool JoystickButtonEvent::lessImpl(const JoystickEvent& other) const
@@ -282,7 +282,7 @@ short JoystickAxisMotionEvent::getValue() const
 void JoystickAxisMotionEvent::toStringImpl(TclObject& result) const
 {
 	toStringHelper(result);
-	result.addListElement("axis" + StringOp::toString(getAxis()));
+	result.addListElement(StringOp::Builder() << "axis" << getAxis());
 	result.addListElement(getValue());
 }
 
@@ -299,7 +299,7 @@ bool JoystickAxisMotionEvent::lessImpl(const JoystickEvent& other) const
 // class FocusEvent
 
 FocusEvent::FocusEvent(bool gain_)
-	: InputEvent(OPENMSX_FOCUS_EVENT), gain(gain_)
+	: Event(OPENMSX_FOCUS_EVENT), gain(gain_)
 {
 }
 
@@ -314,7 +314,7 @@ void FocusEvent::toStringImpl(TclObject& result) const
 	result.addListElement(getGain());
 }
 
-bool FocusEvent::lessImpl(const InputEvent& other) const
+bool FocusEvent::lessImpl(const Event& other) const
 {
 	const FocusEvent* otherFocusEvent =
 		checked_cast<const FocusEvent*>(&other);
@@ -325,7 +325,7 @@ bool FocusEvent::lessImpl(const InputEvent& other) const
 // class ResizeEvent
 
 ResizeEvent::ResizeEvent(unsigned x_, unsigned y_)
-	: InputEvent(OPENMSX_RESIZE_EVENT), x(x_), y(y_)
+	: Event(OPENMSX_RESIZE_EVENT), x(x_), y(y_)
 {
 }
 
@@ -346,7 +346,7 @@ void ResizeEvent::toStringImpl(TclObject& result) const
 	result.addListElement(int(getY()));
 }
 
-bool ResizeEvent::lessImpl(const InputEvent& other) const
+bool ResizeEvent::lessImpl(const Event& other) const
 {
 	const ResizeEvent* otherResizeEvent =
 		checked_cast<const ResizeEvent*>(&other);
@@ -358,7 +358,7 @@ bool ResizeEvent::lessImpl(const InputEvent& other) const
 
 // class QuitEvent
 
-QuitEvent::QuitEvent() : InputEvent(OPENMSX_QUIT_EVENT)
+QuitEvent::QuitEvent() : Event(OPENMSX_QUIT_EVENT)
 {
 }
 
@@ -367,68 +367,9 @@ void QuitEvent::toStringImpl(TclObject& result) const
 	result.addListElement("quit");
 }
 
-bool QuitEvent::lessImpl(const InputEvent& /*other*/) const
+bool QuitEvent::lessImpl(const Event& /*other*/) const
 {
 	return false;
 }
 
-
-// class MSXCommandEvent
-
-MSXCommandEvent::MSXCommandEvent(const vector<string>& tokens_)
-	: InputEvent(OPENMSX_MSX_COMMAND_EVENT)
-	, owned(true)
-{
-	for (vector<string>::const_iterator it = tokens_.begin();
-	     it != tokens_.end(); ++it) {
-		tokens.push_back(new TclObject(0, *it));
-	}
-}
-
-MSXCommandEvent::MSXCommandEvent(const vector<TclObject*>& tokens_)
-	: InputEvent(OPENMSX_MSX_COMMAND_EVENT)
-	, tokens(tokens_)
-	, owned(false)
-{
-}
-
-MSXCommandEvent::~MSXCommandEvent()
-{
-	if (owned) {
-		for (vector<TclObject*>::const_iterator it = tokens.begin();
-		     it != tokens.end(); ++it) {
-			delete *it;
-		}
-	}
-}
-
-const vector<TclObject*>& MSXCommandEvent::getTokens() const
-{
-	return tokens;
-}
-
-void MSXCommandEvent::toStringImpl(TclObject& result) const
-{
-	result.addListElement("command");
-	for (vector<TclObject*>::const_iterator it = tokens.begin();
-	     it != tokens.end(); ++it) {
-		result.addListElement(**it);
-	}
-}
-
-bool MSXCommandEvent::lessImpl(const InputEvent& other) const
-{
-	const MSXCommandEvent* otherCommandEvent =
-		checked_cast<const MSXCommandEvent*>(&other);
-	return getTokens() < otherCommandEvent->getTokens();
-}
-
-
-// class ConsoleEvent
-
-ConsoleEvent::ConsoleEvent(EventType type)
-	: Event(type)
-{
-}
-
 } // namespace openmsx
