@@ -1,4 +1,4 @@
-// $Id: DSKDiskImage.cc 8170 2008-08-21 20:36:38Z m9710797 $
+// $Id: DSKDiskImage.cc 8630 2009-02-21 20:59:32Z m9710797 $
 
 #include "DSKDiskImage.hh"
 #include "File.hh"
@@ -16,13 +16,13 @@ DSKDiskImage::~DSKDiskImage()
 {
 }
 
-void DSKDiskImage::readSectorSBD(unsigned sector, byte* buf)
+void DSKDiskImage::readSectorImpl(unsigned sector, byte* buf)
 {
 	file->seek(sector * SECTOR_SIZE);
 	file->read(buf, SECTOR_SIZE);
 }
 
-void DSKDiskImage::writeSectorSBD(unsigned sector, const byte* buf)
+void DSKDiskImage::writeSectorImpl(unsigned sector, const byte* buf)
 {
 	file->seek(sector * SECTOR_SIZE);
 	file->write(buf, SECTOR_SIZE);
