@@ -1,11 +1,8 @@
-// $Id: DirAsDSK.cc 9312 2009-03-01 21:05:37Z mfeingol $
+// $Id: DirAsDSK.cc 11442 2010-05-24 20:18:27Z m9710797 $
 
 #include "DirAsDSK.hh"
 #include "CliComm.hh"
 #include "BootBlocks.hh"
-#include "GlobalSettings.hh"
-#include "BooleanSetting.hh"
-#include "EnumSetting.hh"
 #include "File.hh"
 #include "FileException.hh"
 #include "FileOperations.hh"
@@ -13,14 +10,34 @@
 #include "StringOp.hh"
 #include "statp.hh"
 #include <algorithm>
-#include <cstring>
-#include <sys/types.h>
+#include <cassert>
 #include <cstdio>
+#include <cstring>
+#include <limits>
+#include <sys/types.h>
 
 using std::string;
 
 namespace openmsx {
 
+static const unsigned NUM_FATS = 2;
+static const unsigned SECTORS_PER_CLUSTER = 2;
+static const unsigned FIRST_FAT_SECTOR = 1;
+static const unsigned FIRST_DIR_SECTOR =
+	FIRST_FAT_SECTOR + NUM_FATS * DirAsDSK::SECTORS_PER_FAT;
+static const unsigned FIRST_DATA_SECTOR =
+	FIRST_DIR_SECTOR + DirAsDSK::SECTORS_PER_DIR;
+
+// first valid regular cluster number
+static const unsigned FIRST_CLUSTER = 2;
+// first cluster number that can NOT be used anymore
+static const unsigned MAX_CLUSTER =
+	(DirAsDSK::NUM_SECTORS - FIRST_DATA_SECTOR) / SECTORS_PER_CLUSTER + FIRST_CLUSTER;
+
+// REDEFINE, see comment in DirAsDSK.hh
+const unsigned DirAsDSK::SECTOR_SIZE = 512;
+
+
 // Wrapper function to easily enable/disable debug prints
 // Don't check-in this code with printing enabled:
 //   printing stuff to stdout breaks stdio CliComm connections!
@@ -35,9 +52,9 @@ static void debug(const char* format, ...)
 #endif
 }
 
-static const unsigned BAD_FAT = 0xFF7;
-static const unsigned EOF_FAT = 0xFFF; // actually 0xFF8-0xFFF
-static const unsigned MAX_CLUSTER = 720;
+static const unsigned FREE_FAT = 0x000;
+static const unsigned BAD_FAT  = 0xFF7;
+static const unsigned EOF_FAT  = 0xFFF; // actually 0xFF8-0xFFF
 
 
 // functions to set/get little endian 16/32 bit values
@@ -71,7 +88,8 @@ static unsigned normalizeFAT(unsigned cluster)
 
 static unsigned readFATHelper(const byte* buf, unsigned cluster)
 {
-	assert(cluster < DirAsDSK::NUM_FAT_ENTRIES);
+	assert(FIRST_CLUSTER <= cluster);
+	assert(cluster < MAX_CLUSTER);
 	const byte* p = buf + (cluster * 3) / 2;
 	unsigned result = (cluster & 1)
 	                ? (p[0] >> 4) + (p[1] << 4)
@@ -81,7 +99,8 @@ static unsigned readFATHelper(const byte* buf, unsigned cluster)
 
 static void writeFATHelper(byte* buf, unsigned cluster, unsigned val)
 {
-	assert(cluster < DirAsDSK::NUM_FAT_ENTRIES);
+	assert(FIRST_CLUSTER <= cluster);
+	assert(cluster < MAX_CLUSTER);
 	byte* p = buf + (cluster * 3) / 2;
 	if (cluster & 1) {
 		p[0] = (p[0] & 0x0F) + (val << 4);
@@ -107,10 +126,11 @@ unsigned DirAsDSK::readFAT2(unsigned cluster)
 	return readFATHelper(fat2, cluster);
 }
 
-// write an entry to FAT in memory
-void DirAsDSK::writeFAT(unsigned cluster, unsigned val)
+// write an entry to both FAT1 and FAT2 in memory
+void DirAsDSK::writeFAT12(unsigned cluster, unsigned val)
 {
-	writeFATHelper(fat, cluster, val);
+	writeFATHelper(fat,  cluster, val);
+	writeFATHelper(fat2, cluster, val);
 }
 
 // write an entry to FAT2 in memory
@@ -120,16 +140,19 @@ void DirAsDSK::writeFAT2(unsigned cluster, unsigned val)
 	writeFATHelper(fat2, cluster, val);
 }
 
+// returns MAX_CLUSTER in case of no more free clusters
 unsigned DirAsDSK::findNextFreeCluster(unsigned curcl)
 {
+	assert(curcl < MAX_CLUSTER);
 	do {
 		++curcl;
-	} while ((curcl <= MAX_CLUSTER) && readFAT(curcl));
+		assert(curcl >= FIRST_CLUSTER);
+	} while ((curcl < MAX_CLUSTER) && (readFAT(curcl) != FREE_FAT));
 	return curcl;
 }
 unsigned DirAsDSK::findFirstFreeCluster()
 {
-	return findNextFreeCluster(1);
+	return findNextFreeCluster(FIRST_CLUSTER - 1);
 }
 
 // get start cluster from a directory entry,
@@ -144,7 +167,7 @@ bool DirAsDSK::checkMSXFileExists(const string& msxfilename)
 {
 	for (unsigned i = 0; i < NUM_DIR_ENTRIES; ++i) {
 		if (strncmp(mapdir[i].msxinfo.filename,
-			    msxfilename.c_str(), 11) == 0) {
+			    msxfilename.c_str(), 8 + 3) == 0) {
 			return true;
 		}
 	}
@@ -182,8 +205,10 @@ static string makeSimpleMSXFileName(string filename)
 
 static unsigned clusterToSector(unsigned cluster)
 {
-	assert(cluster >= 2);
-	return 14 + 2 * (cluster - 2);
+	assert(cluster >= FIRST_CLUSTER);
+	assert(cluster < MAX_CLUSTER);
+	return FIRST_DATA_SECTOR + SECTORS_PER_CLUSTER *
+	            (cluster - FIRST_CLUSTER);
 }
 
 
@@ -227,22 +252,20 @@ void DirAsDSK::cleandisk()
 	fat2[2] = 0xFF;
 
 	// clear the sectormap so that they all point to 'clean' sectors
-	for (int i = 0; i < 1440; ++i) {
+	for (unsigned i = 0; i < NUM_SECTORS; ++i) {
 		sectormap[i].usage = CLEAN;
 		sectormap[i].dirEntryNr = 0;
 		sectormap[i].fileOffset = 0;
 	}
 }
 
-DirAsDSK::DirAsDSK(CliComm& cliComm_, GlobalSettings& globalSettings_,
-                   const Filename& filename)
+DirAsDSK::DirAsDSK(CliComm& cliComm_, const Filename& filename,
+		SyncMode syncMode_, BootSectorType bootSectorType)
 	: SectorBasedDisk(filename)
 	, cliComm(cliComm_)
 	, hostDir(filename.getResolved())
-	, globalSettings(globalSettings_)
+	, syncMode(syncMode_)
 {
-	syncMode = globalSettings.getSyncDirAsDSKSetting().getValue();
-
 	// create the diskimage based upon the files that can be
 	// found in the host directory
 	ReadDir dir(hostDir);
@@ -251,15 +274,15 @@ DirAsDSK::DirAsDSK(CliComm& cliComm_, GlobalSettings& globalSettings_,
 	}
 
 	// First create structure for the fake disk
-	setNbSectors(1440); // asume a DS disk is used
+	setNbSectors(NUM_SECTORS); // asume a DS disk is used
 	setSectorsPerTrack(9);
 	setNbSides(2);
 
 	// use selected bootsector
-	const byte* bootSector
-		= globalSettings.getBootSectorSetting().getValue()
-		? BootBlocks::dos2BootBlock
-		: BootBlocks::dos1BootBlock;
+	const byte* bootSector =
+		  bootSectorType == BOOTSECTOR_DOS1
+		? BootBlocks::dos1BootBlock
+		: BootBlocks::dos2BootBlock;
 	memcpy(bootBlock, bootSector, SECTOR_SIZE);
 
 	// make a clean initial disk
@@ -306,7 +329,7 @@ void DirAsDSK::readSectorImpl(unsigned sector, byte* buf)
 		// copy our fake bootsector into the buffer
 		memcpy(buf, bootBlock, SECTOR_SIZE);
 
-	} else if (sector < (1 + 2 * SECTORS_PER_FAT)) {
+	} else if (sector < FIRST_DIR_SECTOR) {
 		// copy correct sector from FAT
 
 		// quick-and-dirty:
@@ -317,26 +340,27 @@ void DirAsDSK::readSectorImpl(unsigned sector, byte* buf)
 			checkAlterFileInDisk(i);
 		}
 
-		unsigned fatSector = (sector - 1) % SECTORS_PER_FAT;
-		memcpy(buf, fat + fatSector * SECTOR_SIZE, SECTOR_SIZE);
+		unsigned fatSector = (sector - FIRST_FAT_SECTOR) % SECTORS_PER_FAT;
+		memcpy(buf, &fat[fatSector * SECTOR_SIZE], SECTOR_SIZE);
 
-	} else if (sector < 14) {
+	} else if (sector < FIRST_DATA_SECTOR) {
 		// create correct DIR sector
-		sector -= (1 + 2 * SECTORS_PER_FAT);
-		int dirCount = sector * 16;
+		sector -= FIRST_DIR_SECTOR;
+		unsigned dirCount = sector * DIR_ENTRIES_PER_SECTOR;
 		// check if there are new files on the host when we read the
 		// first directory sector
 		if (dirCount == 0) {
 			scanHostDir(true);
 		}
-		for (int i = 0; i < 16; ++i, ++dirCount) {
+		for (unsigned i = 0; i < DIR_ENTRIES_PER_SECTOR; ++i, ++dirCount) {
 			checkAlterFileInDisk(dirCount);
-			memcpy(&buf[32 * i], &(mapdir[dirCount].msxinfo), 32);
+			memcpy(&buf[sizeof(MSXDirEntry) * i], &(mapdir[dirCount].msxinfo), sizeof(MSXDirEntry));
 		}
 
 	} else {
 		// else get map from sector to file and read correct block
 		// folowing same numbering as FAT eg. first data block is cluster 2
+		assert(sector < NUM_SECTORS);
 		if (sectormap[sector].usage == CLEAN) {
 			// return an 'empty' sector
 			// 0xE5 is the value used on the Philips VG8250
@@ -348,7 +372,7 @@ void DirAsDSK::readSectorImpl(unsigned sector, byte* buf)
 			// in case (end of) file only fills partial sector
 			memcpy(buf, cachedSectors[sector].data, SECTOR_SIZE);
 			// read data from host file
-			int offset = sectormap[sector].fileOffset;
+			unsigned offset = sectormap[sector].fileOffset;
 			string shortname = mapdir[sectormap[sector].dirEntryNr].shortname;
 			checkAlterFileInDisk(shortname);
 			// now try to read from file if possible
@@ -357,7 +381,13 @@ void DirAsDSK::readSectorImpl(unsigned sector, byte* buf)
 				File file(fullfilename);
 				unsigned size = file.getSize();
 				file.seek(offset);
-				file.read(buf, std::min<int>(size - offset, SECTOR_SIZE));
+				if (offset < size) {
+					file.read(buf, std::min(size - offset, SECTOR_SIZE));
+				} else {
+					// Normally shouldn't happen because
+					// checkAlterFileInDisk() above synced
+					// host file size with MSX file size.
+				}
 				// and store the newly read data again in the sector cache
 				// since checkAlterFileInDisk => updateFileInDisk only reads
 				// altered data if filesize has been changed and not if only
@@ -422,35 +452,38 @@ void DirAsDSK::updateFileInDisk(unsigned dirindex, struct stat& fst)
 	              : 0;
 	setLE16(mapdir[dirindex].msxinfo.date, t2);
 
-	int fsize = fst.st_size;
+	unsigned fsize = fst.st_size;
 	mapdir[dirindex].filesize = fsize;
 	unsigned curcl = getStartCluster(mapdir[dirindex].msxinfo);
 	// if there is no cluster assigned yet to this file, then find a free cluster
 	bool followFATClusters = true;
-	if (curcl == 0) {
+	if ((curcl < FIRST_CLUSTER) || (curcl >= MAX_CLUSTER)) {
+		// curcl == 0 happens for zero-sized files, but treat invalid
+		// cases in the same way (curcl == 1 or curcl >= MAX_CLUSTER)
 		followFATClusters = false;
 		curcl = findFirstFreeCluster();
-		setLE16(mapdir[dirindex].msxinfo.startcluster, curcl);
 	}
 
-	unsigned size = fsize;
+	unsigned remainingSize = fsize;
 	unsigned prevcl = 0;
 	try {
 		string fullfilename = hostDir + '/' + mapdir[dirindex].shortname;
 		File file(fullfilename, "rb"); // don't uncompress
 
-		while (size && (curcl <= MAX_CLUSTER)) {
+		while (remainingSize && (curcl < MAX_CLUSTER)) {
 			unsigned logicalSector = clusterToSector(curcl);
-			for (int i = 0; i < 2; ++i) {
-				sectormap[logicalSector + i].usage = MIXED;
-				sectormap[logicalSector + i].dirEntryNr = dirindex;
-				sectormap[logicalSector + i].fileOffset = fsize - size;
-				byte* buf = cachedSectors[logicalSector + i].data;
+			for (unsigned i = 0; i < SECTORS_PER_CLUSTER; ++i) {
+				unsigned sector = logicalSector + i;
+				assert(sector < NUM_SECTORS);
+				sectormap[sector].usage = MIXED;
+				sectormap[sector].dirEntryNr = dirindex;
+				sectormap[sector].fileOffset = fsize - remainingSize;
+				byte* buf = cachedSectors[sector].data;
 				memset(buf, 0, SECTOR_SIZE); // in case (end of) file only fills partial sector
-				file.seek(sectormap[logicalSector + i].fileOffset);
-				file.read(buf, std::min<int>(size, SECTOR_SIZE));
-				size -= std::min<int>(size, SECTOR_SIZE);
-				if (size == 0) {
+				file.seek(sectormap[sector].fileOffset);
+				file.read(buf, std::min(remainingSize, SECTOR_SIZE));
+				remainingSize -= std::min(remainingSize, SECTOR_SIZE);
+				if (remainingSize == 0) {
 					// don't fill next sectors in this cluster
 					// if there is no data left
 					break;
@@ -458,7 +491,9 @@ void DirAsDSK::updateFileInDisk(unsigned dirindex, struct stat& fst)
 			}
 
 			if (prevcl) {
-				writeFAT(prevcl, curcl);
+				writeFAT12(prevcl, curcl);
+			} else {
+				setLE16(mapdir[dirindex].msxinfo.startcluster, curcl);
 			}
 			prevcl = curcl;
 
@@ -469,57 +504,67 @@ void DirAsDSK::updateFileInDisk(unsigned dirindex, struct stat& fst)
 				if (curcl == EOF_FAT) {
 					followFATClusters = false;
 					curcl = findFirstFreeCluster();
+				} else if ((curcl < FIRST_CLUSTER) || (curcl >= MAX_CLUSTER)) {
+					// Invalid FAT chain!! Normally this
+					// doesn't happen, but we also shouldn't
+					// crash on it. Treat the same as EOF_FAT
+					followFATClusters = false;
+					curcl = findFirstFreeCluster();
 				}
 			} else {
 				curcl = findNextFreeCluster(curcl);
 			}
 			// Continuing at cluster 'curcl'
 		}
+		if (remainingSize != 0) {
+			cliComm.printWarning("Virtual diskimage full: " +
+			                     mapdir[dirindex].shortname + " truncated.");
+		}
 	} catch (FileException&) {
 		// Error opening or reading host file
 		cliComm.printWarning("Error reading host file: " +
 		                     mapdir[dirindex].shortname +
 		                     ". Truncated file on MSX disk.");
-		size = 0; // truncate MSX file
 	}
-	if ((size == 0) && (curcl <= MAX_CLUSTER)) {
-		// TODO: check what an MSX does with filesize zero and fat allocation
-		if (prevcl == 0) {
-			writeFAT(curcl, EOF_FAT);
-		} else {
-			writeFAT(prevcl, EOF_FAT);
-		}
 
-		// clear remains of FAT if needed
-		if (followFATClusters) {
-			while ((curcl <= MAX_CLUSTER) && (curcl != 0) &&
-			       (curcl != EOF_FAT)) {
-				writeFAT(curcl, 0);
-				unsigned logicalSector = clusterToSector(curcl);
-				for (int i = 0; i < 2; ++i) {
-					sectormap[logicalSector + i].usage = CLEAN;
-					sectormap[logicalSector + i].dirEntryNr = 0;
-					sectormap[logicalSector + i].fileOffset = 0;
-				}
-				prevcl = curcl;
-				curcl = readFAT(curcl);
-			}
-			writeFAT(prevcl, 0);
-			unsigned logicalSector = clusterToSector(prevcl);
-			for (int i = 0; i < 2; ++i) {
-				sectormap[logicalSector + i].usage = CLEAN;
-				sectormap[logicalSector + i].dirEntryNr = 0;
-				sectormap[logicalSector + i].fileOffset = 0;
-			}
-		}
+	// In all cases (no error / image full / host read error) we need to
+	// properly terminate the FAT chain.
+	if (prevcl) {
+		writeFAT12(prevcl, EOF_FAT);
 	} else {
-		// TODO: don't we need a EOF_FAT in this case as well ?
-		// find out and adjust code here
-		cliComm.printWarning("Fake Diskimage full: " +
-		                     mapdir[dirindex].shortname + " truncated.");
+		// Filesize zero: don't allocate any cluster, write zero
+		// cluster number (checked on a MSXTurboR, DOS2 mode).
+		setLE16(mapdir[dirindex].msxinfo.startcluster, FREE_FAT);
 	}
+
+	// clear remains of FAT if needed
+	if (followFATClusters) {
+		while ((FIRST_CLUSTER <= curcl) && (curcl < MAX_CLUSTER)) {
+			writeFAT12(curcl, FREE_FAT);
+			unsigned logicalSector = clusterToSector(curcl);
+			for (unsigned i = 0; i < SECTORS_PER_CLUSTER; ++i) {
+				unsigned sector = logicalSector + i;
+				assert(sector < NUM_SECTORS);
+				sectormap[sector].usage = CLEAN;
+				sectormap[sector].dirEntryNr = 0;
+				sectormap[sector].fileOffset = 0;
+			}
+			prevcl = curcl;
+			curcl = readFAT(curcl);
+		}
+		writeFAT12(prevcl, FREE_FAT);
+		unsigned logicalSector = clusterToSector(prevcl);
+		for (unsigned i = 0; i < SECTORS_PER_CLUSTER; ++i) {
+			unsigned sector = logicalSector + i;
+			assert(sector < NUM_SECTORS);
+			sectormap[sector].usage = CLEAN;
+			sectormap[sector].dirEntryNr = 0;
+			sectormap[sector].fileOffset = 0;
+		}
+	}
+
 	// write (possibly truncated) file size
-	setLE32(mapdir[dirindex].msxinfo.size, fsize - size);
+	setLE32(mapdir[dirindex].msxinfo.size, fsize - remainingSize);
 }
 
 void DirAsDSK::truncateCorrespondingFile(unsigned dirindex)
@@ -535,7 +580,7 @@ void DirAsDSK::truncateCorrespondingFile(unsigned dirindex)
 		debug("      truncateCorrespondingFile of new Host OS file\n");
 	}
 	debug("      truncateCorrespondingFile %s\n", mapdir[dirindex].shortname.c_str());
-	int cursize = getLE32(mapdir[dirindex].msxinfo.size);
+	unsigned cursize = getLE32(mapdir[dirindex].msxinfo.size);
 	mapdir[dirindex].filesize = cursize;
 
 	// stuff below can fail, so do it as the last thing in this method
@@ -574,23 +619,24 @@ void DirAsDSK::extractCacheToFile(unsigned dirindex)
 			return;
 		}
 
-		while ((curcl <= MAX_CLUSTER) && (curcl != EOF_FAT) && (curcl != 0)) {
+		while ((FIRST_CLUSTER <= curcl) && (curcl < MAX_CLUSTER)) {
 			unsigned logicalSector = clusterToSector(curcl);
-			for (int i = 0; i < 2; ++i) {
-				if ((sectormap[logicalSector].usage == CACHED ||
-				     sectormap[logicalSector].usage == MIXED) &&
-				    (cursize >= offset)) {
+			for (unsigned i = 0; i < SECTORS_PER_CLUSTER; ++i) {
+				unsigned sector = logicalSector + i;
+				assert(sector < NUM_SECTORS);
+				if ((sectormap[sector].usage == CACHED ||
+				     sectormap[sector].usage == MIXED) &&
+				    (cursize > offset)) {
 					// transfer data
-					byte* buf = cachedSectors[logicalSector].data;
+					byte* buf = cachedSectors[sector].data;
 					file.seek(offset);
-					unsigned writesize = std::min<int>(cursize - offset, SECTOR_SIZE);
+					unsigned writesize = std::min(cursize - offset, SECTOR_SIZE);
 					file.write(buf, writesize);
 
-					sectormap[logicalSector].usage = MIXED;
-					sectormap[logicalSector].dirEntryNr = dirindex;
-					sectormap[logicalSector].fileOffset = offset;
+					sectormap[sector].usage = MIXED;
+					sectormap[sector].dirEntryNr = dirindex;
+					sectormap[sector].fileOffset = offset;
 				}
-				++logicalSector;
 				offset += SECTOR_SIZE;
 			}
 			curcl = readFAT(curcl);
@@ -605,7 +651,7 @@ void DirAsDSK::extractCacheToFile(unsigned dirindex)
 void DirAsDSK::writeSectorImpl(unsigned sector, const byte* buf)
 {
 	// is this actually needed ?
-	if (syncMode == GlobalSettings::SYNC_READONLY) return;
+	if (syncMode == SYNC_READONLY) return;
 
 	debug("DirAsDSK::writeSectorImpl: %i ", sector);
 	switch (sector) {
@@ -634,7 +680,8 @@ void DirAsDSK::writeSectorImpl(unsigned sector, const byte* buf)
 			debug("data sector\n");
 			break;
 	}
-	if (sector >= 14) {
+	if (sector >= FIRST_DATA_SECTOR) {
+		assert(sector < NUM_SECTORS);
 		debug("  Mode: ");
 		switch (sectormap[sector].usage) {
 		case CLEAN:
@@ -654,9 +701,9 @@ void DirAsDSK::writeSectorImpl(unsigned sector, const byte* buf)
 
 	if (sector == 0) {
 		memcpy(bootBlock, buf, SECTOR_SIZE);
-	} else if (sector < (1 + 2 * SECTORS_PER_FAT)) {
+	} else if (sector < FIRST_DIR_SECTOR) {
 		writeFATSector(sector, buf);
-	} else if (sector < 14) {
+	} else if (sector < FIRST_DATA_SECTOR) {
 		writeDIRSector(sector, buf);
 	} else {
 		writeDataSector(sector, buf);
@@ -672,25 +719,31 @@ void DirAsDSK::writeFATSector(unsigned sector, const byte* buf)
 	// we use writes to the second FAT (which should be an exact backup
 	// of the first FAT to detect changes (files might have
 	// grown/shrunk/added) so that they can be passed on to the HOST-OS
-	if (sector < (1 + SECTORS_PER_FAT)) {
-		unsigned fatSector = sector - 1;
-		memcpy(fat + fatSector * SECTOR_SIZE, buf, SECTOR_SIZE);
+	static const unsigned FIRST_SECTOR_2ND_FAT =
+	       FIRST_FAT_SECTOR + SECTORS_PER_FAT;
+	if (sector < FIRST_SECTOR_2ND_FAT) {
+		unsigned fatSector = sector - FIRST_FAT_SECTOR;
+		memcpy(&fat[fatSector * SECTOR_SIZE], buf, SECTOR_SIZE);
 		return;
 	}
 
 	// writes to the second FAT so we check for changes
 	// but we fully ignore the sectors afterwards (see remark
 	// about identifier bytes above)
-	unsigned startcluster = std::max<int>(2, ((sector - 1 - SECTORS_PER_FAT) * 2) / 3);
-	unsigned endcluster = std::min<int>(startcluster + 342, NUM_FAT_ENTRIES);
+	static const unsigned MAX_FAT_ENTRIES_PER_SECTOR =
+		(SECTOR_SIZE * 2 + (3 - 1)) / 3; // rounded up
+	unsigned startcluster = std::max(FIRST_CLUSTER,
+	                                 ((sector - FIRST_SECTOR_2ND_FAT) * 2) / 3);
+	unsigned endcluster = std::min(startcluster + MAX_FAT_ENTRIES_PER_SECTOR,
+	                               MAX_CLUSTER);
 	for (unsigned i = startcluster; i < endcluster; ++i) {
 		if (readFAT(i) != readFAT2(i)) {
 			updateFileFromAlteredFatOnly(i);
 		}
 	}
 
-	unsigned fatSector = (sector - 1) % SECTORS_PER_FAT;
-	memcpy(fat2 + fatSector * SECTOR_SIZE, buf, SECTOR_SIZE);
+	unsigned fatSector = (sector - FIRST_FAT_SECTOR) % SECTORS_PER_FAT;
+	memcpy(&fat2[fatSector * SECTOR_SIZE], buf, SECTOR_SIZE);
 }
 
 void DirAsDSK::writeDIRSector(unsigned sector, const byte* buf)
@@ -701,11 +754,11 @@ void DirAsDSK::writeDIRSector(unsigned sector, const byte* buf)
 	// fileactions. Of course some diskcaching programs and disk
 	// optimizers can abandon this behaviour and in such case the
 	// logic used here goes haywire!!
-	sector -= (1 + 2 * SECTORS_PER_FAT);
-	for (int i = 0; i < 16; ++i) {
-		unsigned dirindex = sector * 16 + i;
-		const MSXDirEntry& entry = *reinterpret_cast<const MSXDirEntry*>(&buf[32 * i]);
-		if (memcmp(mapdir[dirindex].msxinfo.filename, &entry, 32) != 0) {
+	sector -= FIRST_DIR_SECTOR;
+	for (unsigned i = 0; i < DIR_ENTRIES_PER_SECTOR; ++i) {
+		unsigned dirindex = sector * DIR_ENTRIES_PER_SECTOR + i;
+		const MSXDirEntry& entry = *reinterpret_cast<const MSXDirEntry*>(&buf[sizeof(MSXDirEntry) * i]);
+		if (memcmp(mapdir[dirindex].msxinfo.filename, &entry, sizeof(MSXDirEntry)) != 0) {
 			writeDIREntry(dirindex, entry);
 		}
 	}
@@ -713,13 +766,13 @@ void DirAsDSK::writeDIRSector(unsigned sector, const byte* buf)
 
 void DirAsDSK::writeDIREntry(unsigned dirindex, const MSXDirEntry& entry)
 {
-	int oldClus = getStartCluster(mapdir[dirindex].msxinfo);
-	int newClus = getStartCluster(entry);
-	int oldSize = getLE32(mapdir[dirindex].msxinfo.size);
-	int newSize = getLE32(entry.size);
+	unsigned oldClus = getStartCluster(mapdir[dirindex].msxinfo);
+	unsigned newClus = getStartCluster(entry);
+	unsigned oldSize = getLE32(mapdir[dirindex].msxinfo.size);
+	unsigned newSize = getLE32(entry.size);
 
 	// The 3 vital informations needed
-	bool chgName = memcmp(mapdir[dirindex].msxinfo.filename, entry.filename, 11) != 0;
+	bool chgName = memcmp(mapdir[dirindex].msxinfo.filename, entry.filename, 8 + 3) != 0;
 	bool chgClus = oldClus != newClus;
 	bool chgSize = oldSize != newSize;
 
@@ -743,18 +796,17 @@ void DirAsDSK::writeDIREntry(unsigned dirindex, const MSXDirEntry& entry)
 	debug("  dirindex %i filename: %s\n",
 	      dirindex, mapdir[dirindex].shortname.c_str());
 	debug("  chgName: %i chgClus: %i chgSize: %i\n", chgName, chgClus, chgSize);
-	debug("  Old start %i   New start %i\n", oldClus, newClus);
-	debug("  Old size %i  New size %i\n\n", oldSize, newSize);
+	debug("  Old start %i   New start %i\n",   oldClus, newClus);
+	debug("  Old size  %i   New size  %i\n\n", oldSize, newSize);
 
 	if (chgName && !chgClus && !chgSize) {
-		if ((entry.filename[0] == char(0xE5)) &&
-		    (syncMode == GlobalSettings::SYNC_FULL)) {
+		if (entry.filename[0] == char(0xE5) && syncMode == SYNC_FULL) {
 			// dir entry has been deleted
 			// delete file from host OS and 'clear' all sector
 			// data pointing to this HOST OS file
 			string fullfilename = hostDir + '/' + mapdir[dirindex].shortname;
 			FileOperations::unlink(fullfilename);
-			for (int i = 14; i < 1440; ++i) {
+			for (unsigned i = FIRST_DATA_SECTOR; i < NUM_SECTORS; ++i) {
 				if (sectormap[i].dirEntryNr == dirindex) {
 					 sectormap[i].usage = CACHED;
 				}
@@ -762,8 +814,7 @@ void DirAsDSK::writeDIREntry(unsigned dirindex, const MSXDirEntry& entry)
 			mapdir[dirindex].shortname.clear();
 
 		} else if ((entry.filename[0] != char(0xE5)) &&
-			   ((syncMode == GlobalSettings::SYNC_FULL) ||
-			    (syncMode == GlobalSettings::SYNC_NODELETE))) {
+			   (syncMode == SYNC_FULL || syncMode == SYNC_NODELETE)) {
 			string shname = condenseName(entry.filename);
 			string newfilename = hostDir + '/' + shname;
 			if (newClus == 0 && newSize == 0) {
@@ -798,7 +849,7 @@ void DirAsDSK::writeDIREntry(unsigned dirindex, const MSXDirEntry& entry)
 		// the single shot Dir update when creating new files)
 		if (oldSize < newSize) {
 			// new size is bigger, file has grown
-			memcpy(&(mapdir[dirindex].msxinfo), &entry, 32);
+			memcpy(&(mapdir[dirindex].msxinfo), &entry, sizeof(MSXDirEntry));
 			extractCacheToFile(dirindex);
 		} else {
 			// new size is smaller, file has been reduced
@@ -808,7 +859,7 @@ void DirAsDSK::writeDIREntry(unsigned dirindex, const MSXDirEntry& entry)
 			// the size is set to zero before it is set to the new value. If we
 			// didn't cache this, then all the 'mapped' sectors would lose their
 			// value
-			memcpy(&(mapdir[dirindex].msxinfo), &entry, 32);
+			memcpy(&(mapdir[dirindex].msxinfo), &entry, sizeof(MSXDirEntry));
 			truncateCorrespondingFile(dirindex);
 			if (newSize != 0) {
 				extractCacheToFile(dirindex); // see copy remark above
@@ -823,19 +874,20 @@ void DirAsDSK::writeDIREntry(unsigned dirindex, const MSXDirEntry& entry)
 	}
 
 	// for now blindly take over info
-	memcpy(&(mapdir[dirindex].msxinfo), &entry, 32);
+	memcpy(&(mapdir[dirindex].msxinfo), &entry, sizeof(MSXDirEntry));
 }
 
 void DirAsDSK::writeDataSector(unsigned sector, const byte* buf)
 {
-	assert(sector >= 14);
+	assert(sector >= FIRST_DATA_SECTOR);
+	assert(sector < NUM_SECTORS);
 
 	// first and before all else buffer everything !!!!
 	// check if cachedSectors exists, if not assign memory.
 	memcpy(cachedSectors[sector].data, buf, SECTOR_SIZE);
 
 	// if in SYNC_CACHEDWRITE then simply mark sector as cached and be done with it
-	if (syncMode == GlobalSettings::SYNC_CACHEDWRITE) {
+	if (syncMode == SYNC_CACHEDWRITE) {
 		// change to a regular cached sector
 		sectormap[sector].usage = CACHED;
 		sectormap[sector].dirEntryNr = 0;
@@ -846,14 +898,16 @@ void DirAsDSK::writeDataSector(unsigned sector, const byte* buf)
 	if (sectormap[sector].usage == MIXED) {
 		// save data to host file
 		try {
-			int offset = sectormap[sector].fileOffset;
-			int dirent = sectormap[sector].dirEntryNr;
+			unsigned offset = sectormap[sector].fileOffset;
+			unsigned dirent = sectormap[sector].dirEntryNr;
 			string fullfilename = hostDir + '/' + mapdir[dirent].shortname;
 			File file(fullfilename);
 			file.seek(offset);
-			int cursize = getLE32(mapdir[dirent].msxinfo.size);
-			unsigned writesize = std::min<int>(cursize - offset, SECTOR_SIZE);
-			file.write(buf, writesize);
+			unsigned cursize = getLE32(mapdir[dirent].msxinfo.size);
+			if (cursize > offset) {
+				unsigned writesize = std::min(cursize - offset, SECTOR_SIZE);
+				file.write(buf, writesize);
+			}
 		} catch (FileException&) {
 			cliComm.printWarning("Couldn't write to file.");
 		}
@@ -867,11 +921,11 @@ void DirAsDSK::updateFileFromAlteredFatOnly(unsigned somecluster)
 {
 	// First look for the first cluster in this chain
 	unsigned startcluster = somecluster;
-	for (unsigned i = 2; i < NUM_FAT_ENTRIES; ++i) {
+	for (unsigned i = FIRST_CLUSTER; i < MAX_CLUSTER; ++i) {
 		if (readFAT(i) == startcluster) {
 			// found a predecessor
 			startcluster = i;
-			i = 1; // restart search
+			i = FIRST_CLUSTER - 1; // restart search
 		}
 	}
 
@@ -885,9 +939,9 @@ void DirAsDSK::updateFileFromAlteredFatOnly(unsigned somecluster)
 	}
 
 	// from startcluster and somecluster on, update fat2 so that the check
-	// in writeFATSector() don't call this routine again for the same file
+	// in writeFATSector() doesn't call this routine again for the same file
 	unsigned curcl = startcluster;
-	while ((curcl <= MAX_CLUSTER) && (curcl != EOF_FAT) && (curcl > 1)) {
+	while ((FIRST_CLUSTER <= curcl) && (curcl < MAX_CLUSTER)) {
 		unsigned next = readFAT(curcl);
 		writeFAT2(curcl, next);
 		curcl = next;
@@ -898,7 +952,7 @@ void DirAsDSK::updateFileFromAlteredFatOnly(unsigned somecluster)
 	// the loop above doesn't take care of this, since it will
 	// stop at the new EOF_FAT||curcl==0 condition
 	curcl = somecluster;
-	while ((curcl <= MAX_CLUSTER) && (curcl != EOF_FAT) && (curcl > 1)) {
+	while ((FIRST_CLUSTER <= curcl) && (curcl < MAX_CLUSTER)) {
 		unsigned next = readFAT(curcl);
 		writeFAT2(curcl, next);
 		curcl = next;
@@ -914,7 +968,7 @@ string DirAsDSK::condenseName(const char* buf)
 	}
 	if (buf[8] != ' ') {
 		result += '.';
-		for (unsigned i = 8; (i < 11) && (buf[i] != ' '); ++i) {
+		for (unsigned i = 8; (i < (8 + 3)) && (buf[i] != ' '); ++i) {
 			result += tolower(buf[i]);
 		}
 	}
@@ -923,7 +977,7 @@ string DirAsDSK::condenseName(const char* buf)
 
 bool DirAsDSK::isWriteProtectedImpl() const
 {
-	return (syncMode != GlobalSettings::SYNC_READONLY) ? false : true;
+	return syncMode == SYNC_READONLY;
 }
 
 void DirAsDSK::updateFileInDisk(const string& filename)
@@ -942,6 +996,13 @@ void DirAsDSK::updateFileInDisk(const string& filename)
 		}
 		return;
 	}
+	if (fst.st_size >= std::numeric_limits<int>::max()) {
+		// File sizes are processed using int, so prevent integer
+		// overflow. Files this large won't be not be supported
+		// by an MSX anyway
+		cliComm.printWarning("File too large: " + fullfilename);
+		return;
+	}
 	if (!checkFileUsedInDSK(filename)) {
 		// add file to fakedisk
 		addFileToDSK(filename, fst);
@@ -977,7 +1038,7 @@ void DirAsDSK::addFileToDSK(const string& filename, struct stat& fst)
 	// fill in native file name
 	mapdir[dirindex].shortname = filename;
 	// fill in MSX file name
-	memcpy(&(mapdir[dirindex].msxinfo.filename), MSXfilename.c_str(), 11);
+	memcpy(&(mapdir[dirindex].msxinfo.filename), MSXfilename.c_str(), 8 + 3);
 
 	updateFileInDisk(dirindex, fst);
 }
