@@ -1,18 +1,18 @@
-// $Id: DiskChanger.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: DiskChanger.cc 11148 2010-01-13 16:14:45Z m9710797 $
 
 #include "DiskChanger.hh"
+#include "DiskFactory.hh"
 #include "DummyDisk.hh"
 #include "RamDSKDiskImage.hh"
-#include "XSADiskImage.hh"
 #include "DirAsDSK.hh"
 #include "DSKDiskImage.hh"
-#include "DiskPartition.hh"
 #include "CommandController.hh"
-#include "Command.hh"
-#include "MSXEventDistributor.hh"
+#include "RecordedCommand.hh"
+#include "StateChangeDistributor.hh"
 #include "InputEvents.hh"
 #include "Scheduler.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "DiskManipulator.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
@@ -24,6 +24,7 @@
 #include "serialize.hh"
 #include "serialize_stl.hh"
 #include "serialize_constr.hh"
+#include "shared_ptr.hh"
 #include "ref.hh"
 
 using std::set;
@@ -47,36 +48,55 @@ private:
 
 DiskChanger::DiskChanger(const string& driveName_,
                          CommandController& controller_,
+                         DiskFactory& diskFactory_,
                          DiskManipulator& manipulator_,
-                         MSXMotherBoard* board,
+                         MSXMotherBoard& board,
                          bool createCmd)
 	: controller(controller_)
-	, msxEventDistributor(board ? &board->getMSXEventDistributor() : NULL)
-	, scheduler(board ? &board->getScheduler() : NULL)
+	, stateChangeDistributor(&board.getStateChangeDistributor())
+	, scheduler(&board.getScheduler())
+	, diskFactory(diskFactory_)
 	, manipulator(manipulator_)
 	, driveName(driveName_)
 {
-	init(board, createCmd);
+	init(board.getMachineID() + "::", createCmd);
+}
+
+DiskChanger::DiskChanger(const string& driveName_,
+                         CommandController& controller_,
+                         DiskFactory& diskFactory_,
+                         DiskManipulator& manipulator_,
+                         bool createCmd)
+	: controller(controller_)
+	, stateChangeDistributor(NULL)
+	, scheduler(NULL)
+	, diskFactory(diskFactory_)
+	, manipulator(manipulator_)
+	, driveName(driveName_)
+{
+	init("", createCmd);
 }
 
 // only used for polymorphic de-serialization (for Nowind)
-DiskChanger::DiskChanger(MSXMotherBoard& board, const std::string& driveName_)
+DiskChanger::DiskChanger(MSXMotherBoard& board,
+                         const std::string& driveName_)
 	: controller(board.getCommandController())
-	, msxEventDistributor(&board.getMSXEventDistributor())
+	, stateChangeDistributor(&board.getStateChangeDistributor())
 	, scheduler(&board.getScheduler())
-	, manipulator(board.getDiskManipulator())
+	, diskFactory(board.getReactor().getDiskFactory())
+	, manipulator(board.getReactor().getDiskManipulator())
 	, driveName(driveName_)
 {
-	init(&board, true);
+	init(board.getMachineID() + "::", true);
 }
 
-void DiskChanger::init(MSXMotherBoard* board, bool createCmd)
+void DiskChanger::init(const string& prefix, bool createCmd)
 {
 	if (createCmd) createCommand();
 	ejectDisk();
-	manipulator.registerDrive(*this, board);
-	if (msxEventDistributor) {
-		msxEventDistributor->registerEventListener(*this);
+	manipulator.registerDrive(*this, prefix);
+	if (stateChangeDistributor) {
+		stateChangeDistributor->registerListener(*this);
 	}
 }
 
@@ -88,8 +108,8 @@ void DiskChanger::createCommand()
 
 DiskChanger::~DiskChanger()
 {
-	if (msxEventDistributor) {
-		msxEventDistributor->unregisterEventListener(*this);
+	if (stateChangeDistributor) {
+		stateChangeDistributor->unregisterListener(*this);
 	}
 	manipulator.unregisterDrive(*this);
 }
@@ -137,22 +157,21 @@ const std::string& DiskChanger::getContainerName() const
 void DiskChanger::sendChangeDiskEvent(const vector<string>& args)
 {
 	// note: might throw MSXException
-	MSXEventDistributor::EventPtr event(new MSXCommandEvent(args));
-	if (msxEventDistributor) {
-		msxEventDistributor->distributeEvent(
-			event, scheduler->getCurrentTime());
+	StateChangeDistributor::EventPtr event(
+		new MSXCommandEvent(args, scheduler->getCurrentTime()));
+	if (stateChangeDistributor) {
+		stateChangeDistributor->distributeNew(event);
 	} else {
-		signalEvent(event, EmuTime::zero);
+		signalStateChange(event);
 	}
 }
 
-void DiskChanger::signalEvent(
-	shared_ptr<const Event> event, EmuTime::param /*time*/)
+void DiskChanger::signalStateChange(shared_ptr<StateChange> event)
 {
-	if (event->getType() != OPENMSX_MSX_COMMAND_EVENT) return;
+	MSXCommandEvent* commandEvent =
+		dynamic_cast<MSXCommandEvent*>(event.get());
+	if (!commandEvent) return;
 
-	const MSXCommandEvent* commandEvent =
-		checked_cast<const MSXCommandEvent*>(event.get());
 	const vector<TclObject*>& tokens = commandEvent->getTokens();
 	if (tokens[0]->getString() == getDriveName()) {
 		if (tokens[1]->getString() == "eject") {
@@ -163,6 +182,11 @@ void DiskChanger::signalEvent(
 	}
 }
 
+void DiskChanger::stopReplay(EmuTime::param /*time*/)
+{
+	// nothing
+}
+
 int DiskChanger::insertDisk(const string& filename)
 {
 	TclObject arg0("dummy");
@@ -180,57 +204,8 @@ int DiskChanger::insertDisk(const string& filename)
 
 void DiskChanger::insertDisk(const vector<TclObject*>& args)
 {
-	std::auto_ptr<Disk> newDisk;
 	const string& diskImage = args[1]->getString();
-	if (diskImage == "ramdsk") {
-		newDisk.reset(new RamDSKDiskImage());
-	} else {
-		Filename filename(diskImage, controller);
-		try {
-			// first try XSA
-			newDisk.reset(new XSADiskImage(filename));
-		} catch (MSXException&) {
-		try {
-			// First try the fake disk, because a DSK will always
-			// succeed if diskImage can be resolved
-			// It is simply stat'ed, so even a directory name
-			// can be resolved and will be accepted as dsk name
-			// try to create fake DSK from a dir on host OS
-			newDisk.reset(new DirAsDSK(
-				controller.getCliComm(),
-				controller.getGlobalSettings(),
-				filename));
-		} catch (MSXException&) {
-		try {
-			// then try normal DSK
-			newDisk.reset(new DSKDiskImage(filename));
-		} catch (MSXException& e1) {
-			// Finally try to interpret the filename as
-			//    <filename>:<partition-number>
-			// Try this last because the ':' character could be
-			// part of the filename itself. So only try this if
-			// the name could not be interpreted as a valid
-			// filename.
-			string::size_type pos = diskImage.find_last_of(':');
-			if (pos == string::npos) {
-				// does not contain ':', throw previous exception
-				throw;
-			}
-			shared_ptr<SectorAccessibleDisk> wholeDisk;
-			try {
-				Filename file(diskImage.substr(0, pos));
-				wholeDisk.reset(new DSKDiskImage(file));
-			} catch (MSXException&) {
-				// If this fails we still prefer to show the
-				// previous error message, because it's most
-				// likely more descriptive.
-				throw e1;
-			}
-			unsigned num = StringOp::stringToUint(
-				diskImage.substr(pos + 1));
-			newDisk.reset(new DiskPartition(*wholeDisk, num, wholeDisk));
-		}}}
-	}
+	std::auto_ptr<Disk> newDisk(diskFactory.createDisk(diskImage));
 	for (unsigned i = 2; i < args.size(); ++i) {
 		Filename filename(args[i]->getString(), controller);
 		newDisk->applyPatch(filename);
@@ -373,25 +348,24 @@ template<typename Archive>
 void DiskChanger::serialize(Archive& ar, unsigned version)
 {
 	DiskName diskname = disk->getName();
-	if (version < 2) {
+	if (ar.versionBelow(version, 2)) {
 		// there was no DiskName yet, just a plain Filename
-		assert(ar.isLoader());
 		Filename filename;
-		ar.serializeNoID("disk", filename);
+		ar.serialize("disk", filename);
 		if (filename.getOriginal() == "ramdisk") {
 			diskname = DiskName(Filename(), "ramdisk");
 		} else {
 			diskname = DiskName(filename, "");
 		}
 	} else {
-		ar.serializeNoID("disk", diskname);
+		ar.serialize("disk", diskname);
 	}
 
 	vector<Filename> patches;
 	if (!ar.isLoader()) {
 		disk->getPatches(patches);
 	}
-	ar.serializeNoID("patches", patches);
+	ar.serialize("patches", patches);
 
 	if (ar.isLoader()) {
 		diskname.updateAfterLoadState(controller);
