@@ -1,8 +1,7 @@
-// $Id: DiskManipulator.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: DiskManipulator.cc 10695 2009-10-25 12:49:50Z mthuurne $
 
 #include "DiskManipulator.hh"
 #include "DiskContainer.hh"
-#include "DiskChanger.hh"
 #include "MSXtar.hh"
 #include "DiskImageUtils.hh"
 #include "DSKDiskImage.hh"
@@ -28,34 +27,39 @@ using std::auto_ptr;
 
 namespace openmsx {
 
-DiskManipulator::DiskManipulator(CommandController& commandController)
+#ifndef _MSC_EXTENSIONS
+// #ifdef required to avoid link error with vc++, see also
+//   http://www.codeguru.com/forum/showthread.php?t=430949
+const unsigned DiskManipulator::MAX_PARTITIONS;
+#endif
+
+DiskManipulator::DiskManipulator(CommandController& commandController,
+                                 Reactor& reactor_)
 	: SimpleCommand(commandController, "diskmanipulator")
-	, virtualDrive(new DiskChanger("virtual_drive", commandController,
-	                               *this, NULL))
+	, reactor(reactor_)
 {
 }
 
 DiskManipulator::~DiskManipulator()
 {
-	virtualDrive.reset();
 	assert(drives.empty()); // all DiskContainers must be unregistered
 }
 
 string DiskManipulator::getMachinePrefix() const
 {
-	string id = getCommandController().getReactor().getMachineID();
+	string id = reactor.getMachineID();
 	return id.empty() ? id : id + "::";
 }
 
-void DiskManipulator::registerDrive(DiskContainer& drive, MSXMotherBoard* board)
+void DiskManipulator::registerDrive(
+	DiskContainer& drive, const std::string& prefix)
 {
 	assert(findDriveSettings(drive) == drives.end());
 	DriveSettings driveSettings;
 	driveSettings.drive = &drive;
-	string prefix = board ? (board->getMachineID() + "::") : "";
 	driveSettings.driveName = prefix + drive.getContainerName();
 	driveSettings.partition = 0;
-	for (int i = 0; i < 32; ++i) {
+	for (unsigned i = 0; i <= MAX_PARTITIONS; ++i) {
 		driveSettings.workingDir[i] = "/";
 	}
 	drives.push_back(driveSettings);
@@ -302,9 +306,9 @@ void DiskManipulator::tabCompletion(vector<string>& tokens) const
 			names.insert(name2);
 			// if it has partitions then we also add the partition
 			// numbers to the autocompletion
-			if (SectorAccessibleDisk* disk =
-				it->drive->getSectorAccessibleDisk()) {
-				for (int i = 1; i <= 31; ++i) {
+			SectorAccessibleDisk* disk = it->drive->getSectorAccessibleDisk();
+			if (disk) {
+				for (unsigned i = 1; i <= MAX_PARTITIONS; ++i) {
 					try {
 						DiskImageUtils::checkFAT12Partition(*disk, i);
 						names.insert(name1 + StringOp::toString(i));
@@ -351,9 +355,9 @@ void DiskManipulator::create(const vector<string>& tokens)
 	vector<unsigned> sizes;
 	unsigned totalSectors = 0;
 	for (unsigned i = 3; i < tokens.size(); ++i) {
-		if (sizes.size() >= 31) {
-			throw CommandException(
-				"Maximum number of partitions is 31.");
+		if (sizes.size() >= MAX_PARTITIONS) {
+			throw CommandException(StringOp::Builder() <<
+				"Maximum number of partitions is " << MAX_PARTITIONS);
 		}
 		char* q;
 		int sectors = strtol(tokens[i].c_str(), &q, 0);
