@@ -1,4 +1,4 @@
-// $Id$
+// $Id: DiskName.cc 10532 2009-09-06 23:03:48Z mthuurne $
 
 #include "DiskName.hh"
 #include "serialize.hh"
@@ -41,10 +41,8 @@ const Filename& DiskName::getFilename() const
 template<typename Archive>
 void DiskName::serialize(Archive& ar, unsigned /*version*/)
 {
-	// serializeNoID because we sometimes serialize DiskName objects that
-	// are allocated on the stack
-	ar.serializeNoID("filename", name);
-	ar.serializeNoID("extra", extra);
+	ar.serialize("filename", name);
+	ar.serialize("extra", extra);
 }
 INSTANTIATE_SERIALIZE_METHODS(DiskName);
 
