@@ -1,4 +1,4 @@
-// $Id: DriveMultiplexer.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: DriveMultiplexer.cc 9383 2009-03-17 19:26:19Z m9710797 $
 
 #include "DriveMultiplexer.hh"
 #include "serialize.hh"
@@ -32,9 +32,9 @@ void DriveMultiplexer::selectDrive(DriveNum num, EmuTime::param time)
 	}
 }
 
-bool DriveMultiplexer::isReady() const
+bool DriveMultiplexer::isDiskInserted() const
 {
-	return drive[selected]->isReady();
+	return drive[selected]->isDiskInserted();
 }
 
 bool DriveMultiplexer::isWriteProtected() const
