@@ -1,4 +1,4 @@
-// $Id: DummyDisk.cc 8351 2008-11-12 18:20:29Z m9710797 $
+// $Id: DummyDisk.cc 9383 2009-03-17 19:26:19Z m9710797 $
 
 #include "DummyDisk.hh"
 #include "DiskExceptions.hh"
@@ -11,9 +11,9 @@ DummyDisk::DummyDisk()
 	setNbSectors(0);
 }
 
-bool DummyDisk::isReady() const
+bool DummyDisk::isDummyDisk() const
 {
-	return false;
+	return true;
 }
 
 bool DummyDisk::isWriteProtectedImpl() const
@@ -21,12 +21,12 @@ bool DummyDisk::isWriteProtectedImpl() const
 	return true; // TODO check
 }
 
-void DummyDisk::readSectorSBD(unsigned /*sector*/, byte* /*buf*/)
+void DummyDisk::readSectorImpl(unsigned /*sector*/, byte* /*buf*/)
 {
 	throw DriveEmptyException("No disk in drive");
 }
 
-void DummyDisk::writeSectorSBD(unsigned /*sector*/, const byte* /*buf*/)
+void DummyDisk::writeSectorImpl(unsigned /*sector*/, const byte* /*buf*/)
 {
 	throw DriveEmptyException("No disk in drive");
 }
