@@ -1,4 +1,4 @@
-// $Id: MSXFDC.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: MSXFDC.cc 10672 2009-10-16 21:11:00Z manuelbi $
 
 #include "MSXFDC.hh"
 #include "Rom.hh"
@@ -16,9 +16,9 @@ MSXFDC::MSXFDC(MSXMotherBoard& motherBoard, const XMLElement& config)
 {
 	bool singleSided = config.findChild("singlesided") != NULL;
 	int numDrives = config.getChildDataAsInt("drives", 1);
-	if ((0 >= numDrives) || (numDrives >= 4)) {
-		throw MSXException("Invalid number of drives: " +
-		                   StringOp::toString(numDrives));
+	if ((0 > numDrives) || (numDrives >= 4)) {
+		throw MSXException(StringOp::Builder() <<
+			"Invalid number of drives: " << numDrives);
 	}
 	int i = 0;
 	for ( ; i < numDrives; ++i) {
