@@ -1,4 +1,4 @@
-// $Id: MSXtar.cc 9855 2009-05-23 16:38:31Z mthuurne $
+// $Id: MSXtar.cc 11317 2010-03-02 19:10:40Z m9710797 $
 
 // Note: For Mac OS X 10.3 <ctime> must be included before <utime.h>.
 #include <ctime>
@@ -89,38 +89,44 @@ void MSXtar::parseBootSector(const byte* buf)
 	const MSXBootSector* boot = reinterpret_cast<const MSXBootSector*>(buf);
 	unsigned nbSectors = rdsh(boot->nrsectors);
 	if (nbSectors == 0) { // TODO: check limits more accurately
-		throw MSXException("Illegal number of sectors: " +
-			StringOp::toString(nbSectors));
+		throw MSXException(StringOp::Builder() <<
+			"Illegal number of sectors: " << nbSectors);
 	}
 	unsigned nbSides = rdsh(boot->nrsides);
 	if (nbSides == 0) { // TODO: check limits more accurately
-		throw MSXException("Illegal number of sides: " +
-			StringOp::toString(nbSides));
+		throw MSXException(StringOp::Builder() <<
+			"Illegal number of sides: " << nbSides);
 	}
 	unsigned nbFats = boot->nrfats[0];
 	if (nbFats == 0) { // TODO: check limits more accurately
-		throw MSXException("Illegal number of FATs: " +
-			StringOp::toString(nbFats));
+		throw MSXException(StringOp::Builder() <<
+			"Illegal number of FATs: " << nbFats);
 	}
 	sectorsPerFat = rdsh(boot->sectorsfat);
 	if (sectorsPerFat == 0) { // TODO: check limits more accurately
-		throw MSXException("Illegal number sectors per FAT: " +
-			StringOp::toString(sectorsPerFat));
+		throw MSXException(StringOp::Builder() <<
+			"Illegal number sectors per FAT: " << sectorsPerFat);
 	}
 	unsigned nbRootDirSectors = rdsh(boot->direntries) / 16;
 	if (nbRootDirSectors == 0) { // TODO: check limits more accurately
-		throw MSXException("Illegal number of root dir sectors: " +
-			StringOp::toString(nbRootDirSectors));
+		throw MSXException(StringOp::Builder() <<
+			"Illegal number of root dir sectors: " << nbRootDirSectors);
 	}
 	sectorsPerCluster = boot->spcluster[0];
 	if (sectorsPerCluster == 0) { // TODO: check limits more accurately
-		throw MSXException("Illegal number of sectors per cluster: " +
-			StringOp::toString(sectorsPerCluster));
+		throw MSXException(StringOp::Builder() <<
+			"Illegal number of sectors per cluster: " << sectorsPerCluster);
 	}
 	rootDirStart = 1 + nbFats * sectorsPerFat;
 	chrootSector = rootDirStart;
 	rootDirLast = rootDirStart + nbRootDirSectors - 1;
 	maxCluster = sectorToCluster(nbSectors);
+
+	// Some (invalid) diskimages have a too small FAT to be able to address
+	// all clusters of the image. OpenMSX SVN revisions pre-11326 even
+	// created such invalid images for some disk sizes!!
+	unsigned maxFatCluster = (2 * SECTOR_SIZE * sectorsPerFat) / 3;
+	maxCluster = std::min(maxCluster, maxFatCluster);
 }
 
 void MSXtar::writeLogicalSector(unsigned sector, const byte* buf)
@@ -699,7 +705,7 @@ void MSXtar::changeTime(string resultFile, MSXDirEntry& direntry)
 
 string MSXtar::dir()
 {
-	string result;
+	StringOp::Builder result;
 	for (unsigned sector = chrootSector; sector != 0; sector = getNextSector(sector)) {
 		byte buf[SECTOR_SIZE];
 		readLogicalSector(sector, buf);
@@ -711,16 +717,16 @@ string MSXtar::dir()
 				// filename first (in condensed form for human readablitly)
 				string tmp = condensName(direntry[i]);
 				tmp.resize(13, ' ');
-				result += tmp;
+				result << tmp;
 				// attributes
-				result += (direntry[i].attrib & T_MSX_DIR  ? "d" : "-");
-				result += (direntry[i].attrib & T_MSX_READ ? "r" : "-");
-				result += (direntry[i].attrib & T_MSX_HID  ? "h" : "-");
-				result += (direntry[i].attrib & T_MSX_VOL  ? "v" : "-"); // TODO check if this is the output of files,l
-				result += (direntry[i].attrib & T_MSX_ARC  ? "a" : "-"); // TODO check if this is the output of files,l
-				result += "  ";
+				result << (direntry[i].attrib & T_MSX_DIR  ? 'd' : '-')
+				       << (direntry[i].attrib & T_MSX_READ ? 'r' : '-')
+				       << (direntry[i].attrib & T_MSX_HID  ? 'h' : '-')
+				       << (direntry[i].attrib & T_MSX_VOL  ? 'v' : '-') // TODO check if this is the output of files,l
+				       << (direntry[i].attrib & T_MSX_ARC  ? 'a' : '-') // TODO check if this is the output of files,l
+				       << "  ";
 				// filesize
-				result += StringOp::toString(rdlg(direntry[i].size)) + "\n";
+				result << rdlg(direntry[i].size) << '\n';
 			}
 		}
 	}
