@@ -1,4 +1,4 @@
-// $Id: RamDSKDiskImage.cc 8201 2008-09-04 17:06:27Z m9710797 $
+// $Id: RamDSKDiskImage.cc 8630 2009-02-21 20:59:32Z m9710797 $
 
 #include "RamDSKDiskImage.hh"
 #include "DiskImageUtils.hh"
@@ -7,7 +7,7 @@
 namespace openmsx {
 
 RamDSKDiskImage::RamDSKDiskImage(unsigned size)
-	: SectorBasedDisk(Filename("ramdsk"))
+	: SectorBasedDisk(DiskName(Filename(), "ramdsk"))
 {
 	setNbSectors(size / SECTOR_SIZE);
 	diskdata = new byte[size];
@@ -20,12 +20,12 @@ RamDSKDiskImage::~RamDSKDiskImage()
 	delete[] diskdata;
 }
 
-void RamDSKDiskImage::readSectorSBD(unsigned sector, byte* buf)
+void RamDSKDiskImage::readSectorImpl(unsigned sector, byte* buf)
 {
 	memcpy(buf, &diskdata[sector * SECTOR_SIZE], SECTOR_SIZE);
 }
 
-void RamDSKDiskImage::writeSectorSBD(unsigned sector, const byte* buf)
+void RamDSKDiskImage::writeSectorImpl(unsigned sector, const byte* buf)
 {
 	memcpy(&diskdata[sector * SECTOR_SIZE], buf, SECTOR_SIZE);
 }
