@@ -1,9 +1,10 @@
-// $Id: RealDrive.cc 9383 2009-03-17 19:26:19Z m9710797 $
+// $Id: RealDrive.cc 10367 2009-08-16 11:40:05Z mthuurne $
 
 #include "RealDrive.hh"
 #include "Disk.hh"
 #include "DiskChanger.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "LedStatus.hh"
 #include "CommandController.hh"
 #include "ThrottleManager.hh"
@@ -25,7 +26,7 @@ RealDrive::RealDrive(MSXMotherBoard& motherBoard_, bool doubleSided)
 	: Schedulable(motherBoard_.getScheduler())
 	, motherBoard(motherBoard_)
 	, loadingIndicator(new LoadingIndicator(
-	         motherBoard.getGlobalSettings().getThrottleManager()))
+		motherBoard.getReactor().getGlobalSettings().getThrottleManager()))
 	, motorTimer(motherBoard.getCurrentTime())
 	, headLoadTimer(motherBoard.getCurrentTime())
 	, headPos(0), side(0), motorStatus(false), headLoadStatus(false)
@@ -56,11 +57,15 @@ RealDrive::RealDrive(MSXMotherBoard& motherBoard_, bool doubleSided)
 	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, driveName, "add");
 	changer.reset(new DiskChanger(
 		driveName, motherBoard.getCommandController(),
-		motherBoard.getDiskManipulator(), &motherBoard));
+		motherBoard.getReactor().getDiskFactory(),
+		motherBoard.getReactor().getDiskManipulator(),
+		motherBoard, true));
 }
 
 RealDrive::~RealDrive()
 {
+	setMotor(false, motherBoard.getCurrentTime()); // to send LED event
+
 	MSXMotherBoard::SharedStuff& info =
 		motherBoard.getSharedStuff("drivesInUse");
 	assert(info.counter);
