@@ -1,4 +1,4 @@
-// $Id: TC8566AF.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: TC8566AF.cc 9383 2009-03-17 19:26:19Z m9710797 $
 
 /*
  * Based on code from NLMSX written by Frits Hilderink
@@ -289,10 +289,10 @@ void TC8566AF::writeReg(int reg, byte data, EmuTime::param time)
 {
 	switch (reg) {
 	case 2: // control register 0
-		drive[3]->setMotor(data & 0x80, time);
-		drive[2]->setMotor(data & 0x40, time);
-		drive[1]->setMotor(data & 0x20, time);
-		drive[0]->setMotor(data & 0x10, time);
+		drive[3]->setMotor((data & 0x80) != 0, time);
+		drive[2]->setMotor((data & 0x40) != 0, time);
+		drive[1]->setMotor((data & 0x20) != 0, time);
+		drive[0]->setMotor((data & 0x10) != 0, time);
 		//enableIntDma = data & 0x08;
 		//notReset     = data & 0x04;
 		driveSelect = data & 0x03;
@@ -390,16 +390,16 @@ void TC8566AF::idlePhaseWrite(byte value)
 
 void TC8566AF::commandPhase1(byte value)
 {
-	drive[driveSelect]->setSide(value & 0x04);
+	drive[driveSelect]->setSide((value & 0x04) != 0);
 	status0 &= ~(ST0_DS0 | ST0_DS1 | ST0_IC0 | ST0_IC1);
-	status0 |= //(drive[driveSelect]->isReady() ? 0 : ST0_DS0) |
+	status0 |= //(drive[driveSelect]->isDiskInserted() ? 0 : ST0_DS0) |
 	           (value & (ST0_DS0 | ST0_DS1)) |
 	           (drive[driveSelect]->isDummyDrive() ? ST0_IC1 : 0);
 	status3  = (value & (ST3_DS0 | ST3_DS1)) |
 	           (drive[driveSelect]->isTrack00()        ? ST3_TK0 : 0) |
 	           (drive[driveSelect]->isDoubleSided()    ? ST3_HD  : 0) |
 	           (drive[driveSelect]->isWriteProtected() ? ST3_WP  : 0) |
-	           (drive[driveSelect]->isReady()          ? ST3_RDY : 0);
+	           (drive[driveSelect]->isDiskInserted()   ? ST3_RDY : 0);
 }
 
 void TC8566AF::commandPhaseWrite(byte value, EmuTime::param time)
@@ -445,7 +445,7 @@ void TC8566AF::commandPhaseWrite(byte value, EmuTime::param time)
 						sectorNumber, sectorBuf,
 						onDiskTrack, onDiskSector,
 						onDiskSide,  onDiskSize);
-				} catch (MSXException& e) {
+				} catch (MSXException&) {
 					status0 |= ST0_IC0;
 					status1 |= ST1_ND;
 				}
@@ -571,7 +571,7 @@ void TC8566AF::executionPhaseWrite(byte value)
 						sectorNumber, sectorBuf,
 						onDiskTrack, onDiskSector,
 						onDiskSide,  onDiskSize);
-				} catch (MSXException& e) {
+				} catch (MSXException&) {
 					status1 |= ST1_NW;
 				}
 				phase       = PHASE_RESULT;
@@ -598,7 +598,7 @@ void TC8566AF::executionPhaseWrite(byte value)
 					sectorNumber, sectorBuf,
 					onDiskTrack, onDiskSector,
 					onDiskSide,  onDiskSize);
-			} catch (MSXException& e) {
+			} catch (MSXException&) {
 				status1 |= ST1_NW;
 			}
 			break;
