@@ -1,4 +1,4 @@
-// $Id: TurboRFDC.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: TurboRFDC.cc 10112 2009-06-30 18:29:35Z m9710797 $
 
 /*
  * Based on code from NLMSX written by Frits Hilderink
@@ -50,13 +50,13 @@ byte TurboRFDC::readMem(word address, EmuTime::param time)
 			result = controller->readReg(5, time);
 			break;
 		default:
-			result = peekMem(address, time);
+			result = TurboRFDC::peekMem(address, time);
 			break;
 		}
 		//std::cout << "TurboRFDC read 0x" << std::hex << (int)address <<
 		//                           " 0x" << (int)result << std::dec << std::endl;
 	} else {
-		result = peekMem(address, time);
+		result = TurboRFDC::peekMem(address, time);
 	}
 	return result;
 }
