@@ -1,9 +1,10 @@
-// $Id: WD2793.cc 9383 2009-03-17 19:26:19Z m9710797 $
+// $Id: WD2793.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "WD2793.hh"
 #include "DiskDrive.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
+#include "unreachable.hh"
 #include <cstring>
 
 namespace openmsx {
@@ -482,7 +483,7 @@ void WD2793::executeUntil(EmuTime::param time, int userData)
 			}
 			break;
 		default:
-			assert(false);
+			UNREACHABLE;
 	}
 }
 
