@@ -1,4 +1,4 @@
-// $Id: XSADiskImage.cc 8202 2008-09-04 17:06:56Z m9710797 $
+// $Id: XSADiskImage.cc 9289 2009-02-28 19:28:15Z mfeingol $
 
 #include "XSADiskImage.hh"
 #include "DiskExceptions.hh"
@@ -60,15 +60,15 @@ XSADiskImage::XSADiskImage(const Filename& filename)
 {
 	XSAExtractor extractor(filename);
 	extractor.getData(data);
-	setNbSectors(data.size() / 512);
+	setNbSectors(unsigned(data.size()) / 512);
 }
 
-void XSADiskImage::readSectorSBD(unsigned sector, byte* buf)
+void XSADiskImage::readSectorImpl(unsigned sector, byte* buf)
 {
 	memcpy(buf, &data[sector * SECTOR_SIZE], SECTOR_SIZE);
 }
 
-void XSADiskImage::writeSectorSBD(unsigned /*sector*/, const byte* /*buf*/)
+void XSADiskImage::writeSectorImpl(unsigned /*sector*/, const byte* /*buf*/)
 {
 	throw WriteProtectedException("Write protected");
 }
@@ -176,7 +176,7 @@ int XSAExtractor::rdstrlen()
 
 	int len = 1;
 	while (nrbits--) {
-		len = (len << 1) | bitin();
+		len = (len << 1) | (bitin() ? 1:0);
 	}
 	return (len + 1);
 }
@@ -193,20 +193,20 @@ int XSAExtractor::rdstrpos()
 			hufpos = hufpos->child1;
 		}
 	}
-	byte cpdindex = hufpos-huftbl;
+	byte cpdindex = byte(hufpos-huftbl);
 	++tblsizes[cpdindex];
 
 	int strpos;
 	if (cpdbmask[cpdindex] >= 256) {
 		byte strposlsb = charin();
 		byte strposmsb = 0;
-		for (byte nrbits = cpdext[cpdindex]-8; nrbits--; strposmsb |= bitin()) {
+		for (byte nrbits = cpdext[cpdindex]-8; nrbits--; strposmsb |= (bitin() ? 1:0)) {
 			strposmsb <<= 1;
 		}
 		strpos = strposlsb + 256 * strposmsb;
 	} else {
 		strpos = 0;
-		for (byte nrbits = cpdext[cpdindex]; nrbits--; strpos |= bitin()) {
+		for (byte nrbits = cpdext[cpdindex]; nrbits--; strpos |= (bitin() ? 1:0)) {
 			strpos <<= 1;
 		}
 	}
