@@ -1,4 +1,4 @@
-// $Id: File.cc 8226 2008-09-15 18:02:40Z m9710797 $
+// $Id: File.cc 10111 2009-06-30 18:28:56Z m9710797 $
 
 #include "File.hh"
 #include "Filename.hh"
@@ -59,9 +59,9 @@ void File::write(const void* buffer, unsigned num)
 	file->write(buffer, num);
 }
 
-byte* File::mmap(bool writeBack)
+byte* File::mmap()
 {
-	return file->mmap(writeBack);
+	return file->mmap();
 }
 
 void File::munmap()
