@@ -1,4 +1,4 @@
-// $Id: FileBase.cc 8228 2008-09-15 18:05:13Z m9710797 $
+// $Id: FileBase.cc 10111 2009-06-30 18:28:56Z m9710797 $
 
 #include "FileBase.hh"
 #include "FileOperations.hh"
@@ -19,10 +19,9 @@ FileBase::~FileBase()
 	munmap();
 }
 
-byte* FileBase::mmap(bool writeBack)
+byte* FileBase::mmap()
 {
 	if (!mmem) {
-		mmapWrite = writeBack;
 		mmapSize = getSize();
 		mmem = new byte[mmapSize];
 		read(mmem, mmapSize);
@@ -32,14 +31,8 @@ byte* FileBase::mmap(bool writeBack)
 
 void FileBase::munmap()
 {
-	if (mmem) {
-		if (mmapWrite) {
-			seek(0);
-			write(mmem, mmapSize);
-		}
-		delete[] mmem;
-		mmem = NULL;
-	}
+	delete[] mmem;
+	mmem = NULL;
 }
 
 void FileBase::truncate(unsigned newSize)
