@@ -1,4 +1,4 @@
-// $Id: FileContext.cc 9277 2009-02-28 07:32:48Z m9710797 $
+// $Id: FileContext.cc 10761 2009-11-08 22:10:06Z manuelbi $
 
 #include "FileContext.hh"
 #include "FileOperations.hh"
@@ -28,16 +28,18 @@ const string SYSTEM_DATA  = "{{SYSTEM_DATA}}";
 static void getUserDirs(CommandController& controller, vector<string>& result)
 {
 	assert(&controller);
+	// TODO: Either remove user dirs feature or find a better home for the
+	//       "user_directories" setting.
+	const string& list =
+		""; //controller.getGlobalSettings().getUserDirSetting().getValue();
+	vector<string> dirs;
 	try {
-		const string& list = controller.getGlobalSettings().
-			getUserDirSetting().getValue();
-		vector<string> dirs;
 		controller.splitList(list, dirs);
-		result.insert(result.end(), dirs.begin(), dirs.end());
 	} catch (CommandException& e) {
 		controller.getCliComm().printWarning(
 			"user directories: " + e.getMessage());
 	}
+	result.insert(result.end(), dirs.begin(), dirs.end());
 }
 
 static string subst(const string& path, const string& before,
@@ -183,6 +185,11 @@ UserFileContext::UserFileContext(const string& savePath)
 	}
 }
 
+UserDataFileContext::UserDataFileContext(const string& subDir)
+{
+	paths.push_back(USER_OPENMSX + "/" + subDir);
+}
+
 CurrentDirFileContext::CurrentDirFileContext()
 {
 	paths.push_back("");
