@@ -1,4 +1,4 @@
-// $Id: FilePool.cc 8178 2008-08-26 17:57:53Z m9710797 $
+// $Id: FilePool.cc 9276 2009-02-28 06:58:00Z mfeingol $
 
 #include "FilePool.hh"
 #include "File.hh"
@@ -95,7 +95,8 @@ void FilePool::readSha1sums()
 void FilePool::writeSha1sums()
 {
 	string cacheFile = FileOperations::getUserDataDir() + FILE_CACHE;
-	ofstream file(cacheFile.c_str());
+	ofstream file;
+	FileOperations::openofstream(file, cacheFile);
 	if (!file.is_open()) {
 		return;
 	}
@@ -145,7 +146,7 @@ auto_ptr<File> FilePool::getFromPool(const string& sha1sum)
 			pool.erase(it++);
 			pool.insert(make_pair(newSum,
 			                 make_pair(newTime, filename)));
-		} catch (FileException& e) {
+		} catch (FileException&) {
 			// error reading file, remove from db
 			pool.erase(it++);
 		}
@@ -187,7 +188,7 @@ auto_ptr<File> FilePool::scanFile(const string& sha1sum, const string& filename)
 			if (sum == sha1sum) {
 				return file;
 			}
-		} catch (FileException& e) {
+		} catch (FileException&) {
 			// ignore
 		}
 	} else {
@@ -213,7 +214,7 @@ auto_ptr<File> FilePool::scanFile(const string& sha1sum, const string& filename)
 					return file;
 				}
 			}
-		} catch (FileException& e) {
+		} catch (FileException&) {
 			// error reading file, remove from db
 			pool.erase(it);
 		}
