@@ -1,13 +1,13 @@
-// $Id: LocalFile.cc 9870 2009-05-23 20:43:43Z mthuurne $
+// $Id: LocalFile.cc 11423 2010-05-01 20:02:09Z m9710797 $
 
 #include "systemfuncs.hh"
 #include "unistdp.hh"
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_MMAP
+#if HAVE_MMAP
 #include <sys/mman.h>
 #endif
-#ifdef _WIN32
+#if defined _WIN32
 #include <io.h>
 #include <iostream>
 #endif
@@ -25,7 +25,7 @@ namespace openmsx {
 
 LocalFile::LocalFile(const string& filename_, File::OpenMode mode)
 	: filename(FileOperations::expandTilde(filename_))
-#ifdef _WIN32
+#if defined _WIN32
 	, hMmap(NULL)
 #endif
 	, readOnly(false)
@@ -68,7 +68,12 @@ LocalFile::LocalFile(const string& filename_, File::OpenMode mode)
 	}
 }
 
-LocalFile::LocalFile(const std::string& filename, const char* mode)
+LocalFile::LocalFile(const std::string& filename_, const char* mode)
+	: filename(FileOperations::expandTilde(filename_))
+#if defined _WIN32
+	, hMmap(NULL)
+#endif
+	, readOnly(false)
 {
 	assert(strchr(mode, 'b'));
 	const string name = FileOperations::getNativePath(filename);
@@ -110,8 +115,8 @@ void LocalFile::write(const void* buffer, unsigned num)
 	}
 }
 
-#ifdef _WIN32
-byte* LocalFile::mmap(bool writeBack)
+#if defined _WIN32
+byte* LocalFile::mmap()
 {
 	if (!mmem) {
 		int fd = _fileno(file);
@@ -123,18 +128,18 @@ byte* LocalFile::mmap(bool writeBack)
 			throw FileException("_get_osfhandle failed");
 		}
 		assert(!hMmap);
-		hMmap = CreateFileMapping(hFile, NULL, writeBack ? PAGE_EXECUTE_READWRITE : PAGE_WRITECOPY, 0, 0, NULL);
+		hMmap = CreateFileMapping(hFile, NULL, PAGE_WRITECOPY, 0, 0, NULL);
 		if (!hMmap) {
-			throw FileException("CreateFileMapping failed: " +
-				StringOp::toString(GetLastError()));
+			throw FileException(StringOp::Builder() <<
+				"CreateFileMapping failed: " << GetLastError());
 		}
-		mmem = static_cast<byte*>(MapViewOfFile(hMmap, writeBack ? FILE_MAP_ALL_ACCESS : FILE_MAP_COPY, 0, 0, 0));
+		mmem = static_cast<byte*>(MapViewOfFile(hMmap, FILE_MAP_COPY, 0, 0, 0));
 		if (!mmem) {
 			DWORD gle = GetLastError();
 			CloseHandle(hMmap);
 			hMmap = NULL;
-			throw FileException("MapViewOfFile failed: " +
-				StringOp::toString(gle));
+			throw FileException(StringOp::Builder() <<
+				"MapViewOfFile failed: " << gle);
 		}
 	}
 	return mmem;
@@ -163,14 +168,13 @@ void LocalFile::munmap()
 	}
 }
 
-#elif defined HAVE_MMAP
-byte* LocalFile::mmap(bool writeBack)
+#elif HAVE_MMAP
+byte* LocalFile::mmap()
 {
 	if (!mmem) {
-		int flags = writeBack ? MAP_SHARED : MAP_PRIVATE;
 		mmem = static_cast<byte*>(
 		          ::mmap(0, getSize(), PROT_READ | PROT_WRITE,
-		                 flags, fileno(file), 0));
+		                 MAP_PRIVATE, fileno(file), 0));
 		// MAP_FAILED is #define'd using an old-style cast, we
 		// have to redefine it ourselves to avoid a warning
 		void* MY_MAP_FAILED = reinterpret_cast<void*>(-1);
@@ -212,7 +216,7 @@ unsigned LocalFile::getPos()
 	return unsigned(ftell(file));
 }
 
-#ifdef HAVE_FTRUNCATE
+#if HAVE_FTRUNCATE
 void LocalFile::truncate(unsigned size)
 {
 	int fd = fileno(file);
