@@ -1,4 +1,4 @@
-// $Id: LocalFileReference.cc 8529 2009-01-29 18:08:53Z m9710797 $
+// $Id: LocalFileReference.cc 10420 2009-08-27 13:22:09Z seanyoung $
 
 #include "LocalFileReference.hh"
 #include "File.hh"
@@ -39,7 +39,7 @@ void LocalFileReference::init(const string& url)
 	tmpDir = FileOperations::getTempDir() + FileOperations::nativePathSeparator + "openmsx";
 #else
 	// TODO - why not just use getTempDir()?
-	tmpDir = "/tmp/openmsx." + StringOp::toString(getpid());
+	tmpDir = StringOp::Builder() << "/tmp/openmsx." << int(getpid());
 #endif
 	// it's possible this directory already exists, in that case the
 	// following function does nothing
