@@ -1,8 +1,9 @@
-// $Id: PreCacheFile.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: PreCacheFile.cc 9276 2009-02-28 06:58:00Z mfeingol $
 
 #include "PreCacheFile.hh"
+#include "FileOperations.hh"
+#include "statp.hh"
 #include <cstdio>
-#include <sys/stat.h>
 #include <sys/types.h>
 
 namespace openmsx {
@@ -22,7 +23,7 @@ void PreCacheFile::run()
 		return;
 	}
 
-	FILE* file = fopen(name.c_str(), "rb");
+	FILE* file = FileOperations::openFile(name, "rb");
 	if (!file) return;
 
 	fseek(file, 0, SEEK_END);
@@ -36,7 +37,7 @@ void PreCacheFile::run()
 		while (true) {
 			char buf[BLOCK_SIZE];
 			fseek(file, block * BLOCK_SIZE, SEEK_SET);
-			unsigned read = fread(buf, 1, BLOCK_SIZE, file);
+			size_t read = fread(buf, 1, BLOCK_SIZE, file);
 			if (read != BLOCK_SIZE) {
 				// error or end-of-file reached,
 				// in both cases stop pre-caching
