@@ -1,4 +1,4 @@
-// $Id: ReadDir.cc 5945 2006-12-19 19:20:18Z m9710797 $
+// $Id: ReadDir.cc 9286 2009-02-28 18:15:49Z mfeingol $
 
 #include "ReadDir.hh"
 
@@ -26,7 +26,7 @@ struct dirent* ReadDir::getEntry()
 
 bool ReadDir::isValid() const
 {
-	return dir;
+	return dir != NULL;
 }
 
 } // namespace openmsx
