@@ -1,7 +1,9 @@
-// $Id: ZlibInflate.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: ZlibInflate.cc 11385 2010-04-06 16:09:12Z mthuurne $
 
 #include "ZlibInflate.hh"
 #include "FileException.hh"
+#include "Math.hh"
+#include "StringOp.hh"
 
 namespace openmsx {
 
@@ -75,11 +77,15 @@ std::string ZlibInflate::getCString()
 
 void ZlibInflate::inflate(std::vector<byte>& output, unsigned sizeHint)
 {
-	inflateInit2(&s, -MAX_WBITS);
+	int initErr = inflateInit2(&s, -MAX_WBITS);
+	if (initErr != Z_OK) {
+		throw FileException(StringOp::Builder()
+			<< "Error initializing inflate struct: " << zError(initErr));
+	}
 	wasInit = true;
 
 	std::vector<byte> buf;
-	buf.resize(sizeHint); // initial buffer size
+	buf.resize(Math::clip<4 * 1024, 1024 * 1024>(sizeHint)); // initial buffer size
 	while (true) {
 		s.next_out = &buf[0] + s.total_out;
 		s.avail_out = uInt(buf.size() - s.total_out);
@@ -88,7 +94,8 @@ void ZlibInflate::inflate(std::vector<byte>& output, unsigned sizeHint)
 			break;
 		}
 		if (err != Z_OK) {
-			throw FileException("Error decompressing gzip");
+			throw FileException(StringOp::Builder()
+				<< "Error decompressing gzip: " << zError(err));
 		}
 		buf.resize(buf.size() * 2); // double buffer size
 	}
