@@ -1,8 +1,8 @@
-// $Id: CDImageCLI.cc 7731 2008-04-03 19:58:10Z m9710797 $
+// $Id: CDImageCLI.cc 10715 2009-10-25 17:30:11Z mthuurne $
 
 #include "CDImageCLI.hh"
-#include "CommandController.hh"
 #include "CommandLineParser.hh"
+#include "GlobalCommandController.hh"
 #include "TclObject.hh"
 #include "MSXException.hh"
 
@@ -12,7 +12,7 @@ using std::string;
 namespace openmsx {
 
 CDImageCLI::CDImageCLI(CommandLineParser& commandLineParser)
-	: commandController(commandLineParser.getCommandController())
+	: commandController(commandLineParser.getGlobalCommandController())
 {
 	commandLineParser.registerOption("-cda", *this);
 	// TODO: offer more options in case you want to specify 2 hard disk images?
