@@ -1,10 +1,10 @@
-// $Id: GoudaSCSI.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: GoudaSCSI.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "GoudaSCSI.hh"
 #include "WD33C93.hh"
 #include "Rom.hh"
 #include "serialize.hh"
-#include <cassert>
+#include "unreachable.hh"
 
 namespace openmsx {
 
@@ -35,8 +35,7 @@ byte GoudaSCSI::readIO(word port, EmuTime::param /*time*/)
 	case 2:
 		return 0xb0; // bit 4: 1 = Halt on SCSI parity error
 	default:
-		assert(false);
-		return 0;
+		UNREACHABLE; return 0;
 	}
 }
 
@@ -50,8 +49,7 @@ byte GoudaSCSI::peekIO(word port, EmuTime::param /*time*/) const
 	case 2:
 		return 0xb0; // bit 4: 1 = Halt on SCSI parity error
 	default:
-		assert(false);
-		return 0;
+		UNREACHABLE; return 0;
 	}
 }
 
@@ -68,13 +66,13 @@ void GoudaSCSI::writeIO(word port, byte value, EmuTime::param time)
 		reset(time);
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
 byte GoudaSCSI::readMem(word address, EmuTime::param /*time*/)
 {
-	return *getReadCacheLine(address);
+	return *GoudaSCSI::getReadCacheLine(address);
 }
 
 const byte* GoudaSCSI::getReadCacheLine(word start) const
