@@ -1,4 +1,4 @@
-// $Id: HD.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: HD.cc 10747 2009-11-04 19:21:39Z m9710797 $
 
 #include "HD.hh"
 #include "File.hh"
@@ -7,6 +7,8 @@
 #include "XMLElement.hh"
 #include "CliComm.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
+#include "GlobalSettings.hh"
 #include "MSXException.hh"
 #include "HDCommand.hh"
 #include "serialize.hh"
@@ -61,9 +63,11 @@ HD::HD(MSXMotherBoard& motherBoard_, const XMLElement& config)
 
 	hdInUse[id] = true;
 	hdCommand.reset(new HDCommand(motherBoard.getCommandController(),
-	                              motherBoard.getMSXEventDistributor(),
+	                              motherBoard.getStateChangeDistributor(),
 	                              motherBoard.getScheduler(),
-	                              *this));
+	                              *this,
+	                              motherBoard.getReactor().getGlobalSettings()
+	                                         .getPowerSetting()));
 
 	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, name, "add");
 }
@@ -183,7 +187,7 @@ template<typename Archive>
 void HD::serialize(Archive& ar, unsigned /*version*/)
 {
 	Filename tmp = file.get() ? filename : Filename();
-	ar.serializeNoID("filename", tmp);
+	ar.serialize("filename", tmp);
 	if (ar.isLoader()) {
 		if (tmp.empty()) {
 			// lazily open file specified in config
