@@ -1,13 +1,11 @@
-// $Id: HDCommand.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: HDCommand.cc 10747 2009-11-04 19:21:39Z m9710797 $
 
 #include "HDCommand.hh"
 #include "HD.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
-#include "CommandController.hh"
 #include "CommandException.hh"
-#include "GlobalSettings.hh"
 #include "BooleanSetting.hh"
 #include "TclObject.hh"
 #include <set>
@@ -21,11 +19,13 @@ using std::set;
 // class HDCommand
 
 HDCommand::HDCommand(CommandController& commandController,
-                     MSXEventDistributor& msxEventDistributor,
-                     Scheduler& scheduler, HD& hd_)
-	: RecordedCommand(commandController, msxEventDistributor,
+                     StateChangeDistributor& stateChangeDistributor,
+                     Scheduler& scheduler, HD& hd_,
+                     BooleanSetting& powerSetting_)
+	: RecordedCommand(commandController, stateChangeDistributor,
 	                  scheduler, hd_.getName())
 	, hd(hd_)
+	, powerSetting(powerSetting_)
 {
 }
 
@@ -45,9 +45,7 @@ void HDCommand::execute(const std::vector<TclObject*>& tokens, TclObject& result
 		}
 	} else if ((tokens.size() == 2) ||
 	           ((tokens.size() == 3) && tokens[1]->getString() == "insert")) {
-		CommandController& controller = getCommandController();
-		if (controller.getGlobalSettings().
-				getPowerSetting().getValue()) {
+		if (powerSetting.getValue()) {
 			throw CommandException(
 				"Can only change hard disk image when MSX "
 				"is powered down.");
@@ -61,6 +59,7 @@ void HDCommand::execute(const std::vector<TclObject*>& tokens, TclObject& result
 					"Missing argument to insert subcommand");
 			}
 		}
+		CommandController& controller = getCommandController();
 		try {
 			Filename filename(tokens[fileToken]->getString(), controller);
 			hd.switchImage(filename);
