@@ -1,8 +1,9 @@
-// $Id: IDEHD.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: IDEHD.cc 10337 2009-08-08 10:05:01Z mthuurne $
 
 #include "IDEHD.hh"
 #include "MSXException.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "DiskManipulator.hh"
 #include "serialize.hh"
 #include <cassert>
@@ -12,9 +13,9 @@ namespace openmsx {
 IDEHD::IDEHD(MSXMotherBoard& motherBoard, const XMLElement& config)
 	: HD(motherBoard, config)
 	, AbstractIDEDevice(motherBoard)
-	, diskManipulator(motherBoard.getDiskManipulator())
+	, diskManipulator(motherBoard.getReactor().getDiskManipulator())
 {
-	diskManipulator.registerDrive(*this, &motherBoard);
+	diskManipulator.registerDrive(*this, motherBoard.getMachineID() + "::");
 }
 
 IDEHD::~IDEHD()
