@@ -1,4 +1,4 @@
-// $Id: MB89352.cc 8537 2009-01-31 19:06:14Z mfeingol $
+// $Id: MB89352.cc 10580 2009-09-14 20:37:31Z m9710797 $
 /* Ported from:
 ** Source: /cvsroot/bluemsx/blueMSX/Src/IoDevice/MB89352.c,v
 ** Revision: 1.9
@@ -105,12 +105,13 @@ MB89352::MB89352(MSXMotherBoard& motherBoard, const XMLElement& config)
 		const XMLElement& target = **it;
 		unsigned id = target.getAttributeAsInt("id");
 		if (id >= MAX_DEV) {
-			throw MSXException(
-				"Invalid SCSI id: " + StringOp::toString(id) +
-				" (should be 0.." + StringOp::toString(MAX_DEV - 1) + ")");
+			throw MSXException(StringOp::Builder() <<
+				"Invalid SCSI id: " << id <<
+				" (should be 0.." + MAX_DEV - 1 << ')');
 		}
 		if (dev[id].get()) {
-			throw MSXException("Duplicate SCSI id: " + StringOp::toString(id));
+			throw MSXException(StringOp::Builder() <<
+				"Duplicate SCSI id: " << id);
 		}
 		const XMLElement& typeElem = target.getChild("type");
 		const string& type = typeElem.getData();
@@ -400,7 +401,7 @@ void MB89352::resetACKREQ()
 		return;
 
 	default:
-		//assert(false);
+		//UNREACHABLE;
 		PRT_DEBUG("reset ACK/REQ code error");
 		break;
 	}
