@@ -1,4 +1,4 @@
-// $Id: SCSILS120.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: SCSILS120.cc 10747 2009-11-04 19:21:39Z m9710797 $
 /* Ported from:
 ** Source: /cvsroot/bluemsx/blueMSX/Src/IoDevice/ScsiDevice.c,v
 ** Revision: 1.10
@@ -92,7 +92,7 @@ class LSXCommand : public RecordedCommand
 {
 public:
 	LSXCommand(CommandController& commandController,
-	           MSXEventDistributor& msxEventDistributor,
+	           StateChangeDistributor& stateChangeDistributor,
 	           Scheduler& scheduler, SCSILS120& ls);
 	virtual void execute(const std::vector<TclObject*>& tokens,
 	                     TclObject& result, EmuTime::param time);
@@ -131,7 +131,7 @@ SCSILS120::SCSILS120(MSXMotherBoard& motherBoard_, const XMLElement& targetconfi
 	name = string("ls") + char('a' + id);
 	lsInUse[id] = true;
 	lsxCommand.reset(new LSXCommand(motherBoard.getCommandController(),
-	                                motherBoard.getMSXEventDistributor(),
+	                                motherBoard.getStateChangeDistributor(),
 	                                motherBoard.getScheduler(), *this));
 
 	reset();
@@ -816,9 +816,9 @@ int SCSILS120::insertDisk(const std::string& filename)
 // class LSXCommand
 
 LSXCommand::LSXCommand(CommandController& commandController,
-                       MSXEventDistributor& msxEventDistributor,
+                       StateChangeDistributor& stateChangeDistributor,
                        Scheduler& scheduler, SCSILS120& ls_)
-	: RecordedCommand(commandController, msxEventDistributor,
+	: RecordedCommand(commandController, stateChangeDistributor,
 	                  scheduler, ls_.name)
 	, ls(ls_)
 {
