@@ -1,4 +1,4 @@
-// $Id: WD33C93.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: WD33C93.cc 10392 2009-08-18 19:56:02Z m9710797 $
 /* Ported from:
 ** Source: /cvsroot/bluemsx/blueMSX/Src/IoDevice/wd33c93.c,v
 ** Revision: 1.12
@@ -116,12 +116,13 @@ WD33C93::WD33C93(MSXMotherBoard& motherBoard, const XMLElement& config)
 		const XMLElement& target = **it;
 		unsigned id = target.getAttributeAsInt("id");
 		if (id >= MAX_DEV) {
-			throw MSXException(
-				"Invalid SCSI id: " + StringOp::toString(id) +
-				" (should be 0.." + StringOp::toString(MAX_DEV - 1) + ")");
+			throw MSXException(StringOp::Builder() <<
+				"Invalid SCSI id: " << id <<
+				" (should be 0.." << MAX_DEV - 1 << ')');
 		}
 		if (dev[id].get()) {
-			throw MSXException("Duplicate SCSI id: " + StringOp::toString(id));
+			throw MSXException(StringOp::Builder() <<
+				"Duplicate SCSI id: " << id);
 		}
 		const XMLElement& typeElem = target.getChild("type");
 		const std::string& type = typeElem.getData();
