@@ -1,15 +1,15 @@
-// $Id: DummyJoystick.cc 4839 2005-11-11 22:00:07Z m9710797 $
+// $Id: DummyJoystick.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "DummyJoystick.hh"
 
 namespace openmsx {
 
-byte DummyJoystick::read(const EmuTime& /*time*/)
+byte DummyJoystick::read(EmuTime::param /*time*/)
 {
 	return 0x3F;
 }
 
-void DummyJoystick::write(byte /*value*/, const EmuTime& /*time*/)
+void DummyJoystick::write(byte /*value*/, EmuTime::param /*time*/)
 {
 	// do nothing
 }
@@ -21,11 +21,11 @@ const std::string& DummyJoystick::getDescription() const
 }
 
 void DummyJoystick::plugHelper(Connector& /*connector*/,
-                               const EmuTime& /*time*/)
+                               EmuTime::param /*time*/)
 {
 }
 
-void DummyJoystick::unplugHelper(const EmuTime& /*time*/)
+void DummyJoystick::unplugHelper(EmuTime::param /*time*/)
 {
 }
 
