@@ -1,7 +1,8 @@
-// $Id: JoyTap.cc 5089 2006-01-27 21:53:29Z m9710797 $
+// $Id: JoyTap.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "JoyTap.hh"
 #include "JoystickPort.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
@@ -31,17 +32,17 @@ const std::string& JoyTap::getName() const
 	return name;
 }
 
-void JoyTap::plugHelper(Connector& /*connector*/, const EmuTime& /*time*/)
+void JoyTap::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
 {
 	// TODO move create of the joystickports to here???
 }
 
-void JoyTap::unplugHelper(const EmuTime& /*time*/)
+void JoyTap::unplugHelper(EmuTime::param /*time*/)
 {
 	// TODO move destruction of the joystickports to here???
 }
 
-byte JoyTap::read(const EmuTime& time)
+byte JoyTap::read(EmuTime::param time)
 {
 	byte value = 255;
 	for (int i = 0; i < 4; ++i) {
@@ -50,11 +51,22 @@ byte JoyTap::read(const EmuTime& time)
 	return value;
 }
 
-void JoyTap::write(byte value, const EmuTime& time)
+void JoyTap::write(byte value, EmuTime::param time)
 {
 	for (int i = 0; i < 4; ++i) {
 		slaves[i]->write(value, time);
 	}
 }
 
+template<typename Archive>
+void JoyTap::serialize(Archive& ar, unsigned /*version*/)
+{
+	for (int i = 0; i < 4; ++i) {
+		std::string tag = std::string("port") + char('0' + i);
+		ar.serialize(tag.c_str(), *slaves[i]);
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(JoyTap);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, JoyTap, "JoyTap");
+
 } // namespace openmsx
