@@ -1,4 +1,4 @@
-// $Id: Keyboard.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: Keyboard.cc 11324 2010-03-11 18:49:45Z m9710797 $
 
 #include "Keyboard.hh"
 #include "KeyboardSettings.hh"
@@ -6,21 +6,25 @@
 #include "Clock.hh"
 #include "EventListener.hh"
 #include "EventDistributor.hh"
+#include "InputEventFactory.hh"
 #include "MSXEventDistributor.hh"
+#include "StateChangeDistributor.hh"
+#include "MSXMotherBoard.hh"
+#include "ReverseManager.hh"
 #include "MSXException.hh"
-#include "File.hh"
-#include "FileContext.hh"
-#include "FileException.hh"
 #include "RecordedCommand.hh"
 #include "CommandException.hh"
+#include "SimpleDebuggable.hh"
 #include "InputEvents.hh"
-#include "FilenameSetting.hh"
+#include "StateChange.hh"
 #include "BooleanSetting.hh"
 #include "EnumSetting.hh"
 #include "UnicodeKeymap.hh"
 #include "utf8_checked.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
+#include "serialize_stl.hh"
+#include "serialize_meta.hh"
 #include <SDL.h>
 #include <cstdio>
 #include <cstdlib>
@@ -44,7 +48,7 @@ class KeyMatrixUpCmd : public RecordedCommand
 {
 public:
 	KeyMatrixUpCmd(CommandController& commandController,
-	               MSXEventDistributor& msxEventDistributor,
+	               StateChangeDistributor& stateChangeDistributor,
 	               Scheduler& scheduler, Keyboard& keyboard);
 	virtual string execute(const vector<string>& tokens, EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
@@ -56,7 +60,7 @@ class KeyMatrixDownCmd : public RecordedCommand
 {
 public:
 	KeyMatrixDownCmd(CommandController& commandController,
-	                 MSXEventDistributor& msxEventDistributor,
+	                 StateChangeDistributor& stateChangeDistributor,
 	                 Scheduler& scheduler, Keyboard& keyboard);
 	virtual string execute(const vector<string>& tokens, EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
@@ -64,11 +68,14 @@ private:
 	Keyboard& keyboard;
 };
 
-class MsxKeyEventQueue : private Schedulable
+class MsxKeyEventQueue : public Schedulable
 {
 public:
 	MsxKeyEventQueue(Scheduler& scheduler, Keyboard& keyboard);
 	void process_asap(EmuTime::param time, shared_ptr<const Event> event);
+	void clear();
+	template<typename Archive>
+	void serialize(Archive& ar, unsigned version);
 
 private:
 	// Schedulable
@@ -82,7 +89,7 @@ class KeyInserter : public RecordedCommand, public Schedulable
 {
 public:
 	KeyInserter(CommandController& commandController,
-	            MSXEventDistributor& msxEventDistributor,
+	            StateChangeDistributor& stateChangeDistributor,
 	            Scheduler& scheduler, Keyboard& keyboard);
 	template<typename Archive>
 	void serialize(Archive& ar, unsigned version);
@@ -132,79 +139,73 @@ private:
 	MSXEventDistributor& msxEventDistributor;
 };
 
-
-void Keyboard::parseKeymapfile(const byte* buf, unsigned size)
+class KeybDebuggable : public SimpleDebuggable
 {
-	unsigned i = 0;
-	while  (i < size) {
-		string line;
-		bool done = false;
-		while (!done && (i < size)) {
-			char c = buf[i++];
-			switch (c) {
-				case '#':
-					while ((i < size) &&
-					       (buf[i++] != '\n')) {
-						// skip till end of line
-					}
-					done = true;
-					break;
-				case '\r':
-				case '\t':
-				case ' ':
-					// skip whitespace
-					break;
-				case '\n':
-				case '\0':
-					// end of line
-					done = true;
-					break;
-				default:
-					line += c;
-					break;
-			}
-		}
-		int vkey, row, bit;
-		int rdnum = sscanf(line.c_str(), "%i=%i,%i", &vkey, &row, &bit);
-		if ( rdnum == 3 && 0 <= vkey && vkey < MAX_KEYSYM
-		&& 0 <= row && row < 12 && 0 <= bit && bit < 256 ) {
-			keyTab[vkey][0] = row;
-			keyTab[vkey][1] = bit;
-		}
-	}
-}
+public:
+	KeybDebuggable(MSXMotherBoard& motherBoard, Keyboard& keyboard);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value);
+private:
+	Keyboard& keyboard;
+};
 
-void Keyboard::loadKeymapfile(const string& filename)
+
+class KeyMatrixState : public StateChange
 {
-	try {
-		File file(filename);
-		byte* buf = file.mmap();
-		parseKeymapfile(buf, file.getSize());
-	} catch (FileException&) {
-		throw MSXException("Couldn't load keymap file: " + filename);
+public:
+	KeyMatrixState() {} // for serialize
+	KeyMatrixState(EmuTime::param time, byte row_, byte press_, byte release_)
+		: StateChange(time)
+		, row(row_), press(press_), release(release_)
+	{
+		// disallow useless events
+		assert((press != 0) || (release != 0));
+		// avoid confusion about what happens when some bits are both
+		// set and reset (in other words: don't rely on order of and-
+		// and or-operations)
+		assert((press & release) == 0);
 	}
-}
+	byte getRow()     const { return row; }
+	byte getPress()   const { return press; }
+	byte getRelease() const { return release; }
 
-Keyboard::Keyboard(Scheduler& scheduler,
+	template<typename Archive> void serialize(Archive& ar, unsigned /*version*/)
+	{
+		ar.template serializeBase<StateChange>(*this);
+		ar.serialize("row", row);
+		ar.serialize("press", press);
+		ar.serialize("release", release);
+	}
+private:
+	byte row, press, release;
+};
+REGISTER_POLYMORPHIC_CLASS(StateChange, KeyMatrixState, "KeyMatrixState");
+
+
+Keyboard::Keyboard(MSXMotherBoard& motherBoard,
+                   Scheduler& scheduler,
                    CommandController& commandController_,
                    EventDistributor& eventDistributor,
                    MSXEventDistributor& msxEventDistributor_,
+                   StateChangeDistributor& stateChangeDistributor_,
                    string& keyboardType, bool hasKP, bool keyGhosting_,
                    bool keyGhostSGCprotected, bool codeKanaLocks_,
                    bool graphLocks_)
 	: Schedulable(scheduler)
 	, commandController(commandController_)
 	, msxEventDistributor(msxEventDistributor_)
+	, stateChangeDistributor(stateChangeDistributor_)
 	, keyMatrixUpCmd  (new KeyMatrixUpCmd  (
-		commandController, msxEventDistributor, scheduler, *this))
+		commandController, stateChangeDistributor, scheduler, *this))
 	, keyMatrixDownCmd(new KeyMatrixDownCmd(
-		commandController, msxEventDistributor, scheduler, *this))
+		commandController, stateChangeDistributor, scheduler, *this))
 	, keyTypeCmd(new KeyInserter(
-		commandController, msxEventDistributor, scheduler, *this))
+		commandController, stateChangeDistributor, scheduler, *this))
 	, capsLockAligner(new CapsLockAligner(
 		eventDistributor, msxEventDistributor, scheduler, *this))
 	, keyboardSettings(new KeyboardSettings(commandController))
 	, msxKeyEventQueue(new MsxKeyEventQueue(scheduler, *this))
+	, keybDebuggable(new KeybDebuggable(motherBoard, *this))
 	, unicodeKeymap(new UnicodeKeymap(keyboardType))
 	, hasKeypad(hasKP)
 	, keyGhosting(keyGhosting_)
@@ -220,25 +221,21 @@ Keyboard::Keyboard(Scheduler& scheduler,
 	memset(keyMatrix,     255, sizeof(keyMatrix));
 	memset(cmdKeyMatrix,  255, sizeof(cmdKeyMatrix));
 	memset(userKeyMatrix, 255, sizeof(userKeyMatrix));
+	memset(hostKeyMatrix, 255, sizeof(hostKeyMatrix));
 	memset(dynKeymap, 0, sizeof(dynKeymap));
 
-	SystemFileContext context;
-
-	string keymapFile = keyboardSettings->getKeymapFile().getValueString();
-	if (!keymapFile.empty()) {
-		loadKeymapfile(context.resolve(commandController, keymapFile));
-	}
-
-	keyboardSettings->getKeymapFile().attach(*this);
 	msxEventDistributor.registerEventListener(*this);
+	stateChangeDistributor.registerListener(*this);
 	// We do not listen for CONSOLE_OFF_EVENTS because rescanning the
 	// keyboard can have unwanted side effects
+
+	motherBoard.getReverseManager().registerKeyboard(*this);
 }
 
 Keyboard::~Keyboard()
 {
+	stateChangeDistributor.unregisterListener(*this);
 	msxEventDistributor.unregisterEventListener(*this);
-	keyboardSettings->getKeymapFile().detach(*this);
 }
 
 
@@ -256,7 +253,29 @@ const byte* Keyboard::getKeys()
 	return keyMatrix;
 }
 
-/* Received an MSX event (through EventTranslator class)
+void Keyboard::transferHostKeyMatrix(const Keyboard& source)
+{
+	// This mechanism exists to solve the following problem:
+	//   - play a game where the spacebar is constantly pressed (e.g.
+	//     Road Fighter)
+	//   - go back in time (press the reverse hotkey) while keeping the
+	//     spacebar pressed
+	//   - interrupt replay by pressing the cursor keys, still while
+	//     keeping spacebar pressed
+	// At the moment replay is interrupted, we need to resynchronize the
+	// msx keyboard with the host keyboard. In the past we assumed the host
+	// keyboard had no keys pressed. But this is wrong in the above
+	// scenario. Now we remember the state of the host keyboard and
+	// transfer that to the new keyboard(s) that get created for reverese.
+	// When replay is stopped we restore this host keyboard state, see
+	// stopReplay().
+
+	for (unsigned row = 0; row < NR_KEYROWS; ++row) {
+		hostKeyMatrix[row] = source.hostKeyMatrix[row];
+	}
+}
+
+/* Received an MSX event
  * Following events get processed:
  *  OPENMSX_KEY_DOWN_EVENT
  *  OPENMSX_KEY_UP_EVENT
@@ -275,6 +294,57 @@ void Keyboard::signalEvent(shared_ptr<const Event> event,
 	}
 }
 
+void Keyboard::signalStateChange(shared_ptr<StateChange> event)
+{
+	KeyMatrixState* kms = dynamic_cast<KeyMatrixState*>(event.get());
+	if (!kms) return;
+
+	userKeyMatrix[kms->getRow()] &= ~kms->getPress();
+	userKeyMatrix[kms->getRow()] |=  kms->getRelease();
+	keysChanged = true; // do ghosting at next getKeys()
+}
+
+void Keyboard::stopReplay(EmuTime::param time)
+{
+	for (unsigned row = 0; row < NR_KEYROWS; ++row) {
+		changeKeyMatrixEvent(time, row, hostKeyMatrix[row]);
+	}
+	msxmodifiers = 0xff;
+	msxKeyEventQueue->clear();
+	memset(dynKeymap, 0, sizeof(dynKeymap));
+}
+
+void Keyboard::pressKeyMatrixEvent(EmuTime::param time, byte row, byte press)
+{
+	assert(press);
+	stateChangeDistributor.stopReplay(time);
+	// call stopReplay() before using userKeyMatrix[]
+	changeKeyMatrixEvent(time, row, userKeyMatrix[row] & ~press);
+}
+void Keyboard::releaseKeyMatrixEvent(EmuTime::param time, byte row, byte release)
+{
+	assert(release);
+	stateChangeDistributor.stopReplay(time);
+	// call stopReplay() before using userKeyMatrix[]
+	changeKeyMatrixEvent(time, row, userKeyMatrix[row] | release);
+}
+
+void Keyboard::changeKeyMatrixEvent(EmuTime::param time, byte row, byte newValue)
+{
+	assert(!stateChangeDistributor.isReplaying());
+
+	// This method already updates hostKeyMatrix[],
+	// userKeyMatrix[] will soon be updated via KeyMatrixState events.
+	hostKeyMatrix[row] = newValue;
+
+	byte diff = userKeyMatrix[row] ^ newValue;
+	if (diff == 0) return;
+	byte press   = userKeyMatrix[row] & diff;
+	byte release = newValue           & diff;
+	stateChangeDistributor.distributeNew(shared_ptr<StateChange>(
+		new KeyMatrixState(time, row, press, release)));
+}
+
 bool Keyboard::processQueuedEvent(shared_ptr<const Event> event, EmuTime::param time)
 {
 	bool insertCodeKanaRelease = false;
@@ -296,17 +366,17 @@ bool Keyboard::processQueuedEvent(shared_ptr<const Event> event, EmuTime::param
 	if (key == Keys::K_RCTRL &&
 	    keyboardSettings->getMappingMode().getValue() ==
 	            KeyboardSettings::CHARACTER_MAPPING) {
-		processRightControlEvent(down);
+		processRightControlEvent(time, down);
 	} else if (key == Keys::K_CAPSLOCK) {
 		processCapslockEvent(time);
 	} else if (key == keyboardSettings->getCodeKanaHostKey().getValue()) {
-		processCodeKanaChange(down);
+		processCodeKanaChange(time, down);
 	} else if (key == Keys::K_LALT) {
-		processGraphChange(down);
+		processGraphChange(time, down);
 	} else if (key == Keys::K_KP_ENTER) {
-		processKeypadEnterKey(down);
+		processKeypadEnterKey(time, down);
 	} else {
-		insertCodeKanaRelease = processKeyEvent(down, keyEvent);
+		insertCodeKanaRelease = processKeyEvent(time, down, keyEvent);
 	}
 	return insertCodeKanaRelease;
 }
@@ -316,12 +386,12 @@ bool Keyboard::processQueuedEvent(shared_ptr<const Event> event, EmuTime::param
  * It presses or releases the key in the MSX keyboard matrix
  * and changes the kanalock state in case of a press
  */
-void Keyboard::processCodeKanaChange(bool down)
+void Keyboard::processCodeKanaChange(EmuTime::param time, bool down)
 {
 	if (down) {
 		msxCodeKanaLockOn = !msxCodeKanaLockOn;
 	}
-	updateKeyMatrix(down, 6, CODE_MASK);
+	updateKeyMatrix(time, down, 6, CODE_MASK);
 }
 
 /*
@@ -329,12 +399,12 @@ void Keyboard::processCodeKanaChange(bool down)
  * It presses or releases the key in the MSX keyboard matrix
  * and changes the graphlock state in case of a press
  */
-void Keyboard::processGraphChange(bool down)
+void Keyboard::processGraphChange(EmuTime::param time, bool down)
 {
 	if (down) {
 		msxGraphLockOn = !msxGraphLockOn;
 	}
-	updateKeyMatrix(down, 6, GRAPH_MASK);
+	updateKeyMatrix(time, down, 6, GRAPH_MASK);
 }
 
 /*
@@ -342,10 +412,12 @@ void Keyboard::processGraphChange(bool down)
  * It presses or releases the DEADKEY at the correct location
  * in the keyboard matrix
  */
-void Keyboard::processRightControlEvent(bool down)
+void Keyboard::processRightControlEvent(EmuTime::param time, bool down)
 {
 	UnicodeKeymap::KeyInfo deadkey = unicodeKeymap->getDeadkey();
-	updateKeyMatrix(down, deadkey.row, deadkey.keymask);
+	if (deadkey.keymask) {
+		updateKeyMatrix(time, down, deadkey.row, deadkey.keymask);
+	}
 }
 
 /*
@@ -368,14 +440,14 @@ void Keyboard::processRightControlEvent(bool down)
 void Keyboard::processCapslockEvent(EmuTime::param time)
 {
 	msxCapsLockOn = !msxCapsLockOn;
-	updateKeyMatrix(true, 6, CAPS_MASK);
+	updateKeyMatrix(time, true, 6, CAPS_MASK);
 	Clock<1000> now(time);
 	setSyncPoint(now + 100);
 }
 
-void Keyboard::executeUntil(EmuTime::param /*time*/, int /*userData*/)
+void Keyboard::executeUntil(EmuTime::param time, int /*userData*/)
 {
-	updateKeyMatrix(false, 6, CAPS_MASK);
+	updateKeyMatrix(time, false, 6, CAPS_MASK);
 }
 
 const string& Keyboard::schedName() const
@@ -384,7 +456,7 @@ const string& Keyboard::schedName() const
 	return schedName;
 }
 
-void Keyboard::processKeypadEnterKey(bool down)
+void Keyboard::processKeypadEnterKey(EmuTime::param time, bool down)
 {
 	if (!hasKeypad && !keyboardSettings->getAlwaysEnableKeypad().getValue()) {
 		// User entered on host keypad but this MSX model does not have one
@@ -401,7 +473,7 @@ void Keyboard::processKeypadEnterKey(bool down)
 		row = 7;
 		mask = 0x80;
 	}
-	updateKeyMatrix(down, row, mask);
+	updateKeyMatrix(time, down, row, mask);
 }
 
 /*
@@ -410,23 +482,25 @@ void Keyboard::processKeypadEnterKey(bool down)
  * be unambiguously derived from a unicode character;
  *  Map the SDL key to an equivalent MSX key press/release event
  */
-void Keyboard::processSdlKey(bool down, int key)
+void Keyboard::processSdlKey(EmuTime::param time, bool down, int key)
 {
 	if (key < MAX_KEYSYM) {
 		int row   = keyTab[key][0];
 		byte mask = keyTab[key][1];
-		updateKeyMatrix(down, row, mask);
+		if (mask) {
+			updateKeyMatrix(time, down, row, mask);
+		}
 	}
 }
 
 /*
  * Update the MSX keyboard matrix
  */
-void Keyboard::updateKeyMatrix(bool down, int row, byte mask)
+void Keyboard::updateKeyMatrix(EmuTime::param time, bool down, int row, byte mask)
 {
+	assert(mask);
 	if (down) {
-		// Key pressed: reset bit in user keyMatrix
-		userKeyMatrix[row] &= ~mask;
+		pressKeyMatrixEvent(time, row, mask);
 		if (row == 6) {
 			// Keep track of the MSX modifiers (CTRL, GRAPH, CODE, SHIFT)
 			// The MSX modifiers in row 6 of the matrix sometimes get
@@ -436,13 +510,11 @@ void Keyboard::updateKeyMatrix(bool down, int row, byte mask)
 			msxmodifiers &= ~(mask & 0x17);
 		}
 	} else {
-		// Key released: set bit in keyMatrix
-		userKeyMatrix[row] |= mask;
+		releaseKeyMatrixEvent(time, row, mask);
 		if (row == 6) {
 			msxmodifiers |= (mask & 0x17);
 		}
 	}
-	keysChanged = true; // do ghosting at next getKeys()
 }
 
 /*
@@ -453,7 +525,7 @@ void Keyboard::updateKeyMatrix(bool down, int row, byte mask)
  *  and map the unicode character to the key-combination that must
  *  be pressed to generate the equivalent character on the MSX
  */
-bool Keyboard::processKeyEvent(bool down, const KeyEvent& keyEvent)
+bool Keyboard::processKeyEvent(EmuTime::param time, bool down, const KeyEvent& keyEvent)
 {
 	bool insertCodeKanaRelease = false;
 	Keys::KeyCode keyCode = keyEvent.getKeyCode();
@@ -526,11 +598,11 @@ bool Keyboard::processKeyEvent(bool down, const KeyEvent& keyEvent)
 			// But only when it is not a first keystroke of a
 			// composed key
 			if ((keyCode & Keys::KM_MODE) == 0) {
-				processSdlKey(down, key);
+				processSdlKey(time, down, key);
 			}
 		} else {
 			// It is a unicode character; map it to the right key-combination
-			insertCodeKanaRelease = pressUnicodeByUser(unicode, key, true);
+			insertCodeKanaRelease = pressUnicodeByUser(time, unicode, key, true);
 		}
 	} else {
 		// key was released
@@ -544,11 +616,11 @@ bool Keyboard::processKeyEvent(bool down, const KeyEvent& keyEvent)
 			// But only when it is not a first keystroke of a
 			// composed key
 			if ((keyCode & Keys::KM_MODE) == 0) {
-				processSdlKey(down, key);
+				processSdlKey(time, down, key);
 			}
 		} else {
 			// It was a unicode character; map it to the right key-combination
-			pressUnicodeByUser(unicode, key, false);
+			pressUnicodeByUser(time, unicode, key, false);
 		}
 	}
 	return insertCodeKanaRelease;
@@ -671,11 +743,14 @@ string Keyboard::processCmd(const vector<string>& tokens, bool up)
  *              7    6     5     4    3      2     1    0
  * row  6   |  F3 |  F2 |  F1 | code| caps|graph| ctrl|shift|
  */
-bool Keyboard::pressUnicodeByUser(unsigned unicode, int key, bool down)
+bool Keyboard::pressUnicodeByUser(EmuTime::param time, unsigned unicode, int key, bool down)
 {
 	bool insertCodeKanaRelease = false;
-	byte shiftkeymask = (key >= Keys::K_A && key <= Keys::K_Z) ? 0x00 : SHIFT_MASK;
 	UnicodeKeymap::KeyInfo keyInfo = unicodeKeymap->get(unicode);
+	if (keyInfo.keymask == 0) {
+		return insertCodeKanaRelease;
+	}
+
 	if (down) {
 		if (codeKanaLocks &&
 		    keyboardSettings->getAutoToggleCodeKanaLock().getValue() &&
@@ -685,7 +760,7 @@ bool Keyboard::pressUnicodeByUser(unsigned unicode, int key, bool down)
 			// Toggle it by pressing the lock key and scheduling a
 			// release event
 			msxCodeKanaLockOn = !msxCodeKanaLockOn;
-			userKeyMatrix[6] &= (~CODE_MASK);
+			pressKeyMatrixEvent(time, 6, CODE_MASK);
 			insertCodeKanaRelease = true;
 		} else {
 			// Press the character key and related modifiers
@@ -695,31 +770,42 @@ bool Keyboard::pressUnicodeByUser(unsigned unicode, int key, bool down)
 			// Always ignore CAPSLOCK mask (assume that user will
 			// use real CAPS lock to switch/ between hiragana and
 			// katanana on japanese model)
-			byte modmask = keyInfo.modmask & (~CAPS_MASK);
-			if (codeKanaLocks) {
-				modmask &= (~CODE_MASK);
+			assert(keyInfo.keymask);
+			pressKeyMatrixEvent(time, keyInfo.row, keyInfo.keymask);
+			assert(!stateChangeDistributor.isReplaying());
+
+			byte modmask = keyInfo.modmask & ~CAPS_MASK;
+			if (codeKanaLocks) modmask &= ~CODE_MASK;
+			if (graphLocks)    modmask &= ~GRAPH_MASK;
+			if ((Keys::K_A <= key) && (key <= Keys::K_Z)) {
+				// for A-Z, leave shift unchanged, (other
+				// modifiers are only pressed, never released)
+				byte press = modmask & ~SHIFT_MASK;
+				if (press) {
+					pressKeyMatrixEvent(time, 6, press);
+				}
+			} else {
+				// for other keys, set shift according to modmask
+				// so also release shift when required (other
+				// modifiers are only pressed, never released)
+				byte newRow = (userKeyMatrix[6] | SHIFT_MASK) & ~modmask;
+				changeKeyMatrixEvent(time, 6, newRow);
 			}
-			if (graphLocks) {
-				modmask &= (~GRAPH_MASK);
-			}
-			userKeyMatrix[keyInfo.row] &= ~keyInfo.keymask;
-			userKeyMatrix[6] |= shiftkeymask;
-			userKeyMatrix[6] &= ~(modmask & ((~SHIFT_MASK) | shiftkeymask));
 		}
 	} else {
-		userKeyMatrix[keyInfo.row] |= keyInfo.keymask;
-		// Do not simply unpress graph, ctrl, code and shift
-		// but restore them to the values currently pressed
-		// by the user
+		assert(keyInfo.keymask);
+		releaseKeyMatrixEvent(time, keyInfo.row, keyInfo.keymask);
+		assert(!stateChangeDistributor.isReplaying());
+
+		// Do not simply unpress graph, ctrl, code and shift but
+		// restore them to the values currently pressed by the user
 		byte mask = SHIFT_MASK | CTRL_MASK;
-		if (!codeKanaLocks) {
-			mask |= CODE_MASK;
-		}
-		if (!graphLocks) {
-			mask |= GRAPH_MASK;
-		}
-		userKeyMatrix[6] &= (msxmodifiers | ~mask);
-		userKeyMatrix[6] |= (msxmodifiers & mask);
+		if (!codeKanaLocks) mask |= CODE_MASK;
+		if (!graphLocks)    mask |= GRAPH_MASK;
+		byte newRow = userKeyMatrix[6];
+		newRow &= msxmodifiers | ~mask;
+		newRow |= msxmodifiers &  mask;
+		changeKeyMatrixEvent(time, 6, newRow);
 	}
 	keysChanged = true;
 	return insertCodeKanaRelease;
@@ -815,18 +901,6 @@ bool Keyboard::commonKeys(unsigned unicode1, unsigned unicode2)
 	        (keyInfo1.keymask & keyInfo2.keymask));
 }
 
-void Keyboard::update(const Setting& setting)
-{
-	assert(&setting == &keyboardSettings->getKeymapFile());
-	(void)setting;
-
-	string keymapFile = keyboardSettings->getKeymapFile().getValueString();
-	if (!keymapFile.empty()) {
-		SystemFileContext context;
-		loadKeymapfile(context.resolve(commandController, keymapFile));
-	}
-}
-
 void Keyboard::debug(const char* format, ...)
 {
 	if (keyboardSettings->getTraceKeyPresses().getValue()) {
@@ -841,9 +915,9 @@ void Keyboard::debug(const char* format, ...)
 // class KeyMatrixUpCmd
 
 KeyMatrixUpCmd::KeyMatrixUpCmd(CommandController& commandController,
-		MSXEventDistributor& msxEventDistributor,
+		StateChangeDistributor& stateChangeDistributor,
 		Scheduler& scheduler, Keyboard& keyboard_)
-	: RecordedCommand(commandController, msxEventDistributor,
+	: RecordedCommand(commandController, stateChangeDistributor,
 		scheduler, "keymatrixup")
 	, keyboard(keyboard_)
 {
@@ -865,9 +939,9 @@ string KeyMatrixUpCmd::help(const vector<string>& /*tokens*/) const
 // class KeyMatrixDownCmd
 
 KeyMatrixDownCmd::KeyMatrixDownCmd(CommandController& commandController,
-		MSXEventDistributor& msxEventDistributor,
+		StateChangeDistributor& stateChangeDistributor,
 		Scheduler& scheduler, Keyboard& keyboard_)
-	: RecordedCommand(commandController, msxEventDistributor,
+	: RecordedCommand(commandController, stateChangeDistributor,
 		scheduler, "keymatrixdown")
 	, keyboard(keyboard_)
 {
@@ -903,6 +977,12 @@ void MsxKeyEventQueue::process_asap(EmuTime::param time, shared_ptr<const Event>
 	}
 }
 
+void MsxKeyEventQueue::clear()
+{
+	eventQueue.clear();
+	removeSyncPoint();
+}
+
 void MsxKeyEventQueue::executeUntil(EmuTime::param time, int /*userData*/)
 {
 	// Get oldest event from the queue and process it
@@ -917,7 +997,12 @@ void MsxKeyEventQueue::executeUntil(EmuTime::param time, int /*userData*/)
 			keyboard.keyboardSettings->getCodeKanaHostKey().getValue())));
 	} else {
 		// The event has been completely processed. Delete it from the queue
-		eventQueue.pop_front();
+		if (!eventQueue.empty()) {
+			eventQueue.pop_front();
+		} else {
+			// it's possible clear() has been called
+			// (indirectly from keyboard.processQueuedEvent())
+		}
 	}
 
 	if (!eventQueue.empty()) {
@@ -937,9 +1022,9 @@ const string& MsxKeyEventQueue::schedName() const
 // class KeyInserter
 
 KeyInserter::KeyInserter(CommandController& commandController,
-		MSXEventDistributor& msxEventDistributor,
+		StateChangeDistributor& stateChangeDistributor,
 		Scheduler& scheduler, Keyboard& keyboard_)
-	: RecordedCommand(commandController, msxEventDistributor,
+	: RecordedCommand(commandController, stateChangeDistributor,
 		scheduler, "type")
 	, Schedulable(scheduler)
 	, keyboard(keyboard_)
@@ -1132,6 +1217,27 @@ const string& CapsLockAligner::schedName() const
 	return schedName;
 }
 
+
+// class KeybDebuggable
+
+KeybDebuggable::KeybDebuggable(MSXMotherBoard& motherBoard, Keyboard& keyboard_)
+	: SimpleDebuggable(motherBoard, "keymatrix", "MSX Keyboard Matrix",
+	                   Keyboard::NR_KEYROWS)
+	, keyboard(keyboard_)
+{
+}
+
+byte KeybDebuggable::read(unsigned address)
+{
+	return keyboard.getKeys()[address];
+}
+
+void KeybDebuggable::write(unsigned /*address*/, byte /*value*/)
+{
+	// ignore
+}
+
+
 template<typename Archive>
 void KeyInserter::serialize(Archive& ar, unsigned /*version*/)
 {
@@ -1145,8 +1251,20 @@ void KeyInserter::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("oldCapsLockOn", oldCapsLockOn);
 }
 
+// version 1: Initial version: {userKeyMatrix, dynKeymap, msxmodifiers,
+//            msxKeyEventQueue} was intentionally not serialized. The reason
+//            was that after a loadstate, you want the MSX keyboard to reflect
+//            the state of the host keyboard. So any pressed MSX keys from the
+//            time the savestate was created are cleared.
+// version 2: For reverse-replay it is important that snapshots contain the
+//            full state of the MSX keyboard, so now we do serialize it.
+// TODO Is the assumption in version 1 correct (clear keyb state on load)?
+//      If it is still useful for 'regular' loadstate, then we could implement
+//      it by explicitly clearing the keyb state from the actual loadstate
+//      command. (But let's only do this when experience shows it's really
+//      better).
 template<typename Archive>
-void Keyboard::serialize(Archive& ar, unsigned /*version*/)
+void Keyboard::serialize(Archive& ar, unsigned version)
 {
 	ar.serialize("keyTypeCmd", *keyTypeCmd);
 	ar.serialize("cmdKeyMatrix", cmdKeyMatrix);
@@ -1154,11 +1272,13 @@ void Keyboard::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("msxCodeKanaLockOn", msxCodeKanaLockOn);
 	ar.serialize("msxGraphLockOn", msxGraphLockOn);
 
-	// don't serialize:
-	//   userKeyMatrix
-	//   dynKeymap
-	//   msxmodifiers
-	//   msxKeyEventQueue
+	if (ar.versionAtLeast(version, 2)) {
+		ar.serialize("userKeyMatrix", userKeyMatrix);
+		ar.serialize("dynKeymap", dynKeymap);
+		ar.serialize("msxmodifiers", msxmodifiers);
+		ar.serialize("msxKeyEventQueue", *msxKeyEventQueue);
+	}
+	// don't serialize hostKeyMatrix
 
 	if (ar.isLoader()) {
 		// force recalculation of keyMatrix
@@ -1168,6 +1288,36 @@ void Keyboard::serialize(Archive& ar, unsigned /*version*/)
 }
 INSTANTIATE_SERIALIZE_METHODS(Keyboard);
 
+template<typename Archive>
+void MsxKeyEventQueue::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Schedulable>(*this);
+
+	// serialization of deque<shared_ptr<const Event> > is not directly
+	// supported by the serialization framework (main problem is the
+	// constness, collections of shared_ptr to polymorhpic objects are
+	// not a problem). Worked around this by serializing the events in
+	// ascii format. (In all practical cases this queue will anyway be
+	// empty or contain very few elements).
+	//ar.serialize("eventQueue", eventQueue);
+	vector<string> eventStrs;
+	if (!ar.isLoader()) {
+		for (std::deque<shared_ptr<const Event> >::const_iterator it =
+		       eventQueue.begin(); it != eventQueue.end(); ++it) {
+			eventStrs.push_back((*it)->toString());
+		}
+	}
+	ar.serialize("eventQueue", eventStrs);
+	if (ar.isLoader()) {
+		assert(eventQueue.empty());
+		for (vector<string>::const_iterator it = eventStrs.begin();
+		     it != eventStrs.end(); ++it) {
+			eventQueue.push_back(InputEventFactory::createInputEvent(*it));
+		}
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(MsxKeyEventQueue);
+
 
 /** Keyboard bindings ****************************************/
 
@@ -1201,7 +1351,7 @@ INSTANTIATE_SERIALIZE_METHODS(Keyboard);
 //       +-----+-----+-----+-----+-----+-----+-----+-----+
 
 // Mapping from SDL keys to MSX keys
-byte Keyboard::keyTab[MAX_KEYSYM][2] = {
+const byte Keyboard::keyTab[MAX_KEYSYM][2] = {
 /* 0000 */
   {0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00},
   {7,0x20},{7,0x08},{0,0x00},{0,0x00},{0,0x00},{7,0x80},{0,0x00},{0,0x00},
