@@ -1,39 +1,77 @@
-// $Id: Mouse.cc 9468 2009-04-05 10:52:59Z manuelbi $
+// $Id: Mouse.cc 11148 2010-01-13 16:14:45Z m9710797 $
 
 #include "Mouse.hh"
 #include "MSXEventDistributor.hh"
+#include "StateChangeDistributor.hh"
 #include "InputEvents.hh"
+#include "StateChange.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
+#include "unreachable.hh"
 
 using std::string;
+using std::min;
+using std::max;
 
 namespace openmsx {
 
-const int TRESHOLD = 2;
-const int SCALE = 2;
-const int FAZE_XHIGH = 0;
-const int FAZE_XLOW  = 1;
-const int FAZE_YHIGH = 2;
-const int FAZE_YLOW  = 3;
-const int STROBE = 0x04;
+static const int TRESHOLD = 2;
+static const int SCALE = 2;
+static const int MAX_POS =  127 * SCALE;
+static const int MIN_POS = -128 * SCALE;
+static const int FAZE_XHIGH = 0;
+static const int FAZE_XLOW  = 1;
+static const int FAZE_YHIGH = 2;
+static const int FAZE_YLOW  = 3;
+static const int STROBE = 0x04;
 
 
-Mouse::Mouse(MSXEventDistributor& eventDistributor_)
+class MouseState: public StateChange
+{
+public:
+	MouseState() {} // for serialize
+	MouseState(EmuTime::param time, int deltaX_, int deltaY_,
+	           byte press_, byte release_)
+		: StateChange(time)
+		, deltaX(deltaX_), deltaY(deltaY_)
+		, press(press_), release(release_) {}
+	int  getDeltaX()  const { return deltaX; }
+	int  getDeltaY()  const { return deltaY; }
+	byte getPress()   const { return press; }
+	byte getRelease() const { return release; }
+	template<typename Archive> void serialize(Archive& ar, unsigned /*version*/)
+	{
+		ar.template serializeBase<StateChange>(*this);
+		ar.serialize("deltaX", deltaX);
+		ar.serialize("deltaY", deltaY);
+		ar.serialize("press", press);
+		ar.serialize("release", release);
+	}
+private:
+	int deltaX, deltaY;
+	byte press, release;
+};
+
+REGISTER_POLYMORPHIC_CLASS(StateChange, MouseState, "MouseState");
+
+Mouse::Mouse(MSXEventDistributor& eventDistributor_,
+             StateChangeDistributor& stateChangeDistributor_)
 	: eventDistributor(eventDistributor_)
+	, stateChangeDistributor(stateChangeDistributor_)
 	, lastTime(EmuTime::zero)
 {
 	status = JOY_BUTTONA | JOY_BUTTONB;
 	faze = FAZE_YLOW;
 	xrel = yrel = curxrel = curyrel = 0;
 	mouseMode = true;
-	eventDistributor.registerEventListener(*this);
 }
 
 Mouse::~Mouse()
 {
-	eventDistributor.unregisterEventListener(*this);
+	if (isPluggedIn()) {
+		Mouse::unplugHelper(EmuTime::dummy());
+	}
 }
 
 
@@ -60,10 +98,19 @@ void Mouse::plugHelper(Connector& /*connector*/, EmuTime::param time)
 		// left mouse button pressed, joystick emulation mode
 		mouseMode = false;
 	}
+	plugHelper2();
+}
+
+void Mouse::plugHelper2()
+{
+	eventDistributor.registerEventListener(*this);
+	stateChangeDistributor.registerListener(*this);
 }
 
 void Mouse::unplugHelper(EmuTime::param /*time*/)
 {
+	stateChangeDistributor.unregisterListener(*this);
+	eventDistributor.unregisterEventListener(*this);
 }
 
 
@@ -81,8 +128,7 @@ byte Mouse::read(EmuTime::param /*time*/)
 		case FAZE_YLOW:
 			return  ((yrel / SCALE)       & 0x0F) | status;
 		default:
-			assert(false);
-			return status; // avoid warning
+			UNREACHABLE; return 0;
 		}
 	} else {
 		emulateJoystick();
@@ -176,19 +222,20 @@ void Mouse::write(byte value, EmuTime::param time)
 }
 
 
-// EventListener
-void Mouse::signalEvent(shared_ptr<const Event> event, EmuTime::param /*time*/)
+// MSXEventListener
+void Mouse::signalEvent(shared_ptr<const Event> event, EmuTime::param time)
 {
 	switch (event->getType()) {
 	case OPENMSX_MOUSE_MOTION_EVENT: {
-		const MouseMotionEvent& motionEvent =
+		const MouseMotionEvent& mev =
 			checked_cast<const MouseMotionEvent&>(*event);
-		curxrel -= motionEvent.getX();
-		curyrel -= motionEvent.getY();
-		if (curxrel >  127 * SCALE) curxrel =  127 * SCALE;
-		if (curxrel < -128 * SCALE) curxrel = -128 * SCALE;
-		if (curyrel >  127 * SCALE) curyrel =  127 * SCALE;
-		if (curyrel < -128 * SCALE) curyrel = -128 * SCALE;
+		int newX = max(MIN_POS, min(MAX_POS, curxrel - mev.getX()));
+		int newY = max(MIN_POS, min(MAX_POS, curyrel - mev.getY()));
+		int deltaX = newX - curxrel;
+		int deltaY = newY - curyrel;
+		if (deltaX || deltaY) {
+			createMouseStateChange(time, deltaX, deltaY, 0, 0);
+		}
 		break;
 	}
 	case OPENMSX_MOUSE_BUTTON_DOWN_EVENT: {
@@ -196,10 +243,10 @@ void Mouse::signalEvent(shared_ptr<const Event> event, EmuTime::param /*time*/)
 			checked_cast<const MouseButtonEvent&>(*event);
 		switch (buttonEvent.getButton()) {
 		case MouseButtonEvent::LEFT:
-			status &= ~JOY_BUTTONA;
+			createMouseStateChange(time, 0, 0, JOY_BUTTONA, 0);
 			break;
 		case MouseButtonEvent::RIGHT:
-			status &= ~JOY_BUTTONB;
+			createMouseStateChange(time, 0, 0, JOY_BUTTONB, 0);
 			break;
 		default:
 			// ignore other buttons
@@ -212,10 +259,10 @@ void Mouse::signalEvent(shared_ptr<const Event> event, EmuTime::param /*time*/)
 			checked_cast<const MouseButtonEvent&>(*event);
 		switch (buttonEvent.getButton()) {
 		case MouseButtonEvent::LEFT:
-			status |= JOY_BUTTONA;
+			createMouseStateChange(time, 0, 0, 0, JOY_BUTTONA);
 			break;
 		case MouseButtonEvent::RIGHT:
-			status |= JOY_BUTTONB;
+			createMouseStateChange(time, 0, 0, 0, JOY_BUTTONB);
 			break;
 		default:
 			// ignore other buttons
@@ -229,18 +276,54 @@ void Mouse::signalEvent(shared_ptr<const Event> event, EmuTime::param /*time*/)
 	}
 }
 
+void Mouse::createMouseStateChange(
+	EmuTime::param time, int deltaX, int deltaY, byte press, byte release)
+{
+	stateChangeDistributor.distributeNew(shared_ptr<StateChange>(
+		new MouseState(time, deltaX, deltaY, press, release)));
+}
 
+void Mouse::signalStateChange(shared_ptr<StateChange> event)
+{
+	MouseState* ms = dynamic_cast<MouseState*>(event.get());
+	if (!ms) return;
+
+	curxrel += ms->getDeltaX();
+	curyrel += ms->getDeltaY();
+	status = (status & ~ms->getPress()) | ms->getRelease();
+}
+
+void Mouse::stopReplay(EmuTime::param time)
+{
+	// TODO read actual host mouse button state
+	int dx = 0 - curxrel;
+	int dy = 0 - curyrel;
+	byte release = (JOY_BUTTONA | JOY_BUTTONB) & ~status;
+	if ((dx != 0) || (dy != 0) || (release != 0)) {
+		createMouseStateChange(time, dx, dy, 0, release);
+	}
+}
+
+// version 1: Initial version, the variables curxrel, curyrel and status were
+//            not serialized.
+// version 2: Also serialize the above variables, this is required for
+//            record/replay, see comment in Keyboard.cc for more details.
 template<typename Archive>
-void Mouse::serialize(Archive& ar, unsigned /*version*/)
+void Mouse::serialize(Archive& ar, unsigned version)
 {
 	ar.serialize("lastTime", lastTime);
 	ar.serialize("faze", faze);
 	ar.serialize("xrel", xrel);
 	ar.serialize("yrel", yrel);
 	ar.serialize("mouseMode", mouseMode);
-
-	// Don't serialzie status, curxrel, curyrel.
-	// These are controlled via (mouse button/motion) events
+	if (ar.versionAtLeast(version, 2)) {
+		ar.serialize("curxrel", curxrel);
+		ar.serialize("curyrel", curyrel);
+		ar.serialize("status",  status);
+	}
+	if (ar.isLoader() && isPluggedIn()) {
+		plugHelper2();
+	}
 }
 INSTANTIATE_SERIALIZE_METHODS(Mouse);
 REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, Mouse, "Mouse");
