@@ -1,7 +1,8 @@
-// $Id: NinjaTap.cc 5359 2006-05-06 18:00:30Z m9710797 $
+// $Id: NinjaTap.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "NinjaTap.hh"
 #include "JoystickPort.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
@@ -22,12 +23,12 @@ const std::string& NinjaTap::getDescription() const
 	return desc;
 }
 
-byte NinjaTap::read(const EmuTime& /*time*/)
+byte NinjaTap::read(EmuTime::param /*time*/)
 {
 	return status;
 }
 
-void NinjaTap::write(byte value, const EmuTime& time)
+void NinjaTap::write(byte value, EmuTime::param time)
 {
 	// bit 0 -> pin 6
 	// bit 1 -> pin 7
@@ -66,4 +67,16 @@ void NinjaTap::write(byte value, const EmuTime& time)
 	previous = value;
 }
 
+
+template<typename Archive>
+void NinjaTap::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<JoyTap>(*this);
+	ar.serialize("status", status);
+	ar.serialize("previous", previous);
+	ar.serialize("buf", buf);
+}
+INSTANTIATE_SERIALIZE_METHODS(NinjaTap);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, NinjaTap, "NinjaTap");
+
 } // namespace openmsx
