@@ -1,35 +1,39 @@
-// $Id: RecordedCommand.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: RecordedCommand.cc 10762 2009-11-08 22:20:19Z manuelbi $
 
 #include "RecordedCommand.hh"
 #include "CommandController.hh"
-#include "MSXEventDistributor.hh"
+#include "StateChangeDistributor.hh"
 #include "TclObject.hh"
 #include "Scheduler.hh"
-#include "InputEvents.hh"
+#include "StateChange.hh"
 #include "ScopedAssign.hh"
+#include "serialize.hh"
+#include "serialize_stl.hh"
 #include "checked_cast.hh"
+#include "unreachable.hh"
 
 using std::vector;
 using std::string;
 
 namespace openmsx {
 
+
 RecordedCommand::RecordedCommand(CommandController& commandController,
-                                 MSXEventDistributor& msxEventDistributor_,
+                                 StateChangeDistributor& stateChangeDistributor_,
                                  Scheduler& scheduler_,
                                  const string& name)
 	: Command(commandController, name)
-	, msxEventDistributor(msxEventDistributor_)
+	, stateChangeDistributor(stateChangeDistributor_)
 	, scheduler(scheduler_)
-	, dummyResultObject(new TclObject(commandController.getInterpreter()))
+	, dummyResultObject(new TclObject(getInterpreter()))
 	, currentResultObject(dummyResultObject.get())
 {
-	msxEventDistributor.registerEventListener(*this);
+	stateChangeDistributor.registerListener(*this);
 }
 
 RecordedCommand::~RecordedCommand()
 {
-	msxEventDistributor.unregisterEventListener(*this);
+	stateChangeDistributor.unregisterListener(*this);
 }
 
 void RecordedCommand::execute(const vector<TclObject*>& tokens,
@@ -38,10 +42,9 @@ void RecordedCommand::execute(const vector<TclObject*>& tokens,
 	EmuTime::param time = scheduler.getCurrentTime();
 	if (needRecord(tokens)) {
 		ScopedAssign<TclObject*> sa(currentResultObject, &result);
-		msxEventDistributor.distributeEvent(
-			MSXEventDistributor::EventPtr(
-				new MSXCommandEvent(tokens)),
-			time);
+		stateChangeDistributor.distributeNew(
+			StateChangeDistributor::EventPtr(
+				new MSXCommandEvent(tokens, time)));
 	} else {
 		execute(tokens, result, time);
 	}
@@ -69,16 +72,21 @@ static string getBaseName(const std::string& str)
 	return (pos == string::npos) ? str : str.substr(pos + 2);
 }
 
-void RecordedCommand::signalEvent(
-	shared_ptr<const Event> event, EmuTime::param time)
+void RecordedCommand::signalStateChange(shared_ptr<StateChange> event)
 {
-	if (event->getType() != OPENMSX_MSX_COMMAND_EVENT) return;
-	const MSXCommandEvent* commandEvent =
-		checked_cast<const MSXCommandEvent*>(event.get());
+	MSXCommandEvent* commandEvent =
+		dynamic_cast<MSXCommandEvent*>(event.get());
+	if (!commandEvent) return;
+
 	const vector<TclObject*>& tokens = commandEvent->getTokens();
 	if (getBaseName(tokens[0]->getString()) != getName()) return;
 
-	execute(tokens, *currentResultObject, time);
+	execute(tokens, *currentResultObject, commandEvent->getTime());
+}
+
+void RecordedCommand::stopReplay(EmuTime::param /*time*/)
+{
+	// nothing
 }
 
 void RecordedCommand::execute(const vector<TclObject*>& tokens,
@@ -98,9 +106,65 @@ string RecordedCommand::execute(const vector<string>& /*tokens*/,
 {
 	// either this method or the method above should be reimplemented
 	// by the subclasses
-	assert(false);
-	// avoid warning:
-	return string("");
+	UNREACHABLE; return "";
 }
 
+
+// class MSXCommandEvent
+
+MSXCommandEvent::MSXCommandEvent(const vector<string>& tokens_, EmuTime::param time)
+	: StateChange(time)
+{
+	for (vector<string>::const_iterator it = tokens_.begin();
+	     it != tokens_.end(); ++it) {
+		tokens.push_back(new TclObject(*it));
+	}
+}
+
+MSXCommandEvent::MSXCommandEvent(const vector<TclObject*>& tokens_, EmuTime::param time)
+	: StateChange(time)
+{
+	for (vector<TclObject*>::const_iterator it = tokens_.begin();
+	     it != tokens_.end(); ++it) {
+		tokens.push_back(new TclObject(**it));
+	}
+}
+
+MSXCommandEvent::~MSXCommandEvent()
+{
+	for (vector<TclObject*>::const_iterator it = tokens.begin();
+	     it != tokens.end(); ++it) {
+		delete *it;
+	}
+}
+
+const vector<TclObject*>& MSXCommandEvent::getTokens() const
+{
+	return tokens;
+}
+
+template<typename Archive>
+void MSXCommandEvent::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<StateChange>(*this);
+
+	// serialize vector<TclObject*> as vector<string>
+	vector<string> str;
+	if (!ar.isLoader()) {
+		for (vector<TclObject*>::const_iterator it = tokens.begin();
+		     it != tokens.end(); ++it) {
+			str.push_back((*it)->getString());
+		}
+	}
+	ar.serialize("tokens", str);
+	if (ar.isLoader()) {
+		assert(tokens.empty());
+		for (vector<string>::const_iterator it = str.begin();
+		     it != str.end(); ++it) {
+			tokens.push_back(new TclObject(*it));
+		}
+	}
+}
+REGISTER_POLYMORPHIC_CLASS(StateChange, MSXCommandEvent, "MSXCommandEvent");
+
 } // namespace openmsx
