@@ -1,4 +1,4 @@
-// $Id: ESE_RAM.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: ESE_RAM.cc 10392 2009-08-18 19:56:02Z m9710797 $
 
 /*
  * MEGA-SCSI and ESE-RAM cartridge:
@@ -37,9 +37,10 @@ static SRAM* createSRAM(MSXMotherBoard& motherBoard, const XMLElement& config,
 {
 	unsigned sramSize = config.getChildDataAsInt("sramsize", 256); // size in kb
 	if (sramSize != 1024 && sramSize != 512 && sramSize != 256 && sramSize != 128) {
-		throw MSXException("SRAM size for " + name +
-			" should be 128, 256, 512 or 1024kB and not " +
-			StringOp::toString(sramSize) + "kB!");
+		throw MSXException(StringOp::Builder() <<
+			"SRAM size for " << name <<
+			" should be 128, 256, 512 or 1024kB and not " <<
+			sramSize << "kB!");
 	}
 	sramSize *= 1024; // in bytes
 	return new SRAM(motherBoard, name + " SRAM", sramSize, config);
