@@ -1,35 +1,33 @@
-// $Id: MSXPac.cc 5751 2006-10-05 19:34:54Z m9710797 $
+// $Id: MSXPac.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 #include "MSXPac.hh"
 #include "SRAM.hh"
-#include "MSXCPU.hh"
-#include "MSXMotherBoard.hh"
 #include "CacheLine.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 static const char* const PAC_Header = "PAC2 BACKUP DATA";
 
-MSXPac::MSXPac(MSXMotherBoard& motherBoard, const XMLElement& config,
-               const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+MSXPac::MSXPac(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, sram(new SRAM(motherBoard, getName() + " SRAM", 0x1FFE, config,
 	                PAC_Header))
 {
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
 MSXPac::~MSXPac()
 {
 }
 
-void MSXPac::reset(const EmuTime& /*time*/)
+void MSXPac::reset(EmuTime::param /*time*/)
 {
 	sramEnabled = false;
-	r1ffe = r1fff = 0xFF;	// TODO check
+	r1ffe = r1fff = 0xFF; // TODO check
 }
 
-byte MSXPac::readMem(word address, const EmuTime& /*time*/)
+byte MSXPac::readMem(word address, EmuTime::param /*time*/)
 {
 	byte result;
 	address &= 0x3FFF;
@@ -66,7 +64,7 @@ const byte* MSXPac::getReadCacheLine(word address) const
 	}
 }
 
-void MSXPac::writeMem(word address, byte value, const EmuTime& /*time*/)
+void MSXPac::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	//PRT_DEBUG("PAC write "<<hex<<(int)address<<" "<<(int)value<<dec);
 	address &= 0x3FFF;
@@ -104,8 +102,22 @@ void MSXPac::checkSramEnable()
 	bool newEnabled = (r1ffe == 0x4D) && (r1fff == 0x69);
 	if (sramEnabled != newEnabled) {
 		sramEnabled = newEnabled;
-		getMotherBoard().getCPU().invalidateMemCache(0x0000, 0x10000);
+		invalidateMemCache(0x0000, 0x10000);
 	}
 }
 
+template<typename Archive>
+void MSXPac::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.serialize("SRAM", *sram);
+	ar.serialize("r1ffe", r1ffe);
+	ar.serialize("r1fff", r1fff);
+	if (ar.isLoader()) {
+		checkSramEnable();
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXPac);
+REGISTER_MSXDEVICE(MSXPac, "PAC");
+
 } // namespace openmsx
