@@ -1,25 +1,28 @@
-// $Id: PanasonicMemory.cc 6366 2007-04-24 19:05:58Z m9710797 $
+// $Id: PanasonicMemory.cc 8219 2008-09-13 07:17:42Z m9710797 $
 
 #include "PanasonicMemory.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXCPU.hh"
 #include "Ram.hh"
 #include "Rom.hh"
-#include "MachineConfig.hh"
+#include "HardwareConfig.hh"
 #include "XMLElement.hh"
 #include "MSXException.hh"
 
 namespace openmsx {
 
-static const XMLElement& getRomConfig(MSXMotherBoard& motherBoard)
+static Rom* createRom(MSXMotherBoard& motherBoard)
 {
-	return motherBoard.getMachineConfig().getConfig().getChild("PanasonicRom");
+	const XMLElement* elem = motherBoard.getMachineConfig()->
+	                      getConfig().findChild("PanasonicRom");
+	return elem ? new Rom(motherBoard, "PanasonicRom",
+	                      "Turbor-R main ROM", *elem)
+	            : NULL;
 }
 
 PanasonicMemory::PanasonicMemory(MSXMotherBoard& motherBoard)
 	: msxcpu(motherBoard.getCPU())
-	, rom(new Rom(motherBoard, "PanasonicRom", "Turbo-R main ROM",
-	              getRomConfig(motherBoard)))
+	, rom(createRom(motherBoard))
 	, ram(NULL), dram(false)
 {
 }
@@ -36,6 +39,9 @@ void PanasonicMemory::registerRam(Ram& ram_)
 
 const byte* PanasonicMemory::getRomBlock(unsigned block)
 {
+	if (!rom.get()) {
+		throw MSXException("Missing PanasonicRom.");
+	}
 	if (dram &&
 	    (((0x28 <= block) && (block < 0x2C)) ||
 	     ((0x38 <= block) && (block < 0x3C)))) {
@@ -55,6 +61,9 @@ const byte* PanasonicMemory::getRomBlock(unsigned block)
 
 const byte* PanasonicMemory::getRomRange(unsigned first, unsigned last)
 {
+	if (!rom.get()) {
+		throw MSXException("Missing PanasonicRom.");
+	}
 	if (last < first) {
 		throw MSXException("Error in config file: firstblock must "
 		                   "be smaller than lastblock");
@@ -74,13 +83,20 @@ const byte* PanasonicMemory::getRomRange(unsigned first, unsigned last)
 
 byte* PanasonicMemory::getRamBlock(unsigned block)
 {
-	assert(ram);
+	if (!ram) return NULL;
+
 	unsigned offset = block * 0x2000;
 	if (offset >= ramSize) {
 		offset &= ramSize - 1;
 	}
 	return ram + offset;
 }
+unsigned PanasonicMemory::getRamSize() const
+{
+	if (!ram) return 0;
+
+	return ramSize;
+}
 
 void PanasonicMemory::setDRAM(bool dram_)
 {
