@@ -1,21 +1,22 @@
-// $Id: PanasonicRam.cc 5232 2006-03-10 12:39:49Z m9710797 $
+// $Id: PanasonicRam.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "PanasonicRam.hh"
 #include "MSXMotherBoard.hh"
 #include "PanasonicMemory.hh"
 #include "CheckedRam.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 PanasonicRam::PanasonicRam(MSXMotherBoard& motherBoard,
-                           const XMLElement& config, const EmuTime& time)
-	: MSXMemoryMapper(motherBoard, config, time)
+                           const XMLElement& config)
+	: MSXMemoryMapper(motherBoard, config)
 	, panasonicMemory(motherBoard.getPanasonicMemory())
 {
 	panasonicMemory.registerRam(checkedRam->getUncheckedRam());
 }
 
-void PanasonicRam::writeMem(word address, byte value, const EmuTime& /*time*/)
+void PanasonicRam::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	unsigned addr = calcAddress(address);
 	if (panasonicMemory.isWritable(addr)) {
@@ -33,4 +34,12 @@ byte* PanasonicRam::getWriteCacheLine(word start) const
 	}
 }
 
+template<typename Archive>
+void PanasonicRam::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXMemoryMapper>(*this);
+}
+INSTANTIATE_SERIALIZE_METHODS(PanasonicRam);
+REGISTER_MSXDEVICE(PanasonicRam, "PanasonicRam");
+
 } // namespace openmsx
