@@ -1,4 +1,4 @@
-// $Id: Rom.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: Rom.cc 11011 2009-12-23 11:07:47Z manuelbi $
 
 #include "Rom.hh"
 #include "XMLElement.hh"
@@ -9,6 +9,7 @@
 #include "FileException.hh"
 #include "PanasonicMemory.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "Debugger.hh"
 #include "Debuggable.hh"
 #include "CliComm.hh"
@@ -41,33 +42,33 @@ private:
 };
 
 
-Rom::Rom(MSXMotherBoard& motherBoard_, const string& name_,
+Rom::Rom(MSXMotherBoard& motherBoard, const string& name_,
          const string& description_, const XMLElement& config)
-	: motherBoard(motherBoard_), name(name_), description(description_)
+	: name(name_), description(description_)
 {
-	init(motherBoard.getGlobalCliComm(), config.getChild("rom"));
+	init(motherBoard, config.getChild("rom"));
 }
 
-Rom::Rom(MSXMotherBoard& motherBoard_, const string& name_,
+Rom::Rom(MSXMotherBoard& motherBoard, const string& name_,
          const string& description_, const XMLElement& config,
          const string& id)
-	: motherBoard(motherBoard_), name(name_), description(description_)
+	: name(name_), description(description_)
 {
 	XMLElement::Children romConfigs;
 	config.getChildren("rom", romConfigs);
 	for (XMLElement::Children::const_iterator it = romConfigs.begin();
 	     it != romConfigs.end(); ++it) {
 		if ((*it)->getId() == id) {
-			init(motherBoard.getGlobalCliComm(), **it);
+			init(motherBoard, **it);
 			return;
 		}
 	}
 	throw ConfigException("ROM tag \"" + id + "\" missing.");
 }
 
-void Rom::init(CliComm& cliComm, const XMLElement& config)
+void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config)
 {
-	CommandController& controller = getMotherBoard().getCommandController();
+	CommandController& controller = motherBoard.getCommandController();
 
 	XMLElement::Children sums;
 	config.getChildren("sha1", sums);
@@ -114,6 +115,13 @@ void Rom::init(CliComm& cliComm, const XMLElement& config)
 			}
 		}
 		read(config);
+		// verify SHA1
+		if (!checkSHA1(config)) {
+			motherBoard.getMSXCliComm().printWarning(
+				"SHA1 sum for '" + config.getId() +
+				"' does not match with sum of '" +
+				file->getURL() + "'.");
+		}
 
 	} else if (config.findChild("firstblock")) {
 		// part of the TurboR main ROM
@@ -163,11 +171,11 @@ void Rom::init(CliComm& cliComm, const XMLElement& config)
 			patchedSha1 = SHA1::calc(rom, size);
 		}
 	}
-	info = RomDatabase::instance().fetchRomInfo(cliComm, *this);
 
-	// TODO fix this, this is a hack that depends heavily on MSXRomCLI.cc
-	if (!info->getTitle().empty() && StringOp::startsWith(name, "MSXRom")) {
-		name = info->getTitle();
+	// TODO fix this, this is a hack that depends heavily on HardwareConig::createRomConfig
+	const RomInfo* romInfo = motherBoard.getReactor().getSoftwareDatabase().fetchRomInfo(getOriginalSHA1());
+	if ((romInfo != NULL) && !romInfo->getTitle().empty() && StringOp::startsWith(name, "MSXRom")) {
+		name = romInfo->getTitle();
 	}
 
 	if (size) {
@@ -176,7 +184,7 @@ void Rom::init(CliComm& cliComm, const XMLElement& config)
 			unsigned n = 0;
 			string tmp;
 			do {
-				tmp = name + " (" + StringOp::toString(++n) + ")";
+				tmp = StringOp::Builder() << name << " (" << ++n << ')';
 			} while (debugger.findDebuggable(tmp));
 			name = tmp;
 		}
@@ -227,14 +235,6 @@ void Rom::read(const XMLElement& config)
 		throw MSXException("Error reading ROM image: " +
 		                   file->getURL());
 	}
-
-	// verify SHA1
-	if (!checkSHA1(config)) {
-		motherBoard.getMSXCliComm().printWarning(
-			"SHA1 sum for '" + config.getId() +
-			"' does not match with sum of '" +
-			file->getURL() + "'.");
-	}
 }
 
 bool Rom::checkSHA1(const XMLElement& config)
@@ -258,11 +258,6 @@ Rom::~Rom()
 {
 }
 
-const RomInfo& Rom::getInfo() const
-{
-	return *info;
-}
-
 const string& Rom::getName() const
 {
 	return name;
@@ -288,11 +283,6 @@ const string& Rom::getPatchedSHA1() const
 	return patchedSha1;
 }
 
-MSXMotherBoard& Rom::getMotherBoard() const
-{
-	return motherBoard;
-}
-
 
 RomDebuggable::RomDebuggable(Debugger& debugger_, Rom& rom_)
 	: debugger(debugger_), rom(rom_)
