@@ -1,4 +1,4 @@
-// $Id: RomAscii16kB.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomAscii16kB.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // ASCII 16kB cartridges
 //
@@ -26,10 +26,10 @@ RomAscii16kB::RomAscii16kB(
 
 void RomAscii16kB::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setRom (1, 0);
-	setRom (2, 0);
-	setBank(3, unmappedRead);
+	setUnmapped(0);
+	setRom(1, 0);
+	setRom(2, 0);
+	setUnmapped(3);
 }
 
 void RomAscii16kB::writeMem(word address, byte value, EmuTime::param /*time*/)
