@@ -1,26 +1,62 @@
-// $Id: RomBlocks.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: RomBlocks.cc 11148 2010-01-13 16:14:45Z m9710797 $
 
 #include "RomBlocks.hh"
 #include "Rom.hh"
 #include "SRAM.hh"
+#include "SimpleDebuggable.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
 #include "serialize.hh"
+#include "unreachable.hh"
 
 namespace openmsx {
 
+class RomBlockDebuggable : public SimpleDebuggable
+{
+public:
+	RomBlockDebuggable(MSXRom& rom, const byte* blockNr_,
+	                   unsigned bankSizeShift_, unsigned debugShift_)
+		: SimpleDebuggable(rom.getMotherBoard(),
+		                   rom.getName() + " romblocks",
+		                   "Shows for each byte of the mapper which memory block is selected.",
+		                   0x10000)
+		, blockNr(blockNr_), bankSizeShift(bankSizeShift_)
+		, debugShift(debugShift_), debugMask(~((1 << debugShift) - 1))
+	{
+	}
+
+	virtual byte read(unsigned address)
+	{
+		byte tmp = blockNr[(address >> bankSizeShift) & debugMask];
+		return (tmp != 255) ? (tmp >> debugShift) : tmp;
+	}
+
+private:
+	const byte* blockNr;
+	const unsigned bankSizeShift;
+	const unsigned debugShift;
+	const unsigned debugMask;
+};
+
+template<bool C, class T, class F> struct if_log2_             : F {};
+template<        class T, class F> struct if_log2_<true, T, F> : T {};
+template<int I> struct int_ { static const int value = I; };
+template<unsigned A, unsigned R = 0> struct log2
+	: if_log2_<A == 1, int_<R>, log2<A / 2, R + 1> > {};
+
 template <unsigned BANK_SIZE>
 RomBlocks<BANK_SIZE>::RomBlocks(
 		MSXMotherBoard& motherBoard, const XMLElement& config,
-		std::auto_ptr<Rom> rom_)
+		std::auto_ptr<Rom> rom_, unsigned debugBankSizeShift)
 	: MSXRom(motherBoard, config, rom_)
+	, romBlockDebug(new RomBlockDebuggable(*this, blockNr,
+	                         log2<BANK_SIZE>::value, debugBankSizeShift))
 	, nrBlocks(rom->getSize() / BANK_SIZE)
 {
 	if ((nrBlocks * BANK_SIZE) != rom->getSize()) {
-		throw MSXException(
-			"(uncompressed) ROM image filesize must be a multiple of " +
-			StringOp::toString(BANK_SIZE / 1024) + " kB (for this "
-			"mapper type).");
+		throw MSXException(StringOp::Builder() <<
+			"(uncompressed) ROM image filesize must be a multiple of " <<
+			BANK_SIZE / 1024 << " kB (for this mapper type).");
 	}
 	// by default no extra mappable memory block
 	extraMem = NULL;
@@ -51,7 +87,7 @@ const byte* RomBlocks<BANK_SIZE>::getReadCacheLine(word address) const
 }
 
 template <unsigned BANK_SIZE>
-void RomBlocks<BANK_SIZE>::setBank(byte region, const byte* adr)
+void RomBlocks<BANK_SIZE>::setBank(byte region, const byte* adr, int block)
 {
 	assert("address passed to setBank() is not serializable" &&
 	       ((adr == unmappedRead) ||
@@ -60,9 +96,16 @@ void RomBlocks<BANK_SIZE>::setBank(byte region, const byte* adr)
 	                       (adr <= &(*sram)[sram->getSize() - 1])) ||
 	        ((extraMem <= adr) && (adr <= &extraMem[extraSize - 1]))));
 	bank[region] = adr;
+	blockNr[region] = block; // only for debuggable
 	invalidateMemCache(region * BANK_SIZE, BANK_SIZE);
 }
 
+template <unsigned BANK_SIZE>
+void RomBlocks<BANK_SIZE>::setUnmapped(byte region)
+{
+	setBank(region, unmappedRead, 255);
+}
+
 template <unsigned BANK_SIZE>
 void RomBlocks<BANK_SIZE>::setBlockMask(int mask)
 {
@@ -83,12 +126,14 @@ void RomBlocks<BANK_SIZE>::setRom(byte region, int block)
 	//       for those we have to make an exception for "block < nrBlocks".
 	block = (block < nrBlocks) ? block : block & blockMask;
 	if (block < nrBlocks) {
-		setBank(region, &(*rom)[block * BANK_SIZE]);
+		setBank(region, &(*rom)[block * BANK_SIZE], block);
 	} else {
-		setBank(region, unmappedRead);
+		setBank(region, unmappedRead, 255);
 	}
 }
 
+// version 1: initial version
+// version 2: added blockNr
 template <unsigned BANK_SIZE>
 template<typename Archive>
 void RomBlocks<BANK_SIZE>::serialize(Archive& ar, unsigned /*version*/)
@@ -116,7 +161,8 @@ void RomBlocks<BANK_SIZE>::serialize(Archive& ar, unsigned /*version*/)
 			} else if (offsets[i] < (romSize + sramSize + extraSize)) {
 				bank[i] = &extraMem[offsets[i] - romSize - sramSize];
 			} else {
-				assert(false); // TODO throw
+				// TODO throw
+				UNREACHABLE;
 			}
 		}
 	} else {
@@ -134,11 +180,23 @@ void RomBlocks<BANK_SIZE>::serialize(Archive& ar, unsigned /*version*/)
 			           (bank[i] <= &extraMem[extraSize - 1])) {
 				offsets[i] = unsigned(bank[i] - extraMem + romSize + sramSize);
 			} else {
-				assert(false);
+				UNREACHABLE;
 			}
 		}
 		ar.serialize("banks", offsets);
 	}
+
+	// Commented out because versioning doesn't work correct on subclasses
+	// that don't override the serialize() method (e.g. RomPlain)
+	/*if (ar.versionAtLeast(version, 2)) {
+		ar.serialize("blockNr", blockNr);
+	} else {
+		assert(ar.isLoader());
+		// set dummy value, anyway only used for debuggable
+		for (unsigned i = 0; i < NUM_BANKS; ++i) {
+			blockNr[i] = 255;
+		}
+	}*/
 }
 
 template class RomBlocks<0x1000>;
