@@ -1,19 +1,20 @@
-// $Id: RomDatabase.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: RomDatabase.cc 11122 2010-01-10 13:49:02Z manuelbi $
 
 #include "RomDatabase.hh"
 #include "RomInfo.hh"
 #include "Rom.hh"
+#include "InfoTopic.hh"
+#include "CommandException.hh"
+#include "TclObject.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "FileOperations.hh"
 #include "LocalFileReference.hh"
+#include "GlobalCommandController.hh"
 #include "CliComm.hh"
-#include "StringOp.hh"
-#include "StringPool.hh"
 #include "XMLLoader.hh"
 #include "XMLElement.hh"
 #include "XMLException.hh"
-#include <map>
 #include <cassert>
 
 using std::auto_ptr;
@@ -22,38 +23,26 @@ using std::set;
 using std::string;
 using std::vector;
 
-// Note: Using something like boost::shared_ptr would simplify memory management
-//       but it does add another dependency. So atm we don't use it yet.
-
 namespace openmsx {
 
-typedef map<string, RomInfo*, StringOp::caseless> DBMap;
-static DBMap romDBSHA1;
-typedef map<string, unsigned> UnknownTypes;
-UnknownTypes unknownTypes;
-
-RomDatabase::RomDatabase()
+class SoftwareInfoTopic : public InfoTopic
 {
-	// make sure StringPool has longer lifetime than RomDatabase
-	// (needed because of lazy initialization of RomDatabase)
-	StringPool::instance();
-}
+public:
+        SoftwareInfoTopic(InfoCommand& openMSXInfoCommand, RomDatabase& romDatabase);
 
-RomDatabase::~RomDatabase()
-{
-	for (DBMap::const_iterator it = romDBSHA1.begin();
-	     it != romDBSHA1.end(); ++it) {
-		delete it->second;
-	}
-}
+        virtual void execute(const std::vector<TclObject*>& tokens,
+                             TclObject& result) const;
+        virtual std::string help(const std::vector<std::string>& tokens) const;
+        virtual void tabCompletion(std::vector<std::string>& tokens) const;
 
-RomDatabase& RomDatabase::instance()
-{
-	static RomDatabase oneInstance;
-	return oneInstance;
-}
+private:
+	const RomDatabase& romDatabase;
+};
 
 
+
+typedef std::map<std::string, unsigned> UnknownTypes;
+
 static string parseRemarks(const XMLElement& elem)
 {
 	string result;
@@ -74,7 +63,7 @@ static string parseRemarks(const XMLElement& elem)
 }
 
 static void addEntry(CliComm& cliComm, auto_ptr<RomInfo> romInfo,
-                     const string& sha1, DBMap& result)
+                     const string& sha1, RomDatabase::DBMap& result)
 {
 	assert(romInfo.get());
 	if (result.find(sha1) == result.end()) {
@@ -85,12 +74,13 @@ static void addEntry(CliComm& cliComm, auto_ptr<RomInfo> romInfo,
 	}
 }
 
-static void parseEntry(CliComm& cliComm,
-	const XMLElement& rom, DBMap& result,
+static void parseEntry(CliComm& cliComm, const XMLElement& rom,
+	RomDatabase::DBMap& result, UnknownTypes& unknownTypes,
 	const string& title,   const string& year,
 	const string& company, const string& country,
 	bool original,         const string& origType,
-	const string& remark,  const string& type)
+	const string& remark,  const string& type,
+	int genMSXid)
 {
 	XMLElement::Children hashTags;
 	rom.getChildren("hash", hashTags);
@@ -105,7 +95,7 @@ static void parseEntry(CliComm& cliComm,
 		}
 		auto_ptr<RomInfo> romInfo(new RomInfo(
 			title, year, company, country, original, origType,
-			remark, romType));
+			remark, romType, genMSXid));
 		string sha1 = (*it2)->getData();
 		addEntry(cliComm, romInfo, sha1, result);
 	}
@@ -121,21 +111,21 @@ static string parseStart(const XMLElement& rom)
 	else return "";
 }
 
-static void parseDump(CliComm& cliComm,
-	const XMLElement& dump, DBMap& result,
+static void parseDump(CliComm& cliComm, const XMLElement& dump,
+	RomDatabase::DBMap& result, UnknownTypes& unknownTypes,
 	const string& title,   const string& year,
 	const string& company, const string& country,
-	const string& remark)
+	const string& remark,  int genMSXid)
 {
-	// TODO make <original> tag not required?
-	const XMLElement& originalTag = dump.getChild("original");
-	bool original = originalTag.getAttributeAsBool("value");
-	string origType = originalTag.getData();
+	const XMLElement* originalTag = dump.findChild("original");
+	bool original = originalTag ? originalTag->getAttributeAsBool("value")
+	                            : false;
+	string origType = originalTag ? originalTag->getData() : "";
 
 	if (const XMLElement* megarom = dump.findChild("megarom")) {
-		parseEntry(cliComm, *megarom, result, title, year,
-			   company, country, original, origType,
-			   remark, megarom->getChildData("type"));
+		parseEntry(cliComm, *megarom, result, unknownTypes,
+		           title, year, company, country, original, origType,
+		           remark, megarom->getChildData("type"), genMSXid);
 	} else if (const XMLElement* rom = dump.findChild("rom")) {
 		string type = rom->getChildData("type", "Mirrored");
 		if (type == "Normal") {
@@ -143,13 +133,15 @@ static void parseDump(CliComm& cliComm,
 		} else if (type == "Mirrored") {
 			type += parseStart(*rom);
 		}
-		parseEntry(cliComm, *rom, result, title, year, company,
-		           country, original, origType, remark, type);
+		parseEntry(cliComm, *rom, result, unknownTypes,
+		           title, year, company, country, original, origType,
+		           remark, type, genMSXid);
 	}
 }
 
 static void parseSoftware(CliComm& cliComm, const string& filename,
-                          const XMLElement& soft, DBMap& result)
+                          const XMLElement& soft, RomDatabase::DBMap& result,
+                          UnknownTypes& unknownTypes)
 {
 	try {
 		const XMLElement* system = soft.findChild("system");
@@ -164,13 +156,19 @@ static void parseSoftware(CliComm& cliComm, const string& filename,
 		string company = soft.getChildData("company", "");
 		string country = soft.getChildData("country", "");
 		string remark  = parseRemarks(soft);
+		int genMSXid   = soft.getChildDataAsInt("genmsxid", 0);
+		if ((genMSXid == 0) && (soft.findChild("genmsxid") != NULL)) {
+			cliComm.printWarning("Ignoring bad Generation MSX id (genmsxid) "
+			             "in entry with title '" + title +
+		                     "' in " + filename + ": " + soft.getChildData("genmsxid"));
+		}
 
 		XMLElement::Children dumps;
 		soft.getChildren("dump", dumps);
 		for (XMLElement::Children::const_iterator it = dumps.begin();
 		     it != dumps.end(); ++it) {
-			parseDump(cliComm, **it, result, title, year,
-			          company, country, remark);
+			parseDump(cliComm, **it, result, unknownTypes,
+			          title, year, company, country, remark, genMSXid);
 		}
 	} catch (MSXException& e) {
 		string title = soft.getChildData("title", "<missing-title>");
@@ -180,13 +178,14 @@ static void parseSoftware(CliComm& cliComm, const string& filename,
 }
 
 static void parseDB(CliComm& cliComm, const string& filename,
-                    const XMLElement& doc, DBMap& result)
+                    const XMLElement& doc, RomDatabase::DBMap& result,
+                    UnknownTypes& unknownTypes)
 {
 	const XMLElement::Children& children = doc.getChildren();
 	for (XMLElement::Children::const_iterator it = children.begin();
 	     it != children.end(); ++it) {
 		// Parse all <software> tags
-		parseSoftware(cliComm, filename, **it, result);
+		parseSoftware(cliComm, filename, **it, result, unknownTypes);
 	}
 }
 
@@ -207,12 +206,11 @@ static auto_ptr<XMLElement> openDB(CliComm& cliComm, const string& filename,
 	return doc;
 }
 
-static void initDatabase(CliComm& cliComm)
+RomDatabase::RomDatabase(GlobalCommandController& commandController, CliComm& cliComm)
+	: softwareInfoTopic(new SoftwareInfoTopic(
+		commandController.getOpenMSXInfoCommand(), *this))
 {
-	static bool init = false;
-	if (init) return;
-	init = true;
-
+	UnknownTypes unknownTypes;
 	SystemFileContext context;
 	CommandController* controller = NULL; // ok for SystemFileContext
 	vector<string> paths = context.getPaths(*controller);
@@ -223,7 +221,7 @@ static void initDatabase(CliComm& cliComm)
 			openDB(cliComm, filename, "softwaredb1.dtd"));
 		if (doc.get()) {
 			DBMap tmp;
-			parseDB(cliComm, filename, *doc, tmp);
+			parseDB(cliComm, filename, *doc, tmp, unknownTypes);
 			for (DBMap::const_iterator it = tmp.begin();
 			     it != tmp.end(); ++it) {
 				if (romDBSHA1.find(it->first) == romDBSHA1.end()) {
@@ -242,38 +240,86 @@ static void initDatabase(CliComm& cliComm)
 			"This may cause incorrect ROM mapper types to be used.");
 	}
 	if (!unknownTypes.empty()) {
-		string output = "Unknown mapper types in software database: ";
+		StringOp::Builder output;
+		output << "Unknown mapper types in software database: ";
 		for (UnknownTypes::iterator it = unknownTypes.begin();
 		     it != unknownTypes.end(); ++it) {
-			output += it->first +
-			       " (" + StringOp::toString(it->second) + "x); ";
+			output << it->first << " (" << it->second << "x); ";
 		}
 		cliComm.printWarning(output);
 	}
 }
 
-auto_ptr<RomInfo> RomDatabase::fetchRomInfo(CliComm& cliComm, const Rom& rom)
+RomDatabase::~RomDatabase()
 {
-	// Note: RomInfo is copied only to make ownership managment easier
+	for (DBMap::const_iterator it = romDBSHA1.begin();
+	     it != romDBSHA1.end(); ++it) {
+		delete it->second;
+	}
+}
 
-	initDatabase(cliComm);
+const RomInfo* RomDatabase::fetchRomInfo(const string& sha1sum) const
+{
+	DBMap::const_iterator it = romDBSHA1.find(sha1sum);
+	if (it == romDBSHA1.end()) {
+		return NULL;
+	}
+	return it->second;
+}
 
-	if (rom.getSize() == 0) {
-		return auto_ptr<RomInfo>(
-			new RomInfo("", "", "", "", false, "", "Empty ROM",
-			            ROM_UNKNOWN));
+
+// SoftwareInfoTopic
+
+SoftwareInfoTopic::SoftwareInfoTopic(InfoCommand& openMSXInfoCommand,
+                                     RomDatabase& romDatabase_)
+	: InfoTopic(openMSXInfoCommand, "software")
+	, romDatabase(romDatabase_)
+{
+}
+
+void SoftwareInfoTopic::execute(const vector<TclObject*>& tokens,
+                                TclObject& result) const
+{
+	if (tokens.size() != 3) {
+		throw CommandException("Wrong number of parameters");
 	}
 
-	const string& sha1sum = rom.getOriginalSHA1();
-	if (romDBSHA1.find(sha1sum) != romDBSHA1.end()) {
-		romDBSHA1[sha1sum]->print(cliComm);
-		// Return a copy of the DB entry.
-		return auto_ptr<RomInfo>(new RomInfo(*romDBSHA1[sha1sum]));
+	const string& sha1sum = tokens[2]->getString();
+	const RomInfo* romInfo = romDatabase.fetchRomInfo(sha1sum);
+	if (romInfo == NULL) {
+		// no match found
+		throw CommandException("Software with sha1sum " + sha1sum + " not found");
 	}
 
-	// no match found
-	return auto_ptr<RomInfo>(new RomInfo("", "", "", "", false, "", "",
-	                                     ROM_UNKNOWN));
+	result.addListElement("title");
+	result.addListElement(romInfo->getTitle());
+	result.addListElement("year");
+	result.addListElement(romInfo->getYear());
+	result.addListElement("company");
+	result.addListElement(romInfo->getCompany());
+	result.addListElement("country");
+	result.addListElement(romInfo->getCountry());
+	result.addListElement("orig_type");
+	result.addListElement(romInfo->getOrigType());
+	result.addListElement("remark");
+	result.addListElement(romInfo->getRemark());
+	result.addListElement("original");
+	result.addListElement(romInfo->getOriginal());
+	result.addListElement("mapper_type_name");
+	result.addListElement(RomInfo::romTypeToName(romInfo->getRomType()));
+	result.addListElement("genmsxid");
+	result.addListElement(romInfo->getGenMSXid());
+}
+
+string SoftwareInfoTopic::help(const vector<string>& /*tokens*/) const
+{
+	return "Returns information about the software "
+	       "given its sha1sum, in a paired list.";
+}
+
+void SoftwareInfoTopic::tabCompletion(vector<string>& /*tokens*/) const
+{
+	// no useful completion possible
 }
 
 } // namespace openmsx
