@@ -1,4 +1,4 @@
-// $Id: RomFSA1FM.cc 9444 2009-03-31 19:41:38Z m9710797 $
+// $Id: RomFSA1FM.cc 10112 2009-06-30 18:29:35Z m9710797 $
 
 // FS-A1FM mappers
 //   based on info from:
@@ -130,7 +130,7 @@ byte RomFSA1FM1::peekMem(word address, EmuTime::param /*time*/) const
 
 byte RomFSA1FM1::readMem(word address, EmuTime::param time)
 {
-	return peekMem(address, time);
+	return RomFSA1FM1::peekMem(address, time);
 }
 
 const byte* RomFSA1FM1::getReadCacheLine(word address) const
@@ -233,7 +233,7 @@ byte RomFSA1FM2::peekMem(word address, EmuTime::param time) const
 
 byte RomFSA1FM2::readMem(word address, EmuTime::param time)
 {
-	return peekMem(address, time);
+	return RomFSA1FM2::peekMem(address, time);
 }
 
 const byte* RomFSA1FM2::getReadCacheLine(word address) const
