@@ -1,4 +1,4 @@
-// $Id: RomFactory.cc 8208 2008-09-09 12:51:28Z m9710797 $
+// $Id: RomFactory.cc 11146 2010-01-13 13:49:17Z m9710797 $
 
 #include "RomFactory.hh"
 #include "RomTypes.hh"
@@ -34,10 +34,15 @@
 #include "RomZemina90in1.hh"
 #include "RomZemina126in1.hh"
 #include "RomHolyQuran.hh"
+#include "RomHolyQuran2.hh"
 #include "RomFSA1FM.hh"
 #include "RomManbow2.hh"
 #include "RomMatraInk.hh"
+#include "RomArc.hh"
 #include "Rom.hh"
+#include "MSXMotherBoard.hh"
+#include "Reactor.hh"
+#include "RomDatabase.hh"
 #include "XMLElement.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
@@ -149,9 +154,11 @@ auto_ptr<MSXDevice> create(MSXMotherBoard& motherBoard, const XMLElement& config
 	string typestr = config.getChildData("mappertype", "Mirrored");
 	if (typestr == "auto") {
 		// Guess mapper type, if it was not in DB.
-		type = rom->getInfo().getRomType();
-		if (type == ROM_UNKNOWN) {
+		const RomInfo* romInfo = motherBoard.getReactor().getSoftwareDatabase().fetchRomInfo(rom->getOriginalSHA1());
+		if (romInfo == NULL) {
 			type = guessRomType(*rom);
+		} else {
+			type = romInfo->getRomType();
 		}
 	} else {
 		// Use mapper type from config, even if this overrides DB.
@@ -308,6 +315,9 @@ auto_ptr<MSXDevice> create(MSXMotherBoard& motherBoard, const XMLElement& config
 	case ROM_HOLY_QURAN:
 		result.reset(new RomHolyQuran(motherBoard, config, rom));
 		break;
+	case ROM_HOLY_QURAN2:
+		result.reset(new RomHolyQuran2(motherBoard, config, rom));
+		break;
 	case ROM_FSA1FM1:
 		result.reset(new RomFSA1FM1(motherBoard, config, rom));
 		break;
@@ -322,10 +332,20 @@ auto_ptr<MSXDevice> create(MSXMotherBoard& motherBoard, const XMLElement& config
 	case ROM_MATRAINK:
 		result.reset(new RomMatraInk(motherBoard, config, rom));
 		break;
+	case ROM_ARC:
+		result.reset(new RomArc(motherBoard, config, rom));
+		break;
 	default:
 		throw MSXException("Unknown ROM type");
 	}
-	result->setRomType(type);
+
+	// Store actual detected mapper type in config (override the possible
+	// 'auto' value). This way we're sure that on savestate/loadstate we're
+	// using the same mapper type (for example when the user's rom-database
+	// was updated).
+	XMLElement& writableConfig = const_cast<XMLElement&>(config);
+	writableConfig.setChildData("mappertype", RomInfo::romTypeToName(type));
+
 	return auto_ptr<MSXDevice>(result);
 }
 
