@@ -1,4 +1,4 @@
-// $Id: RomHarryFox.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomHarryFox.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // This mapper is used for the game "Harry Fox Yki no Maoh"
 
@@ -24,10 +24,10 @@ RomHarryFox::RomHarryFox(MSXMotherBoard& motherBoard, const XMLElement& config,
 
 void RomHarryFox::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setRom (1, 0);
-	setRom (2, 1);
-	setBank(3, unmappedRead);
+	setUnmapped(0);
+	setRom(1, 0);
+	setRom(2, 1);
+	setUnmapped(3);
 }
 
 void RomHarryFox::writeMem(word address, byte value, EmuTime::param /*time*/)
