@@ -1,4 +1,4 @@
-// $Id: RomHolyQuran.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomHolyQuran.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // Holy Qu'ran  cartridge
 //  It is like an ASCII 8KB, but using the 5000h, 5400h, 5800h and 5C00h
@@ -21,13 +21,13 @@ RomHolyQuran::RomHolyQuran(
 
 void RomHolyQuran::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setBank(1, unmappedRead);
+	setUnmapped(0);
+	setUnmapped(1);
 	for (int i = 2; i < 6; i++) {
 		setRom(i, 0);
 	}
-	setBank(6, unmappedRead);
-	setBank(7, unmappedRead);
+	setUnmapped(6);
+	setUnmapped(7);
 }
 
 void RomHolyQuran::writeMem(word address, byte value, EmuTime::param /*time*/)
