@@ -1,8 +1,9 @@
-// $Id: RomInfo.cc 8292 2008-10-10 20:32:52Z m9710797 $
+// $Id: RomInfo.cc 11122 2010-01-10 13:49:02Z manuelbi $
 
 #include "RomInfo.hh"
 #include "CliComm.hh"
 #include "StringOp.hh"
+#include "unreachable.hh"
 #include <cassert>
 #include <map>
 
@@ -49,7 +50,8 @@ static const RomTypeMap& getRomTypeMap()
 		romTypeMap["Synthesizer"]    = ROM_SYNTHESIZER;
 		romTypeMap["PlayBall"]       = ROM_PLAYBALL;
 		romTypeMap["NettouYakyuu"]   = ROM_NETTOU_YAKYUU;
-		romTypeMap["HolyQuran"]      = ROM_HOLY_QURAN;
+		romTypeMap["AlQuranDecoded"] = ROM_HOLY_QURAN;
+		romTypeMap["AlQuran"]        = ROM_HOLY_QURAN2;
 		romTypeMap["Padial8"]        = ROM_PADIAL8;
 		romTypeMap["Padial16"]       = ROM_PADIAL16;
 		romTypeMap["SuperLodeRunner"]= ROM_SUPERLODERUNNER;
@@ -57,6 +59,7 @@ static const RomTypeMap& getRomTypeMap()
 		romTypeMap["Manbow2"]        = ROM_MANBOW2;
 		romTypeMap["MegaFlashRomScc"]= ROM_MEGAFLASHROMSCC;
 		romTypeMap["MatraInk"]       = ROM_MATRAINK;
+		romTypeMap["Arc"]            = ROM_ARC;
 
 		// ROM mapper types used for system ROMs in machines
 		romTypeMap["Panasonic"]      = ROM_PANASONIC;
@@ -93,7 +96,8 @@ static const RomTypeMap& getRomTypeMap()
 RomInfo::RomInfo(const string& ntitle,   const string& nyear,
                  const string& ncompany, const string& ncountry,
                  bool noriginal,         const string& norigType,
-                 const string& nremark,  const RomType& nromType)
+                 const string& nremark,  const RomType& nromType,
+                 int ngenMSXid)
 	: title(ntitle)
 	, year(nyear)
 	, company(ncompany)
@@ -102,6 +106,7 @@ RomInfo::RomInfo(const string& ntitle,   const string& nyear,
 	, remark(nremark)
 	, romType(nromType)
 	, original(noriginal)
+	, genMSXid(ngenMSXid)
 {
 }
 
@@ -138,6 +143,7 @@ RomType RomInfo::nameToRomType(string name)
 		aliasMap["KOREAN80IN1"]   = "Zemina80in1";
 		aliasMap["KOREAN90IN1"]   = "Zemina90in1";
 		aliasMap["KOREAN126IN1"]  = "Zemina126in1";
+		aliasMap["HolyQuran"]     = "AlQuranDecoded";
 	}
 	const RomTypeMap& romTypeMap = getRomTypeMap();
 	AliasMap::const_iterator alias_it = aliasMap.find(name);
@@ -161,8 +167,7 @@ string RomInfo::romTypeToName(RomType type)
 			return it->first;
 		}
 	}
-	assert(false);
-	return "";
+	UNREACHABLE; return "";
 }
 
 void RomInfo::getAllRomTypes(set<string>& result)
@@ -174,43 +179,5 @@ void RomInfo::getAllRomTypes(set<string>& result)
 	}
 }
 
-void RomInfo::print(CliComm& cliComm) const
-{
-	string year(getYear());
-	if (year.empty()) {
-		year = "(info not available)";
-	}
-	string company(getCompany());
-	if (company.empty()) {
-		company = "(info not available)";
-	}
-	string status;
-	if (getOriginal()) {
-		// this is an unmodified original dump
-		status = "Unmodified dump (confirmed by " + getOrigType() + ")";
-	} else {
-		// not original or unknown
-		if (getOrigType() == "broken") {
-			status = "Bad dump (game is broken)";
-		} else if (getOrigType() == "translated") {
-			status = "Translated from original";
-		} else if (getOrigType() == "working") {
-			status = "Modified but confirmed working";
-		} else {
-			status = "Unknown";
-		}
-	}
-	string info = "Found this ROM in the database:\n"
-	              "  Title:    " + getTitle() + "\n"
-	              "  Year:     " + getYear() + "\n"
-	              "  Company:  " + getCompany() + "\n"
-	              "  Country:  " + getCountry() + "\n"
-	              "  Status:   " + status;
-	if (!getRemark().empty()) {
-		info += "\n  Remark:   " + getRemark();
-	}
-	cliComm.printInfo(info);
-}
-
 } // namespace openmsx
 
