@@ -1,4 +1,4 @@
-// $Id: RomKonami.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: RomKonami.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // KONAMI 8kB cartridges (without SCC)
 //
@@ -30,13 +30,13 @@ RomKonami::RomKonami(MSXMotherBoard& motherBoard, const XMLElement& config,
 
 void RomKonami::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setBank(1, unmappedRead);
+	setUnmapped(0);
+	setUnmapped(1);
 	for (int i = 2; i < 6; i++) {
 		setRom(i, i - 2);
 	}
-	setBank(6, unmappedRead);
-	setBank(7, unmappedRead);
+	setUnmapped(6);
+	setUnmapped(7);
 }
 
 void RomKonami::writeMem(word address, byte value, EmuTime::param /*time*/)
