@@ -1,11 +1,11 @@
-// $Id: RomMSXDOS2.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomMSXDOS2.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 #include "RomMSXDOS2.hh"
 #include "Rom.hh"
 #include "CacheLine.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
-#include <cassert>
+#include "unreachable.hh"
 
 namespace openmsx {
 
@@ -23,10 +23,10 @@ RomMSXDOS2::RomMSXDOS2(
 
 void RomMSXDOS2::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setRom (1, 0);
-	setBank(2, unmappedRead);
-	setBank(3, unmappedRead);
+	setUnmapped(0);
+	setRom(1, 0);
+	setUnmapped(2);
+	setUnmapped(3);
 }
 
 void RomMSXDOS2::writeMem(word address, byte value, EmuTime::param /*time*/)
@@ -42,7 +42,7 @@ void RomMSXDOS2::writeMem(word address, byte value, EmuTime::param /*time*/)
 		if (address != 0x7ffe) return;
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 	setRom(1, value);
 }
@@ -60,7 +60,7 @@ byte* RomMSXDOS2::getWriteCacheLine(word address) const
 		if (address == (0x7ffe & CacheLine::HIGH)) return NULL;
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 	return unmappedWrite;
 }
