@@ -1,4 +1,4 @@
-// $Id: RomManbow2.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: RomManbow2.cc 10129 2009-07-02 00:09:56Z mthuurne $
 
 #include "RomManbow2.hh"
 #include "Rom.hh"
@@ -20,7 +20,8 @@ RomManbow2::RomManbow2(MSXMotherBoard& motherBoard, const XMLElement& config,
                        std::auto_ptr<Rom> rom_, RomType type)
 	: MSXRom(motherBoard, config, rom_)
 	, scc(new SCC(motherBoard, "SCC", config, getCurrentTime()))
-	, flash(new AmdFlash(*rom, 16, 512 / 64, getWriteProtected(type), config))
+	, flash(new AmdFlash(
+		motherBoard, *rom, 16, 512 / 64, getWriteProtected(type), config))
 {
 	reset(getCurrentTime());
 }
