@@ -1,4 +1,4 @@
-// $Id: RomMatraInk.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomMatraInk.cc 10129 2009-07-02 00:09:56Z mthuurne $
 
 #include "RomMatraInk.hh"
 #include "AmdFlash.hh"
@@ -11,7 +11,7 @@ RomMatraInk::RomMatraInk(
 		MSXMotherBoard& motherBoard, const XMLElement& config,
 		std::auto_ptr<Rom> rom_)
 	: MSXRom(motherBoard, config, rom_)
-	, flash(new AmdFlash(*rom, 16, 2, 0)) // don't load/save
+	, flash(new AmdFlash(motherBoard, *rom, 16, 2, 0)) // don't load/save
 {
 	reset(EmuTime::dummy());
 }
