@@ -1,4 +1,4 @@
-// $Id: RomPadial16kB.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomPadial16kB.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // Padial 16kB
 //
@@ -22,10 +22,10 @@ RomPadial16kB::RomPadial16kB(
 
 void RomPadial16kB::reset(EmuTime::param /*time*/)
 {
-	setRom (0, 0);
-	setRom (1, 0);
-	setRom (2, 2);
-	setBank(3, unmappedRead);
+	setRom(0, 0);
+	setRom(1, 0);
+	setRom(2, 2);
+	setUnmapped(3);
 }
 
 REGISTER_MSXDEVICE(RomPadial16kB, "RomPadial16kB");
