@@ -1,4 +1,4 @@
-// $Id: RomPanasonic.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomPanasonic.cc 10739 2009-11-03 12:05:29Z m9710797 $
 
 #include "RomPanasonic.hh"
 #include "PanasonicMemory.hh"
@@ -33,11 +33,13 @@ RomPanasonic::RomPanasonic(
 		maxSRAMBank = SRAM_BASE + (sramSize / 8);
 	}
 
-	// Only lazily tell baseclass about about PanasonicMemory (for
-	// serialization). We cannot do this in the constructor yet, because
-	// we're not certain the PanasonicRam device is already constructed.
-	// Actual registration is done in changeBank(), right before we
-	// actually select ram pages.
+	// Inform baseclass about PanasonicMemory (needed for serialization).
+	// This relies on the order of MSXDevice instantiation, the PanasonicRam
+	// device must be create before this one. (In the hardwareconfig.xml
+	// we added a device-reference from this device to the PanasonicRam
+	// device, this should protected against wrong future edits in the
+	// config file).
+	setExtraMemory(panasonicMem.getRamBlock(0), panasonicMem.getRamSize());
 
 	reset(EmuTime::dummy());
 }
@@ -83,7 +85,7 @@ byte RomPanasonic::peekMem(word address, EmuTime::param time) const
 
 byte RomPanasonic::readMem(word address, EmuTime::param time)
 {
-	return peekMem(address, time);
+	return RomPanasonic::peekMem(address, time);
 }
 
 const byte* RomPanasonic::getReadCacheLine(word address) const
@@ -174,16 +176,12 @@ void RomPanasonic::changeBank(byte region, int bank)
 		if (offset >= sramSize) {
 			offset &= (sramSize - 1);
 		}
-		setBank(region, &sram->operator[](offset));
+		// TODO romblock debuggable is only 8 bits, here bank is 9 bits
+		setBank(region, &sram->operator[](offset), bank);
 	} else if (panasonicMem.getRamSize() && (RAM_BASE <= bank)) {
 		// RAM
-		// Only lazily (=not in constructor) inform base class about
-		// RAM region, to avoid dependency on the order of devices
-		// in the config file (otherwise PanasonicRam must come before
-		// this device). It's ok to call setExtraMemory() multiple times.
-		setExtraMemory(panasonicMem.getRamBlock(0),
-		               panasonicMem.getRamSize());
-		setBank(region, panasonicMem.getRamBlock(bank - RAM_BASE));
+		// TODO romblock debuggable is only 8 bits, here bank is 9 bits
+		setBank(region, panasonicMem.getRamBlock(bank - RAM_BASE), bank);
 	} else {
 		// ROM
 		setRom(region, bank);
