@@ -1,4 +1,4 @@
-// $Id: RomPlain.cc 8237 2008-09-18 17:54:54Z m9710797 $
+// $Id: RomPlain.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 #include "RomPlain.hh"
 #include "Rom.hh"
@@ -65,7 +65,7 @@ RomPlain::RomPlain(MSXMotherBoard& motherBoard, const XMLElement& config,
 			if (mirrored == MIRRORED) {
 				setRom(page, romPage & (numPages - 1));
 			} else {
-				setBank(page, unmappedRead);
+				setUnmapped(page);
 			}
 		}
 
