@@ -1,4 +1,4 @@
-// $Id: RomRType.cc 9361 2009-03-09 20:57:09Z m9710797 $
+// $Id: RomRType.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // R-Type cartridges
 //
@@ -25,10 +25,10 @@ RomRType::RomRType(MSXMotherBoard& motherBoard, const XMLElement& config,
 
 void RomRType::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setRom (1, 0x17);
-	setRom (2, 0);
-	setBank(3, unmappedRead);
+	setUnmapped(0);
+	setRom(1, 0x17);
+	setRom(2, 0);
+	setUnmapped(3);
 }
 
 void RomRType::writeMem(word address, byte value, EmuTime::param /*time*/)
