@@ -1,4 +1,4 @@
-// $Id: RomSuperLodeRunner.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: RomSuperLodeRunner.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 #include "RomSuperLodeRunner.hh"
 #include "MSXMotherBoard.hh"
@@ -25,10 +25,10 @@ RomSuperLodeRunner::~RomSuperLodeRunner()
 
 void RomSuperLodeRunner::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setBank(1, unmappedRead);
-	setRom (2, 0);
-	setBank(3, unmappedRead);
+	setUnmapped(0);
+	setUnmapped(1);
+	setRom(2, 0);
+	setUnmapped(3);
 }
 
 void RomSuperLodeRunner::globalWrite(word address, byte value, EmuTime::param /*time*/)
