@@ -1,4 +1,4 @@
-// $Id: RomZemina126in1.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomZemina126in1.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // Zemina 126-in-1 cartridge
 //
@@ -24,10 +24,10 @@ RomZemina126in1::RomZemina126in1(
 
 void RomZemina126in1::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setRom (1, 0);
-	setRom (2, 1);
-	setBank(3, unmappedRead);
+	setUnmapped(0);
+	setRom(1, 0);
+	setRom(2, 1);
+	setUnmapped(3);
 }
 
 void RomZemina126in1::writeMem(word address, byte value, EmuTime::param /*time*/)
