@@ -1,4 +1,4 @@
-// $Id: RomZemina80in1.cc 8353 2008-11-13 18:38:44Z m9710797 $
+// $Id: RomZemina80in1.cc 10587 2009-09-15 19:35:34Z m9710797 $
 
 // Zemina 80-in-1 cartridge
 //
@@ -25,13 +25,13 @@ RomZemina80in1::RomZemina80in1(
 
 void RomZemina80in1::reset(EmuTime::param /*time*/)
 {
-	setBank(0, unmappedRead);
-	setBank(1, unmappedRead);
+	setUnmapped(0);
+	setUnmapped(1);
 	for (int i = 2; i < 6; i++) {
 		setRom(i, i - 2);
 	}
-	setBank(6, unmappedRead);
-	setBank(7, unmappedRead);
+	setUnmapped(6);
+	setUnmapped(7);
 }
 
 void RomZemina80in1::writeMem(word address, byte value, EmuTime::param /*time*/)
