@@ -1,7 +1,8 @@
-// $Id: ClockPin.cc 5071 2006-01-23 19:35:27Z m9710797 $
+// $Id: ClockPin.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
-#include <cassert>
 #include "ClockPin.hh"
+#include "serialize.hh"
+#include <cassert>
 
 using std::string;
 
@@ -14,7 +15,7 @@ ClockPin::ClockPin(Scheduler& scheduler, ClockPinListener* listener_)
 {
 }
 
-void ClockPin::setState(bool newStatus, const EmuTime& time)
+void ClockPin::setState(bool newStatus, EmuTime::param time)
 {
 	periodic = false;
 	if (signalEdge) {
@@ -34,8 +35,8 @@ void ClockPin::setState(bool newStatus, const EmuTime& time)
 	}
 }
 
-void ClockPin::setPeriodicState(const EmuDuration& total,
-	const EmuDuration& hi, const EmuTime& time)
+void ClockPin::setPeriodicState(EmuDuration::param total,
+	EmuDuration::param hi, EmuTime::param time)
 {
 	referenceTime = time;
 	totalDur = total;
@@ -56,7 +57,7 @@ void ClockPin::setPeriodicState(const EmuDuration& total,
 }
 
 
-bool ClockPin::getState(const EmuTime& time) const
+bool ClockPin::getState(EmuTime::param time) const
 {
 	if (!periodic) {
 		return status;
@@ -70,19 +71,19 @@ bool ClockPin::isPeriodic() const
 	return periodic;
 }
 
-const EmuDuration& ClockPin::getTotalDuration() const
+EmuDuration::param ClockPin::getTotalDuration() const
 {
 	assert(periodic);
 	return totalDur;
 }
 
-const EmuDuration& ClockPin::getHighDuration() const
+EmuDuration::param ClockPin::getHighDuration() const
 {
 	assert(periodic);
 	return hiDur;
 }
 
-int ClockPin::getTicksBetween(const EmuTime& begin, const EmuTime& end) const
+int ClockPin::getTicksBetween(EmuTime::param begin, EmuTime::param end) const
 {
 	assert(begin <= end);
 	if (!periodic) {
@@ -100,7 +101,7 @@ int ClockPin::getTicksBetween(const EmuTime& begin, const EmuTime& end) const
 }
 
 
-void ClockPin::generateEdgeSignals(bool wanted, const EmuTime& time)
+void ClockPin::generateEdgeSignals(bool wanted, EmuTime::param time)
 {
 	if (signalEdge != wanted) {
 		signalEdge = wanted;
@@ -125,13 +126,13 @@ void ClockPin::unschedule()
 	removeSyncPoint();
 }
 
-void ClockPin::schedule(const EmuTime& time)
+void ClockPin::schedule(EmuTime::param time)
 {
 	assert(signalEdge && periodic && listener);
 	setSyncPoint(time);
 }
 
-void ClockPin::executeUntil(const EmuTime& time, int /*userdata*/)
+void ClockPin::executeUntil(EmuTime::param time, int /*userdata*/)
 {
 	assert(signalEdge && periodic && listener);
 	listener->signalPosEdge(*this, time);
@@ -146,4 +147,18 @@ const string& ClockPin::schedName() const
 	return name;
 }
 
+
+template<typename Archive>
+void ClockPin::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Schedulable>(*this);
+	ar.serialize("totalDur", totalDur);
+	ar.serialize("hiDur", hiDur);
+	ar.serialize("referenceTime", referenceTime);
+	ar.serialize("periodic", periodic);
+	ar.serialize("status", status);
+	ar.serialize("signalEdge", signalEdge);
+}
+INSTANTIATE_SERIALIZE_METHODS(ClockPin);
+
 } // namespace openmsx
