@@ -1,4 +1,4 @@
-// $Id: MSXRS232.cc 9310 2009-03-01 08:04:24Z m9710797 $
+// $Id: MSXRS232.cc 11388 2010-04-07 07:40:27Z auroramsx $
 
 #include "MSXRS232.hh"
 #include "RS232Device.hh"
@@ -9,6 +9,7 @@
 #include "Rom.hh"
 #include "XMLElement.hh"
 #include "serialize.hh"
+#include "unreachable.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -85,13 +86,21 @@ MSXRS232::MSXRS232(MSXMotherBoard& motherBoard, const XMLElement& config)
 	i8254->getClockPin(1).setPeriodicState(total, hi, time);
 	i8254->getClockPin(2).setPeriodicState(total, hi, time);
 
-	reset(time);
+	powerUp(time);
 }
 
 MSXRS232::~MSXRS232()
 {
 }
 
+void MSXRS232::powerUp(EmuTime::param time)
+{
+	if(ram.get()) {
+		ram->clear();
+	}
+	reset(time);
+}
+
 void MSXRS232::reset(EmuTime::param /*time*/)
 {
 	rxrdyIRQlatch = false;
@@ -167,8 +176,7 @@ byte MSXRS232::readIO(word port, EmuTime::param time)
 			result = i8254->readIO(port - 4, time);
 			break;
 		default:
-			assert(false);
-			result = 0xFF; // avoid warning
+			UNREACHABLE; return 0;
 	}
 	//PRT_DEBUG("MSXRS232 read " << (int)port << " " << (int)result);
 	return result;
@@ -196,8 +204,7 @@ byte MSXRS232::peekIO(word port, EmuTime::param time) const
 			result = i8254->peekIO(port - 4, time);
 			break;
 		default:
-			assert(false);
-			result = 0xFF; // avoid warning
+			UNREACHABLE; return 0;
 	}
 	return result;
 }
@@ -353,7 +360,7 @@ void Counter0::signal(ClockPin& pin, EmuTime::param time)
 
 void Counter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 {
-	assert(false);
+	UNREACHABLE;
 }
 
 
@@ -381,7 +388,7 @@ void Counter1::signal(ClockPin& pin, EmuTime::param time)
 
 void Counter1::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 {
-	assert(false);
+	UNREACHABLE;
 }
 
 
