@@ -1,4 +1,4 @@
-// $Id: MidiInReader.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: MidiInReader.cc 10736 2009-11-02 19:25:01Z m9710797 $
 
 #include "MidiInReader.hh"
 #include "MidiInConnector.hh"
@@ -89,12 +89,12 @@ void MidiInReader::run()
 		if (num != 1) {
 			continue;
 		}
-		assert(getConnector());
+		assert(isPluggedIn());
 
 		ScopedLock l(lock);
 		queue.push_back(buf);
 		eventDistributor.distributeEvent(
-			new SimpleEvent<OPENMSX_MIDI_IN_READER_EVENT>());
+			new SimpleEvent(OPENMSX_MIDI_IN_READER_EVENT));
 	}
 }
 
@@ -120,7 +120,7 @@ void MidiInReader::signal(EmuTime::param time)
 // EventListener
 bool MidiInReader::signalEvent(shared_ptr<const Event> /*event*/)
 {
-	if (getConnector()) {
+	if (isPluggedIn()) {
 		signal(scheduler.getCurrentTime());
 	} else {
 		ScopedLock l(lock);
