@@ -1,4 +1,4 @@
-// $Id: MidiInWindows.cc 9300 2009-03-01 01:03:07Z mfeingol $
+// $Id: MidiInWindows.cc 10736 2009-11-02 19:25:01Z m9710797 $
 
 #if defined(_WIN32)
 #include "MidiInWindows.hh"
@@ -100,7 +100,7 @@ void MidiInWindows::procLongMsg(LPMIDIHDR p)
 			queue.push_back(p->lpData[i]);
 		}
 		eventDistributor.distributeEvent(
-			new SimpleEvent<OPENMSX_MIDI_IN_WINDOWS_EVENT>());
+			new SimpleEvent(OPENMSX_MIDI_IN_WINDOWS_EVENT));
 	}
 }
 
@@ -121,13 +121,13 @@ void MidiInWindows::procShortMsg(DWORD param)
 		param >>= 8;
 	}
 	eventDistributor.distributeEvent(
-		new SimpleEvent<OPENMSX_MIDI_IN_WINDOWS_EVENT>());
+		new SimpleEvent(OPENMSX_MIDI_IN_WINDOWS_EVENT));
 }
 
 // Runnable
 void MidiInWindows::run()
 {
-	assert(getConnector());
+	assert(isPluggedIn());
 	thrdid = SDL_ThreadID();
 
 	MSG msg;
@@ -180,7 +180,7 @@ void MidiInWindows::signal(EmuTime::param time)
 // EventListener
 bool MidiInWindows::signalEvent(shared_ptr<const Event> /*event*/)
 {
-	if (getConnector()) {
+	if (isPluggedIn()) {
 		signal(scheduler.getCurrentTime());
 	} else {
 		ScopedLock l(lock);
