@@ -1,4 +1,4 @@
-// $Id: RS232Tester.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: RS232Tester.cc 11480 2010-06-05 08:05:43Z manuelbi $
 
 #include "RS232Tester.hh"
 #include "RS232Connector.hh"
@@ -97,16 +97,16 @@ void RS232Tester::run()
 {
 	byte buf;
 	if (!inFile) return;
-	while (true) {
+	while (!feof(inFile)) {
 		size_t num = fread(&buf, 1, 1, inFile);
 		if (num != 1) {
 			continue;
 		}
-		assert(getConnector());
+		assert(isPluggedIn());
 		ScopedLock l(lock);
 		queue.push_back(buf);
 		eventDistributor.distributeEvent(
-			new SimpleEvent<OPENMSX_RS232_TESTER_EVENT>());
+			new SimpleEvent(OPENMSX_RS232_TESTER_EVENT));
 	}
 }
 
@@ -130,7 +130,7 @@ void RS232Tester::signal(EmuTime::param time)
 // EventListener
 bool RS232Tester::signalEvent(shared_ptr<const Event> /*event*/)
 {
-	if (getConnector()) {
+	if (isPluggedIn()) {
 		signal(scheduler.getCurrentTime());
 	} else {
 		ScopedLock l(lock);
