@@ -1,4 +1,4 @@
-// $Id: serialize.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: serialize.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "serialize.hh"
 #include "Base64.hh"
@@ -6,6 +6,7 @@
 #include "XMLLoader.hh"
 #include "XMLElement.hh"
 #include "XMLException.hh"
+#include "lzo.hh"
 #include "StringOp.hh"
 #include <cstring>
 #include <zlib.h>
@@ -15,11 +16,60 @@ using std::string;
 namespace openmsx {
 
 template<typename Derived>
-OutputArchiveBase<Derived>::OutputArchiveBase()
+void ArchiveBase<Derived>::attribute(const char* name, const char* value)
+{
+	string valueStr(value);
+	self().attribute(name, valueStr);
+}
+template class ArchiveBase<MemOutputArchive>;
+template class ArchiveBase<XmlOutputArchive>;
+
+////
+
+OutputArchiveBase2::OutputArchiveBase2()
 	: lastId(0)
 {
 }
 
+unsigned OutputArchiveBase2::generateID1(const void* p)
+{
+	#ifdef linux
+	assert("Can't serialize ID of object located on the stack" &&
+	       !addressOnStack(p));
+	#endif
+	++lastId;
+	assert(polyIdMap.find(p) == polyIdMap.end());
+	polyIdMap[p] = lastId;
+	return lastId;
+}
+unsigned OutputArchiveBase2::generateID2(
+	const void* p, const std::type_info& typeInfo)
+{
+	#ifdef linux
+	assert("Can't serialize ID of object located on the stack" &&
+	       !addressOnStack(p));
+	#endif
+	++lastId;
+	IdKey key = std::make_pair(p, TypeInfo(typeInfo));
+	assert(idMap.find(key) == idMap.end());
+	idMap[key] = lastId;
+	return lastId;
+}
+
+unsigned OutputArchiveBase2::getID1(const void* p)
+{
+	PolyIdMap::const_iterator it = polyIdMap.find(p);
+	return it != polyIdMap.end() ? it->second : 0;
+}
+unsigned OutputArchiveBase2::getID2(
+	const void* p, const std::type_info& typeInfo)
+{
+	IdKey key = std::make_pair(p, TypeInfo(typeInfo));
+	IdMap::const_iterator it = idMap.find(key);
+	return it != idMap.end() ? it->second : 0;
+}
+
+
 template<typename Derived>
 void OutputArchiveBase<Derived>::serialize_blob(
 	const char* tag, const void* data, unsigned len)
@@ -56,9 +106,16 @@ template class OutputArchiveBase<XmlOutputArchive>;
 
 ////
 
-template<typename Derived>
-InputArchiveBase<Derived>::InputArchiveBase()
+void* InputArchiveBase2::getPointer(unsigned id)
 {
+	IdMap::const_iterator it = idMap.find(id);
+	return it != idMap.end() ? it->second : NULL;
+}
+
+void InputArchiveBase2::addPointer(unsigned id, const void* p)
+{
+	assert(idMap.find(id) == idMap.end());
+	idMap[id] = const_cast<void*>(p);
 }
 
 template<typename Derived>
@@ -90,11 +147,9 @@ void InputArchiveBase<Derived>::serialize_blob(
 			tmp = Base64::decode(tmp);
 		}
 		if (tmp.size() != len) {
-			throw XMLException(
-				"Length of decoded blob: " +
-				StringOp::toString(tmp.size()) +
-				" not identical to expected value: " +
-				StringOp::toString(len));
+			throw XMLException(StringOp::Builder()
+				<< "Length of decoded blob: " << tmp.size()
+				<< " not identical to expected value: " << len);
 		}
 		memcpy(data, tmp.data(), len);
 	} else {
@@ -102,25 +157,96 @@ void InputArchiveBase<Derived>::serialize_blob(
 	}
 }
 
-template<typename Derived>
-void* InputArchiveBase<Derived>::getPointer(unsigned id)
-{
-	IdMap::const_iterator it = idMap.find(id);
-	return it != idMap.end() ? it->second : NULL;
-}
-
-template<typename Derived>
-void InputArchiveBase<Derived>::addPointer(unsigned id, const void* p)
-{
-	assert(idMap.find(id) == idMap.end());
-	idMap[id] = const_cast<void*>(p);
-}
-
 template class InputArchiveBase<MemInputArchive>;
 template class InputArchiveBase<XmlInputArchive>;
 
 ////
 
+void MemOutputArchive::save(const std::string& s)
+{
+	unsigned size = unsigned(s.size());
+	save(size);
+	put(s.data(), size);
+}
+
+////
+
+void MemInputArchive::load(std::string& s)
+{
+	unsigned length;
+	load(length);
+	s.resize(length);
+	if (length) {
+		get(&s[0], length);
+	}
+}
+
+////
+
+void MemOutputArchive::serialize_blob(const char*, const void* data, unsigned len)
+{
+	// Compress in-memory blobs:
+	//
+	// This is a bit slower than memcpy, but it uses a lot less memory.
+	// Memory usage is important for the reverse feature, where we keep a
+	// lot of savestates in memory.
+	//
+	// I compared 'gzip level=1' (fastest version with lowest compression
+	// ratio) with 'lzo'. lzo was considerably faster. Compression ratio
+	// was about the same (maybe lzo was slightly better (OTOH on higher
+	// levels gzip compresses better)). So I decided to go with lzo.
+#if 0
+	// gzip
+	uLongf dstLen = len + len / 1000 + 12; // upper bound for required size
+	char* buf = buffer.allocate(sizeof(uLongf) + dstLen);
+
+	if (compress2(reinterpret_cast<Bytef*>(&buf[sizeof(uLongf)]), &dstLen,
+	              reinterpret_cast<const Bytef*>(data), len, 1)
+	    != Z_OK) {
+		UNREACHABLE;
+	}
+
+	memcpy(buf, &dstLen, sizeof(uLongf)); // fill-in actual size
+	buffer.deallocate(&buf[sizeof(uLongf) + dstLen]); // dealloc unused portion
+#else
+	// lzo
+	unsigned dstLen = len + len / 16 + 64 + 3; // upper bound
+	char* buf = buffer.allocate(sizeof(unsigned) + dstLen);
+
+	lzo1x_1_compress(reinterpret_cast<const byte*>(data), len,
+	                 reinterpret_cast<byte*>(&buf[sizeof(unsigned)]),
+	                 dstLen);
+
+	memcpy(buf, &dstLen, sizeof(unsigned)); // fill-in actual size
+	buffer.deallocate(&buf[sizeof(unsigned) + dstLen]); // dealloc unused portion
+#endif
+}
+
+void MemInputArchive::serialize_blob(const char*, void* data, unsigned len)
+{
+#if 0
+	// gzip
+	uLongf srcLen; load(srcLen);
+	uLongf dstLen = len;
+	if (uncompress(reinterpret_cast<Bytef*>(data), &dstLen,
+		       reinterpret_cast<const Bytef*>(buffer.getCurrentPos()), srcLen)
+	    != Z_OK) {
+		UNREACHABLE;
+	}
+	buffer.skip(srcLen);
+#else
+	// lzo
+	unsigned srcLen; load(srcLen);
+	unsigned dstLen = len;
+	lzo1x_decompress(reinterpret_cast<const byte*>(buffer.getCurrentPos()),
+	                 srcLen, reinterpret_cast<byte*>(data), dstLen);
+	assert(dstLen == len);
+	buffer.skip(srcLen);
+#endif
+}
+
+////
+
 XmlOutputArchive::XmlOutputArchive(const string& filename)
 	: current(new XMLElement("serial"))
 {
@@ -162,15 +288,35 @@ void XmlOutputArchive::save(signed char c)
 {
 	save(int(c));
 }
+void XmlOutputArchive::save(int i)
+{
+	saveImpl(i);
+}
+void XmlOutputArchive::save(unsigned u)
+{
+	saveImpl(u);
+}
+void XmlOutputArchive::save(unsigned long long ull)
+{
+	saveImpl(ull);
+}
 
-void XmlOutputArchive::attribute(const string& name, const string& str)
+void XmlOutputArchive::attribute(const char* name, const string& str)
 {
 	assert(current);
 	assert(!current->hasAttribute(name));
 	current->addAttribute(name, str);
 }
+void XmlOutputArchive::attribute(const char* name, int i)
+{
+	attributeImpl(name, i);
+}
+void XmlOutputArchive::attribute(const char* name, unsigned u)
+{
+	attributeImpl(name, u);
+}
 
-void XmlOutputArchive::beginTag(const string& tag)
+void XmlOutputArchive::beginTag(const char* tag)
 {
 	XMLElement* elem = new XMLElement(tag);
 	if (current) {
@@ -178,7 +324,7 @@ void XmlOutputArchive::beginTag(const string& tag)
 	}
 	current = elem;
 }
-void XmlOutputArchive::endTag(const string& tag)
+void XmlOutputArchive::endTag(const char* tag)
 {
 	assert(current);
 	assert(current->getName() == tag);
@@ -190,7 +336,6 @@ void XmlOutputArchive::endTag(const string& tag)
 
 XmlInputArchive::XmlInputArchive(const string& filename)
 	: elem(XMLLoader::load(filename, "openmsx-serialize.dtd"))
-	, pos(0)
 {
 	init(elem.get());
 }
@@ -233,18 +378,30 @@ void XmlInputArchive::load(signed char& c)
 	load(i);
 	c = i;
 }
-
-void XmlInputArchive::beginTag(const string& tag)
+void XmlInputArchive::load(int& i)
 {
+	loadImpl(i);
+}
+void XmlInputArchive::load(unsigned& u)
+{
+	loadImpl(u);
+}
+void XmlInputArchive::load(unsigned long long& ull)
+{
+	loadImpl(ull);
+}
+
+void XmlInputArchive::beginTag(const char* tag_)
+{
+	string tag(tag_);
 	const XMLElement* child = elems.back()->findChild(tag);
 	if (!child) {
 		throw XMLException("No child tag found in begin tag \"" + tag + "\"");
 	}
 	elems.push_back(child);
 }
-void XmlInputArchive::endTag(const string& tag)
+void XmlInputArchive::endTag(const char* tag)
 {
-	(void)tag;
 	if (elems.back()->getName() != tag) {
 		throw XMLException("End tag \"" + elems.back()->getName() + "\" not equal to begin tag \"" + tag + "\"");
 	}
@@ -253,14 +410,22 @@ void XmlInputArchive::endTag(const string& tag)
 	elems.pop_back();
 }
 
-void XmlInputArchive::attribute(const string& name, string& t)
+void XmlInputArchive::attribute(const char* name, string& t)
 {
 	if (!hasAttribute(name)) {
-		throw XMLException("Missing attribute \"" + name + "\"");
+		throw XMLException("Missing attribute \"" + string(name) + "\"");
 	}
 	t = elems.back()->getAttribute(name);
 }
-bool XmlInputArchive::hasAttribute(const string& name)
+void XmlInputArchive::attribute(const char* name, int& i)
+{
+	attributeImpl(name, i);
+}
+void XmlInputArchive::attribute(const char* name, unsigned& u)
+{
+	attributeImpl(name, u);
+}
+bool XmlInputArchive::hasAttribute(const char* name)
 {
 	return elems.back()->hasAttribute(name);
 }
