@@ -1,6 +1,10 @@
-// $Id: serialize_meta.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: serialize_meta.cc 11145 2010-01-13 13:48:46Z m9710797 $
 
 #include "serialize_meta.hh"
+#include "serialize.hh"
+#include "MSXException.hh"
+#include "StringOp.hh"
+#include "unreachable.hh"
 #include <cassert>
 #include <iostream>
 
@@ -28,16 +32,34 @@ PolymorphicSaverRegistry<Archive>& PolymorphicSaverRegistry<Archive>::instance()
 }
 
 template<typename Archive>
-const PolymorphicSaverBase<Archive>&
-PolymorphicSaverRegistry<Archive>::getSaver(TypeInfo typeInfo)
+void PolymorphicSaverRegistry<Archive>::registerHelper(
+	const std::type_info& type, PolymorphicSaverBase<Archive>* saver)
 {
-	typename SaverMap::const_iterator it = saverMap.find(typeInfo);
-	if (it == saverMap.end()) {
+	assert(saverMap.find(type) == saverMap.end());
+	saverMap[type] = saver;
+}
+
+template<typename Archive>
+void PolymorphicSaverRegistry<Archive>::save(
+	Archive& ar, const void* t, const std::type_info& typeInfo)
+{
+	PolymorphicSaverRegistry<Archive>& reg =
+		PolymorphicSaverRegistry<Archive>::instance();
+	typename SaverMap::const_iterator it = reg.saverMap.find(typeInfo);
+	if (it == reg.saverMap.end()) {
 		std::cerr << "Trying to save an unregistered polymorphic type: "
 			  << typeInfo.name() << std::endl;
-		assert(false);
+		UNREACHABLE;
 	}
-	return *it->second;
+	it->second->save(ar, t);
+}
+template<typename Archive>
+void PolymorphicSaverRegistry<Archive>::save(
+	const char* tag, Archive& ar, const void* t, const std::type_info& typeInfo)
+{
+	ar.beginTag(tag);
+	save(ar, t, typeInfo);
+	ar.endTag(tag);
 }
 
 template class PolymorphicSaverRegistry<MemOutputArchive>;
@@ -67,12 +89,25 @@ PolymorphicLoaderRegistry<Archive>& PolymorphicLoaderRegistry<Archive>::instance
 }
 
 template<typename Archive>
-const PolymorphicLoaderBase<Archive>&
-PolymorphicLoaderRegistry<Archive>::getLoader(const std::string& type)
+void PolymorphicLoaderRegistry<Archive>::registerHelper(
+	const char* name_, PolymorphicLoaderBase<Archive>* loader)
 {
-	typename LoaderMap::const_iterator it = loaderMap.find(type);
-	assert(it != loaderMap.end());
-	return *it->second;
+	std::string name(name_);
+	assert(loaderMap.find(name) == loaderMap.end());
+	loaderMap[name] = loader;
+}
+
+template<typename Archive>
+void* PolymorphicLoaderRegistry<Archive>::load(
+	Archive& ar, unsigned id, TupleBase& args)
+{
+	std::string type;
+	ar.attribute("type", type);
+	PolymorphicLoaderRegistry<Archive>& reg =
+		PolymorphicLoaderRegistry<Archive>::instance();
+	typename LoaderMap::const_iterator it = reg.loaderMap.find(type);
+	assert(it != reg.loaderMap.end());
+	return it->second->load(ar, id, args);
 }
 
 template class PolymorphicLoaderRegistry<MemInputArchive>;
@@ -80,6 +115,12 @@ template class PolymorphicLoaderRegistry<XmlInputArchive>;
 
 ////
 
+void polyInitError(const char* expected, const char* actual)
+{
+	throw MSXException(StringOp::Builder() <<
+		"Expected type: " << expected << " but got: " << actual << ".");
+}
+
 template<typename Archive>
 PolymorphicInitializerRegistry<Archive>::PolymorphicInitializerRegistry()
 {
@@ -102,12 +143,32 @@ PolymorphicInitializerRegistry<Archive>& PolymorphicInitializerRegistry<Archive>
 }
 
 template<typename Archive>
-const PolymorphicInitializerBase<Archive>&
-PolymorphicInitializerRegistry<Archive>::getInitializer(const std::string& type)
+void PolymorphicInitializerRegistry<Archive>::registerHelper(
+	const char* name_, PolymorphicInitializerBase<Archive>* initializer)
 {
-	typename InitializerMap::const_iterator it = initializerMap.find(type);
-	assert(it != initializerMap.end());
-	return *it->second;
+	std::string name(name_);
+	assert(initializerMap.find(name) == initializerMap.end());
+	initializerMap[name] = initializer;
+}
+
+template<typename Archive>
+void PolymorphicInitializerRegistry<Archive>::init(
+	const char* tag, Archive& ar, void* t)
+{
+	ar.beginTag(tag);
+	unsigned id;
+	ar.attribute("id", id);
+	assert(id);
+	std::string type;
+	ar.attribute("type", type);
+
+	PolymorphicInitializerRegistry<Archive>& reg =
+		PolymorphicInitializerRegistry<Archive>::instance();
+	typename InitializerMap::const_iterator it = reg.initializerMap.find(type);
+	assert(it != reg.initializerMap.end());
+	it->second->init(ar, t, id);
+
+	ar.endTag(tag);
 }
 
 template class PolymorphicInitializerRegistry<MemInputArchive>;
