@@ -1,4 +1,4 @@
-// $Id: FilenameSetting.cc 7924 2008-06-28 19:14:09Z m9710797 $
+// $Id: FilenameSetting.cc 10226 2009-07-17 11:09:58Z m9710797 $
 
 #include "FilenameSetting.hh"
 #include "Completer.hh"
@@ -34,4 +34,14 @@ FilenameSetting::FilenameSetting(
 {
 }
 
+FilenameSetting::FilenameSetting(
+		CommandController& commandController,
+		const char* name, const char* description,
+		const char* initialValue)
+	: SettingImpl<FilenameSettingPolicy>(
+		commandController, name, description, initialValue,
+		Setting::SAVE)
+{
+}
+
 } // namespace openmsx
