@@ -1,4 +1,4 @@
-// $Id: SettingImpl.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: SettingImpl.cc 10709 2009-10-25 16:31:28Z mthuurne $
 
 #include "SettingImpl.hh"
 #include "SettingsConfig.hh"
@@ -22,10 +22,9 @@ SettingImplBase::SettingImplBase(
 
 void SettingImplBase::init()
 {
-	CommandController& commandController = Setting::getCommandController();
-	XMLElement& settingsConfig =
-		commandController.getSettingsConfig().getXMLElement();
 	if (needLoadSave()) {
+		XMLElement& settingsConfig = Setting::getGlobalCommandController()
+			.getSettingsConfig().getXMLElement();
 		const XMLElement* config = settingsConfig.findChild("settings");
 		if (config) {
 			const XMLElement* elem = config->findChildWithAttribute(
@@ -39,7 +38,7 @@ void SettingImplBase::init()
 			}
 		}
 	}
-	commandController.registerSetting(*this);
+	Setting::getCommandController().registerSetting(*this);
 
 	// This is needed to for example inform catapult of the new setting
 	// value when a setting was destroyed/recreated (by a machine switch
@@ -49,32 +48,27 @@ void SettingImplBase::init()
 
 void SettingImplBase::destroy()
 {
-	CommandController& commandController = Setting::getCommandController();
-	sync(commandController.getSettingsConfig().getXMLElement());
-	commandController.unregisterSetting(*this);
+	sync(Setting::getGlobalCommandController()
+		.getSettingsConfig().getXMLElement());
+	Setting::getCommandController().unregisterSetting(*this);
 }
 
 void SettingImplBase::syncProxy()
 {
 	MSXCommandController* controller =
-	    dynamic_cast<MSXCommandController*>(&Setting::getCommandController());
+		dynamic_cast<MSXCommandController*>(&Setting::getCommandController());
 	if (!controller) {
-		// not a machine specific setting
+		// This is not a machine specific setting.
 		return;
 	}
-	GlobalCommandController& globalController =
-		controller->getGlobalCommandController();
-	MSXMotherBoard* mb = globalController.getReactor().getMotherBoard();
-	if (!mb) {
-		// no active MSXMotherBoard
-		return;
-	}
-	if (&mb->getMSXCommandController() != controller) {
-		// this setting does not belong to active MSXMotherBoard
+	if (!controller->isActive()) {
+		// This setting does not belong to the active machine.
 		return;
 	}
 
-	// Tcl already makes sure this doesn't result in an endless loop
+	GlobalCommandController& globalController =
+		controller->getGlobalCommandController();
+	// Tcl already makes sure this doesn't result in an endless loop.
 	globalController.changeSetting(getName(), getValueString());
 }
 
