@@ -1,4 +1,4 @@
-// $Id: StringSetting.cc 6330 2007-04-21 08:23:51Z m9710797 $
+// $Id: StringSetting.cc 10226 2009-07-17 11:09:58Z m9710797 $
 
 #include "StringSetting.hh"
 
@@ -38,4 +38,12 @@ StringSetting::StringSetting(CommandController& commandController,
 {
 }
 
+StringSetting::StringSetting(CommandController& commandController,
+                             const char* name, const char* description,
+                             const char* initialValue, SaveSetting save)
+	: SettingImpl<StringSettingPolicy>(
+		commandController, name, description, initialValue, save)
+{
+}
+
 } // namespace openmsx
