@@ -1,4 +1,4 @@
-// $Id: AY8910.cc 9289 2009-02-28 19:28:15Z mfeingol $
+// $Id: AY8910.cc 10628 2009-09-30 21:17:12Z seanyoung $
 
 /*
  * Emulation of the AY-3-8910
@@ -13,6 +13,8 @@
 #include "AY8910.hh"
 #include "AY8910Periphery.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "CliComm.hh"
 #include "SimpleDebuggable.hh"
 #include "XMLElement.hh"
@@ -482,7 +484,7 @@ inline void AY8910::Envelope::advanceFast(unsigned duration)
 AY8910::AY8910(MSXMotherBoard& motherBoard, AY8910Periphery& periphery_,
                const XMLElement& config, EmuTime::param time)
 	: SoundDevice(motherBoard.getMSXMixer(), "PSG", "PSG", 3)
-	, Resample(motherBoard.getGlobalSettings(), 1)
+	, Resample(motherBoard.getReactor().getGlobalSettings().getResampleSetting())
 	, cliComm(motherBoard.getMSXCliComm())
 	, periphery(periphery_)
 	, debuggable(new AY8910Debuggable(motherBoard, *this))
@@ -667,7 +669,7 @@ void AY8910::wrtReg(unsigned reg, byte value, EmuTime::param time)
 void AY8910::setOutputRate(unsigned sampleRate)
 {
 	setInputRate(NATIVE_FREQ_INT);
-	setResampleRatio(NATIVE_FREQ_DOUBLE, sampleRate);
+	setResampleRatio(NATIVE_FREQ_DOUBLE, sampleRate, isStereo());
 }
 
 static void addFill(int*& buf, int val, unsigned num)
