@@ -1,4 +1,4 @@
-// $Id: BlipBuffer.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: BlipBuffer.cc 10114 2009-06-30 18:30:38Z m9710797 $
 
 #include "BlipBuffer.hh"
 #include <algorithm>
@@ -121,7 +121,7 @@ static inline int sign(int x)
 	return (x != 0) | (x >> 31);
 }
 
-bool BlipBuffer::readSamples(int* out, unsigned samples, unsigned pitch)
+bool BlipBuffer::readSamples(int* __restrict out, unsigned samples, unsigned pitch)
 {
 	static const int SAMPLE_SHIFT = BLIP_SAMPLE_BITS - 16;
 	static const int BASS_SHIFT = 9;
