@@ -1,4 +1,4 @@
-// $Id: DummyAudioInputDevice.cc 4836 2005-11-11 20:53:06Z m9710797 $
+// $Id: DummyAudioInputDevice.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "DummyAudioInputDevice.hh"
 
@@ -11,15 +11,15 @@ const std::string& DummyAudioInputDevice::getDescription() const
 }
 
 void DummyAudioInputDevice::plugHelper(Connector& /*connector*/,
-                                       const EmuTime& /*time*/)
+                                       EmuTime::param /*time*/)
 {
 }
 
-void DummyAudioInputDevice::unplugHelper(const EmuTime& /*time*/)
+void DummyAudioInputDevice::unplugHelper(EmuTime::param /*time*/)
 {
 }
 
-short DummyAudioInputDevice::readSample(const EmuTime& /*time*/)
+short DummyAudioInputDevice::readSample(EmuTime::param /*time*/)
 {
 	return 0; // silence
 }
