@@ -1,4 +1,4 @@
-// $Id: MSXMixer.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: MSXMixer.cc 11367 2010-03-26 01:52:30Z mthuurne $
 
 #include "MSXMixer.hh"
 #include "Mixer.hh"
@@ -18,6 +18,8 @@
 #include "Math.hh"
 #include "StringOp.hh"
 #include "vla.hh"
+#include "unreachable.hh"
+#include "build-info.hh"
 #include <algorithm>
 #include <cmath>
 #include <cstring>
@@ -44,15 +46,14 @@ private:
 
 
 MSXMixer::MSXMixer(Mixer& mixer_, Scheduler& scheduler,
-                   MSXCommandController& msxCommandController_)
+                   MSXCommandController& msxCommandController_,
+                   GlobalSettings& globalSettings)
 	: Schedulable(scheduler)
 	, mixer(mixer_)
 	, commandController(msxCommandController_)
 	, masterVolume(mixer.getMasterVolume())
-	, speedSetting(
-		commandController.getGlobalSettings().getSpeedSetting())
-	, throttleManager(
-		commandController.getGlobalSettings().getThrottleManager())
+	, speedSetting(globalSettings.getSpeedSetting())
+	, throttleManager(globalSettings.getThrottleManager())
 	, prevTime(EmuTime::zero)
 	, soundDeviceInfo(new SoundDeviceInfoTopic(
 	              msxCommandController_.getMachineInfoCommand(), *this))
@@ -104,7 +105,7 @@ void MSXMixer::registerSound(SoundDevice& device, double volume,
 
 	for (unsigned i = 0; i < numChannels; ++i) {
 		SoundDeviceInfo::ChannelSettings channelSettings;
-		string ch_name = name + "_ch" + StringOp::toString(i + 1);
+		string ch_name = StringOp::Builder() << name << "_ch" << i + 1;
 
 		channelSettings.recordSetting = new StringSetting(
 			commandController, ch_name + "_record",
@@ -217,17 +218,22 @@ void MSXMixer::generate(short* output, unsigned samples,
 
 	VLA(int, stereoBuf, 2 * samples + 3);
 	VLA(int, monoBuf, samples + 3);
+#if ASM_X86
 	VLA_ALIGNED(int, tmpBuf, 2 * samples + 3, 16);
+#else
+	VLA(int, tmpBuf, 2 * samples + 3);
+#endif
 
 	static const unsigned HAS_MONO_FLAG = 1;
 	static const unsigned HAS_STEREO_FLAG = 2;
 	unsigned usedBuffers = 0;
 
+	// FIXME: The Infos should be ordered such that all the mono
+	// devices are handled first
 	for (Infos::const_iterator it = infos.begin();
 	     it != infos.end(); ++it) {
 		SoundDevice& device = *it->first;
-		if (device.updateBuffer(samples, tmpBuf,
-					start, sampDur)) {
+		if (device.updateBuffer(samples, tmpBuf, start, sampDur)) {
 			if (!device.isStereo()) {
 				int l1 = it->second.left1;
 				int r1 = it->second.right1;
@@ -476,6 +482,18 @@ void MSXMixer::generate(short* output, unsigned samples,
 	}
 }
 
+bool MSXMixer::anyStereoDevice() const
+{
+	for (Infos::const_iterator it = infos.begin();
+	     it != infos.end(); ++it) {
+		SoundDevice& device = *it->first;
+		if (device.isStereo()) {
+			return true;
+		}
+	}
+	return false;
+}
+
 void MSXMixer::mute()
 {
 	if (muteCount == 0) {
@@ -580,7 +598,7 @@ void MSXMixer::update(const Setting& setting)
 	} else if (dynamic_cast<const BooleanSetting*>(&setting)) {
 		changeMuteSetting(setting);
 	} else {
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -600,7 +618,7 @@ void MSXMixer::changeRecordSetting(const Setting& setting)
 			}
 		}
 	}
-	assert(false);
+	UNREACHABLE;
 }
 
 void MSXMixer::changeMuteSetting(const Setting& setting)
@@ -618,7 +636,7 @@ void MSXMixer::changeMuteSetting(const Setting& setting)
 			}
 		}
 	}
-	assert(false);
+	UNREACHABLE;
 }
 
 void MSXMixer::update(const ThrottleManager& /*throttleManager*/)
