@@ -1,4 +1,4 @@
-// $Id: MSXSCCPlusCart.cc 9276 2009-02-28 06:58:00Z mfeingol $
+// $Id: MSXSCCPlusCart.cc 10112 2009-06-30 18:29:35Z m9710797 $
 
 // Note: this device is actually called SCC-I. But this would take a lot of
 // renaming, which isn't worth it right now. TODO rename this :)
@@ -61,13 +61,19 @@ MSXSCCPlusCart::MSXSCCPlusCart(MSXMotherBoard& motherBoard,
 		mapper[i] = 0;
 	}
 
-	reset(getCurrentTime());
+	powerUp(getCurrentTime());
 }
 
 MSXSCCPlusCart::~MSXSCCPlusCart()
 {
 }
 
+void MSXSCCPlusCart::powerUp(EmuTime::param time)
+{
+	scc->powerUp(time);
+	reset(time);
+}
+
 void MSXSCCPlusCart::reset(EmuTime::param time)
 {
 	setModeRegister(0);
@@ -86,7 +92,7 @@ byte MSXSCCPlusCart::readMem(word addr, EmuTime::param time)
 	    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
 		result = scc->readMem(addr & 0xFF, time);
 	} else {
-		result = peekMem(addr, time);
+		result = MSXSCCPlusCart::peekMem(addr, time);
 	}
 	//PRT_DEBUG("SCC+ read "<< hex << (int)addr << " " << (int)result << dec);
 	return result;
