@@ -1,10 +1,11 @@
-// $Id: ResampleBlip.cc 9571 2009-04-13 19:37:05Z mthuurne $
+// $Id: ResampleBlip.cc 11367 2010-03-26 01:52:30Z mthuurne $
 
 #include "ResampleBlip.hh"
 #include "Resample.hh"
 #include "static_assert.hh"
 #include "likely.hh"
 #include "vla.hh"
+#include "build-info.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -30,7 +31,11 @@ bool ResampleBlip<CHANNELS>::generateOutput(int* dataOut, unsigned num)
 	int required = int(ceil(len - lastPos));
 	if (required > 0) {
 		// 3 extra for padding, CHANNELS extra for sentinel
+#if ASM_X86
 		VLA_ALIGNED(int, buf, required * CHANNELS + std::max(3u, CHANNELS), 16);
+#else
+		VLA(int, buf, required * CHANNELS + std::max(3u, CHANNELS));
+#endif
 		if (input.generateInput(buf, required)) {
 			for (unsigned ch = 0; ch < CHANNELS; ++ch) {
 				// In case of PSG (and to a lesser degree SCC) it happens
