@@ -19,6 +19,7 @@
 #include "HostCPU.hh"
 #include "noncopyable.hh"
 #include "vla.hh"
+#include "build-info.hh"
 #include <algorithm>
 #include <map>
 #include <cmath>
@@ -45,7 +46,7 @@ static const unsigned TAB_LEN = 4096;
 extern "C"
 {
 	void __cdecl ResampleHQ_calcOutput_1_SSE(
-		const void* bufferOffset, const void* tableOffset, 
+		const void* bufferOffset, const void* tableOffset,
 		void* output, long filterLen16Product, unsigned filterLenRest);
 }
 #endif
@@ -194,7 +195,7 @@ ResampleHQ<CHANNELS>::~ResampleHQ()
 }
 
 template <unsigned CHANNELS>
-void ResampleHQ<CHANNELS>::calcOutput(float lastPos, int* output)
+void ResampleHQ<CHANNELS>::calcOutput(float lastPos, int* __restrict output)
 {
 	assert((filterLen & 3) == 0);
 	int t = int(lastPos * TAB_LEN + 0.5f) % TAB_LEN;
@@ -202,7 +203,7 @@ void ResampleHQ<CHANNELS>::calcOutput(float lastPos, int* output)
 	int tabIdx = t * filterLen;
 	int bufIdx = bufStart * CHANNELS;
 
-	#if defined(ASM_X86) && !defined(__APPLE__)
+	#if ASM_X86 && !defined(__APPLE__)
 	// On Mac OS X, we are one register short, because EBX is not available.
 	// We disable this piece of assembly and fall back to the C++ code.
 	const HostCPU& cpu = HostCPU::getInstance();
@@ -403,7 +404,11 @@ void ResampleHQ<CHANNELS>::prepareData(unsigned request)
 		bufEnd = available;
 		missing = std::min(missing, BUF_LEN - bufEnd);
 	}
+#if ASM_X86
 	VLA_ALIGNED(int, tmpBuf, missing * CHANNELS + 3, 16);
+#else
+	VLA(int, tmpBuf, missing * CHANNELS + 3);
+#endif
 	if (input.generateInput(tmpBuf, missing)) {
 		for (unsigned i = 0; i < missing * CHANNELS; ++i) {
 			buffer[bufEnd * CHANNELS + i] = float(tmpBuf[i]);
@@ -421,7 +426,7 @@ void ResampleHQ<CHANNELS>::prepareData(unsigned request)
 }
 
 template <unsigned CHANNELS>
-bool ResampleHQ<CHANNELS>::generateOutput(int* dataOut, unsigned num)
+bool ResampleHQ<CHANNELS>::generateOutput(int* __restrict dataOut, unsigned num)
 {
 	bool anyNonZero = false;
 
