@@ -1,4 +1,4 @@
-// $Id: SCC.cc 9300 2009-03-01 01:03:07Z mfeingol $
+// $Id: SCC.cc 10628 2009-09-30 21:17:12Z seanyoung $
 
 //-----------------------------------------------------------------------------
 //
@@ -101,8 +101,11 @@
 #include "SCC.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "serialize.hh"
 #include "likely.hh"
+#include "unreachable.hh"
 
 using std::string;
 
@@ -127,10 +130,25 @@ static string calcDescription(SCC::ChipMode mode)
 SCC::SCC(MSXMotherBoard& motherBoard, const string& name,
          const XMLElement& config, EmuTime::param time, ChipMode mode)
 	: SoundDevice(motherBoard.getMSXMixer(), name, calcDescription(mode), 5)
-	, Resample(motherBoard.getGlobalSettings(), 1)
+	, Resample(motherBoard.getReactor().getGlobalSettings().getResampleSetting())
 	, debuggable(new SCCDebuggable(motherBoard, *this))
 	, deformTimer(time)
 	, currentChipMode(mode)
+{
+	// Make valgrind happy
+	for (int i = 0; i < 5; ++i) {
+		orgPeriod[i] = 0;
+	}
+	powerUp(time);
+	registerSound(config);
+}
+
+SCC::~SCC()
+{
+	unregisterSound();
+}
+
+void SCC::powerUp(EmuTime::param time)
 {
 	// Power on values, tested by enen (log from IRC #openmsx):
 	//
@@ -141,12 +159,9 @@ SCC::SCC(MSXMotherBoard& motherBoard, const string& name,
 	//    ...
 	//  <enen>    filled with $FF, some bits cleared but that seems random
 
-	// Make valgrind happy
-	for (int i = 0; i < 5; ++i) {
-		orgPeriod[i] = 0;
-	}
 	// Initialize ch_enable, deform (initialize this before period)
 	reset(time);
+
 	// Initialize waveform (initialize before volumes)
 	for (unsigned i = 0; i < 5; ++i) {
 		for (unsigned j = 0; j < 32; ++j) {
@@ -166,13 +181,6 @@ SCC::SCC(MSXMotherBoard& motherBoard, const string& name,
 	for (int i = 0; i < 2 * 5; ++i) {
 		setFreqVol(i, 0, time);
 	}
-
-	registerSound(config);
-}
-
-SCC::~SCC()
-{
-	unregisterSound();
 }
 
 void SCC::reset(EmuTime::param /*time*/)
@@ -189,7 +197,7 @@ void SCC::setOutputRate(unsigned sampleRate)
 {
 	double input = 3579545.0 / 32;
 	setInputRate(int(input + 0.5));
-	setResampleRatio(input, sampleRate);
+	setResampleRatio(input, sampleRate, isStereo());
 }
 
 void SCC::setChipMode(ChipMode newMode)
@@ -258,8 +266,7 @@ byte SCC::peekMem(byte address, EmuTime::param time) const
 		}
 		break;
 	default:
-		assert(false);
-		result = 0xFF;
+		UNREACHABLE; return 0;
 	}
 	//PRT_DEBUG("SCC: read " << (int)address << " " << (int)result);
 	return result;
@@ -351,7 +358,7 @@ void SCC::writeMem(byte address, byte value, EmuTime::param time)
 		}
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -475,7 +482,7 @@ void SCC::setDeformRegHelper(byte value)
 		}
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
