@@ -1,7 +1,9 @@
-// $Id: SamplePlayer.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: SamplePlayer.cc 10628 2009-09-30 21:17:12Z seanyoung $
 
 #include "SamplePlayer.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "WavData.hh"
 #include "MSXCliComm.hh"
 #include "FileContext.hh"
@@ -16,7 +18,7 @@ SamplePlayer::SamplePlayer(MSXMotherBoard& motherBoard, const std::string& name,
                            const std::string& desc, const XMLElement& config,
                            const std::string& samplesBaseName, unsigned numSamples)
 	: SoundDevice(motherBoard.getMSXMixer(), name, desc, 1)
-	, Resample(motherBoard.getGlobalSettings(), 1)
+	, Resample(motherBoard.getReactor().getGlobalSettings().getResampleSetting())
 	, inFreq(44100)
 {
 	bool alreadyWarned = false;
@@ -24,8 +26,8 @@ SamplePlayer::SamplePlayer(MSXMotherBoard& motherBoard, const std::string& name,
 	for (unsigned i = 0; i < numSamples; ++i) {
 		try {
 			SystemFileContext context;
-			std::string filename =
-				samplesBaseName + StringOp::toString(i) + ".wav";
+			std::string filename = StringOp::Builder() <<
+				samplesBaseName << i << ".wav";
 			samples[i].reset(new WavData(context.resolve(
 				motherBoard.getCommandController(), filename)));
 		} catch (MSXException& e) {
@@ -63,7 +65,7 @@ void SamplePlayer::setOutputRate(unsigned outFreq_)
 {
 	outFreq = outFreq_;
 	setInputRate(inFreq);
-	setResampleRatio(inFreq, outFreq);
+	setResampleRatio(inFreq, outFreq, isStereo());
 }
 
 void SamplePlayer::play(unsigned sampleNum)
