@@ -1,4 +1,4 @@
-// $Id: VLM5030.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: VLM5030.cc 10628 2009-09-30 21:17:12Z seanyoung $
 
 /*
     vlm5030.c
@@ -81,6 +81,8 @@ chirp 12-..: vokume   0   : silent
 #include "Resample.hh"
 #include "Rom.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "XMLElement.hh"
 #include "FileContext.hh"
 #include "serialize.hh"
@@ -585,7 +587,7 @@ void VLM5030Impl::setST(bool pin)
 VLM5030Impl::VLM5030Impl(MSXMotherBoard& motherBoard, const std::string& name,
                  const std::string& desc, const XMLElement& config)
 	: SoundDevice(motherBoard.getMSXMixer(), name, desc, 1)
-	, Resample(motherBoard.getGlobalSettings(), 1)
+	, Resample(motherBoard.getReactor().getGlobalSettings().getResampleSetting())
 {
 	XMLElement voiceROMconfig(name);
 	voiceROMconfig.addAttribute("id", "name");
@@ -622,7 +624,7 @@ void VLM5030Impl::setOutputRate(unsigned sampleRate)
        const int CLOCK_FREQ = 3579545;
        double input = CLOCK_FREQ / 440.0;
        setInputRate(int(input + 0.5));
-       setResampleRatio(input, sampleRate);
+       setResampleRatio(input, sampleRate, isStereo());
 }
 
 bool VLM5030Impl::generateInput(int* buffer, unsigned length)
