@@ -1,4 +1,4 @@
-// $Id: WavAudioInput.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: WavAudioInput.cc 10736 2009-11-02 19:25:01Z m9710797 $
 
 #include "WavAudioInput.hh"
 #include "CommandController.hh"
@@ -68,7 +68,7 @@ void WavAudioInput::update(const Setting& setting)
 {
 	(void)setting;
 	assert(&setting == audioInputFilenameSetting.get());
-	if (getConnector()) {
+	if (isPluggedIn()) {
 		try {
 			loadWave();
 		} catch (MSXException& e) {
