@@ -1,4 +1,4 @@
-// $Id: $
+// $Id: WavData.cc 11190 2010-01-25 00:47:05Z mthuurne $
 
 #include "WavData.hh"
 #include "MSXException.hh"
@@ -26,6 +26,7 @@ WavData::WavData(const string& filename, unsigned wantedBits, unsigned wantedFre
 	freq = (wantedFreq == 0) ? wavSpec.freq : wantedFreq;
 	bits = (wantedBits == 0) ? (is8Bit(wavSpec.format) ? 8 : 16)
 	                         : wantedBits;
+	channels = wavSpec.channels;
 	assert((bits == 8) || (bits == 16));
 	Uint16 format = (bits == 8) ? AUDIO_U8 : AUDIO_S16SYS;
 
@@ -63,6 +64,11 @@ unsigned WavData::getSize() const
 	return length;
 }
 
+unsigned WavData::getChannels() const
+{
+	return channels;
+}
+
 const void* WavData::getData() const
 {
 	return &buffer[0];
