@@ -1,4 +1,4 @@
-// $Id: Y8950.cc 9329 2009-03-03 20:23:24Z m9710797 $
+// $Id: Y8950.cc 10719 2009-10-26 23:58:11Z mthuurne $
 
 /*
   * Based on:
@@ -10,12 +10,15 @@
 #include "Y8950Adpcm.hh"
 #include "Y8950KeyboardConnector.hh"
 #include "Y8950Periphery.hh"
+#include "MSXAudio.hh"
 #include "SoundDevice.hh"
 #include "EmuTimer.hh"
 #include "Resample.hh"
 #include "SimpleDebuggable.hh"
 #include "IRQHelper.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "DACSound16S.hh"
 #include "FixedPoint.hh"
 #include "Math.hh"
@@ -142,11 +145,12 @@ class Y8950Impl : private SoundDevice, private EmuTimerCallback, private Resampl
 public:
 	Y8950Impl(Y8950& self, MSXMotherBoard& motherBoard,
 	          const std::string& name, const XMLElement& config,
-	          unsigned sampleRam, Y8950Periphery& perihery);
+	          unsigned sampleRam, MSXAudio& audio);
 	void init(const XMLElement& config, EmuTime::param time);
 	virtual ~Y8950Impl();
 
 	void setEnabled(bool enabled, EmuTime::param time);
+	void clearRam();
 	void reset(EmuTime::param time);
 	void writeReg(byte reg, byte data, EmuTime::param time);
 	byte readReg(byte reg, EmuTime::param time);
@@ -192,7 +196,7 @@ private:
 
 
 	MSXMotherBoard& motherBoard;
-	Y8950Periphery& perihery;
+	Y8950Periphery& periphery;
 	const std::auto_ptr<Y8950Adpcm> adpcm;
 	const std::auto_ptr<Y8950KeyboardConnector> connector;
 	const std::auto_ptr<DACSound16S> dac13; // 13-bit (exponential) DAC
@@ -239,11 +243,6 @@ static const double PM_SPEED  = 6.4;
 static const double PM_DEPTH  = 13.75 / 2;
 static const double PM_DEPTH2 = 13.75;
 
-// AM speed(Hz) and depth(dB)
-static const double AM_SPEED  = 3.7;
-static const double AM_DEPTH  = 1.0;
-static const double AM_DEPTH2 = 4.8;
-
 // Dynamic range of sustine level
 static const int SL_BITS = 4;
 static const int SL_MUTE = 1 << SL_BITS;
@@ -254,9 +253,6 @@ static const int PG_MASK = PG_WIDTH - 1;
 // Phase increment counter
 static const int DP_BITS = 19;
 static const int DP_BASE_BITS = DP_BITS - PG_BITS;
-// Dynamic range of total level
-static const int TL_BITS = 6;
-static const int TL_MUTE = 1 << TL_BITS;
 
 // WaveTable for each envelope amp.
 //  values are in range[        0,   DB_MUTE)   (for positive values)
@@ -269,10 +265,7 @@ static EnvPhaseIndex dphaseARTable[16][16];
 static EnvPhaseIndex dphaseDRTable[16][16];
 
 // TL Table.
-static int tllTable[16 * 8][1 << TL_BITS][4];
-
-// Phase incr table for PG.
-static unsigned dphaseTable[1024 * 8][16];
+static int tllTable[16 * 8][4];
 
 // Liner to Log curve conversion table (for Attack rate).
 //   values are in the range [0 .. EG_MUTE]
@@ -303,14 +296,84 @@ static const int AM_DP_WIDTH = 1 << AM_DP_BITS;
 
 // LFO Table
 static const unsigned PM_DPHASE = unsigned(PM_SPEED * PM_DP_WIDTH / (Y8950::CLOCK_FREQ / double(Y8950::CLOCK_FREQ_DIV)));
-static const unsigned AM_DPHASE = unsigned(AM_SPEED * AM_DP_WIDTH / (Y8950::CLOCK_FREQ / double(Y8950::CLOCK_FREQ_DIV)));
 static int pmtable[2][PM_PG_WIDTH];
-static int amtable[2][AM_PG_WIDTH];
 
 // dB to Liner table
 static int dB2LinTab[(2 * DB_MUTE) * 2];
 
 
+// LFO Amplitude Modulation table (verified on real YM3812)
+// 27 output levels (triangle waveform);
+// 1 level takes one of: 192, 256 or 448 samples
+//
+// Length: 210 elements.
+//  Each of the elements has to be repeated
+//  exactly 64 times (on 64 consecutive samples).
+//  The whole table takes: 64 * 210 = 13440 samples.
+//
+// Verified on real YM3812 (OPL2), but I believe it's the same for Y8950
+// because it closely matches the Y8950 AM parameters:
+//    speed = 3.7Hz
+//    depth = 4.875dB
+// Also this approch can be easily implemented in HW, the previous one (see SVN
+// history) could not.
+static const unsigned LFO_AM_TAB_ELEMENTS = 210;
+static const byte lfo_am_table[LFO_AM_TAB_ELEMENTS] =
+{
+	0,0,0,0,0,0,0,
+	1,1,1,1,
+	2,2,2,2,
+	3,3,3,3,
+	4,4,4,4,
+	5,5,5,5,
+	6,6,6,6,
+	7,7,7,7,
+	8,8,8,8,
+	9,9,9,9,
+	10,10,10,10,
+	11,11,11,11,
+	12,12,12,12,
+	13,13,13,13,
+	14,14,14,14,
+	15,15,15,15,
+	16,16,16,16,
+	17,17,17,17,
+	18,18,18,18,
+	19,19,19,19,
+	20,20,20,20,
+	21,21,21,21,
+	22,22,22,22,
+	23,23,23,23,
+	24,24,24,24,
+	25,25,25,25,
+	26,26,26,
+	25,25,25,25,
+	24,24,24,24,
+	23,23,23,23,
+	22,22,22,22,
+	21,21,21,21,
+	20,20,20,20,
+	19,19,19,19,
+	18,18,18,18,
+	17,17,17,17,
+	16,16,16,16,
+	15,15,15,15,
+	14,14,14,14,
+	13,13,13,13,
+	12,12,12,12,
+	11,11,11,11,
+	10,10,10,10,
+	9,9,9,9,
+	8,8,8,8,
+	7,7,7,7,
+	6,6,6,6,
+	5,5,5,5,
+	4,4,4,4,
+	3,3,3,3,
+	2,2,2,2,
+	1,1,1,1
+};
+
 //**************************************************//
 //                                                  //
 //  Helper functions                                //
@@ -400,54 +463,19 @@ static void makePmTable()
 	}
 }
 
-// Table for Amp Modulator
-static void makeAmTable()
-{
-	for (int i = 0; i < AM_PG_WIDTH; ++i) {
-		amtable[0][i] = int(double(AM_DEPTH)  / 2 / DB_STEP * (1.0 + sin(2.0 * M_PI * i / PM_PG_WIDTH)));
-		amtable[1][i] = int(double(AM_DEPTH2) / 2 / DB_STEP * (1.0 + sin(2.0 * M_PI * i / PM_PG_WIDTH)));
-	}
-}
-
-// Phase increment counter table
-static void makeDphaseTable()
-{
-	int mltable[16] = {
-		  1, 1*2,  2*2,  3*2,  4*2,  5*2,  6*2 , 7*2,
-		8*2, 9*2, 10*2, 10*2, 12*2, 12*2, 15*2, 15*2
-	};
-
-	for (unsigned freq = 0; freq < 8 * 1024; ++freq) {
-		unsigned fnum  = freq % 1024;
-		unsigned block = freq / 1024;
-		for (int ML = 0; ML < 16; ++ML) {
-			dphaseTable[freq][ML] =
-				((fnum * mltable[ML]) << block) >> (21 - DP_BITS);
-		}
-	}
-}
-
 static void makeTllTable()
 {
-	#define dB2(x) int((x) * 2)
-	static int kltable[16] = {
-		dB2( 0.000), dB2( 9.000), dB2(12.000), dB2(13.875),
-		dB2(15.000), dB2(16.125), dB2(16.875), dB2(17.625),
-		dB2(18.000), dB2(18.750), dB2(19.125), dB2(19.500),
-		dB2(19.875), dB2(20.250), dB2(20.625), dB2(21.000)
+	// Processed version of Table 3.5 from the Application Manual
+	static const unsigned kltable[16] = {
+		0, 24, 32, 37, 40, 43, 45, 47, 48, 50, 51, 52, 53, 54, 55, 56
 	};
 
 	for (unsigned freq = 0; freq < 16 * 8; ++freq) {
 		unsigned fnum  = freq % 16;
 		unsigned block = freq / 16;
-		for (unsigned TL = 0; TL < 64; ++TL) {
-			tllTable[freq][TL][0] = TL * TL_PER_EG;
-			for (unsigned KL = 1; KL < 4; ++KL) {
-				int tmp = kltable[fnum] - dB2(3.000) * (7 - block);
-				tllTable[freq][TL][KL] = (tmp <= 0)
-					? TL * TL_PER_EG
-					: int((tmp >> (3 - KL)) / EG_STEP) + (TL * TL_PER_EG);
-			}
+		int tmp = 4 * kltable[fnum] - 32 * (7 - block);
+		for (unsigned KL = 0; KL < 4; ++KL) {
+			tllTable[freq][KL] = (tmp <= 0 || KL == 0) ? 0 : (tmp >> (3 - KL));
 		}
 	}
 }
@@ -525,12 +553,19 @@ void Y8950Slot::reset()
 
 void Y8950Slot::updatePG(unsigned freq)
 {
-	dphase = dphaseTable[freq][patch.ML];
+	static const int mltable[16] = {
+		  1, 1*2,  2*2,  3*2,  4*2,  5*2,  6*2 , 7*2,
+		8*2, 9*2, 10*2, 10*2, 12*2, 12*2, 15*2, 15*2
+	};
+
+	unsigned fnum  = freq % 1024;
+	unsigned block = freq / 1024;
+	dphase = ((fnum * mltable[patch.ML]) << block) >> (21 - DP_BITS);
 }
 
 void Y8950Slot::updateTLL(unsigned freq)
 {
-	tll = tllTable[freq >> 6][patch.TL][patch.KL];
+	tll = tllTable[freq >> 6][patch.KL] + patch.TL * TL_PER_EG;
 }
 
 void Y8950Slot::updateRKS(unsigned freq)
@@ -636,11 +671,11 @@ void Y8950Channel::keyOff()
 
 Y8950Impl::Y8950Impl(Y8950& self, MSXMotherBoard& motherBoard_,
                      const std::string& name, const XMLElement& config,
-                     unsigned sampleRam, Y8950Periphery& perihery_)
-	: SoundDevice(motherBoard_.getMSXMixer(), name, "MSX-AUDIO", 12)
-	, Resample(motherBoard_.getGlobalSettings(), 1)
+                     unsigned sampleRam, MSXAudio& audio)
+	: SoundDevice(motherBoard_.getMSXMixer(), name, "MSX-AUDIO", 9 + 5 + 1)
+	, Resample(motherBoard_.getReactor().getGlobalSettings().getResampleSetting())
 	, motherBoard(motherBoard_)
-	, perihery(perihery_)
+	, periphery(audio.createPeriphery(getName()))
 	, adpcm(new Y8950Adpcm(self, motherBoard, name, sampleRam))
 	, connector(new Y8950KeyboardConnector(motherBoard.getPluggingController()))
 	, dac13(new DACSound16S(motherBoard.getMSXMixer(), name + " DAC",
@@ -660,13 +695,11 @@ Y8950Impl::Y8950Impl(Y8950& self, MSXMotherBoard& motherBoard_,
 void Y8950Impl::init(const XMLElement& config, EmuTime::param time)
 {
 	makePmTable();
-	makeAmTable();
 	makeAdjustTable();
 	makeDB2LinTable();
 	makeTllTable();
 	makeSinTable();
 
-	makeDphaseTable();
 	makeDphaseARTable();
 	makeDphaseDRTable();
 
@@ -683,7 +716,12 @@ void Y8950Impl::setOutputRate(unsigned sampleRate)
 {
 	double input = Y8950::CLOCK_FREQ / double(Y8950::CLOCK_FREQ_DIV);
 	setInputRate(int(input + 0.5));
-	setResampleRatio(input, sampleRate);
+	setResampleRatio(input, sampleRate, isStereo());
+}
+
+void Y8950Impl::clearRam()
+{
+	adpcm->clearRam();
 }
 
 // Reset whole of opl except patch datas.
@@ -928,20 +966,28 @@ bool Y8950Impl::checkMuteHelper()
 
 void Y8950Impl::generateChannels(int** bufs, unsigned num)
 {
+	// TODO implement per-channel mute (instead of all-or-nothing)
 	if (checkMuteHelper()) {
 		// TODO update internal state even when muted
-		for (int i = 0; i < 12; ++i) {
+		// during mute pm_phase, am_phase, noiseA_phase, noiseB_phase
+		// and noise_seed aren't updated, probably ok
+		for (int i = 0; i < 9 + 5 + 1; ++i) {
 			bufs[i] = 0;
 		}
 		return;
 	}
 
 	for (unsigned sample = 0; sample < num; ++sample) {
-		// during mute pm_phase, am_phase, noiseA_phase, noiseB_phase
-		// and noise_seed aren't updated, probably ok
+		// Amplitude modulation: 27 output levels (triangle waveform);
+		// 1 level takes one of: 192, 256 or 448 samples
+		// One entry from LFO_AM_TABLE lasts for 64 samples
+		// lfo_am_table is 210 elements long
+		++am_phase;
+		if (am_phase == (LFO_AM_TAB_ELEMENTS * 64)) am_phase = 0;
+		unsigned tmp = lfo_am_table[am_phase / 64];
+		int lfo_am = am_mode ? tmp : tmp / 4;
+
 		pm_phase = (pm_phase + PM_DPHASE) & (PM_DP_WIDTH - 1);
-		am_phase = (am_phase + AM_DPHASE) & (AM_DP_WIDTH - 1);
-		int lfo_am = amtable[am_mode][am_phase >> (AM_DP_BITS - AM_PG_BITS)];
 		int lfo_pm = pmtable[pm_mode][pm_phase >> (PM_DP_BITS - PM_PG_BITS)];
 
 		if (noise_seed & 1) {
@@ -974,32 +1020,39 @@ void Y8950Impl::generateChannels(int** bufs, unsigned num)
 			}
 		}
 		if (rythm_mode) {
+			//bufs[6][sample] += 0;
+			//bufs[7][sample] += 0;
+			//bufs[8][sample] += 0;
+
 			// TODO wasn't in original source either
 			ch[7].slot[MOD].calc_phase(lfo_pm);
 			ch[8].slot[CAR].calc_phase(lfo_pm);
 
-			bufs[ 6][sample] += (ch[6].slot[CAR].isActive())
+			bufs[ 9][sample] += (ch[6].slot[CAR].isActive())
 				? 2 * ch[6].slot[CAR].calc_slot_car(lfo_pm, lfo_am,
 						    ch[6].slot[MOD].calc_slot_mod(lfo_pm, lfo_am))
 				: 0;
-			bufs[ 7][sample] += (ch[7].slot[CAR].isActive())
+			bufs[10][sample] += (ch[7].slot[CAR].isActive())
 				? 2 * ch[7].slot[CAR].calc_slot_snare(lfo_pm, lfo_am, whitenoise)
 				: 0;
-			bufs[ 8][sample] += (ch[8].slot[CAR].isActive())
+			bufs[11][sample] += (ch[8].slot[CAR].isActive())
 				? 2 * ch[8].slot[CAR].calc_slot_cym(lfo_am, noiseA, noiseB)
 				: 0;
-			bufs[ 9][sample] += (ch[7].slot[MOD].isActive())
+			bufs[12][sample] += (ch[7].slot[MOD].isActive())
 				? 2 * ch[7].slot[MOD].calc_slot_hat(lfo_am, noiseA, noiseB, whitenoise)
 				: 0;
-			bufs[10][sample] += (ch[8].slot[MOD].isActive())
+			bufs[13][sample] += (ch[8].slot[MOD].isActive())
 				? 2 * ch[8].slot[MOD].calc_slot_tom(lfo_pm, lfo_am)
 				: 0;
 		} else {
 			//bufs[ 9] += 0;
 			//bufs[10] += 0;
+			//bufs[11] += 0;
+			//bufs[12] += 0;
+			//bufs[13] += 0;
 		}
 
-		bufs[11][sample] += adpcm->calcSample();
+		bufs[14][sample] += adpcm->calcSample();
 	}
 }
 
@@ -1097,7 +1150,7 @@ void Y8950Impl::writeReg(byte rg, byte data, EmuTime::param time)
 			break;
 
 		case 0x07: // START/REC/MEM DATA/REPEAT/SP-OFF/-/-/RESET
-			perihery.setSPOFF((data & 8) != 0, time); // bit 3
+			periphery.setSPOFF((data & 8) != 0, time); // bit 3
 			// fall-through
 
 		case 0x08: // CSM/KEY BOARD SPLIT/-/-/SAMPLE/DA AD/64K/ROM
@@ -1137,12 +1190,12 @@ void Y8950Impl::writeReg(byte rg, byte data, EmuTime::param time)
 			// 0 -> input
 			// 1 -> output
 			reg[rg] = data;
-			perihery.write(reg[0x18], reg[0x19], time);
+			periphery.write(reg[0x18], reg[0x19], time);
 			break;
 
 		case 0x19: // I/O-DATA (bit3-0)
 			reg[rg] = data;
-			perihery.write(reg[0x18], reg[0x19], time);
+			periphery.write(reg[0x18], reg[0x19], time);
 			break;
 		}
 		break;
@@ -1294,7 +1347,7 @@ byte Y8950Impl::peekReg(byte rg, EmuTime::param time) const
 			return adpcm->peekReg(rg, time);
 
 		case 0x19: { // I/O DATA
-			byte input = perihery.read(time);
+			byte input = periphery.read(time);
 			byte output = reg[0x19];
 			byte enable = reg[0x18];
 			return (output & enable) | (input & ~enable) | 0xF0;
@@ -1465,9 +1518,8 @@ void Y8950Debuggable::write(unsigned address, byte value, EmuTime::param time)
 
 Y8950::Y8950(MSXMotherBoard& motherBoard, const std::string& name,
              const XMLElement& config, unsigned sampleRam, EmuTime::param time,
-             Y8950Periphery& perihery)
-	: pimple(new Y8950Impl(*this, motherBoard, name, config, sampleRam,
-	                       perihery))
+             MSXAudio& audio)
+	: pimple(new Y8950Impl(*this, motherBoard, name, config, sampleRam, audio))
 {
 	pimple->init(config, time);
 }
@@ -1481,6 +1533,11 @@ void Y8950::setEnabled(bool enabled, EmuTime::param time)
 	pimple->setEnabled(enabled, time);
 }
 
+void Y8950::clearRam()
+{
+	pimple->clearRam();
+}
+
 void Y8950::reset(EmuTime::param time)
 {
 	pimple->reset(time);
