@@ -1,4 +1,4 @@
-// $Id: Y8950Adpcm.cc 9581 2009-04-15 16:34:02Z m9710797 $
+// $Id: Y8950Adpcm.cc 11148 2010-01-13 16:14:45Z m9710797 $
 
 // The actual sample playing part is duplicated for the 'emu' domain and the
 // 'audio' domain. The emu part is responsible for cycle accurate sample
@@ -53,13 +53,18 @@ Y8950Adpcm::Y8950Adpcm(Y8950& y8950_, MSXMotherBoard& motherBoard,
 	, clock(motherBoard.getCurrentTime())
 	, volume(0)
 {
-	memset(&(*ram)[0], 0xFF, ram->getSize());
+	clearRam();
 }
 
 Y8950Adpcm::~Y8950Adpcm()
 {
 }
 
+void Y8950Adpcm::clearRam()
+{
+	memset(&(*ram)[0], 0xFF, ram->getSize());
+}
+
 void Y8950Adpcm::reset(EmuTime::param time)
 {
 	removeSyncPoint();
@@ -502,18 +507,17 @@ void Y8950Adpcm::serialize(Archive& ar, unsigned version)
 		aud = emu;
 	}
 
-	if (version >= 2) {
-		ar.serialize("clock", clock);
-	} else {
-		assert(ar.isLoader());
+	if (ar.versionBelow(version, 2)) {
 		clock.reset(getCurrentTime());
 
-		// reschedule, because automatically derserialized sync-point
+		// reschedule, because automatically deserialized sync-point
 		// can be off, because clock.getTime() != getCurrentTime()
 		removeSyncPoint();
 		if (isPlaying()) {
 			schedule();
 		}
+	} else {
+		ar.serialize("clock", clock);
 	}
 }
 INSTANTIATE_SERIALIZE_METHODS(Y8950Adpcm);
