@@ -1,4 +1,4 @@
-// $Id: YMF262.cc 9300 2009-03-01 01:03:07Z mfeingol $
+// $Id: YMF262.cc 11148 2010-01-13 16:14:45Z m9710797 $
 
 /*
  *
@@ -47,7 +47,10 @@
 #include "FixedPoint.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "serialize.hh"
+#include "serialize_meta.hh"
 #include <cmath>
 #include <cstring>
 
@@ -182,7 +185,7 @@ class YMF262Impl : private SoundDevice, private EmuTimerCallback, private Resamp
 {
 public:
 	YMF262Impl(MSXMotherBoard& motherBoard, const std::string& name,
-	           const XMLElement& config);
+	           const XMLElement& config, bool isYMF278);
 	virtual ~YMF262Impl();
 
 	void reset(EmuTime::param time);
@@ -275,7 +278,12 @@ private:
 	byte status;			// status flag
 	byte status2;
 	byte statusMask;		// status mask
+
+	bool alreadySignaledNEW2;
+	const bool isYMF278;		// true iff this is actually a YMF278
+					// ATM only used for NEW2 bit
 };
+SERIALIZE_CLASS_VERSION(YMF262Impl, 2);
 
 
 // envelope output entries
@@ -794,6 +802,12 @@ inline int YMF262Slot::op_calc(unsigned phase, unsigned lfo_am)
 // (or 1st part of a 4-op channel)
 void YMF262Channel::chan_calc(unsigned lfo_am)
 {
+	// !! something is wrong with this, it caused bug
+	// !!    [2823673] moonsound 4 operator FM fail
+	// !! optimization disabled for now
+	// !! TODO investigate
+	// !!   maybe this micro optimization isn't worth the trouble/risk
+	// !!
 	// - mod.connect can point to 'phase_modulation'  or 'ch0-output'
 	// - car.connect can point to 'phase_modulation2' or 'ch0-output'
 	//    (see register #C0-#C8 writes)
@@ -802,14 +816,14 @@ void YMF262Channel::chan_calc(unsigned lfo_am)
 	//   an addition for car.connect (and initialize phase_modulation2 to
 	//   zero). For mod.connect we can directly assign the value.
 
+	// ?? is this paragraph correct ??
 	// phase_modulation should be initialized to zero here. But there seems
 	// to be an optimization bug in gcc-4.2: it *seems* that when we
 	// initialize phase_modulation to zero in this function, the optimizer
 	// assumes it still has value zero at the end of this function (where
 	// it's used to calculate car.connect). As a workaround we initialize
 	// phase_modulation each time before calling this function.
-	//phase_modulation = 0;
-
+	phase_modulation = 0;
 	phase_modulation2 = 0;
 
 	YMF262Slot& mod = slot[MOD];
@@ -818,7 +832,7 @@ void YMF262Channel::chan_calc(unsigned lfo_am)
 		: 0;
 	mod.op1_out[0] = mod.op1_out[1];
 	mod.op1_out[1] = mod.op_calc(mod.Cnt.toInt() + (out >> mod.fb_shift), lfo_am);
-	*mod.connect = mod.op1_out[1];
+	*mod.connect += mod.op1_out[1];
 
 	YMF262Slot& car = slot[CAR];
 	*car.connect += car.op_calc(car.Cnt.toInt() + phase_modulation, lfo_am);
@@ -827,6 +841,9 @@ void YMF262Channel::chan_calc(unsigned lfo_am)
 // calculate output of a 2nd part of 4-op channel
 void YMF262Channel::chan_calc_ext(unsigned lfo_am)
 {
+	// !! see remark in chan_cal(), something is wrong with this
+	// !! optimization disabled for now
+	// !!
 	// - mod.connect can point to 'phase_modulation' or 'ch3-output'
 	// - car.connect always points to 'ch3-output'  (always 4op-mode)
 	//    (see register #C0-#C8 writes)
@@ -834,11 +851,10 @@ void YMF262Channel::chan_calc_ext(unsigned lfo_am)
 	//   an addition for car.connect. For mod.connect we can directly assign
 	//   the value.
 
-	// gcc-4.2 workaround, see comments in chan_calc()
-	//phase_modulation = 0;
+	phase_modulation = 0;
 
 	YMF262Slot& mod = slot[MOD];
-	*mod.connect = mod.op_calc(mod.Cnt.toInt() + phase_modulation2, lfo_am);
+	*mod.connect += mod.op_calc(mod.Cnt.toInt() + phase_modulation2, lfo_am);
 
 	YMF262Slot& car = slot[CAR];
 	*car.connect += car.op_calc(car.Cnt.toInt() + phase_modulation, lfo_am);
@@ -1098,7 +1114,7 @@ void YMF262Impl::setOutputRate(unsigned sampleRate)
 	const int CLOCK_FREQ = 3579545 * 4;
 	double input = CLOCK_FREQ / (8.0 * 36.0);
 	setInputRate(int(input + 0.5));
-	setResampleRatio(input, sampleRate);
+	setResampleRatio(input, sampleRate, isStereo());
 }
 
 void YMF262Slot::FM_KEYON(byte key_set)
@@ -1295,8 +1311,17 @@ void YMF262Impl::writeRegDirect(unsigned r, byte v, EmuTime::param time)
 	case 0x105:
 		// OPL3 mode when bit0=1 otherwise it is OPL2 mode
 		OPL3_mode = v & 0x01;
-		if (OPL3_mode) {
+
+		// When NEW2 bit is first set, a read from the status register
+		// (once) returns bit 1 set (0x02). This only happens once after
+		// reset, so clearing NEW2 and setting it again doesn't cause
+		// another change in the status register.
+		// This seems strange behaviour to me, but it is what I saw on
+		// a real YMF278. Also see page 10 in the 'OPL4 YMF278B
+		// Application Manual' (though it's not clear on the details).
+		if ((v & 0x02) && !alreadySignaledNEW2 && isYMF278) {
 			status2 = 0x02;
+			alreadySignaledNEW2 = true;
 		}
 
 		// following behaviour was tested on real YMF262,
@@ -1622,6 +1647,7 @@ void YMF262Impl::reset(EmuTime::param time)
 
 	noise_rng = 1; // noise shift register
 	nts = false; // note split
+	alreadySignaledNEW2 = false;
 	resetStatus(0x60);
 
 	// reset with register write
@@ -1651,15 +1677,16 @@ void YMF262Impl::reset(EmuTime::param time)
 }
 
 YMF262Impl::YMF262Impl(MSXMotherBoard& motherBoard, const std::string& name,
-                       const XMLElement& config)
+                       const XMLElement& config, bool isYMF278_)
 	: SoundDevice(motherBoard.getMSXMixer(), name, "MoonSound FM-part",
 	              18, true)
-	, Resample(motherBoard.getGlobalSettings(), 2)
+	, Resample(motherBoard.getReactor().getGlobalSettings().getResampleSetting())
 	, debuggable(new YMF262Debuggable(motherBoard, *this))
 	, timer1(motherBoard.getScheduler(), *this)
 	, timer2(motherBoard.getScheduler(), *this)
 	, irq(motherBoard, getName() + ".IRQ")
 	, lfo_am_cnt(0), lfo_pm_cnt(0)
+	, isYMF278(isYMF278_)
 {
 	lfo_am_depth = false;
 	lfo_pm_depth_range = 0;
@@ -1718,6 +1745,8 @@ int YMF262Impl::getAmplificationFactor() const
 
 void YMF262Impl::generateChannels(int** bufs, unsigned num)
 {
+	// TODO implement per-channel mute (instead of all-or-nothing)
+	// TODO output rhythm on seperate channels?
 	if (checkMuteHelper()) {
 		// TODO update internal state, even if muted
 		for (int i = 0; i < 18; ++i) {
@@ -1750,15 +1779,12 @@ void YMF262Impl::generateChannels(int** bufs, unsigned num)
 				YMF262Channel& ch0 = channel[k + i + 0];
 				YMF262Channel& ch3 = channel[k + i + 3];
 				// extended 4op ch#0 part 1 or 2op ch#0
-				phase_modulation = 0; // gcc-4.2 workaround
 				ch0.chan_calc(lfo_am);
 				if (ch0.extended) {
 					// extended 4op ch#0 part 2
-					phase_modulation = 0; // gcc-4.2 workaround
 					ch3.chan_calc_ext(lfo_am);
 				} else {
 					// standard 2op ch#3
-					phase_modulation = 0; // gcc-4.2 workaround
 					ch3.chan_calc(lfo_am);
 				}
 			}
@@ -1766,11 +1792,8 @@ void YMF262Impl::generateChannels(int** bufs, unsigned num)
 
 		// channels 6,7,8 rhythm or 2op mode
 		if (!rhythmEnabled) {
-			phase_modulation = 0; // gcc-4.2 workaround
 			channel[6].chan_calc(lfo_am);
-			phase_modulation = 0; // gcc-4.2 workaround
 			channel[7].chan_calc(lfo_am);
-			phase_modulation = 0; // gcc-4.2 workaround
 			channel[8].chan_calc(lfo_am);
 		} else {
 			// Rhythm part
@@ -1778,11 +1801,8 @@ void YMF262Impl::generateChannels(int** bufs, unsigned num)
 		}
 
 		// channels 15,16,17 are fixed 2-operator channels only
-		phase_modulation = 0; // gcc-4.2 workaround
 		channel[15].chan_calc(lfo_am);
-		phase_modulation = 0; // gcc-4.2 workaround
 		channel[16].chan_calc(lfo_am);
-		phase_modulation = 0; // gcc-4.2 workaround
 		channel[17].chan_calc(lfo_am);
 
 		for (int i = 0; i < 18; ++i) {
@@ -1872,8 +1892,10 @@ void YMF262Channel::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("extended", extended);
 }
 
+// version 1: initial version
+// version 2: added alreadySignaledNEW2
 template<typename Archive>
-void YMF262Impl::serialize(Archive& ar, unsigned /*version*/)
+void YMF262Impl::serialize(Archive& ar, unsigned version)
 {
 	ar.serialize("timer1", timer1);
 	ar.serialize("timer2", timer2);
@@ -1893,6 +1915,9 @@ void YMF262Impl::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("status", status);
 	ar.serialize("status2", status2);
 	ar.serialize("statusMask", statusMask);
+	if (ar.versionAtLeast(version, 2)) {
+		ar.serialize("alreadySignaledNEW2", alreadySignaledNEW2);
+	}
 
 	// TODO restore more state by rewriting register values
 	//   this handles pan
@@ -1927,8 +1952,8 @@ void YMF262Debuggable::write(unsigned address, byte value, EmuTime::param time)
 // class YMF262
 
 YMF262::YMF262(MSXMotherBoard& motherBoard, const std::string& name,
-               const XMLElement& config)
-	: pimple(new YMF262Impl(motherBoard, name, config))
+               const XMLElement& config, bool isYMF278)
+	: pimple(new YMF262Impl(motherBoard, name, config, isYMF278))
 {
 }
 
