@@ -1,4 +1,4 @@
-// $Id: YMF278.cc 9401 2009-03-20 05:37:27Z mfeingol $
+// $Id: YMF278.cc 10628 2009-09-30 21:17:12Z seanyoung $
 
 // Based on ymf278b.c written by R. Belmont and O. Galibert
 
@@ -8,6 +8,8 @@
 #include "Rom.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include "GlobalSettings.hh"
+#include "Reactor.hh"
 #include "Clock.hh"
 #include "serialize.hh"
 #include <algorithm>
@@ -96,6 +98,7 @@ public:
 	YMF278Impl(MSXMotherBoard& motherBoard, const std::string& name,
 	       int ramSize, const XMLElement& config);
 	virtual ~YMF278Impl();
+	void clearRam();
 	void reset(EmuTime::param time);
 	void writeRegOPL4(byte reg, byte data, EmuTime::param time);
 	byte readReg(byte reg, EmuTime::param time);
@@ -537,8 +540,7 @@ void YMF278Impl::advance()
 			break;
 
 		default:
-			assert(false);
-			break;
+			UNREACHABLE;
 		}
 	}
 }
@@ -930,7 +932,7 @@ YMF278Impl::YMF278Impl(MSXMotherBoard& motherBoard_, const std::string& name,
                        int ramSize, const XMLElement& config)
 	: SoundDevice(motherBoard_.getMSXMixer(), name, "MoonSound wave-part",
 	              24, true)
-	, Resample(motherBoard_.getGlobalSettings(), 2)
+	, Resample(motherBoard_.getReactor().getGlobalSettings().getResampleSetting())
 	, motherBoard(motherBoard_)
 	, debugRegisters(new DebugRegisters(*this, motherBoard))
 	, debugMemory   (new DebugMemory   (*this, motherBoard))
@@ -961,9 +963,14 @@ YMF278Impl::~YMF278Impl()
 	unregisterSound();
 }
 
+void YMF278Impl::clearRam()
+{
+	memset(&ram[0], 0, ram.size());
+}
+
 void YMF278Impl::reset(EmuTime::param time)
 {
-	eg_cnt   = 0;
+	eg_cnt = 0;
 
 	for (int i = 0; i < 24; ++i) {
 		slots[i].reset();
@@ -980,7 +987,7 @@ void YMF278Impl::reset(EmuTime::param time)
 void YMF278Impl::setOutputRate(unsigned sampleRate)
 {
 	setInputRate(44100);
-	setResampleRatio(44100, sampleRate);
+	setResampleRatio(44100, sampleRate, isStereo());
 }
 
 byte YMF278Impl::readMem(unsigned address) const
@@ -1129,6 +1136,11 @@ YMF278::~YMF278()
 {
 }
 
+void YMF278::clearRam()
+{
+	pimple->clearRam();
+}
+
 void YMF278::reset(EmuTime::param time)
 {
 	pimple->reset(time);
