@@ -1,6 +1,7 @@
-// $Id: Semaphore.cc 5533 2006-07-24 20:36:47Z m9710797 $
+// $Id: Semaphore.cc 8375 2008-11-29 21:25:27Z manuelbi $
 
 #include "Semaphore.hh"
+#include "openmsx.hh"
 
 namespace openmsx {
 
@@ -11,7 +12,9 @@ Semaphore::Semaphore(unsigned value)
 
 Semaphore::~Semaphore()
 {
+	PRT_DEBUG("Destroying semaphore thread: " << SDL_ThreadID() << "   lock: " << this);
 	SDL_DestroySemaphore(semaphore);
+	PRT_DEBUG("DONE destroying semaphore thread: " << SDL_ThreadID() << "   lock: " << this);
 }
 
 void Semaphore::up()
@@ -19,6 +22,7 @@ void Semaphore::up()
 	while (SDL_SemPost(semaphore)) {
 		// the SDL doc lists no reason why this call could fail,
 		// but just in case we try till it succeeds
+		PRT_DEBUG("Semaphore up of thread: " << SDL_ThreadID() << " and lock: " << this << " failed, error: " << SDL_GetError());
 	}
 }
 
@@ -29,6 +33,7 @@ void Semaphore::down()
 		// example when another thread exits.
 		// We don't want to leave before we actually acquired the semaphore,
 		// so try again until we have it.
+		PRT_DEBUG("Semaphore down of thread: " << SDL_ThreadID() << " and lock: " << this << " failed, error: " << SDL_GetError());
 	}
 }
 
