@@ -1,7 +1,8 @@
-// $Id: Thread.cc 9771 2009-05-07 15:14:07Z m9710797 $
+// $Id: Thread.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "Thread.hh"
 #include "MSXException.hh"
+#include "unreachable.hh"
 #include <iostream>
 #include <cassert>
 #include <SDL_thread.h>
@@ -71,11 +72,11 @@ int Thread::startThread(void* runnable)
 	} catch (FatalError& e) {
 		std::cerr << "Fatal error in subthread: "
 		     << e.getMessage() << std::endl;
-		assert(false);
+		UNREACHABLE;
 	} catch (MSXException& e) {
 		std::cerr << "Uncaught exception in subthread: "
 		     << e.getMessage() << std::endl;
-		assert(false);
+		UNREACHABLE;
 	} // don't catch(..), thread cancelation seems to depend on it
 	return 0;
 }
