@@ -1,4 +1,4 @@
-// $Id: HostCPU.cc 9570 2009-04-13 19:27:41Z mthuurne $
+// $Id: HostCPU.cc 10773 2009-11-09 22:20:11Z mthuurne $
 
 #include "HostCPU.hh"
 #include "openmsx.hh"
@@ -21,7 +21,7 @@ HostCPU::HostCPU()
 	mmxFlag  = true;
 	sseFlag  = true;
 	sse2Flag = true;
-	#elif defined ASM_X86_32
+	#elif ASM_X86_32
 	#ifdef _MSC_VER
 	unsigned hasCPUID;
 	__asm {
