@@ -1,10 +1,10 @@
-// $Id:$
+// $Id: uint128.cc 9571 2009-04-13 19:37:05Z mthuurne $
 
-#ifdef __x86_64
+#if defined __x86_64 && !defined _MSC_VER
 
 // nothing
 
-#else // __x86_64
+#else // __x86_64 && !_MSC_VER
 
 #include "uint128.hh"
 
@@ -60,9 +60,9 @@ uint128 uint128::div(const uint128& ds, uint128& remainder) const
 bool uint128::bit(unsigned n) const
 {
 	if (n < 64) {
-		return lo & (1ull << n);
+		return (lo & (1ull << n)) != 0;
 	} else {
-		return hi & (1ull << (n - 64));
+		return (hi & (1ull << (n - 64))) != 0;
 	}
 }
 
@@ -75,4 +75,4 @@ void uint128::setBit(unsigned n)
 	}
 }
 
-#endif // __x86_64
+#endif // __x86_64 && !_MSC_VER
