@@ -1,10 +1,11 @@
-// $Id: AviRecorder.cc 9753 2009-05-04 20:32:07Z m9710797 $
+// $Id: AviRecorder.cc 11390 2010-04-10 09:03:27Z m9710797 $
 
 #include "AviRecorder.hh"
 #include "AviWriter.hh"
 #include "WavWriter.hh"
 #include "Reactor.hh"
 #include "MSXMotherBoard.hh"
+#include "FileContext.hh"
 #include "Command.hh"
 #include "CommandException.hh"
 #include "Display.hh"
@@ -16,6 +17,7 @@
 #include "CliComm.hh"
 #include "FileOperations.hh"
 #include "StringOp.hh"
+#include "vla.hh"
 #include <cassert>
 
 using std::string;
@@ -53,8 +55,8 @@ AviRecorder::~AviRecorder()
 	assert(!wavWriter.get());
 }
 
-void AviRecorder::start(bool recordAudio, bool recordVideo,
-                        const Filename& filename)
+void AviRecorder::start(bool recordAudio, bool recordVideo, bool recordMono,
+                        bool recordStereo, const Filename& filename)
 {
 	stop();
 	MSXMotherBoard* motherBoard = reactor.getMotherBoard();
@@ -63,6 +65,15 @@ void AviRecorder::start(bool recordAudio, bool recordVideo,
 	}
 	if (recordAudio) {
 		mixer = &motherBoard->getMSXMixer();
+		warnedStereo = false;
+		if (recordStereo) {
+			stereo = true;
+		} else if (recordMono) {
+			stereo = false;
+			warnedStereo = true; // no warning if data is actually stereo
+		} else {
+			stereo = mixer->anyStereoDevice();
+		}		
 		sampleRate = mixer->getSampleRate();
 		warnedSampleRate = false;
 	}
@@ -86,15 +97,17 @@ void AviRecorder::start(bool recordAudio, bool recordVideo,
 		prevTime = EmuTime::infinity;
 
 		try {
-			aviWriter.reset(new AviWriter(filename, frameWidth, frameHeight,
-						      bpp, sampleRate));
+			aviWriter.reset(new AviWriter(filename, frameWidth,
+			        frameHeight, bpp,
+			        (recordAudio && stereo) ? 2 : 1, sampleRate));
 		} catch (MSXException& e) {
 			throw CommandException("Can't start recording: " +
 			                       e.getMessage());
 		}
 	} else {
 		assert(recordAudio);
-		wavWriter.reset(new WavWriter(filename, 2, 16, sampleRate));
+		wavWriter.reset(new Wav16Writer(filename, stereo ? 2 : 1,
+		                                sampleRate));
 	}
 	// only set recorders when all errors are checked for
 	if (postProcessor1) {
@@ -136,15 +149,42 @@ void AviRecorder::addWave(unsigned num, short* data)
 			"avi recording. Audio/video might get out of sync "
 			"because of this.");
 	}
-	if (wavWriter.get()) {
-		wavWriter->write16stereo(data, num);
+	if (stereo) {
+		if (wavWriter.get()) {
+			wavWriter->write(data, 2, num);
+		} else {
+			assert(aviWriter.get());
+			audioBuf.insert(audioBuf.end(), data, data + 2 * num);
+		}
 	} else {
-		assert(aviWriter.get());
-		audioBuf.insert(audioBuf.end(), data, data + 2 * num);
+		VLA(short, buf, num);
+		unsigned i = 0;
+		for (/**/; !warnedStereo && i < num; ++i) {
+			if (data[2 * i + 0] != data[2 * i + 1]) {
+				reactor.getCliComm().printWarning(
+				    "Detected stereo sound during mono recording. "
+				    "Channels will be mixed down to mono. To "
+				    "avoid this warning you can explicity pass the "
+				    "-mono or -stereo flag to the record command.");
+				warnedStereo = true;
+				break;
+			}
+			buf[i] = data[2 * i];
+		}
+		for (/**/; i < num; ++i) {
+			buf[i] = (int(data[2 * i + 0]) + int(data[2 * i + 1])) / 2;
+		}
+
+		if (wavWriter.get()) {
+			wavWriter->write(buf, 1, num);
+		} else {
+			assert(aviWriter.get());
+			audioBuf.insert(audioBuf.end(), buf, buf + num);
+		}
 	}
 }
 
-void AviRecorder::addImage(const void** lines, EmuTime::param time)
+void AviRecorder::addImage(FrameSource* frame, EmuTime::param time)
 {
 	assert(!wavWriter.get());
 	if (duration != EmuDuration::infinity) {
@@ -162,14 +202,15 @@ void AviRecorder::addImage(const void** lines, EmuTime::param time)
 	prevTime = time;
 
 	if (mixer) {
-		MSXMotherBoard* motherBoard = reactor.getMotherBoard();
-		assert(motherBoard);
-		mixer->updateStream(motherBoard->getCurrentTime());
+		mixer->updateStream(time);
 	}
-	aviWriter->addFrame(lines, unsigned(audioBuf.size()) / 2, &audioBuf[0]);
+	// TODO vector::data() is not yet supported in gcc-3.4
+	short* audioData = audioBuf.empty() ? NULL : &audioBuf[0];
+	aviWriter->addFrame(frame, unsigned(audioBuf.size()), audioData);
 	audioBuf.clear();
 }
 
+// TODO: Can this be dropped?
 unsigned AviRecorder::getFrameHeight() const {
 	assert (frameHeight != 0); // someone uses the getter too early?
 	return frameHeight;
@@ -181,6 +222,8 @@ string AviRecorder::processStart(const vector<string>& tokens)
 	string prefix = "openmsx";
 	bool recordAudio = true;
 	bool recordVideo = true;
+	bool recordMono = false;
+	bool recordStereo = false;
 	frameWidth = 320;
 	frameHeight = 240;
 
@@ -199,6 +242,10 @@ string AviRecorder::processStart(const vector<string>& tokens)
 				prefix = tokens[i];
 			} else if (tokens[i] == "-audioonly") {
 				recordVideo = false;
+			} else if (tokens[i] == "-mono") {
+				recordMono = true;
+			} else if (tokens[i] == "-stereo") {
+				recordStereo = true;
 			} else if (tokens[i] == "-videoonly") {
 				recordAudio = false;
 			} else if (tokens[i] == "-doublesize") {
@@ -214,6 +261,12 @@ string AviRecorder::processStart(const vector<string>& tokens)
 	if (!recordAudio && !recordVideo) {
 		throw CommandException("Can't have both -videoonly and -audioonly.");
 	}
+	if (recordStereo && recordMono) {
+		throw CommandException("Can't have both -mono and -stereo.");
+	}
+	if (!recordAudio && (recordStereo || recordMono)) {
+		throw CommandException("Can't have both -videoonly and -stereo or -mono.");
+	}
 	switch (arguments.size()) {
 	case 0:
 		// nothing
@@ -225,17 +278,27 @@ string AviRecorder::processStart(const vector<string>& tokens)
 		throw SyntaxError();
 	}
 
+	string directory = recordVideo ? "videos" : "soundlogs";
+	string extension = recordVideo ? ".avi"   : ".wav";
 	if (filename.empty()) {
-		string directory = recordVideo ? "videos" : "soundlogs";
-		string extension = recordVideo ? ".avi"   : ".wav";
 		filename = FileOperations::getNextNumberedFileName(
 			directory, prefix, extension);
+	} else {
+		if (FileOperations::getExtension(filename).empty()) {
+			// no extension given, append standard one
+			filename += extension;
+		}
+		if (FileOperations::getBaseName(filename).empty()) {
+			// no dir given, use standard dir
+			filename = FileOperations::getUserOpenMSXDir() + "/" + directory + "/" + filename;
+		}
 	}
 
 	if (aviWriter.get() || wavWriter.get()) {
 		return "Already recording.";
 	}
-	start(recordAudio, recordVideo, Filename(filename));
+	start(recordAudio, recordVideo, recordMono, recordStereo,
+	      Filename(filename));
 	return "Recording to " + filename;
 }
 
@@ -293,8 +356,8 @@ string RecordCommand::help(const vector<string>& /*tokens*/) const
 	       "record stop               Stop recording\n"
 	       "record toggle             Toggle recording (useful as keybinding)\n"
 	       "\n"
-	       "The start subcommand also accepts an optional -audioonly, -videoonly and "
-	       "a -doublesize flag.\n"
+	       "The start subcommand also accepts an optional -audioonly, -videoonly, "
+	       " -mono, -stereo, -doublesize flag.\n"
 	       "Videos are recorded in a 320x240 size by default and at 640x480 when the "
 	       "-doublesize flag is used.";
 }
@@ -306,10 +369,11 @@ void RecordCommand::tabCompletion(vector<string>& tokens) const
 		std::set<string> cmds(str, str + 3);
 		completeString(tokens, cmds);
 	} else if ((tokens.size() >= 3) && (tokens[1] == "start")) {
-		const char* const str[4] = { "-prefix", "-videoonly", "-audioonly",
-						"-doublesize"};
-		std::set<string> cmds(str, str + 4);
-		completeString(tokens, cmds);
+		const char* const str[6] = { "-prefix", "-videoonly", 
+			"-audioonly", "-doublesize", "-mono", "-stereo" };
+		std::set<string> cmds(str, str + 6);
+		UserFileContext context;
+		completeFileName(getCommandController(), tokens, context, cmds);
 	}
 }
 
