@@ -1,4 +1,4 @@
-// $Id: BitmapConverter.cc 8462 2009-01-10 19:52:00Z m9710797 $
+// $Id: BitmapConverter.cc 10421 2009-08-27 13:34:27Z mthuurne $
 
 #include "BitmapConverter.hh"
 #include "GLUtil.hh"
@@ -286,13 +286,15 @@ template class BitmapConverter<word>;
 template class BitmapConverter<unsigned>;
 #endif
 
-#ifdef COMPONENT_GL
-#ifdef _MSC_VER
+#if COMPONENT_GL
+#if defined(_MSC_VER)
 // see comment in V9990BitmapConverter
 STATIC_ASSERT((is_same_type<unsigned, GLuint>::value));
-#else
+#elif HAVE_32BPP
 template<> class BitmapConverter<GLUtil::NoExpansion> {};
 template class BitmapConverter<GLUtil::ExpandGL>;
+#else
+template class BitmapConverter<GLuint>;
 #endif
 #endif // COMPONENT_GL
 
