@@ -1,4 +1,4 @@
-// $Id: CharacterConverter.cc 8462 2009-01-10 19:52:00Z m9710797 $
+// $Id: CharacterConverter.cc 10602 2009-09-18 11:20:13Z seanyoung $
 
 /*
 TODO:
@@ -55,10 +55,10 @@ void CharacterConverter<Pixel>::setDisplayMode(DisplayMode mode)
 
 template <class Pixel>
 void CharacterConverter<Pixel>::renderText1(
-	Pixel* pixelPtr, int line)
+	Pixel* __restrict pixelPtr, int line)
 {
-	Pixel fg = palFg[vdp.getForegroundColour()];
-	Pixel bg = palBg[vdp.getBackgroundColour()];
+	Pixel fg = palFg[vdp.getForegroundColor()];
+	Pixel bg = palFg[vdp.getBackgroundColor()];
 
 	// 8 * 256 is small enough to always be contiguous
 	const byte* patternArea = vram.patternTable.getReadArea(0, 256 * 8);
@@ -84,10 +84,10 @@ void CharacterConverter<Pixel>::renderText1(
 
 template <class Pixel>
 void CharacterConverter<Pixel>::renderText1Q(
-	Pixel* pixelPtr, int line)
+	Pixel* __restrict pixelPtr, int line)
 {
-	Pixel fg = palFg[vdp.getForegroundColour()];
-	Pixel bg = palBg[vdp.getBackgroundColour()];
+	Pixel fg = palFg[vdp.getForegroundColor()];
+	Pixel bg = palFg[vdp.getBackgroundColor()];
 
 	unsigned patternBaseLine = (-1 << 13) | ((line + vdp.getVerticalScroll()) & 7);
 
@@ -114,15 +114,15 @@ void CharacterConverter<Pixel>::renderText1Q(
 
 template <class Pixel>
 void CharacterConverter<Pixel>::renderText2(
-	Pixel* pixelPtr, int line)
+	Pixel* __restrict pixelPtr, int line)
 {
-	Pixel plainFg = palFg[vdp.getForegroundColour()];
-	Pixel plainBg = palBg[vdp.getBackgroundColour()];
+	Pixel plainFg = palFg[vdp.getForegroundColor()];
+	Pixel plainBg = palFg[vdp.getBackgroundColor()];
 	Pixel blinkFg, blinkBg;
 	if (vdp.getBlinkState()) {
-		int fg = vdp.getBlinkForegroundColour();
-		blinkFg = palBg[fg ? fg : vdp.getBlinkBackgroundColour()];
-		blinkBg = palBg[vdp.getBlinkBackgroundColour()];
+		int fg = vdp.getBlinkForegroundColor();
+		blinkFg = palBg[fg ? fg : vdp.getBlinkBackgroundColor()];
+		blinkBg = palBg[vdp.getBlinkBackgroundColor()];
 	} else {
 		blinkFg = plainFg;
 		blinkBg = plainBg;
@@ -135,7 +135,7 @@ void CharacterConverter<Pixel>::renderText2(
 	unsigned colorStart = (line / 8) * (80 / 8);
 	unsigned nameStart  = (line / 8) * 80;
 	for (unsigned i = 0; i < (80 / 8); ++i) {
-		unsigned colorPattern = vram.colourTable.readNP(
+		unsigned colorPattern = vram.colorTable.readNP(
 			(colorStart + i) | (-1 << 9));
 		const byte* nameArea = vram.nameTable.getReadArea(
 			(nameStart + 8 * i) | (-1 << 12), 8);
@@ -234,11 +234,11 @@ const byte* CharacterConverter<Pixel>::getNamePtr(int line, int scroll)
 }
 template <class Pixel>
 void CharacterConverter<Pixel>::renderGraphic1(
-	Pixel* pixelPtr, int line)
+	Pixel* __restrict pixelPtr, int line)
 {
 	const byte* patternArea = vram.patternTable.getReadArea(0, 256 * 8);
 	patternArea += line & 7;
-	const byte* colorArea = vram.colourTable.getReadArea(0, 256 / 8);
+	const byte* colorArea = vram.colorTable.getReadArea(0, 256 / 8);
 
 	int scroll = vdp.getHorizontalScrollHigh();
 	const byte* namePtr = getNamePtr(line, scroll);
@@ -264,7 +264,7 @@ void CharacterConverter<Pixel>::renderGraphic1(
 
 template <class Pixel>
 void CharacterConverter<Pixel>::renderGraphic2(
-	Pixel* pixelPtr, int line)
+	Pixel* __restrict pixelPtr, int line)
 {
 	int quarter = ((line / 8) * 32) & ~0xFF;
 	int baseLine = (-1 << 13) | (quarter * 8) | (line & 7);
@@ -280,7 +280,7 @@ void CharacterConverter<Pixel>::renderGraphic2(
 		unsigned charCode8 = namePtr[scroll & 0x1F] * 8;
 		unsigned pattern = patternArea[charCode8];
 		unsigned index = charCode8 | baseLine;
-		unsigned color = vram.colourTable.readNP(index);
+		unsigned color = vram.colorTable.readNP(index);
 		Pixel fg = palFg[color >> 4];
 		Pixel bg = palFg[color & 0x0F];
 #ifdef __arm__
@@ -339,7 +339,7 @@ void CharacterConverter<Pixel>::renderGraphic2(
 
 template <class Pixel>
 void CharacterConverter<Pixel>::renderMultiHelper(
-	Pixel* pixelPtr, int line, int mask, int patternQuarter)
+	Pixel* __restrict pixelPtr, int line, int mask, int patternQuarter)
 {
 	unsigned baseLine = mask | ((line / 4) & 7);
 	unsigned scroll = vdp.getHorizontalScrollHigh();
@@ -359,7 +359,7 @@ void CharacterConverter<Pixel>::renderMultiHelper(
 }
 template <class Pixel>
 void CharacterConverter<Pixel>::renderMulti(
-	Pixel* pixelPtr, int line)
+	Pixel* __restrict pixelPtr, int line)
 {
 	int mask = (-1 << 11);
 	renderMultiHelper(pixelPtr, line, mask, 0);
@@ -367,7 +367,7 @@ void CharacterConverter<Pixel>::renderMulti(
 
 template <class Pixel>
 void CharacterConverter<Pixel>::renderMultiQ(
-	Pixel* pixelPtr, int line)
+	Pixel* __restrict pixelPtr, int line)
 {
 	int mask = (-1 << 13);
 	int patternQuarter = (line * 4) & ~0xFF;  // (line / 8) * 32
@@ -376,10 +376,10 @@ void CharacterConverter<Pixel>::renderMultiQ(
 
 template <class Pixel>
 void CharacterConverter<Pixel>::renderBogus(
-	Pixel* pixelPtr, int /*line*/)
+	Pixel* __restrict pixelPtr, int /*line*/)
 {
-	Pixel fg = palFg[vdp.getForegroundColour()];
-	Pixel bg = palBg[vdp.getBackgroundColour()];
+	Pixel fg = palFg[vdp.getForegroundColor()];
+	Pixel bg = palFg[vdp.getBackgroundColor()];
 	for (int n = 8; n--; ) *pixelPtr++ = bg;
 	for (int c = 20; c--; ) {
 		for (int n = 4; n--; ) *pixelPtr++ = fg;
@@ -396,13 +396,15 @@ template class CharacterConverter<word>;
 template class CharacterConverter<unsigned>;
 #endif
 
-#ifdef COMPONENT_GL
-#ifdef _MSC_VER
+#if COMPONENT_GL
+#if defined(_MSC_VER)
 // see comment in V9990BitmapConverter
 STATIC_ASSERT((is_same_type<unsigned, GLuint>::value));
-#else
+#elif HAVE_32BPP
 template<> class CharacterConverter<GLUtil::NoExpansion> {};
 template class CharacterConverter<GLUtil::ExpandGL>;
+#else
+template class CharacterConverter<GLuint>;
 #endif
 #endif // COMPONENT_GL
 
