@@ -1,11 +1,11 @@
-// $Id: DummyRenderer.cc 9404 2009-03-20 21:34:28Z m9710797 $
+// $Id: DummyRenderer.cc 10786 2009-11-16 15:28:16Z m9710797 $
 
 #include "DummyRenderer.hh"
 #include "DisplayMode.hh"
 
 namespace openmsx {
 
-void DummyRenderer::reset(EmuTime::param /*time*/) {
+void DummyRenderer::reInit() {
 }
 
 void DummyRenderer::frameStart(EmuTime::param /*time*/) {
@@ -17,16 +17,20 @@ void DummyRenderer::frameEnd(EmuTime::param /*time*/) {
 void DummyRenderer::updateTransparency(bool /*enabled*/, EmuTime::param /*time*/) {
 }
 
-void DummyRenderer::updateForegroundColour(int /*colour*/, EmuTime::param /*time*/) {
+void DummyRenderer::updateSuperimposing(const RawFrame* /*videoSource*/,
+                                        EmuTime::param /*time*/) {
 }
 
-void DummyRenderer::updateBackgroundColour(int /*colour*/, EmuTime::param /*time*/) {
+void DummyRenderer::updateForegroundColor(int /*color*/, EmuTime::param /*time*/) {
 }
 
-void DummyRenderer::updateBlinkForegroundColour(int /*colour*/, EmuTime::param /*time*/) {
+void DummyRenderer::updateBackgroundColor(int /*color*/, EmuTime::param /*time*/) {
 }
 
-void DummyRenderer::updateBlinkBackgroundColour(int /*colour*/, EmuTime::param /*time*/) {
+void DummyRenderer::updateBlinkForegroundColor(int /*color*/, EmuTime::param /*time*/) {
+}
+
+void DummyRenderer::updateBlinkBackgroundColor(int /*color*/, EmuTime::param /*time*/) {
 }
 
 void DummyRenderer::updateBlinkState(bool /*enabled*/, EmuTime::param /*time*/) {
@@ -65,7 +69,7 @@ void DummyRenderer::updateNameBase(int /*addr*/, EmuTime::param /*time*/) {
 void DummyRenderer::updatePatternBase(int /*addr*/, EmuTime::param /*time*/) {
 }
 
-void DummyRenderer::updateColourBase(int /*addr*/, EmuTime::param /*time*/) {
+void DummyRenderer::updateColorBase(int /*addr*/, EmuTime::param /*time*/) {
 }
 
 void DummyRenderer::updateSpritesEnabled(bool /*enabled*/, EmuTime::param /*time*/) {
