@@ -1,21 +1,31 @@
-// $Id: DummyVideoSystem.cc 8278 2008-10-01 17:10:24Z m9710797 $
+// $Id: DummyVideoSystem.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "DummyVideoSystem.hh"
-#include "DummyRasterizer.hh"
-#include "V9990DummyRasterizer.hh"
+#include "components.hh"
+#include "unreachable.hh"
 
 namespace openmsx {
 
 Rasterizer* DummyVideoSystem::createRasterizer(VDP& /*vdp*/)
 {
-	return new DummyRasterizer();
+	UNREACHABLE;
+	return NULL;
 }
 
 V9990Rasterizer* DummyVideoSystem::createV9990Rasterizer(V9990& /*vdp*/)
 {
-	return new V9990DummyRasterizer();
+	UNREACHABLE;
+	return NULL;
 }
 
+#if COMPONENT_LASERDISC
+LDRasterizer* DummyVideoSystem::createLDRasterizer(LaserdiscPlayer& /*ld*/)
+{
+	UNREACHABLE;
+	return NULL;
+}
+#endif
+
 void DummyVideoSystem::flush()
 {
 }
