@@ -1,11 +1,13 @@
-// $Id: FrameSource.cc 7638 2008-03-08 13:49:51Z manuelbi $
+// $Id: FrameSource.cc 11103 2010-01-06 22:07:41Z m9710797 $
 
 #include "FrameSource.hh"
 #include "PixelOperations.hh"
 #include "MemoryOps.hh"
 #include "LineScalers.hh"
 #include "openmsx.hh"
+#include "unreachable.hh"
 #include "build-info.hh"
+#include "components.hh"
 
 namespace openmsx {
 
@@ -34,7 +36,18 @@ void FrameSource::setHeight(unsigned height_)
 }
 
 template <typename Pixel>
-const Pixel* FrameSource::getLinePtr320_240(unsigned line)
+const Pixel* FrameSource::blendLines(
+	const Pixel* line1, const Pixel* line2, unsigned width) const
+{
+	PixelOperations<Pixel> pixelOps(pixelFormat);
+	BlendLines<Pixel> blend(pixelOps);
+	Pixel* out = reinterpret_cast<Pixel*>(getTempBuffer());
+	blend(line1, line2, out, width);
+	return out;
+}
+
+template <typename Pixel>
+const Pixel* FrameSource::getLinePtr320_240(unsigned line) const
 {
 	if (getHeight() == 240) {
 		return getLinePtr<Pixel>(line, 320);
@@ -42,16 +55,12 @@ const Pixel* FrameSource::getLinePtr320_240(unsigned line)
 		assert(getHeight() == 480);
 		const Pixel* line1 = getLinePtr<Pixel>(2 * line + 0, 320);
 		const Pixel* line2 = getLinePtr<Pixel>(2 * line + 1, 320);
-		PixelOperations<Pixel> pixelOps(pixelFormat);
-		BlendLines<Pixel> blend(pixelOps);
-		Pixel* out = reinterpret_cast<Pixel*>(getTempBuffer());
-		blend(line1, line2, out, 320);
-		return out;
+		return blendLines(line1, line2, 320);
 	}
 }
 
 template <typename Pixel>
-const Pixel* FrameSource::getLinePtr640_480(unsigned line)
+const Pixel* FrameSource::getLinePtr640_480(unsigned line) const
 {
 	if (getHeight() == 480) {
 		return getLinePtr<Pixel>(line, 640);
@@ -61,23 +70,42 @@ const Pixel* FrameSource::getLinePtr640_480(unsigned line)
 	}
 }
 
-void* FrameSource::getTempBuffer()
+template <typename Pixel>
+const Pixel* FrameSource::getLinePtr960_720(unsigned line) const
+{
+	if (getHeight() == 480) {
+		unsigned l2 = (2 * line) / 3;
+		const Pixel* line0 = getLinePtr<Pixel>(l2 + 0, 960);
+		if ((line % 3) == 1) {
+			const Pixel* line1 = getLinePtr<Pixel>(l2 + 1, 960);
+			return blendLines(line0, line1, 960);
+		} else {
+			return line0;
+		}
+	} else {
+		assert(getHeight() == 240);
+		return getLinePtr<Pixel>(line / 3, 960);
+	}
+}
+
+void* FrameSource::getTempBuffer() const
 {
 	if (tempCounter == tempBuffers.size()) {
-		void* buf = MemoryOps::mallocAligned(64, getLineBufferSize());
+		unsigned size = 1280 * pixelFormat.BytesPerPixel;
+		void* buf = MemoryOps::mallocAligned(64, size);
 		tempBuffers.push_back(buf);
 	}
 	return tempBuffers[tempCounter++];
 }
 
-void FrameSource::freeLineBuffers()
+void FrameSource::freeLineBuffers() const
 {
 	tempCounter = 0; // reuse tempBuffers
 }
 
 template <typename Pixel>
 const Pixel* FrameSource::scaleLine(
-		const Pixel* in, unsigned inWidth, unsigned outWidth)
+		const Pixel* in, unsigned inWidth, unsigned outWidth) const
 {
 	PixelOperations<Pixel> pixelOps(pixelFormat);
 	Pixel* out = reinterpret_cast<Pixel*>(getTempBuffer());
@@ -94,7 +122,7 @@ const Pixel* FrameSource::scaleLine(
 			out[0] = in[0];
 			break;
 		case 213:
-			assert(false);
+			UNREACHABLE;
 		case 320: {
 			Scale_2on3<Pixel> scale(pixelOps);
 			scale(in, out, outWidth);
@@ -126,7 +154,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		default:
-			assert(false);
+			UNREACHABLE;
 		}
 		break;
 	case 320:
@@ -140,7 +168,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		case 320:
-			assert(false);
+			UNREACHABLE;
 		case 426: {
 			Scale_3on4<Pixel> scale(pixelOps);
 			scale(in, out, outWidth);
@@ -167,7 +195,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		default:
-			assert(false);
+			UNREACHABLE;
 		}
 		break;
 	case 426:
@@ -186,7 +214,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		case 426:
-			assert(false);
+			UNREACHABLE;
 		case 640: {
 			Scale_2on3<Pixel> scale(pixelOps);
 			scale(in, out, outWidth);
@@ -208,7 +236,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		default:
-			assert(false);
+			UNREACHABLE;
 		}
 		break;
 	case 640:
@@ -232,7 +260,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		case 640:
-			assert(false);
+			UNREACHABLE;
 		case 853: {
 			Scale_3on4<Pixel> scale(pixelOps);
 			scale(in, out, outWidth);
@@ -249,7 +277,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		default:
-			assert(false);
+			UNREACHABLE;
 		}
 		break;
 	case 853:
@@ -278,7 +306,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		case 853:
-			assert(false);
+			UNREACHABLE;
 		case 960: {
 			Scale_8on9<Pixel> scale(pixelOps);
 			scale(in, out, outWidth);
@@ -290,7 +318,7 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		default:
-			assert(false);
+			UNREACHABLE;
 		}
 		break;
 	case 1280:
@@ -329,13 +357,13 @@ const Pixel* FrameSource::scaleLine(
 			break;
 		}
 		case 1280:
-			assert(false);
+			UNREACHABLE;
 		default:
-			assert(false);
+			UNREACHABLE;
 		}
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 
 	return out;
@@ -344,14 +372,14 @@ const Pixel* FrameSource::scaleLine(
 
 // Force template method instantiation
 #if HAVE_16BPP
-template const word* FrameSource::scaleLine(const word*, unsigned, unsigned);
-template const word* FrameSource::getLinePtr320_240<word>(unsigned);
-template const word* FrameSource::getLinePtr640_480<word>(unsigned);
+template const word* FrameSource::getLinePtr320_240<word>(unsigned) const;
+template const word* FrameSource::getLinePtr640_480<word>(unsigned) const;
+template const word* FrameSource::getLinePtr960_720<word>(unsigned) const;
 #endif
-#if HAVE_32BPP
-template const unsigned* FrameSource::scaleLine(const unsigned*, unsigned, unsigned);
-template const unsigned* FrameSource::getLinePtr320_240<unsigned>(unsigned);
-template const unsigned* FrameSource::getLinePtr640_480<unsigned>(unsigned);
+#if HAVE_32BPP || COMPONENT_GL
+template const unsigned* FrameSource::getLinePtr320_240<unsigned>(unsigned) const;
+template const unsigned* FrameSource::getLinePtr640_480<unsigned>(unsigned) const;
+template const unsigned* FrameSource::getLinePtr960_720<unsigned>(unsigned) const;
 #endif
 
 } // namespace openmsx
