@@ -1,4 +1,4 @@
-// $Id: GLPostProcessor.cc 10052 2009-06-24 07:47:43Z m9710797 $
+// $Id: GLPostProcessor.cc 11381 2010-04-04 17:02:06Z mthuurne $
 
 #include "GLPostProcessor.hh"
 #include "GLScaler.hh"
@@ -7,6 +7,7 @@
 #include "FloatSetting.hh"
 #include "EnumSetting.hh"
 #include "OutputSurface.hh"
+#include "RawFrame.hh"
 #include "Math.hh"
 #include "MemoryOps.hh"
 #include "InitException.hh"
@@ -47,6 +48,8 @@ GLPostProcessor::GLPostProcessor(
 	noiseY = 0.0;
 	preCalcNoise(renderSettings.getNoise().getValue());
 
+	superImposeFrame = NULL;
+
 	storedFrame = false;
 	for (int i = 0; i < 2; ++i) {
 		colorTex[i].reset(new Texture());
@@ -151,9 +154,21 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 	if (scaleAlgorithm != algo) {
 		scaleAlgorithm = algo;
 		currScaler = GLScalerFactory::createScaler(renderSettings);
+
+		// Re-upload frame data, this is both
+		//  - Chunks of RawFrame with a specific linewidth, possibly
+		//    with some extra lines above and below each chunk that are
+		//    also converted to this linewidth.
+		//  - Extra data that is specific for the scaler (ATM only the
+		//    hq and hqlite scalers require this).
+		// Re-uploading the first is not strictly needed. But switching
+		// scalers doesn't happen that often, so it also doesn't hurt
+		// and it keeps the code simpler.
+		uploadFrame();
 	}
 
 	if (renderToTexture) {
+		glViewport(0, 0, screen.getWidth(), screen.getHeight());
 		glBindTexture(GL_TEXTURE_2D, 0);
 		fbo[frameCounter & 1]->push();
 	}
@@ -165,10 +180,13 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 		//fprintf(stderr, "post processing lines %d-%d: %d\n",
 		//	it->srcStartY, it->srcEndY, it->lineWidth);
 		assert(textures.find(it->lineWidth) != textures.end());
+		ColorTexture* superImpose = superImposeFrame
+		                          ? superImposeTex.get() : NULL;
 		currScaler->scaleImage(
-			*textures[it->lineWidth].tex,
-			it->srcStartY, it->srcEndY, it->lineWidth,      // src
-			it->dstStartY, it->dstEndY, screen.getWidth()); // dst
+			*textures[it->lineWidth].tex, superImpose,
+			it->srcStartY, it->srcEndY, it->lineWidth,     // src
+			it->dstStartY, it->dstEndY, screen.getWidth(), // dst
+			paintFrame->getHeight()); // dst
 		//GLUtil::checkGLError("GLPostProcessor::paint");
 	}
 
@@ -180,6 +198,8 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 	if (renderToTexture) {
 		fbo[frameCounter & 1]->pop();
 		colorTex[frameCounter & 1]->bind();
+		glViewport(screen.getX(), screen.getY(),
+		           screen.getWidth(), screen.getHeight());
 
 		glEnable(GL_TEXTURE_2D);
 		if (deform == RenderSettings::DEFORM_3D) {
@@ -216,6 +236,11 @@ RawFrame* GLPostProcessor::rotateFrames(
 	return reuseFrame;
 }
 
+void GLPostProcessor::setSuperimposeFrame(const RawFrame* videoSource)
+{
+	superImposeFrame = videoSource;
+}
+
 void GLPostProcessor::update(const Setting& setting)
 {
 	VideoLayer::update(setting);
@@ -243,6 +268,28 @@ void GLPostProcessor::uploadFrame()
 		            std::min<int>(srcHeight, it->srcEndY   + after),
 		            it->lineWidth);
 	}
+
+	if (superImposeFrame) {
+		int width  = superImposeFrame->getWidth();
+		int height = superImposeFrame->getHeight();
+		if (!superImposeTex.get() ||
+		    superImposeTex->getWidth()  != width ||
+		    superImposeTex->getHeight() != height) {
+			superImposeTex.reset(new ColorTexture(width, height));
+			superImposeTex->enableInterpolation();
+		}
+		superImposeTex->bind();
+		glTexSubImage2D(
+			GL_TEXTURE_2D,     // target
+			0,                 // level
+			0,                 // offset x
+			0,                 // offset y
+			width,             // width
+			height,            // height
+			GL_BGRA,           // format
+			GL_UNSIGNED_BYTE,  // type
+			const_cast<RawFrame*>(superImposeFrame)->getLinePtrDirect<unsigned>(0)); // data
+	}
 }
 
 void GLPostProcessor::uploadBlock(
