@@ -1,4 +1,4 @@
-// $Id: GLScalerFactory.cc 8223 2008-09-14 19:17:24Z m9710797 $
+// $Id: GLScalerFactory.cc 11170 2010-01-17 22:03:11Z mthuurne $
 
 #include "GLScalerFactory.hh"
 #include "GLSimpleScaler.hh"
@@ -10,7 +10,7 @@
 #include "GLHQLiteScaler.hh"
 #include "RenderSettings.hh"
 #include "EnumSetting.hh"
-#include <cassert>
+#include "unreachable.hh"
 
 using std::auto_ptr;
 
@@ -34,13 +34,13 @@ auto_ptr<GLScaler> createScaler(RenderSettings& renderSettings)
 	case RenderSettings::SCALER_SCALE:
 		return auto_ptr<GLScaler>(new GLScaleNxScaler());
 	case RenderSettings::SCALER_TV:
-		return auto_ptr<GLScaler>(new GLTVScaler());
+		return auto_ptr<GLScaler>(new GLTVScaler(renderSettings));
 	case RenderSettings::SCALER_HQ:
 		return auto_ptr<GLScaler>(new GLHQScaler());
 	case RenderSettings::SCALER_HQLITE:
 		return auto_ptr<GLScaler>(new GLHQLiteScaler());
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 	return auto_ptr<GLScaler>(); // avoid warning
 }
