@@ -1,4 +1,4 @@
-// $Id: GLUtil.cc 9511 2009-04-09 18:34:13Z m9710797 $
+// $Id: GLUtil.cc 10787 2009-11-16 15:32:54Z m9710797 $
 
 #include "GLUtil.hh"
 #include "File.hh"
@@ -155,120 +155,6 @@ void LuminanceTexture::updateImage(
 }
 
 
-// class BitmapTexture
-
-BitmapTexture::BitmapTexture()
-	: Texture(GL_TEXTURE_2D)
-{
-	static const unsigned SIZE = WIDTH * HEIGHT * 4;
-	std::vector<byte> blackness(SIZE); // zero-initialized
-	bind();
-	glTexImage2D(
-		GL_TEXTURE_2D,    // target
-		0,                // level
-		GL_RGBA8,         // internal format
-		512,              // width
-		1024,             // height
-		0,                // border
-		GL_BGRA,          // format
-		GL_UNSIGNED_BYTE, // type
-		&blackness[0]);   // data
-}
-
-void BitmapTexture::update(int y, const GLuint* dataBGR, int lineWidth)
-{
-	assert(0 <= y && y < HEIGHT);
-	assert(lineWidth <= WIDTH);
-	bind();
-	// Replace line in existing texture.
-	glTexSubImage2D(
-		GL_TEXTURE_2D,    // target
-		0,                // level
-		0,                // x-offset
-		y,                // y-offset
-		lineWidth,        // width
-		1,                // height
-		GL_BGRA,          // format
-		GL_UNSIGNED_BYTE, // type
-		dataBGR);         // data
-}
-
-void BitmapTexture::draw(
-	int srcL, int srcT, int srcR, int srcB,
-	int dstL, int dstT, int dstR, int dstB)
-{
-	static const GLfloat fx = 1.0f / GLfloat(WIDTH);
-	static const GLfloat fy = 1.0f / GLfloat(HEIGHT);
-	const GLfloat srcLF = srcL * fx;
-	const GLfloat srcRF = srcR * fx;
-	const GLfloat srcTF = srcT * fy;
-	const GLfloat srcBF = srcB * fy;
-	bind();
-	glBegin(GL_QUADS);
-	glTexCoord2f(srcLF, srcTF); glVertex2i(dstL, dstT);
-	glTexCoord2f(srcRF, srcTF); glVertex2i(dstR, dstT);
-	glTexCoord2f(srcRF, srcBF); glVertex2i(dstR, dstB);
-	glTexCoord2f(srcLF, srcBF); glVertex2i(dstL, dstB);
-	glEnd();
-}
-
-
-// class LineTexture
-
-LineTexture::LineTexture()
-	: Texture(GL_TEXTURE_2D) // TODO use 1D texture?
-	, prevLineWidth(-1)
-{
-}
-
-void LineTexture::update(const GLuint* dataBGR, int lineWidth)
-{
-	bind();
-	if (prevLineWidth == lineWidth) {
-		// reuse existing texture
-		glTexSubImage2D(
-			GL_TEXTURE_2D,    // target
-			0,                // level
-			0,                // x-offset
-			0,                // y-offset
-			lineWidth,        // width
-			1,                // height
-			GL_BGRA,          // format
-			GL_UNSIGNED_BYTE, // type
-			dataBGR);         // data
-	} else {
-		// create new texture
-		prevLineWidth = lineWidth;
-		glTexImage2D(
-			GL_TEXTURE_2D,    // target
-			0,                // level
-			GL_RGBA8,         // internal format
-			lineWidth,        // width
-			1,                // height
-			0,                // border
-			GL_BGRA,          // format
-			GL_UNSIGNED_BYTE, // type
-			dataBGR);         // data
-	}
-}
-
-void LineTexture::draw(
-	int texX, int screenX, int screenY, int width, int height)
-{
-	GLfloat texL = texX / 512.0f;
-	GLfloat texR = (texX + width) / 512.0f;
-	int screenL = screenX;
-	int screenR = screenL + width;
-	bind();
-	glBegin(GL_QUADS);
-	glTexCoord2f(texL, 1.0f); glVertex2i(screenL, screenY);
-	glTexCoord2f(texR, 1.0f); glVertex2i(screenR, screenY);
-	glTexCoord2f(texR, 0.0f); glVertex2i(screenR, screenY + height);
-	glTexCoord2f(texL, 0.0f); glVertex2i(screenL, screenY + height);
-	glEnd();
-}
-
-
 // class StoredFrame
 
 StoredFrame::StoredFrame()
@@ -336,7 +222,7 @@ void StoredFrame::drawBlend(
 {
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	// RGB come from texture, alpha comes from fragment colour.
+	// RGB come from texture, alpha comes from fragment color.
 	glColor4f(1.0, 0.0, 0.0, GLfloat(alpha));
 	draw(offsetX, offsetY, width, height);
 	glDisable(GL_BLEND);
@@ -348,19 +234,17 @@ void StoredFrame::drawBlend(
 static GLuint currentId = 0;
 static std::vector<GLuint> stack;
 
-FrameBufferObject::FrameBufferObject(Texture& texture, bool doPush)
+FrameBufferObject::FrameBufferObject(Texture& texture)
 {
 	assert(texture.type == GL_TEXTURE_2D);
 	glGenFramebuffersEXT(1, &bufferId);
-	push();
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, bufferId);
 	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,
 	                          GL_COLOR_ATTACHMENT0_EXT,
 	                          GL_TEXTURE_2D, texture.textureId, 0);
 	bool success = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT) ==
 	               GL_FRAMEBUFFER_COMPLETE_EXT;
-	if (!doPush) {
-		pop();
-	}
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, currentId);
 	if (!success) {
 		throw InitException(
 			"Your OpenGL implementation support for "
@@ -414,6 +298,16 @@ static string readTextFile(const string& filename)
 
 #ifdef GL_VERSION_2_0
 Shader::Shader(GLenum type, const string& filename)
+{
+	init(type, "", filename);
+}
+
+Shader::Shader(GLenum type, const string& header, const string& filename)
+{
+	init(type, header, filename);
+}
+
+void Shader::init(GLenum type, const string& header, const string& filename)
 {
 	// Check if GL 2.0 is present on this machine.
 	if (!GLEW_VERSION_2_0) {
@@ -422,9 +316,9 @@ Shader::Shader(GLenum type, const string& filename)
 	}
 
 	// Load shader source.
-	string source;
+	string source = header;
 	try {
-		source = readTextFile("shaders/" + filename);
+		source += readTextFile("shaders/" + filename);
 	} catch (FileException& e) {
 		std::cerr << "Cannot find shader: " << e.getMessage() << std::endl;
 		handle = 0;
@@ -499,6 +393,11 @@ VertexShader::VertexShader(const string& filename)
 {
 }
 
+VertexShader::VertexShader(const string& header, const string& filename)
+	: Shader(GL_VERTEX_SHADER, header, filename)
+{
+}
+
 
 // class FragmentShader
 
@@ -512,6 +411,11 @@ FragmentShader::FragmentShader(const string& filename)
 {
 }
 
+FragmentShader::FragmentShader(const string& header, const string& filename)
+	: Shader(GL_FRAGMENT_SHADER, header, filename)
+{
+}
+
 
 // class ShaderProgram
 
