@@ -1,4 +1,4 @@
-// $Id: PixelRenderer.cc 8466 2009-01-11 09:15:14Z m9710797 $
+// $Id: PixelRenderer.cc 11182 2010-01-24 08:51:16Z mthuurne $
 
 /*
 TODO:
@@ -20,7 +20,9 @@ TODO:
 #include "FinishFrameEvent.hh"
 #include "RealTime.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "Timer.hh"
+#include "unreachable.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -112,18 +114,21 @@ void PixelRenderer::subdivide(
 
 PixelRenderer::PixelRenderer(VDP& vdp_, Display& display)
 	: vdp(vdp_), vram(vdp.getVRAM())
-	, eventDistributor(vdp.getMotherBoard().getEventDistributor())
+	, eventDistributor(vdp.getMotherBoard().getReactor().getEventDistributor())
 	, realTime(vdp.getMotherBoard().getRealTime())
 	, renderSettings(display.getRenderSettings())
 	, spriteChecker(vdp.getSpriteChecker())
 	, rasterizer(display.getVideoSystem().createRasterizer(vdp))
 {
+	// In case of loadstate we can't yet query any state from the VDP
+	// (because that object is not yet fully deserialized). But
+	// VDP::serialize() will call Renderer::reInit() again when it is
+	// safe to query.
+	reInit();
+
 	finishFrameDuration = 0;
 	frameSkipCounter = 999; // force drawing of frame
-	renderFrame = false; // don't draw before frameStart is called
 	prevRenderFrame = false;
-	displayEnabled = vdp.isDisplayEnabled();
-	rasterizer->reset();
 
 	renderSettings.getMaxFrameSkip().attach(*this);
 	renderSettings.getMinFrameSkip().attach(*this);
@@ -135,11 +140,15 @@ PixelRenderer::~PixelRenderer()
 	renderSettings.getMaxFrameSkip().detach(*this);
 }
 
-void PixelRenderer::reset(EmuTime::param time)
+void PixelRenderer::reInit()
 {
+	// Don't draw before frameStart() is called.
+	// This for example can happen after a loadstate or after switching
+	// renderer in the middle of a frame.
+	renderFrame = false;
+
 	rasterizer->reset();
 	displayEnabled = vdp.isDisplayEnabled();
-	frameStart(time);
 }
 
 void PixelRenderer::updateDisplayEnabled(bool enabled, EmuTime::param time)
@@ -254,29 +263,36 @@ void PixelRenderer::updateTransparency(
 	rasterizer->setTransparency(enabled);
 }
 
-void PixelRenderer::updateForegroundColour(
-	int /*colour*/, EmuTime::param time)
+void PixelRenderer::updateSuperimposing(
+	const RawFrame* videoSource, EmuTime::param time)
+{
+	if (displayEnabled) sync(time);
+	rasterizer->setSuperimposeFrame(videoSource);
+}
+
+void PixelRenderer::updateForegroundColor(
+	int /*color*/, EmuTime::param time)
 {
 	if (displayEnabled) sync(time);
 }
 
-void PixelRenderer::updateBackgroundColour(
-	int colour, EmuTime::param time)
+void PixelRenderer::updateBackgroundColor(
+	int color, EmuTime::param time)
 {
 	sync(time);
 	if (vdp.getDisplayMode().getByte() != DisplayMode::GRAPHIC7) {
-		rasterizer->setBackgroundColour(colour);
+		rasterizer->setBackgroundColor(color);
 	}
 }
 
-void PixelRenderer::updateBlinkForegroundColour(
-	int /*colour*/, EmuTime::param time)
+void PixelRenderer::updateBlinkForegroundColor(
+	int /*color*/, EmuTime::param time)
 {
 	if (displayEnabled) sync(time);
 }
 
-void PixelRenderer::updateBlinkBackgroundColour(
-	int /*colour*/, EmuTime::param time)
+void PixelRenderer::updateBlinkBackgroundColor(
+	int /*color*/, EmuTime::param time)
 {
 	if (displayEnabled) sync(time);
 }
@@ -297,15 +313,15 @@ void PixelRenderer::updatePalette(
 	if (displayEnabled) {
 		sync(time);
 	} else {
-		// Only sync if border colour changed.
+		// Only sync if border color changed.
 		DisplayMode mode = vdp.getDisplayMode();
 		if (mode.getBase() == DisplayMode::GRAPHIC5) {
-			int bgColour = vdp.getBackgroundColour();
-			if (index == (bgColour & 3) || (index == (bgColour >> 2))) {
+			int bgColor = vdp.getBackgroundColor();
+			if (index == (bgColor & 3) || (index == (bgColor >> 2))) {
 				sync(time);
 			}
 		} else if (mode.getByte() != DisplayMode::GRAPHIC7) {
-			if (index == vdp.getBackgroundColour()) {
+			if (index == vdp.getBackgroundColor()) {
 				sync(time);
 			}
 		}
@@ -352,7 +368,7 @@ void PixelRenderer::updatePatternBase(
 	if (displayEnabled) sync(time);
 }
 
-void PixelRenderer::updateColourBase(
+void PixelRenderer::updateColorBase(
 	int /*addr*/, EmuTime::param time)
 {
 	if (displayEnabled) sync(time);
@@ -405,14 +421,14 @@ inline bool PixelRenderer::checkSync(int offset, EmuTime::param time)
 	switch(vdp.getDisplayMode().getBase()) {
 	case DisplayMode::GRAPHIC2:
 	case DisplayMode::GRAPHIC3:
-		if (vram.colourTable.isInside(offset)) {
+		if (vram.colorTable.isInside(offset)) {
 			int vramQuarter = (offset & 0x1800) >> 11;
-			int mask = (vram.colourTable.getMask() & 0x1800) >> 11;
+			int mask = (vram.colorTable.getMask() & 0x1800) >> 11;
 			for (int i = 0; i < 4; i++) {
 				if ( (i & mask) == vramQuarter
 				&& overlap(displayY0, displayY1, i * 64, (i + 1) * 64) ) {
 					/*fprintf(stderr,
-						"colour table: %05X %04X - quarter %d\n",
+						"color table: %05X %04X - quarter %d\n",
 						offset, offset & 0x1FFF, i
 						);*/
 					return true;
@@ -463,7 +479,7 @@ inline bool PixelRenderer::checkSync(int offset, EmuTime::param time)
 	default:
 		// Range unknown; assume full range.
 		return vram.nameTable.isInside(offset)
-			|| vram.colourTable.isInside(offset)
+			|| vram.colorTable.isInside(offset)
 			|| vram.patternTable.isInside(offset);
 	}
 }
@@ -471,7 +487,7 @@ inline bool PixelRenderer::checkSync(int offset, EmuTime::param time)
 void PixelRenderer::updateVRAM(unsigned offset, EmuTime::param time)
 {
 	// Note: No need to sync if display is disabled, because then the
-	//       output does not depend on VRAM (only on background colour).
+	//       output does not depend on VRAM (only on background color).
 	if (renderFrame && displayEnabled && checkSync(offset, time)) {
 		//fprintf(stderr, "vram sync @ line %d\n",
 		//	vdp.getTicksThisFrame(time) / VDP::TICKS_PER_LINE);
@@ -533,7 +549,7 @@ void PixelRenderer::renderUntil(EmuTime::param time)
 		break;
 	}
 	default:
-		assert(false);
+		UNREACHABLE;
 		limitX = limitY = 0; // avoid warning
 	}
 
@@ -552,9 +568,9 @@ void PixelRenderer::renderUntil(EmuTime::param time)
 
 		// Calculate start and end of borders in ticks since start of line.
 		// The 0..7 extra horizontal scroll low pixels should be drawn in
-		// border colour. These will be drawn together with the border,
+		// border color. These will be drawn together with the border,
 		// but sprites above these pixels are clipped at the actual border
-		// rather than the end of the border coloured area.
+		// rather than the end of the border colored area.
 		// TODO: Move these calculations and getDisplayLeft() to VDP.
 		int borderL = vdp.getLeftBorder();
 		int displayL =
@@ -586,7 +602,7 @@ void PixelRenderer::update(const Setting& setting)
 		// Force drawing of frame.
 		frameSkipCounter = 999;
 	} else {
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
