@@ -1,8 +1,9 @@
-// $Id: RGBTriplet3xScaler.cc 8452 2009-01-08 22:38:26Z m9710797 $
+// $Id: RGBTriplet3xScaler.cc 11107 2010-01-06 22:09:08Z m9710797 $
 
 #include "RGBTriplet3xScaler.hh"
+#include "SuperImposedFrame.hh"
 #include "LineScalers.hh"
-#include "FrameSource.hh"
+#include "RawFrame.hh"
 #include "OutputSurface.hh"
 #include "RenderSettings.hh"
 #include "vla.hh"
@@ -41,7 +42,7 @@ void RGBTriplet3xScaler<Pixel>::calcSpil(unsigned x, unsigned& r, unsigned& s)
 
 template <class Pixel>
 void RGBTriplet3xScaler<Pixel>::rgbify(
-		const Pixel* in, Pixel* out, unsigned inwidth)
+		const Pixel* __restrict in, Pixel* __restrict out, unsigned inwidth)
 {
 	unsigned r, g, b, rs, gs, bs;
 	unsigned i = 0;
@@ -71,12 +72,11 @@ void RGBTriplet3xScaler<Pixel>::rgbify(
 }
 
 template <typename Pixel>
-template <typename ScaleOp>
 void RGBTriplet3xScaler<Pixel>::scaleLine(
-		const Pixel* srcLine, Pixel* dstLine, ScaleOp scale,
+		const Pixel* srcLine, Pixel* dstLine, PolyLineScaler<Pixel>& scale,
 		unsigned tmpWidth)
 {
-	if (IsTagged<ScaleOp, Copy>::result) {
+	if (scale.isCopy()) {
 		rgbify(srcLine, dstLine, tmpWidth);
 	} else {
 		VLA(Pixel, tmp, tmpWidth);
@@ -89,11 +89,10 @@ void RGBTriplet3xScaler<Pixel>::scaleLine(
 // to output-width / 3. So, when calling this, keep this in mind and pass a
 // scale functor that scales the input with correctly.
 template <typename Pixel>
-template <typename ScaleOp>
 void RGBTriplet3xScaler<Pixel>::doScale1(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
-	ScaleOp scale)
+	PolyLineScaler<Pixel>& scale)
 {
 	dst.lock();
 	recalcBlur();
@@ -133,11 +132,10 @@ void RGBTriplet3xScaler<Pixel>::doScale1(FrameSource& src,
 }
 
 template <typename Pixel>
-template <typename ScaleOp>
 void RGBTriplet3xScaler<Pixel>::doScale2(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
-	ScaleOp scale)
+	PolyLineScaler<Pixel>& scale)
 {
 	dst.lock();
 	recalcBlur();
@@ -165,8 +163,9 @@ void RGBTriplet3xScaler<Pixel>::scale2x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_2on3<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_2on3<Pixel> > op(pixelOps);
+	doScale1(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -174,8 +173,9 @@ void RGBTriplet3xScaler<Pixel>::scale2x2to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_2on3<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_2on3<Pixel> > op(pixelOps);
+	doScale2(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -183,8 +183,9 @@ void RGBTriplet3xScaler<Pixel>::scale1x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_1on1<Pixel>());
+	PolyScale<Pixel, Scale_1on1<Pixel> > op;
+	doScale1(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -192,8 +193,9 @@ void RGBTriplet3xScaler<Pixel>::scale1x2to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_1on1<Pixel>());
+	PolyScale<Pixel, Scale_1on1<Pixel> > op;
+	doScale2(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -201,8 +203,9 @@ void RGBTriplet3xScaler<Pixel>::scale4x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_4on3<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_4on3<Pixel> > op(pixelOps);
+	doScale1(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -210,8 +213,9 @@ void RGBTriplet3xScaler<Pixel>::scale4x2to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_4on3<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_4on3<Pixel> > op(pixelOps);
+	doScale2(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -219,8 +223,9 @@ void RGBTriplet3xScaler<Pixel>::scale2x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_2on1<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_2on1<Pixel> > op(pixelOps);
+	doScale1(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -228,8 +233,9 @@ void RGBTriplet3xScaler<Pixel>::scale2x2to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_2on1<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_2on1<Pixel> > op(pixelOps);
+	doScale2(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -237,8 +243,9 @@ void RGBTriplet3xScaler<Pixel>::scale8x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_8on3<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_8on3<Pixel> > op(pixelOps);
+	doScale1(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -246,8 +253,9 @@ void RGBTriplet3xScaler<Pixel>::scale8x2to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_8on3<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_8on3<Pixel> > op(pixelOps);
+	doScale2(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -255,8 +263,9 @@ void RGBTriplet3xScaler<Pixel>::scale4x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_4on1<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_4on1<Pixel> > op(pixelOps);
+	doScale1(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <class Pixel>
@@ -264,12 +273,14 @@ void RGBTriplet3xScaler<Pixel>::scale4x2to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-	         Scale_4on1<Pixel>(pixelOps));
+	PolyScale<Pixel, Scale_4on1<Pixel> > op(pixelOps);
+	doScale2(src, srcStartY, srcEndY, srcWidth,
+	         dst, dstStartY, dstEndY, op);
 }
 
 template <typename Pixel>
-static void fillLoop(const Pixel* in, Pixel* out, unsigned dstWidth)
+static void fillLoop(const Pixel* __restrict in, Pixel* __restrict out,
+                     unsigned dstWidth)
 {
 	out[0] = in[0];
 	out[1] = in[1];
@@ -320,8 +331,8 @@ void RGBTriplet3xScaler<Pixel>::scaleBlank1to3(
 		unsigned nextLineWidth = src.getLineWidth(srcY + 1);
 		assert(src.getLineWidth(srcY) == 1);
 		assert(nextLineWidth != 1);
-		this->scaleImage(src, srcY, srcEndY, nextLineWidth,
-		                 dst, dstY, dstEndY);
+		this->dispatchScale(src, srcY, srcEndY, nextLineWidth,
+		                    dst, dstY, dstEndY);
 	}
 }
 
@@ -363,6 +374,24 @@ void RGBTriplet3xScaler<Pixel>::scaleBlank2to3(
 	}
 }
 
+template <class Pixel>
+void RGBTriplet3xScaler<Pixel>::scaleImage(FrameSource& src, const RawFrame* superImpose,
+	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
+	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
+{
+	if (superImpose) {
+		SuperImposedFrame<Pixel> sf(src, *superImpose, pixelOps.getSDLPixelFormat());
+		srcWidth = sf.getLineWidth(srcStartY);
+		this->dispatchScale(sf,  srcStartY, srcEndY, srcWidth,
+		                    dst, dstStartY, dstEndY);
+		src.freeLineBuffers();
+		superImpose->freeLineBuffers();
+	} else {
+		this->dispatchScale(src, srcStartY, srcEndY, srcWidth,
+		                    dst, dstStartY, dstEndY);
+	}
+}
+
 // Force template instantiation.
 #if HAVE_16BPP
 template class RGBTriplet3xScaler<word>;
