@@ -1,4 +1,4 @@
-// $Id: RenderSettings.cc 10003 2009-06-07 20:54:16Z m9710797 $
+// $Id: RenderSettings.cc 11170 2010-01-17 22:03:11Z mthuurne $
 
 #include "RenderSettings.hh"
 #include "IntegerSetting.hh"
@@ -6,9 +6,10 @@
 #include "BooleanSetting.hh"
 #include "StringSetting.hh"
 #include "VideoSourceSetting.hh"
-#include "CommandController.hh"
 #include "CommandException.hh"
+#include "unreachable.hh"
 #include "build-info.hh"
+#include <algorithm>
 #include <cmath>
 
 namespace openmsx {
@@ -24,64 +25,66 @@ private:
 };
 
 
-RenderSettings::RenderSettings(CommandController& commandController_)
-	: commandController(commandController_)
+RenderSettings::RenderSettings(CommandController& commandController)
 {
 	EnumSetting<Accuracy>::Map accMap;
 	accMap["screen"] = ACC_SCREEN;
 	accMap["line"]   = ACC_LINE;
 	accMap["pixel"]  = ACC_PIXEL;
-	accuracy.reset(new EnumSetting<Accuracy>(commandController,
+	accuracySetting.reset(new EnumSetting<Accuracy>(commandController,
 		"accuracy", "rendering accuracy", ACC_PIXEL, accMap));
 
-	deinterlace.reset(new BooleanSetting(commandController,
+	deinterlaceSetting.reset(new BooleanSetting(commandController,
 		"deinterlace", "deinterlacing on/off", true));
 
-	maxFrameSkip.reset(new IntegerSetting(commandController,
+	maxFrameSkipSetting.reset(new IntegerSetting(commandController,
 		"maxframeskip", "set the max amount of frameskip", 3, 0, 100));
 
-	minFrameSkip.reset(new IntegerSetting(commandController,
+	minFrameSkipSetting.reset(new IntegerSetting(commandController,
 		"minframeskip", "set the min amount of frameskip", 0, 0, 100));
 
-	fullScreen.reset(new BooleanSetting(commandController,
+	fullScreenSetting.reset(new BooleanSetting(commandController,
 		"fullscreen", "full screen display on/off", false));
 
-	gamma.reset(new FloatSetting(commandController, "gamma",
+	gammaSetting.reset(new FloatSetting(commandController, "gamma",
 		"amount of gamma correction: low is dark, high is bright",
 		1.1, 0.1, 5.0));
 
-	brightness.reset(new FloatSetting(commandController, "brightness",
+	brightnessSetting.reset(new FloatSetting(commandController, "brightness",
 		"brightness video setting: "
 		"0 is normal, lower is darker, higher is brighter",
 		0.0, -100.0, 100.0));
-
-	contrast.reset(new FloatSetting(commandController, "contrast",
+	contrastSetting.reset(new FloatSetting(commandController, "contrast",
 		"contrast video setting: "
 		"0 is normal, lower is less contrast, higher is more contrast",
 		0.0, -100.0, 100.0));
+	brightnessSetting->attach(*this);
+	contrastSetting->attach(*this);
+	updateBrightnessAndContrast();
 
-	colorMatrix.reset(new StringSetting(commandController, "color_matrix",
+	colorMatrixSetting.reset(new StringSetting(commandController,
+		"color_matrix",
 		"3x3 matrix to transform MSX RGB to host RGB, see manual for details",
 		"{ 1 0 0 } { 0 1 0 } { 0 0 1 }"));
 	colorMatrixChecker.reset(new ColorMatrixChecker(*this));
-	colorMatrix->setChecker(colorMatrixChecker.get());
+	colorMatrixSetting->setChecker(colorMatrixChecker.get());
 
-	glow.reset(new IntegerSetting(commandController,
+	glowSetting.reset(new IntegerSetting(commandController,
 		"glow", "amount of afterglow effect: 0 = none, 100 = lots",
 		0, 0, 100));
 
-	noise.reset(new FloatSetting(commandController,
+	noiseSetting.reset(new FloatSetting(commandController,
 		"noise", "amount of noise to add to the frame",
 		0.0, 0.0, 100.0));
 
-	horizontalBlur.reset(new IntegerSetting(commandController,
+	horizontalBlurSetting.reset(new IntegerSetting(commandController,
 		"blur", "amount of horizontal blur effect: 0 = none, 100 = full",
 		50, 0, 100));
 
-	videoSource.reset(new VideoSourceSetting(commandController));
+	videoSourceSetting.reset(new VideoSourceSetting(commandController));
 
 	// Get user-preferred renderer from config.
-	renderer = RendererFactory::createRendererSetting(commandController);
+	rendererSetting = RendererFactory::createRendererSetting(commandController);
 
 	EnumSetting<ScaleAlgorithm>::Map scalerMap;
 	scalerMap["simple"] = SCALER_SIMPLE;
@@ -93,37 +96,37 @@ RenderSettings::RenderSettings(CommandController& commandController_)
 		scalerMap["RGBtriplet"] = SCALER_RGBTRIPLET;
 		scalerMap["TV"] = SCALER_TV;
 	}
-	scaleAlgorithm.reset(new EnumSetting<ScaleAlgorithm>(commandController,
+	scaleAlgorithmSetting.reset(new EnumSetting<ScaleAlgorithm>(commandController,
 		"scale_algorithm", "scale algorithm",
 		SCALER_SIMPLE, scalerMap));
 
-	scaleFactor.reset(new IntegerSetting(commandController,
-		"scale_factor", "scale factor", 2,
-		MIN_SCALE_FACTOR, MAX_SCALE_FACTOR));
+	scaleFactorSetting.reset(new IntegerSetting(commandController,
+		"scale_factor", "scale factor",
+		std::min(2, MAX_SCALE_FACTOR), MIN_SCALE_FACTOR, MAX_SCALE_FACTOR));
 
-	scanlineAlpha.reset(new IntegerSetting(commandController,
+	scanlineAlphaSetting.reset(new IntegerSetting(commandController,
 		"scanline", "amount of scanline effect: 0 = none, 100 = full",
 		20, 0, 100));
 
-	limitSprites.reset(new BooleanSetting(commandController,
+	limitSpritesSetting.reset(new BooleanSetting(commandController,
 		"limitsprites", "limit number of sprites per line "
 		"(on for realism, off to reduce sprite flashing)", true));
 
 	EnumSetting<bool>::Map cmdMap;
 	cmdMap["real"]   = false;
 	cmdMap["broken"] = true;
-	cmdTiming.reset(new EnumSetting<bool>(commandController,
+	cmdTimingSetting.reset(new EnumSetting<bool>(commandController,
 		"cmdtiming", "VDP command timing", false, cmdMap,
 		Setting::DONT_SAVE));
 
 	EnumSetting<DisplayDeform>::Map deformMap;
 	deformMap["normal"] = DEFORM_NORMAL;
 	deformMap["3d"] = DEFORM_3D;
-	displayDeform.reset(new EnumSetting<DisplayDeform>(commandController,
+	displayDeformSetting.reset(new EnumSetting<DisplayDeform>(commandController,
 		"display_deform", "Display deform (for the moment this only "
 		"works with the SDLGL-PP renderer", DEFORM_NORMAL, deformMap));
 
-	horizontalStretch.reset(new FloatSetting(commandController,
+	horizontalStretchSetting.reset(new FloatSetting(commandController,
 		"horizontal_stretch",
 		"Amount of horizontal stretch: this many MSX pixels will be "
 		"stretched over the complete width of the output screen.\n"
@@ -133,7 +136,7 @@ RenderSettings::RenderSettings(CommandController& commandController_)
 		"This setting has only effect when using the SDLGL-PP renderer.",
 		284.0, 256.0, 320.0));
 
-	pointerHideDelay.reset(new FloatSetting(commandController,
+	pointerHideDelaySetting.reset(new FloatSetting(commandController,
 		"pointer_hide_delay",
 		"number of seconds after which the pointer is hidden in the openMSX "
 		"window; negative = no hiding, 0 = immediately",
@@ -142,21 +145,43 @@ RenderSettings::RenderSettings(CommandController& commandController_)
 
 RenderSettings::~RenderSettings()
 {
+	brightnessSetting->detach(*this);
+	contrastSetting->detach(*this);
 }
 
 int RenderSettings::getBlurFactor() const
 {
-	return (horizontalBlur->getValue()) * 256 / 100;
+	return (horizontalBlurSetting->getValue()) * 256 / 100;
 }
 
 int RenderSettings::getScanlineFactor() const
 {
-	return 255 - ((scanlineAlpha->getValue() * 255) / 100);
+	return 255 - ((scanlineAlphaSetting->getValue() * 255) / 100);
 }
 
-static double conv1(double x, double brightness, double contrast)
+float RenderSettings::getScanlineGap() const
 {
-	return (x + brightness - 0.5) * contrast + 0.5;
+	return scanlineAlphaSetting->getValue() * 0.01f;
+}
+
+void RenderSettings::update(const Setting& setting)
+{
+	if (&setting == brightnessSetting.get()) {
+		updateBrightnessAndContrast();
+	} else if (&setting == contrastSetting.get()) {
+		updateBrightnessAndContrast();
+	} else {
+		UNREACHABLE;
+	}
+}
+
+void RenderSettings::updateBrightnessAndContrast()
+{
+	double contrastValue = getContrast().getValue();
+	contrast = (contrastValue >= 0.0) ? (1.0 + contrastValue / 25.0)
+	                                  : (1.0 + contrastValue / 125.0);
+	double brightnessValue = getBrightness().getValue();
+	brightness = (brightnessValue / 100.0 - 0.5) * contrast + 0.5;
 }
 
 static double conv2(double x, double gamma)
@@ -164,19 +189,22 @@ static double conv2(double x, double gamma)
 	return ::pow(std::min(std::max(0.0, x), 1.0), gamma);
 }
 
+double RenderSettings::transformComponent(double c) const
+{
+	double c2 = c * contrast + brightness;
+	double gamma = 1.0 / getGamma().getValue();
+	return conv2(c2, gamma);
+}
+
 void RenderSettings::transformRGB(double& r, double& g, double& b) const
 {
-	double brightness = getBrightness().getValue() / 100.0;
-	double contrast = getContrast().getValue();
-	contrast = (contrast >= 0.0) ? (1 + contrast / 25.0)
-	                             : (1 + contrast / 125.0);
-	r = conv1(r, brightness, contrast);
-	g = conv1(g, brightness, contrast);
-	b = conv1(b, brightness, contrast);
+	double rbc = r * contrast + brightness;
+	double gbc = g * contrast + brightness;
+	double bbc = b * contrast + brightness;
 
-	double r2 = cm[0][0] * r + cm[0][1] * g + cm[0][2] * b;
-	double g2 = cm[1][0] * r + cm[1][1] * g + cm[1][2] * b;
-	double b2 = cm[2][0] * r + cm[2][1] * g + cm[2][2] * b;
+	double r2 = cm[0][0] * rbc + cm[0][1] * gbc + cm[0][2] * bbc;
+	double g2 = cm[1][0] * rbc + cm[1][1] * gbc + cm[1][2] * bbc;
+	double b2 = cm[2][0] * rbc + cm[2][1] * gbc + cm[2][2] * bbc;
 
 	double gamma = 1.0 / getGamma().getValue();
 	r = conv2(r2, gamma);
@@ -186,11 +214,12 @@ void RenderSettings::transformRGB(double& r, double& g, double& b) const
 
 void RenderSettings::parseColorMatrix(const std::string& value)
 {
-	TclObject matrix(commandController.getInterpreter());
+	TclObject matrix(colorMatrixSetting->getInterpreter());
 	matrix.setString(value);
 	if (matrix.getListLength() != 3) {
 		throw CommandException("must have 3 rows");
 	}
+	bool identity = true;
 	for (int i = 0; i < 3; ++i) {
 		TclObject row = matrix.getListIndex(i);
 		if (row.getListLength() != 3) {
@@ -198,9 +227,12 @@ void RenderSettings::parseColorMatrix(const std::string& value)
 		}
 		for (int j = 0; j < 3; ++j) {
 			TclObject element = row.getListIndex(j);
-			cm[i][j] = element.getDouble();
+			double value = element.getDouble();
+			cm[i][j] = value;
+			identity &= (value == (i == j ? 1.0 : 0.0));
 		}
 	}
+	cmIdentity = identity;
 }
 
 
