@@ -1,4 +1,4 @@
-// $Id: SDLRasterizer.cc 8466 2009-01-11 09:15:14Z m9710797 $
+// $Id: SDLRasterizer.cc 11182 2010-01-24 08:51:16Z mthuurne $
 
 #include "SDLRasterizer.hh"
 #include "VDP.hh"
@@ -7,6 +7,7 @@
 #include "CharacterConverter.hh"
 #include "BitmapConverter.hh"
 #include "SpriteConverter.hh"
+#include "MSXMotherBoard.hh"
 #include "Display.hh"
 #include "Renderer.hh"
 #include "RenderSettings.hh"
@@ -16,6 +17,7 @@
 #include "MemoryOps.hh"
 #include "VisibleSurface.hh"
 #include "build-info.hh"
+#include "components.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -76,7 +78,7 @@ SDLRasterizer<Pixel>::SDLRasterizer(
 	, renderSettings(display.getRenderSettings())
 	, characterConverter(new CharacterConverter<Pixel>(vdp, palFg, palBg))
 	, bitmapConverter(new BitmapConverter<Pixel>(
-	                                    palFg, PALETTE256, V9958_COLOURS))
+	                                    palFg, PALETTE256, V9958_COLORS))
 	, spriteConverter(new SpriteConverter<Pixel>(vdp.getSpriteChecker()))
 {
 	workFrame = new RawFrame(screen.getSDLFormat(), 640, 240);
@@ -88,7 +90,7 @@ SDLRasterizer<Pixel>::SDLRasterizer(
 	if (!vdp.isMSX1VDP()) {
 		for (int i = 0; i < 16; ++i) {
 			palFg[i] = palFg[i + 16] = palBg[i] =
-				V9938_COLOURS[0][0][0];
+				V9938_COLORS[0][0][0];
 		}
 	}
 
@@ -112,7 +114,8 @@ SDLRasterizer<Pixel>::~SDLRasterizer()
 template <class Pixel>
 bool SDLRasterizer<Pixel>::isActive()
 {
-	return postProcessor->getZ() != Layer::Z_MSX_PASSIVE;
+	return (postProcessor->getZ() != Layer::Z_MSX_PASSIVE) &&
+	       vdp.getMotherBoard().isActive();
 }
 
 template <class Pixel>
@@ -136,6 +139,14 @@ void SDLRasterizer<Pixel>::resetPalette()
 	}
 }
 
+template<class Pixel>
+void SDLRasterizer<Pixel>::setSuperimposeFrame(const RawFrame* videoSource)
+{
+	postProcessor->setSuperimposeFrame(videoSource);
+	precalcColorIndex0(vdp.getDisplayMode(), vdp.getTransparency(),
+	                   videoSource, vdp.getBackgroundColor());
+}
+
 template <class Pixel>
 void SDLRasterizer<Pixel>::frameStart(EmuTime::param time)
 {
@@ -167,8 +178,8 @@ void SDLRasterizer<Pixel>::setDisplayMode(DisplayMode mode)
 	} else {
 		characterConverter->setDisplayMode(mode);
 	}
-	precalcColourIndex0(mode, vdp.getTransparency(),
-	                    vdp.getBackgroundColour());
+	precalcColorIndex0(mode, vdp.getTransparency(),
+	                   vdp.isSuperimposing(), vdp.getBackgroundColor());
 	spriteConverter->setDisplayMode(mode);
 	spriteConverter->setPalette(mode.getByte() == DisplayMode::GRAPHIC7
 	                            ? palGraphic7Sprites : palBg);
@@ -177,30 +188,30 @@ void SDLRasterizer<Pixel>::setDisplayMode(DisplayMode mode)
 template <class Pixel>
 void SDLRasterizer<Pixel>::setPalette(int index, int grb)
 {
-	// Update SDL colours in palette.
-	Pixel newColor = V9938_COLOURS[(grb >> 4) & 7][grb >> 8][grb & 7];
+	// Update SDL colors in palette.
+	Pixel newColor = V9938_COLORS[(grb >> 4) & 7][grb >> 8][grb & 7];
 	palFg[index     ] = newColor;
 	palFg[index + 16] = newColor;
 	palBg[index     ] = newColor;
 	bitmapConverter->palette16Changed();
 
-	precalcColourIndex0(vdp.getDisplayMode(), vdp.getTransparency(),
-	                    vdp.getBackgroundColour());
+	precalcColorIndex0(vdp.getDisplayMode(), vdp.getTransparency(),
+	                   vdp.isSuperimposing(), vdp.getBackgroundColor());
 }
 
 template <class Pixel>
-void SDLRasterizer<Pixel>::setBackgroundColour(int index)
+void SDLRasterizer<Pixel>::setBackgroundColor(int index)
 {
-	precalcColourIndex0(
-		vdp.getDisplayMode(), vdp.getTransparency(), index);
+	precalcColorIndex0(vdp.getDisplayMode(), vdp.getTransparency(),
+	                   vdp.isSuperimposing(), index);
 }
 
 template <class Pixel>
 void SDLRasterizer<Pixel>::setTransparency(bool enabled)
 {
 	spriteConverter->setTransparency(enabled);
-	precalcColourIndex0(
-		vdp.getDisplayMode(), enabled, vdp.getBackgroundColour());
+	precalcColorIndex0(vdp.getDisplayMode(), enabled,
+	                   vdp.isSuperimposing(), vdp.getBackgroundColor());
 }
 
 template <class Pixel>
@@ -215,38 +226,89 @@ void SDLRasterizer<Pixel>::precalcPalette()
 			double db = rgb[2] / 255.0;
 			renderSettings.transformRGB(dr, dg, db);
 			palFg[i] = palFg[i + 16] = palBg[i] =
-				screen.mapRGB(dr, dg, db);
+				screen.mapKeyedRGB<Pixel>(dr, dg, db);
 		}
 	} else {
-		// Precalculate palette for V9938 colours.
-		for (int r = 0; r < 8; r++) {
-			for (int g = 0; g < 8; g++) {
-				for (int b = 0; b < 8; b++) {
-					double dr = r / 7.0;
-					double dg = g / 7.0;
-					double db = b / 7.0;
-					renderSettings.transformRGB(dr, dg, db);
-					V9938_COLOURS[r][g][b] =
-						screen.mapRGB(dr, dg, db);
+		if (vdp.hasYJK()) {
+			// Precalculate palette for V9958 colors.
+			if (renderSettings.isColorMatrixIdentity()) {
+				// Most users use the "normal" monitor type; making this a
+				// special case speeds up palette precalculation a lot.
+				int intensity[32];
+				for (int i = 0; i < 32; i++) {
+					intensity[i] =
+						int(255 * renderSettings.transformComponent(i / 31.0));
+				}
+				for (int rgb = 0; rgb < (1 << 15); rgb++) {
+					V9958_COLORS[rgb] = screen.mapKeyedRGB<Pixel>(
+						intensity[(rgb >> 10)     ],
+						intensity[(rgb >>  5) & 31],
+						intensity[ rgb        & 31]);
+				}
+			} else {
+				for (int r5 = 0; r5 < 32; r5++) {
+					for (int g5 = 0; g5 < 32; g5++) {
+						for (int b5 = 0; b5 < 32; b5++) {
+							double dr = r5 / 31.0;
+							double dg = g5 / 31.0;
+							double db = b5 / 31.0;
+							renderSettings.transformRGB(dr, dg, db);
+							V9958_COLORS[(r5<<10) + (g5<<5) + b5] =
+								screen.mapKeyedRGB<Pixel>(dr, dg, db);
+						}
+					}
 				}
 			}
-		}
-		// Precalculate palette for V9958 colours.
-		for (int r = 0; r < 32; r++) {
-			for (int g = 0; g < 32; g++) {
-				for (int b = 0; b < 32; b++) {
-					double dr = r / 31.0;
-					double dg = g / 31.0;
-					double db = b / 31.0;
-					renderSettings.transformRGB(dr, dg, db);
-					V9958_COLOURS[(r<<10) + (g<<5) + b] =
-						screen.mapRGB(dr, dg, db);
+			// Precalculate palette for V9938 colors.
+			// Based on comparing red and green gradients, using palette and
+			// YJK, in SCREEN11 on a real turbo R.
+			for (int r3 = 0; r3 < 8; r3++) {
+				int r5 = (r3 << 2) | (r3 >> 1);
+				for (int g3 = 0; g3 < 8; g3++) {
+					int g5 = (g3 << 2) | (g3 >> 1);
+					for (int b3 = 0; b3 < 8; b3++) {
+						int b5 = (b3 << 2) | (b3 >> 1);
+						V9938_COLORS[r3][g3][b3] =
+							V9958_COLORS[(r5<<10) + (g5<<5) + b5];
+					}
+				}
+			}
+		} else {
+			// Precalculate palette for V9938 colors.
+			if (renderSettings.isColorMatrixIdentity()) {
+				int intensity[8];
+				for (int i = 0; i < 8; i++) {
+					intensity[i] =
+						int(255 * renderSettings.transformComponent(i / 7.0));
+				}
+				for (int r3 = 0; r3 < 8; r3++) {
+					for (int g3 = 0; g3 < 8; g3++) {
+						for (int b3 = 0; b3 < 8; b3++) {
+							V9938_COLORS[r3][g3][b3] =
+								screen.mapKeyedRGB<Pixel>(
+									intensity[r3], intensity[g3], intensity[b3]
+									);
+						}
+					}
+				}
+			} else {
+				for (int r3 = 0; r3 < 8; r3++) {
+					for (int g3 = 0; g3 < 8; g3++) {
+						for (int b3 = 0; b3 < 8; b3++) {
+							double dr = r3 / 7.0;
+							double dg = g3 / 7.0;
+							double db = b3 / 7.0;
+							renderSettings.transformRGB(dr, dg, db);
+							V9938_COLORS[r3][g3][b3] =
+								screen.mapKeyedRGB<Pixel>(dr, dg, db);
+						}
+					}
 				}
 			}
 		}
 		// Precalculate Graphic 7 bitmap palette.
 		for (int i = 0; i < 256; i++) {
-			PALETTE256[i] = V9938_COLOURS
+			PALETTE256[i] = V9938_COLORS
 				[(i & 0x1C) >> 2]
 				[(i & 0xE0) >> 5]
 				[(i & 0x03) == 3 ? 7 : (i & 0x03) * 2];
@@ -255,14 +317,14 @@ void SDLRasterizer<Pixel>::precalcPalette()
 		for (int i = 0; i < 16; i++) {
 			word grb = Renderer::GRAPHIC7_SPRITE_PALETTE[i];
 			palGraphic7Sprites[i] =
-				V9938_COLOURS[(grb >> 4) & 7][grb >> 8][grb & 7];
+				V9938_COLORS[(grb >> 4) & 7][grb >> 8][grb & 7];
 		}
 	}
 }
 
 template <class Pixel>
-void SDLRasterizer<Pixel>::precalcColourIndex0(
-	DisplayMode mode, bool transparency, byte bgcolorIndex)
+void SDLRasterizer<Pixel>::precalcColorIndex0(DisplayMode mode,
+		bool transparency, const RawFrame* superimposing, byte bgcolorIndex)
 {
 	// Graphic7 mode doesn't use transparency.
 	if (mode.getByte() == DisplayMode::GRAPHIC7) {
@@ -271,11 +333,16 @@ void SDLRasterizer<Pixel>::precalcColourIndex0(
 
 	int tpIndex = transparency ? bgcolorIndex : 0;
 	if (mode.getBase() != DisplayMode::GRAPHIC5) {
-		if (palFg[0] != palBg[tpIndex]) {
-			palFg[0] = palBg[tpIndex];
+		Pixel c = ((superimposing != 0) && bgcolorIndex == 0)
+		        ? screen.getKeyColor<Pixel>()
+		        : palBg[tpIndex];
+
+		if (palFg[0] != c) {
+			palFg[0] = c;
 			bitmapConverter->palette16Changed();
 		}
 	} else {
+		// TODO: superimposing
 		if ((palFg[ 0] != palBg[tpIndex >> 2]) ||
 		    (palFg[16] != palBg[tpIndex &  3])) {
 			palFg[ 0] = palBg[tpIndex >> 2];
@@ -291,7 +358,7 @@ void SDLRasterizer<Pixel>::drawBorder(
 {
 	DisplayMode mode = vdp.getDisplayMode();
 	byte modeBase = mode.getBase();
-	int bgColor = vdp.getBackgroundColour();
+	int bgColor = vdp.getBackgroundColor();
 	Pixel border0, border1;
 	if (modeBase == DisplayMode::GRAPHIC5) {
 		// border in SCREEN6 has separate color for even and odd pixels.
@@ -301,7 +368,11 @@ void SDLRasterizer<Pixel>::drawBorder(
 	} else if (modeBase == DisplayMode::GRAPHIC7) {
 		border0 = border1 = PALETTE256[bgColor];
 	} else {
-		border0 = border1 = palBg[bgColor & 0x0F];
+		if (!bgColor && vdp.isSuperimposing()) {
+			border0 = border1 = screen.getKeyColor<Pixel>();
+		} else {
+			border0 = border1 = palBg[bgColor];
+		}
 	}
 
 	int startY = std::max(fromY - lineRenderTop, 0);
@@ -532,7 +603,7 @@ void SDLRasterizer<Pixel>::update(const Setting& setting)
 #if HAVE_16BPP
 template class SDLRasterizer<word>;
 #endif
-#if HAVE_32BPP
+#if HAVE_32BPP || COMPONENT_GL
 template class SDLRasterizer<unsigned>;
 #endif
 
