@@ -1,4 +1,4 @@
-// $Id: SDLVideoSystem.cc 10004 2009-06-07 21:08:28Z m9710797 $
+// $Id: SDLVideoSystem.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "SDLVideoSystem.hh"
 #include "SDLVisibleSurface.hh"
@@ -17,6 +17,7 @@
 #include "V9990.hh"
 #include "build-info.hh"
 #include "openmsx.hh"
+#include "unreachable.hh"
 #include <memory>
 #include <cassert>
 
@@ -25,24 +26,28 @@
 #include "sdlwin32.hh"
 #endif
 #include "components.hh"
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 #include "SDLGLVisibleSurface.hh"
 #include "GLPostProcessor.hh"
 #endif
+#if COMPONENT_LASERDISC
+#include "LaserdiscPlayer.hh"
+#include "LDSDLRasterizer.hh"
+#endif
 
 namespace openmsx {
 
-SDLVideoSystem::SDLVideoSystem(Reactor& reactor)
+SDLVideoSystem::SDLVideoSystem(Reactor& reactor, CommandConsole& console)
 	: reactor(reactor)
 	, display(reactor.getDisplay())
 	, renderSettings(reactor.getDisplay().getRenderSettings())
 {
 	resize();
 
-	console = screen->createConsoleLayer(reactor);
+	consoleLayer = screen->createConsoleLayer(reactor, console);
 	snowLayer = screen->createSnowLayer(display);
 	osdGuiLayer = screen->createOSDGUILayer(display.getOSDGUI());
-	display.addLayer(*console);
+	display.addLayer(*consoleLayer);
 	display.addLayer(*snowLayer);
 	display.addLayer(*osdGuiLayer);
 
@@ -73,7 +78,7 @@ SDLVideoSystem::~SDLVideoSystem()
 
 	display.removeLayer(*osdGuiLayer);
 	display.removeLayer(*snowLayer);
-	display.removeLayer(*console);
+	display.removeLayer(*consoleLayer);
 }
 
 Rasterizer* SDLVideoSystem::createRasterizer(VDP& vdp)
@@ -101,10 +106,9 @@ Rasterizer* SDLVideoSystem::createRasterizer(VDP& vdp)
 					VIDEO_MSX, 640, 240)));
 #endif
 		default:
-			assert(false);
-			return NULL;
+			UNREACHABLE; return NULL;
 		}
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	case RendererFactory::SDLGL_PP:
 		return new SDLRasterizer<unsigned>(
 			vdp, display, *screen,
@@ -113,8 +117,7 @@ Rasterizer* SDLVideoSystem::createRasterizer(VDP& vdp)
 				VIDEO_MSX, 640, 240)));
 #endif
 	default:
-		assert(false);
-		return NULL;
+		UNREACHABLE; return NULL;
 	}
 }
 
@@ -143,10 +146,9 @@ V9990Rasterizer* SDLVideoSystem::createV9990Rasterizer(V9990& vdp)
 					VIDEO_GFX9000, 1280, 240)));
 #endif
 		default:
-			assert(false);
-			return NULL;
+			UNREACHABLE; return NULL;
 		}
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	case RendererFactory::SDLGL_PP:
 		return new V9990SDLRasterizer<unsigned>(
 			vdp, display, *screen,
@@ -155,11 +157,54 @@ V9990Rasterizer* SDLVideoSystem::createV9990Rasterizer(V9990& vdp)
 				VIDEO_GFX9000, 1280, 240)));
 #endif
 	default:
-		assert(false);
-		return NULL;
+		UNREACHABLE; return NULL;
 	}
 }
 
+#if COMPONENT_LASERDISC
+LDRasterizer* SDLVideoSystem::createLDRasterizer(LaserdiscPlayer& ld)
+{
+	MSXMotherBoard& motherBoard = ld.getMotherBoard();
+	switch (renderSettings.getRenderer().getValue()) {
+	case RendererFactory::SDL:
+	case RendererFactory::SDLGL_FB16:
+	case RendererFactory::SDLGL_FB32:
+		switch (screen->getSDLFormat().BytesPerPixel) {
+#if HAVE_16BPP
+		case 2:
+			return new LDSDLRasterizer<word>(
+				*screen,
+				std::auto_ptr<PostProcessor>(
+					new FBPostProcessor<word>(
+					motherBoard, display, *screen,
+					VIDEO_LASERDISC, 640, 480)));
+#endif
+#if HAVE_32BPP
+		case 4:
+			return new LDSDLRasterizer<unsigned>(
+				*screen,
+				std::auto_ptr<PostProcessor>(
+					new FBPostProcessor<unsigned>(
+					motherBoard, display, *screen,
+					VIDEO_LASERDISC, 640, 480)));
+#endif
+		default:
+			UNREACHABLE; return NULL;
+		}
+#if COMPONENT_GL
+	case RendererFactory::SDLGL_PP:
+		return new LDSDLRasterizer<unsigned>(
+			*screen,
+			std::auto_ptr<PostProcessor>(new GLPostProcessor(
+				motherBoard, display, *screen,
+				VIDEO_LASERDISC, 640, 480)));
+#endif
+	default:
+		UNREACHABLE; return NULL;
+	}
+}
+#endif
+
 void SDLVideoSystem::getWindowSize(unsigned& width, unsigned& height)
 {
 	unsigned factor = renderSettings.getScaleFactor().getValue();
@@ -177,7 +222,7 @@ void SDLVideoSystem::getWindowSize(unsigned& width, unsigned& height)
 		factor = 0;
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 	width  = 320 * factor;
 	height = 240 * factor;
@@ -212,7 +257,7 @@ void SDLVideoSystem::takeScreenShot(const std::string& filename, bool withOsd)
 	} else {
 		// we first need to re-render to an off-screen surface
 		// with OSD layers disabled
-		ScopedLayerHider hideConsole(*console);
+		ScopedLayerHider hideConsole(*consoleLayer);
 		ScopedLayerHider hideOsd(*osdGuiLayer);
 		std::auto_ptr<OutputSurface> surf = screen->createOffScreenSurface();
 		display.repaint(*surf);
@@ -246,7 +291,7 @@ void SDLVideoSystem::resize()
 		screen.reset(new SDLVisibleSurface(width, height, fullscreen,
 				renderSettings, eventDistributor, inputEventGenerator));
 		break;
-#ifdef COMPONENT_GL
+#if COMPONENT_GL
 	case RendererFactory::SDLGL_PP:
 		screen.reset(new SDLGLVisibleSurface(width, height, fullscreen,
 				renderSettings, eventDistributor, inputEventGenerator));
@@ -263,7 +308,7 @@ void SDLVideoSystem::resize()
 		break;
 #endif
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 	inputEventGenerator.reinit();
 }
@@ -275,7 +320,7 @@ void SDLVideoSystem::update(const Setting& subject)
 		//       but is that still needed?
 		//resize();
 	} else {
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
