@@ -1,13 +1,13 @@
-// $Id: SDLVisibleSurface.cc 10004 2009-06-07 21:08:28Z m9710797 $
+// $Id: SDLVisibleSurface.cc 11365 2010-03-25 01:35:11Z mthuurne $
 
 #include "SDLVisibleSurface.hh"
 #include "SDLOffScreenSurface.hh"
-#include "ScreenShotSaver.hh"
+#include "PNG.hh"
 #include "SDLSnow.hh"
 #include "OSDConsoleRenderer.hh"
 #include "OSDGUILayer.hh"
+#include "unreachable.hh"
 #include "build-info.hh"
-#include <cstring>
 #include <cassert>
 #if PLATFORM_GP2X
 #include "GP2XMMUHack.hh"
@@ -31,18 +31,24 @@ SDLVisibleSurface::SDLVisibleSurface(
 	// the work surface to the display surface (the HW blitter on GP2X is
 	// fast enough).
 	int flags = SDL_HWSURFACE; // | SDL_DOUBLEBUF;
+#elif PLATFORM_DINGUX
+	// Double buffering hangs openMSX (tested 2009-07-22).
+	// The Dingoo seems to have a hardware problem that makes it hard or
+	// impossible to know when vsync happens:
+	//   http://www.dingux.com/2009/07/on-screen-tearing.html
+	// Hardware buffer without double buffering works but then there is very
+	// annoying flickering of the OSD elements.
+	int flags = SDL_SWSURFACE;
 #else
 	int flags = SDL_SWSURFACE; // Why did we use a SW surface again?
 #endif
 	if (fullscreen) flags |= SDL_FULLSCREEN;
 
 	createSurface(width, height, flags);
-
-	SDL_PixelFormat& format = getSDLFormat();
-	SDL_Surface* displaySurface = getSDLDisplaySurface();
-	memcpy(&format, displaySurface->format, sizeof(SDL_PixelFormat));
+	setSDLFormat(*getSDLDisplaySurface()->format);
 
 #if PLATFORM_GP2X
+	const SDL_PixelFormat& format = getSDLFormat();
 	SDL_Surface* workSurface = SDL_CreateRGBSurface(SDL_HWSURFACE,
 		width, height, format.BitsPerPixel, format.Rmask,
 		format.Gmask, format.Bmask, format.Amask);
@@ -52,7 +58,7 @@ SDLVisibleSurface::SDLVisibleSurface(
 #else
 	// on non-GP2X platforms, work and displaySurfaces are the same,
 	// see remark above
-	SDL_Surface* workSurface = displaySurface;
+	SDL_Surface* workSurface = getSDLDisplaySurface();
 #endif
 	setSDLWorkSurface(workSurface);
 	setBufferPtr(static_cast<char*>(workSurface->pixels), workSurface->pitch);
@@ -83,17 +89,16 @@ std::auto_ptr<Layer> SDLVisibleSurface::createSnowLayer(Display& display)
 		return std::auto_ptr<Layer>(new SDLSnow<unsigned>(*this, display));
 #endif
 	default:
-		assert(false);
-		return std::auto_ptr<Layer>(); // avoid warning
+		UNREACHABLE; return std::auto_ptr<Layer>();
 	}
 }
 
 std::auto_ptr<Layer> SDLVisibleSurface::createConsoleLayer(
-		Reactor& reactor)
+		Reactor& reactor, CommandConsole& console)
 {
 	const bool openGL = false;
 	return std::auto_ptr<Layer>(new OSDConsoleRenderer(
-		reactor, getWidth(), getHeight(), openGL));
+		reactor, console, getWidth(), getHeight(), openGL));
 }
 
 std::auto_ptr<Layer> SDLVisibleSurface::createOSDGUILayer(OSDGUI& gui)
@@ -110,7 +115,7 @@ std::auto_ptr<OutputSurface> SDLVisibleSurface::createOffScreenSurface()
 void SDLVisibleSurface::saveScreenshot(const std::string& filename)
 {
 	lock();
-	ScreenShotSaver::save(getSDLWorkSurface(), filename);
+	PNG::save(getSDLWorkSurface(), filename);
 }
 
 } // namespace openmsx
