@@ -1,4 +1,4 @@
-// $Id: SaI2xScaler.cc 8223 2008-09-14 19:17:24Z m9710797 $
+// $Id: SaI2xScaler.cc 11106 2010-01-06 22:08:48Z m9710797 $
 
 // 2xSaI is Copyright (c) 1999-2001 by Derek Liauw Kie Fa.
 //   http://elektron.its.tudelft.nl/~dalikifa/
@@ -44,8 +44,8 @@ void SaI2xScaler<Pixel>::scaleBlank1to2(
 		unsigned nextLineWidth = src.getLineWidth(srcY + 1);
 		assert(src.getLineWidth(srcY) == 1);
 		assert(nextLineWidth != 1);
-		this->scaleImage(src, srcY, srcEndY, nextLineWidth,
-		                 dst, dstY, dstEndY);
+		this->dispatchScale(src, srcY, srcEndY, nextLineWidth,
+		                    dst, dstY, dstEndY);
 	}
 }
 
@@ -57,9 +57,10 @@ inline Pixel SaI2xScaler<Pixel>::blend(Pixel p1, Pixel p2)
 
 template <class Pixel>
 void SaI2xScaler<Pixel>::scaleLine1on2(
-	const Pixel* srcLine0, const Pixel* srcLine1,
-	const Pixel* srcLine2, const Pixel* srcLine3,
-	Pixel* dstUpper, Pixel* dstLower, unsigned srcWidth)
+	const Pixel* __restrict srcLine0, const Pixel* __restrict srcLine1,
+	const Pixel* __restrict srcLine2, const Pixel* __restrict srcLine3,
+	Pixel* __restrict dstUpper, Pixel* __restrict dstLower,
+	unsigned srcWidth)
 {
 	// TODO: Scale border pixels as well.
 	for (unsigned x = 0; x < srcWidth; x++) {
@@ -76,7 +77,7 @@ void SaI2xScaler<Pixel>::scaleLine1on2(
 		// TODO: Possible performance improvements:
 		// - Play with order of fetching (effect on data cache).
 		// - Try not fetching at all (using srcLineN[x] in algorithm).
-		// - Try rotating the fetched colours (either in vars or in array).
+		// - Try rotating the fetched colors (either in vars or in array).
 		Pixel colorI = srcLine0[xl];
 		Pixel colorE = srcLine0[x];
 		Pixel colorF = srcLine0[xr];
@@ -191,9 +192,10 @@ void SaI2xScaler<Pixel>::scaleLine1on2(
 
 template <class Pixel>
 void SaI2xScaler<Pixel>::scaleLine1on1(
-	const Pixel* srcLine0, const Pixel* srcLine1,
-	const Pixel* srcLine2, const Pixel* srcLine3,
-	Pixel* dstUpper, Pixel* dstLower, unsigned srcWidth)
+	const Pixel* __restrict srcLine0, const Pixel* __restrict srcLine1,
+	const Pixel* __restrict srcLine2, const Pixel* __restrict srcLine3,
+	Pixel* __restrict dstUpper, Pixel* __restrict dstLower,
+	unsigned srcWidth)
 {
 	// Apply 2xSaI and keep the bottom-left pixel.
 	// It's not great, but at least it looks better than doubling the pixel
