@@ -1,4 +1,4 @@
-// $Id: SaI3xScaler.cc 9300 2009-03-01 01:03:07Z mfeingol $
+// $Id: SaI3xScaler.cc 11106 2010-01-06 22:08:48Z m9710797 $
 
 // 2xSaI is Copyright (c) 1999-2001 by Derek Liauw Kie Fa.
 //   http://elektron.its.tudelft.nl/~dalikifa/
@@ -46,8 +46,8 @@ void SaI3xScaler<Pixel>::scaleBlank1to3(
 		unsigned nextLineWidth = src.getLineWidth(srcY + 1);
 		assert(src.getLineWidth(srcY) == 1);
 		assert(nextLineWidth != 1);
-		this->scaleImage(src, srcY, srcEndY, nextLineWidth,
-		                 dst, dstY, dstEndY);
+		this->dispatchScale(src, srcY, srcEndY, nextLineWidth,
+		                    dst, dstY, dstEndY);
 	}
 }
 
@@ -86,10 +86,10 @@ unsigned bilinear<unsigned>(unsigned a, unsigned b, unsigned x)
 	const unsigned areaA = 0x100 - areaB;
 
 	const unsigned result0 =
-		(a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB;
+		((a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB) >> 8;
 	const unsigned result1 =
-		(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB;
-	return ((result0 & (0xFF00FFu << 8)) | (result1 & (0x00FF00 << 8))) >> 8;
+		((a & 0xFF00FF00) >> 8) * areaA + ((b & 0xFF00FF00) >> 8) * areaB;
+	return (result0 & 0x00FF00FF) | (result1 & 0xFF00FF00);
 }
 
 template <typename Pixel>
@@ -133,12 +133,12 @@ unsigned bilinear4<unsigned>(
 	const unsigned areaD = xy;
 
 	const unsigned result0 =
-		(a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB +
-		(c & 0x00FF00FF) * areaC + (d & 0x00FF00FF) * areaD;
+		((a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB +
+		 (c & 0x00FF00FF) * areaC + (d & 0x00FF00FF) * areaD) >> 8;
 	const unsigned result1 =
-		(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB +
-		(c & 0x0000FF00) * areaC + (d & 0x0000FF00) * areaD;
-	return ((result0 & (0xFF00FFu << 8)) | (result1 & (0x00FF00 << 8))) >> 8;
+		((a & 0xFF00FF00) >> 8) * areaA + ((b & 0xFF00FF00) >> 8) * areaB +
+		((c & 0xFF00FF00) >> 8) * areaC + ((d & 0xFF00FF00) >> 8) * areaD;
+	return (result0 & 0x00FF00FF) | (result1 & 0xFF00FF00);
 }
 
 template <typename Pixel>
@@ -176,11 +176,12 @@ inline Pixel Blender<Pixel>::blend(unsigned a, unsigned b)
 		return (result & redblueMask) | ((result >> 16) & greenMask);
 	} else {
 		const unsigned result0 =
-			(a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB;
+			((a & 0x00FF00FF) * areaA +
+			 (b & 0x00FF00FF) * areaB) >> bits;
 		const unsigned result1 =
-			(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB;
-		return ((result0 & (0xFF00FFu << bits)) |
-		        (result1 & (0x00FF00u << bits))) >> bits;
+			((a & 0xFF00FF00) >> bits) * areaA +
+			((b & 0xFF00FF00) >> bits) * areaB;
+		return (result0 & 0x00FF00FF) | (result1 & 0xFF00FF00);
 	}
 }
 
@@ -207,13 +208,16 @@ inline Pixel Blender<Pixel>::blend(
 		return (result & redblueMask) | ((result >> 16) & greenMask);
 	} else {
 		const unsigned result0 =
-			(a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB +
-			(c & 0x00FF00FF) * areaC + (d & 0x00FF00FF) * areaD;
+			((a & 0x00FF00FF) * areaA +
+			 (b & 0x00FF00FF) * areaB +
+			 (c & 0x00FF00FF) * areaC +
+			 (d & 0x00FF00FF) * areaD) >> bits;
 		const unsigned result1 =
-			(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB +
-			(c & 0x0000FF00) * areaC + (d & 0x0000FF00) * areaD;
-		return ((result0 & (0xFF00FFu << bits)) |
-		        (result1 & (0x00FF00u << bits))) >> bits;
+			((a & 0xFF00FF00) >> bits) * areaA +
+			((b & 0xFF00FF00) >> bits) * areaB +
+			((c & 0xFF00FF00) >> bits) * areaC +
+			((d & 0xFF00FF00) >> bits) * areaD;
+		return (result0 & 0x00FF00FF) | (result1 & 0xFF00FF00);
 	}
 }
 
@@ -324,9 +328,9 @@ class LineRepeater
 public:
 	template <unsigned NX, unsigned NY, typename Pixel>
 	inline static void scaleFixedLine(
-		const Pixel* src0, const Pixel* src1, const Pixel* src2,
-		const Pixel* src3, unsigned srcWidth,
-		OutputSurface& dst, unsigned& dstY)
+		const Pixel* __restrict src0, const Pixel* __restrict src1,
+		const Pixel* __restrict src2, const Pixel* __restrict src3,
+		unsigned srcWidth, OutputSurface& dst, unsigned& dstY)
 	{
 		Pixel* dp = dst.getLinePtrDirect<Pixel>(dstY++);
 		// Calculate fixed point coordinate.
@@ -348,9 +352,9 @@ public:
 			const Pixel sc = sd;
 			sd = src2[pos2];
 
-			// Compute and write colour of destination pixel.
+			// Compute and write color of destination pixel.
 			if (sa == sb && sc == sd && sa == sc) {
-				// All the same colour; fill.
+				// All the same color; fill.
 				PixelStripRepeater<NX>::template fill<Pixel>(dp, sa);
 			} else if (sa == sd && sb != sc) {
 				// Pattern in the form of a backslash.
@@ -428,13 +432,13 @@ void SaI3xScaler<Pixel>::scaleAny(FrameSource& src,
 		// Get source line pointers.
 		int line = srcStartY + (h >> 16);
 		// TODO possible optimization: reuse srcN from previous step
-		const Pixel* src0 = src.getLinePtr<Pixel>(line - 1, srcWidth);
-		const Pixel* src1 = src.getLinePtr<Pixel>(line + 0, srcWidth);
-		const Pixel* src2 = src.getLinePtr<Pixel>(line + 1, srcWidth);
-		const Pixel* src3 = src.getLinePtr<Pixel>(line + 2, srcWidth);
+		const Pixel* __restrict src0 = src.getLinePtr<Pixel>(line - 1, srcWidth);
+		const Pixel* __restrict src1 = src.getLinePtr<Pixel>(line + 0, srcWidth);
+		const Pixel* __restrict src2 = src.getLinePtr<Pixel>(line + 1, srcWidth);
+		const Pixel* __restrict src3 = src.getLinePtr<Pixel>(line + 2, srcWidth);
 
 		// Get destination line pointer.
-		Pixel* dp = dst.getLinePtrDirect<Pixel>(dstY);
+		Pixel* __restrict dp = dst.getLinePtrDirect<Pixel>(dstY);
 
 		// Fractional parts of the fixed point Y coordinates.
 		const unsigned y1 = h & 0xffff;
@@ -447,7 +451,7 @@ void SaI3xScaler<Pixel>::scaleAny(FrameSource& src,
 		unsigned pos3 = 1;
 		Pixel B = src1[0];
 		Pixel D = src2[0];
-		for (unsigned w = 0; w < wfinish; ) {
+		for (unsigned w = 0; w < wfinish; /**/) {
 			const unsigned pos0 = pos1;
 			pos1 = pos2;
 			pos2 = pos3;
@@ -458,7 +462,7 @@ void SaI3xScaler<Pixel>::scaleAny(FrameSource& src,
 			const Pixel C = D;
 			D = src2[pos2];
 
-			// Compute and write colour of destination pixel.
+			// Compute and write color of destination pixel.
 			if (A == B && C == D && A == C) { // 0
 				do {
 					*dp++ = A;
