@@ -1,7 +1,8 @@
-// $Id: Scanline.cc 8561 2009-02-06 07:28:43Z mfeingol $
+// $Id: Scanline.cc 10798 2009-11-17 12:53:10Z m9710797 $
 
 #include "Scanline.hh"
 #include "HostCPU.hh"
+#include "unreachable.hh"
 #include "build-info.hh"
 #include <cassert>
 #include <cstring>
@@ -63,8 +64,8 @@ void Multiply<unsigned>::setFactor(unsigned f)
 
 inline unsigned Multiply<unsigned>::multiply(unsigned p, unsigned f) const
 {
-	return ((((p & 0xFF00FF) * f) & 0xFF00FF00) |
-	        (((p & 0x00FF00) * f) & 0x00FF0000)) >> 8;
+	return ((((p       & 0x00FF00FF) * f) & 0xFF00FF00) >> 8)
+	     | ((((p >> 8) & 0x00FF00FF) * f) & 0xFF00FF00);
 }
 
 inline unsigned Multiply<unsigned>::multiply(unsigned p) const
@@ -74,8 +75,7 @@ inline unsigned Multiply<unsigned>::multiply(unsigned p) const
 
 const unsigned* Multiply<unsigned>::getTable() const
 {
-	assert(false);
-	return NULL;
+	UNREACHABLE; return NULL;
 }
 
 
@@ -98,10 +98,11 @@ Scanline<Pixel>::Scanline(const PixelOperations<Pixel>& pixelOps)
 }
 
 template <class Pixel>
-void Scanline<Pixel>::draw(const Pixel* src1, const Pixel* src2,
-		           Pixel* dst, unsigned factor, unsigned long width)
+void Scanline<Pixel>::draw(
+	const Pixel* __restrict src1, const Pixel* __restrict src2,
+	Pixel* __restrict dst, unsigned factor, unsigned long width)
 {
-#ifdef ASM_X86
+#if ASM_X86
 #ifdef _MSC_VER
 	const HostCPU& cpu = HostCPU::getInstance();
 	if ((sizeof(Pixel) == 4) && cpu.hasSSE2()) {
@@ -296,7 +297,7 @@ void Scanline<Pixel>::draw(const Pixel* src1, const Pixel* src2,
 		return;
 	}
 
-	#if !defined(__APPLE__) && !defined(ASM_X86_64)
+	#if !defined(__APPLE__) && !ASM_X86_64
 	// On Mac OS X, we are one register short, because EBX is not available.
 	// We disable this piece of assembly and fall back to the C++ code.
 	// It's unlikely modern Macs will be running in 16bpp anyway.
@@ -306,7 +307,7 @@ void Scanline<Pixel>::draw(const Pixel* src1, const Pixel* src2,
 
 		darkener.setFactor(factor);
 		const Pixel* table = darkener.getTable();
-		Pixel mask = ~pixelOps.getBlendMask();
+		Pixel mask = pixelOps.getBlendMask();
 
 		asm (
 			"movd	%5, %%mm7;"
