@@ -1,8 +1,9 @@
-// $Id: Simple3xScaler.cc 9751 2009-05-04 10:34:42Z m9710797 $
+// $Id: Simple3xScaler.cc 11107 2010-01-06 22:09:08Z m9710797 $
 
 #include "Simple3xScaler.hh"
+#include "SuperImposedFrame.hh"
 #include "LineScalers.hh"
-#include "FrameSource.hh"
+#include "RawFrame.hh"
 #include "OutputSurface.hh"
 #include "RenderSettings.hh"
 #include "MemoryOps.hh"
@@ -41,11 +42,10 @@ Simple3xScaler<Pixel>::Simple3xScaler(
 }
 
 template <typename Pixel>
-template <typename ScaleOp>
 void Simple3xScaler<Pixel>::doScale1(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
-	ScaleOp& scale)
+	PolyLineScaler<Pixel>& scale)
 {
 	dst.lock();
 	int scanlineFactor = settings.getScanlineFactor();
@@ -82,11 +82,10 @@ void Simple3xScaler<Pixel>::doScale1(FrameSource& src,
 }
 
 template <typename Pixel>
-template <typename ScaleOp>
 void Simple3xScaler<Pixel>::doScale2(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
-	ScaleOp& scale)
+	PolyLineScaler<Pixel>& scale)
 {
 	dst.lock();
 	int scanlineFactor = settings.getScanlineFactor();
@@ -111,7 +110,7 @@ void Simple3xScaler<Pixel>::scale2x1to9x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_2on9<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_2on9<Pixel> > op(pixelOps);
 	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -120,7 +119,7 @@ void Simple3xScaler<Pixel>::scale2x2to9x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_2on9<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_2on9<Pixel> > op(pixelOps);
 	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -132,15 +131,16 @@ void Simple3xScaler<Pixel>::scale1x1to3x3(FrameSource& src,
 	unsigned blur = settings.getBlurFactor() / 3;
 	if (blur) {
 		blur_1on3->setBlur(blur);
-		doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-			*blur_1on3);
+		PolyScaleRef<Pixel, Blur_1on3<Pixel> > op(*blur_1on3);
+		doScale1(src, srcStartY, srcEndY, srcWidth,
+		         dst, dstStartY, dstEndY, op);
 	} else {
 		// No blurring: this is an optimization but it's also needed
 		// for correctness (otherwise there's an overflow in 0.16 fixed
 		// point arithmetic).
-		Scale_1on3<Pixel> op;
-		doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
-			op);
+		PolyScale<Pixel, Scale_1on3<Pixel> > op;
+		doScale1(src, srcStartY, srcEndY, srcWidth,
+		         dst, dstStartY, dstEndY, op);
 	}
 }
 
@@ -149,7 +149,7 @@ void Simple3xScaler<Pixel>::scale1x2to3x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_1on3<Pixel> op;
+	PolyScale<Pixel, Scale_1on3<Pixel> > op;
 	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -158,7 +158,7 @@ void Simple3xScaler<Pixel>::scale4x1to9x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_4on9<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_4on9<Pixel> > op(pixelOps);
 	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -167,7 +167,7 @@ void Simple3xScaler<Pixel>::scale4x2to9x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_4on9<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_4on9<Pixel> > op(pixelOps);
 	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -176,7 +176,7 @@ void Simple3xScaler<Pixel>::scale2x1to3x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_2on3<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_2on3<Pixel> > op(pixelOps);
 	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -185,7 +185,7 @@ void Simple3xScaler<Pixel>::scale2x2to3x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_2on3<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_2on3<Pixel> > op(pixelOps);
 	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -194,7 +194,7 @@ void Simple3xScaler<Pixel>::scale8x1to9x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_8on9<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_8on9<Pixel> > op(pixelOps);
 	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -203,7 +203,7 @@ void Simple3xScaler<Pixel>::scale8x2to9x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_8on9<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_8on9<Pixel> > op(pixelOps);
 	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -212,7 +212,7 @@ void Simple3xScaler<Pixel>::scale4x1to3x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_4on3<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_4on3<Pixel> > op(pixelOps);
 	doScale1(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -221,7 +221,7 @@ void Simple3xScaler<Pixel>::scale4x2to3x3(FrameSource& src,
 		unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Scale_4on3<Pixel> op(pixelOps);
+	PolyScale<Pixel, Scale_4on3<Pixel> > op(pixelOps);
 	doScale2(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY, op);
 }
 
@@ -251,8 +251,8 @@ void Simple3xScaler<Pixel>::scaleBlank1to3(
 		unsigned nextLineWidth = src.getLineWidth(srcY + 1);
 		assert(src.getLineWidth(srcY) == 1);
 		assert(nextLineWidth != 1);
-		this->scaleImage(src, srcY, srcEndY, nextLineWidth,
-		                 dst, dstY, dstEndY);
+		this->dispatchScale(src, srcY, srcEndY, nextLineWidth,
+		                    dst, dstY, dstEndY);
 	}
 }
 
@@ -299,7 +299,9 @@ Blur_1on3<Pixel>::Blur_1on3(const PixelOperations<Pixel>& pixelOps)
 }
 
 template <class Pixel>
-void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned long dstWidth)
+void Blur_1on3<Pixel>::operator()(
+	const Pixel* __restrict in, Pixel* __restrict out,
+	unsigned long dstWidth)
 {
 	/* The following code is equivalent to this loop. It is 2x unrolled
 	 * and common subexpressions have been eliminated. The last iteration
@@ -321,7 +323,7 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned long dst
 	 *      curr = next;
 	 *  }
 	 */
-	#ifdef ASM_X86
+	#if ASM_X86
 	const HostCPU& cpu = HostCPU::getInstance();
 	if ((sizeof(Pixel) == 4) && cpu.hasSSE()) {
 		// MMX-EXT routine, 32bpp
@@ -345,7 +347,7 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned long dst
 		Blur_1on3_4_SSE(in, out, dstWidth, &c);
 		return;
 	#else
-		unsigned t0, t1, t2, t3;
+		unsigned long t0, t1, t2, t3;
 		asm volatile (
 			"pxor      %%mm0, %%mm0;"
 			"pshufw    $0,40(%[CNST]),%%mm1;"
@@ -506,6 +508,25 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned long dst
 	out[3 * x + 5] = p1;
 }
 
+template <class Pixel>
+void Simple3xScaler<Pixel>::scaleImage(
+	FrameSource& src, const RawFrame* superImpose,
+	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
+	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
+{
+	if (superImpose) {
+		SuperImposedFrame<Pixel> sf(src, *superImpose, pixelOps.getSDLPixelFormat());
+		srcWidth = sf.getLineWidth(srcStartY);
+		this->dispatchScale(sf,  srcStartY, srcEndY, srcWidth,
+		                    dst, dstStartY, dstEndY);
+		src.freeLineBuffers();
+		superImpose->freeLineBuffers();
+	} else {
+		this->dispatchScale(src, srcStartY, srcEndY, srcWidth,
+		                    dst, dstStartY, dstEndY);
+	}
+}
+
 // Force template instantiation.
 #if HAVE_16BPP
 template class Simple3xScaler<word>;
