@@ -1,4 +1,4 @@
-// $Id: VDP.cc 9984 2009-06-04 21:53:10Z m9710797 $
+// $Id: VDP.cc 11217 2010-01-31 13:07:08Z seanyoung $
 
 /*
 TODO:
@@ -14,7 +14,7 @@ TODO:
   Current implementation would return FH=0 both times.
 - Check how Z80 should treat interrupts occurring during DI.
 - Bottom erase suspends display even on overscan.
-  However, it shows black, not border colour.
+  However, it shows black, not border color.
   How to handle this? Currently it is treated as "overscan" which
   falls outside of the rendered screen area.
 */
@@ -29,8 +29,10 @@ TODO:
 #include "Renderer.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "MSXException.hh"
 #include "CliComm.hh"
+#include "unreachable.hh"
 #include <sstream>
 #include <cstring>
 #include <cassert>
@@ -81,6 +83,7 @@ VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config)
 	: MSXDevice(motherBoard, config)
 	, Schedulable(motherBoard.getScheduler())
 	, cliComm(motherBoard.getMSXCliComm())
+	, display(motherBoard.getReactor().getDisplay())
 	, vdpRegDebug      (new VDPRegDebug      (*this))
 	, vdpStatusRegDebug(new VDPStatusRegDebug(*this))
 	, vdpPaletteDebug  (new VDPPaletteDebug  (*this))
@@ -136,7 +139,6 @@ VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config)
 	}
 	vram.reset(new VDPVRAM(*this, vramSize * 1024, time));
 
-	Display& display = getMotherBoard().getDisplay();
 	RenderSettings& renderSettings = display.getRenderSettings();
 
 	// Create sprite checker.
@@ -152,14 +154,14 @@ VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config)
 	createRenderer();
 
 	// Reset state.
-	reset(time);
+	powerUp(time);
 
 	display.attach(*this);
 }
 
 VDP::~VDP()
 {
-	getMotherBoard().getDisplay().detach(*this);
+	display.detach(*this);
 }
 
 void VDP::preVideoSystemChange()
@@ -174,7 +176,6 @@ void VDP::postVideoSystemChange()
 
 void VDP::createRenderer()
 {
-	Display& display = getMotherBoard().getDisplay();
 	renderer.reset(RendererFactory::createRenderer(*this, display));
 	// TODO: Is it safe to use frameStartTime,
 	//       which is most likely in the past?
@@ -193,7 +194,7 @@ void VDP::resetInit()
 		// Boots (and remains) in PAL mode, all other VDPs boot in NTSC.
 		controlRegs[9] |= 0x02;
 	}
-	// According to page 6 of the V9938 data book the colour burst registers
+	// According to page 6 of the V9938 data book the color burst registers
 	// are loaded with these values at power on.
 	controlRegs[21] = 0x3B;
 	controlRegs[22] = 0x05;
@@ -216,6 +217,8 @@ void VDP::resetInit()
 	// TODO: Real VDP probably resets timing as well.
 	isDisplayArea = false;
 	displayEnabled = false;
+	superimposing = false;
+	externalVideo = NULL;
 
 	// Init status registers.
 	statusReg0 = 0x00;
@@ -240,7 +243,7 @@ void VDP::resetMasks(EmuTime::param time)
 	// TODO: Use the updateNameBase method instead of duplicating the effort
 	//       here for the initial state.
 	vram->nameTable.setMask(~(-1 << 10), -1 << 17, time);
-	updateColourBase(time);
+	updateColorBase(time);
 	updatePatternBase(time);
 	updateSpriteAttributeBase(time);
 	updateSpritePatternBase(time);
@@ -249,6 +252,12 @@ void VDP::resetMasks(EmuTime::param time)
 	//vram->bitmapWindow.setMask(~(-1 << 17), -1 << 17, time);
 }
 
+void VDP::powerUp(EmuTime::param time)
+{
+	vram->clear();
+	reset(time);
+}
+
 void VDP::reset(EmuTime::param time)
 {
 	removeSyncPoint(VSYNC);
@@ -264,7 +273,7 @@ void VDP::reset(EmuTime::param time)
 	resetInit();
 	spriteChecker->reset(time);
 	cmdEngine->reset(time);
-	renderer->reset(time);
+	renderer->reInit();
 
 	// Tell the subsystems of the new mask values.
 	resetMasks(time);
@@ -355,7 +364,7 @@ void VDP::executeUntil(EmuTime::param time, int userData)
 		break;
 	}
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -513,6 +522,17 @@ void VDP::frameStart(EmuTime::param time)
 		}
 	}
 
+	// TODO: Presumably this is done here
+	// Note that if superimposing is enabled but no external video 
+	// signal is provided then the VDP stops producing a signal
+	// (at least on an MSX1, VDP(0)=1 produces "signal lost" on my
+	// monitor)
+	const RawFrame* newSuperimposing = (controlRegs[0] & 1) ? externalVideo : NULL;
+	if (superimposing != newSuperimposing) {
+		superimposing = newSuperimposing;
+		renderer->updateSuperimposing(superimposing, time);
+	}
+
 	// Schedule next VSYNC.
 	frameStartTime.advance_fast(time);
 	setSyncPoint(frameStartTime + getTicksPerFrame(), VSYNC);
@@ -765,8 +785,7 @@ byte VDP::readIO(word port, EmuTime::param time)
 		return readStatusReg(controlRegs[15], time);
 	default:
 		// These ports should not be registered for reading.
-		assert(false);
-		return 0xFF;
+		UNREACHABLE; return 0xFF;
 	}
 }
 
@@ -808,10 +827,10 @@ void VDP::changeRegister(byte reg, byte val, EmuTime::param time)
 		}
 
 		if ((val & 0xF0) && (val & 0x0F)) {
-			// Alternating colours, start with ON.
+			// Alternating colors, start with ON.
 			blinkCount = (val >> 4) * 10;
 		} else {
-			// Stable colour.
+			// Stable color.
 			blinkCount = 0;
 		}
 	}
@@ -858,13 +877,13 @@ void VDP::changeRegister(byte reg, byte val, EmuTime::param time)
 	case 7:
 		if (getDisplayMode().getByte() != DisplayMode::GRAPHIC7) {
 			if (change & 0xF0) {
-				renderer->updateForegroundColour(val >> 4, time);
+				renderer->updateForegroundColor(val >> 4, time);
 			}
 			if (change & 0x0F) {
-				renderer->updateBackgroundColour(val & 0x0F, time);
+				renderer->updateBackgroundColor(val & 0x0F, time);
 			}
 		} else {
-			renderer->updateBackgroundColour(val, time);
+			renderer->updateBackgroundColor(val, time);
 		}
 		break;
 	case 8:
@@ -880,10 +899,10 @@ void VDP::changeRegister(byte reg, byte val, EmuTime::param time)
 		break;
 	case 12:
 		if (change & 0xF0) {
-			renderer->updateBlinkForegroundColour(val >> 4, time);
+			renderer->updateBlinkForegroundColor(val >> 4, time);
 		}
 		if (change & 0x0F) {
-			renderer->updateBlinkBackgroundColour(val & 0x0F, time);
+			renderer->updateBlinkBackgroundColor(val & 0x0F, time);
 		}
 		break;
 	case 16:
@@ -962,7 +981,7 @@ void VDP::changeRegister(byte reg, byte val, EmuTime::param time)
 		break;
 	case 3:
 	case 10:
-		updateColourBase(time);
+		updateColorBase(time);
 		break;
 	case 4:
 		updatePatternBase(time);
@@ -1050,25 +1069,25 @@ void VDP::updateNameBase(EmuTime::param time)
 	vram->nameTable.setMask(base, indexMask, time);
 }
 
-void VDP::updateColourBase(EmuTime::param time)
+void VDP::updateColorBase(EmuTime::param time)
 {
 	int base = (controlRegs[10] << 14) | (controlRegs[3] << 6) | ~(-1 << 6);
-	renderer->updateColourBase(base, time);
+	renderer->updateColorBase(base, time);
 	switch (displayMode.getBase()) {
 	case 0x09: // Text 2.
 		// TODO: Enable this only if dual color is actually active.
-		vram->colourTable.setMask(base, -1 << 9, time);
+		vram->colorTable.setMask(base, -1 << 9, time);
 		break;
 	case 0x00: // Graphic 1.
-		vram->colourTable.setMask(base, -1 << 6, time);
+		vram->colorTable.setMask(base, -1 << 6, time);
 		break;
 	case 0x04: // Graphic 2.
 	case 0x08: // Graphic 3.
-		vram->colourTable.setMask(base, -1 << 13, time);
+		vram->colorTable.setMask(base, -1 << 13, time);
 		break;
 	default:
-		// Other display modes do not use a colour table.
-		vram->colourTable.disable(time);
+		// Other display modes do not use a color table.
+		vram->colorTable.disable(time);
 	}
 }
 
@@ -1081,8 +1100,8 @@ void VDP::updatePatternBase(EmuTime::param time)
 	case 0x05: // Text 1 Q.
 	case 0x09: // Text 2.
 	case 0x00: // Graphic 1.
-	case 0x02: // Multicolour.
-	case 0x06: // Multicolour Q.
+	case 0x02: // Multicolor.
+	case 0x06: // Multicolor Q.
 		vram->patternTable.setMask(base, -1 << 11, time);
 		break;
 	case 0x04: // Graphic 2.
@@ -1144,7 +1163,7 @@ void VDP::updateDisplayMode(DisplayMode newMode, EmuTime::param time)
 	// leave last used character mode active.
 	// TODO: Disable it if not used for some time.
 	if (!displayMode.isBitmapMode()) {
-		updateColourBase(time);
+		updateColorBase(time);
 		updatePatternBase(time);
 	}
 	if (planarChange || spriteModeChange) {
@@ -1160,6 +1179,11 @@ void VDP::updateDisplayMode(DisplayMode newMode, EmuTime::param time)
 	//       It's one line of code and overhead is not huge either.
 }
 
+void VDP::setExternalVideoSource(const RawFrame* externalSource)
+{
+	externalVideo = externalSource;
+}
+
 // VDPRegDebug
 
 VDPRegDebug::VDPRegDebug(VDP& vdp_)
@@ -1302,10 +1326,19 @@ void VDP::serialize(Archive& ar, unsigned /*version*/)
 	displayMode.setByte(mode);
 
 	ar.serialize("cmdEngine", *cmdEngine);
-	ar.serialize("vram", *vram); // must come after controlRegs
 	ar.serialize("spriteChecker", *spriteChecker); // must come after displayMode
+	ar.serialize("vram", *vram); // must come after controlRegs and after spriteChecker
+
+	// externalVideo does not need serializing. It is set on load by the
+	// external video source (e.g. PioneerLDControl).
+	//
+	// TODO should superimposing be serialized? It cannot be recalculated
+	// from register values (it depends on the register values at the start
+	// of this frame). But it will be correct at the start of the next
+	// frame. Probably good enough.
+
 	if (ar.isLoader()) {
-		renderer->reset(Schedulable::getCurrentTime());
+		renderer->reInit();
 	}
 }
 INSTANTIATE_SERIALIZE_METHODS(VDP);
