@@ -1,4 +1,4 @@
-// $Id: VDPVRAM.cc 9835 2009-05-22 14:42:56Z m9710797 $
+// $Id: VDPVRAM.cc 11143 2010-01-12 20:51:25Z m9710797 $
 
 #include "VDPVRAM.hh"
 #include "SpriteChecker.hh"
@@ -131,13 +131,13 @@ VDPVRAM::VDPVRAM(VDP& vdp_, unsigned size, EmuTime::param time)
 	, logicalVRAMDebug (new LogicalVRAMDebuggable (vdp))
 	, physicalVRAMDebug(new PhysicalVRAMDebuggable(vdp, *this, size))
 	#ifdef DEBUG
-	, vramTime(time)
+	, vramTime(EmuTime::zero)
 	#endif
 	, actualSize(size)
 	, cmdReadWindow(data)
 	, cmdWriteWindow(data)
 	, nameTable(data)
-	, colourTable(data)
+	, colorTable(data)
 	, patternTable(data)
 	, bitmapVisibleWindow(data)
 	, bitmapCacheWindow(data)
@@ -149,6 +149,18 @@ VDPVRAM::VDPVRAM(VDP& vdp_, unsigned size, EmuTime::param time)
 	vrMode = vdp.getVRMode();
 	setSizeMask(time);
 
+	// Whole VRAM is cachable.
+	// Because this window has no observer, any EmuTime can be passed.
+	// TODO: Move this to cache registration.
+	bitmapCacheWindow.setMask(0x1FFFF, -1 << 17, EmuTime::zero);
+}
+
+VDPVRAM::~VDPVRAM()
+{
+}
+
+void VDPVRAM::clear()
+{
 	// Initialise VRAM data array.
 	// TODO: Fill with checkerboard pattern NMS8250 has.
 	memset(&data[0], 0, data.getSize());
@@ -159,15 +171,6 @@ VDPVRAM::VDPVRAM(VDP& vdp_, unsigned size, EmuTime::param time)
 		// give the same value.
 		memset(&data[actualSize], 0xFF, data.getSize() - actualSize);
 	}
-
-	// Whole VRAM is cachable.
-	// Because this window has no observer, any EmuTime can be passed.
-	// TODO: Move this to cache registration.
-	bitmapCacheWindow.setMask(0x1FFFF, -1 << 17, EmuTime::zero);
-}
-
-VDPVRAM::~VDPVRAM()
-{
 }
 
 void VDPVRAM::updateDisplayMode(DisplayMode mode, EmuTime::param time)
@@ -202,7 +205,7 @@ void VDPVRAM::setSizeMask(EmuTime::param time)
 	cmdReadWindow.setSizeMask(sizeMask, time);
 	cmdWriteWindow.setSizeMask(sizeMask, time);
 	nameTable.setSizeMask(sizeMask, time);
-	colourTable.setSizeMask(sizeMask, time);
+	colorTable.setSizeMask(sizeMask, time);
 	patternTable.setSizeMask(sizeMask, time);
 	bitmapVisibleWindow.setSizeMask(sizeMask, time);
 	bitmapCacheWindow.setSizeMask(sizeMask, time);
@@ -339,21 +342,23 @@ void VRAMWindow::serialize(Archive& ar, unsigned /*version*/)
 template<typename Archive>
 void VDPVRAM::serialize(Archive& ar, unsigned /*version*/)
 {
+	if (ar.isLoader()) {
+		vrMode = vdp.getVRMode();
+		setSizeMask(static_cast<MSXDevice&>(vdp).getCurrentTime());
+	}
+
 	ar.serialize_blob("data", &data[0], actualSize);
 	ar.serialize("cmdReadWindow",       cmdReadWindow);
 	ar.serialize("cmdWriteWindow",      cmdWriteWindow);
 	ar.serialize("nameTable",           nameTable);
-	ar.serialize("colourTable",         colourTable);
+	// TODO: Find a way of changing the line below to "colorTable",
+	// without breaking backwards compatibility
+	ar.serialize("colourTable",         colorTable);
 	ar.serialize("patternTable",        patternTable);
 	ar.serialize("bitmapVisibleWindow", bitmapVisibleWindow);
 	ar.serialize("bitmapCacheWindow",   bitmapCacheWindow);
 	ar.serialize("spriteAttribTable",   spriteAttribTable);
 	ar.serialize("spritePatternTable",  spritePatternTable);
-
-	if (ar.isLoader()) {
-		vrMode = vdp.getVRMode();
-		setSizeMask(static_cast<MSXDevice&>(vdp).getCurrentTime());
-	}
 }
 INSTANTIATE_SERIALIZE_METHODS(VDPVRAM);
 
