@@ -1,17 +1,24 @@
-// $Id: VisibleSurface.cc 10004 2009-06-07 21:08:28Z m9710797 $
+// $Id: VisibleSurface.cc 11292 2010-02-22 23:18:36Z mthuurne $
 
 #include "VisibleSurface.hh"
 #include "InitException.hh"
 #include "Icon.hh"
 #include "RenderSettings.hh"
+#include "SDLSurfacePtr.hh"
 #include "FloatSetting.hh"
 #include "BooleanSetting.hh"
+#include "AlarmEvent.hh"
 #include "Event.hh"
 #include "EventDistributor.hh"
 #include "InputEventGenerator.hh"
 #include "build-info.hh"
 #include "openmsx.hh"
 
+#if PLATFORM_MAEMO5
+#include <SDL_syswm.h>
+#define _NET_WM_STATE_ADD 1
+#endif
+
 #ifdef _WIN32
 #include <windows.h>
 static int lastWindowX = 0;
@@ -20,12 +27,64 @@ static int lastWindowY = 0;
 
 namespace openmsx {
 
+#if PLATFORM_MAEMO5
+static void setMaemo5WMHints(bool fullscreen)
+{
+	if (!fullscreen) {
+		// In windowed mode, stick with default settings.
+		return;
+	}
+	SDL_SysWMinfo info;
+	SDL_VERSION(&info.version);
+	// In SDL 1.2.14, the header states that 0 is returned for all failures,
+	// but the implementation returns 0 for not implemented and -1 for invalid
+	// version. Reported as bug 957:
+	//   http://bugzilla.libsdl.org/show_bug.cgi?id=957
+	if (SDL_GetWMInfo(&info) > 0) {
+		::Display* dpy = info.info.x11.display;
+		Window win = info.info.x11.fswindow;
+		if (win) {
+			// Tell the SDL event thread not to touch the X server until we are
+			// done with it.
+			info.info.x11.lock_func();
+			// Fetch atoms; create them if necessary.
+			Atom wmStateAtom =
+				XInternAtom(dpy, "_NET_WM_STATE", False);
+			Atom fullscreenAtom =
+				XInternAtom(dpy, "_NET_WM_STATE_FULLSCREEN", False);
+			Atom nonCompositedAtom =
+				XInternAtom(dpy, "_HILDON_NON_COMPOSITED_WINDOW", False);
+			// Unmap window, so we can remap it when properly configured.
+			XUnmapWindow(dpy, win);
+			// Tell window manager that we are running fullscreen.
+			// TODO: Wouldn't it be better if SDL did this?
+			XChangeProperty(
+				dpy, win, wmStateAtom, XA_ATOM, 32, PropModeReplace,
+				(unsigned char *)&fullscreenAtom, 1
+				);
+			// Disable compositor to improve painting performance.
+			int one = 1;
+			XChangeProperty(
+				dpy, win, nonCompositedAtom, XA_INTEGER, 32, PropModeReplace,
+				(unsigned char *)&one, 1
+				);
+			// Remap the window with the new settings.
+			XMapWindow(dpy, win);
+			// Resume the SDL event thread.
+			info.info.x11.unlock_func();
+		}
+	}
+}
+#endif
+
 VisibleSurface::VisibleSurface(RenderSettings& renderSettings_,
 		EventDistributor& eventDistributor_,
 		InputEventGenerator& inputEventGenerator_)
 	: renderSettings(renderSettings_)
 	, eventDistributor(eventDistributor_)
 	, inputEventGenerator(inputEventGenerator_)
+	, alarm(new AlarmEvent(eventDistributor, *this,
+	                       OPENMSX_POINTER_TIMER_EVENT))
 {
 	if (!SDL_WasInit(SDL_INIT_VIDEO) &&
 	    SDL_InitSubSystem(SDL_INIT_VIDEO) < 0) {
@@ -35,7 +94,7 @@ VisibleSurface::VisibleSurface(RenderSettings& renderSettings_,
 
 	// set icon
 	if (OPENMSX_SET_WINDOW_ICON) {
-		SDL_Surface* iconSurf = SDL_CreateRGBSurfaceFrom(
+		SDLSurfacePtr iconSurf(SDL_CreateRGBSurfaceFrom(
 			const_cast<char*>(openMSX_icon.pixel_data),
 			openMSX_icon.width, openMSX_icon.height,
 			openMSX_icon.bytes_per_pixel * 8,
@@ -43,18 +102,14 @@ VisibleSurface::VisibleSurface(RenderSettings& renderSettings_,
 			OPENMSX_BIGENDIAN ? 0xFF000000 : 0x000000FF,
 			OPENMSX_BIGENDIAN ? 0x00FF0000 : 0x0000FF00,
 			OPENMSX_BIGENDIAN ? 0x0000FF00 : 0x00FF0000,
-			OPENMSX_BIGENDIAN ? 0x000000FF : 0xFF000000
-			);
-		SDL_SetColorKey(iconSurf, SDL_SRCCOLORKEY, 0);
-		SDL_WM_SetIcon(iconSurf, NULL);
-		SDL_FreeSurface(iconSurf);
+			OPENMSX_BIGENDIAN ? 0x000000FF : 0xFF000000));
+		SDL_SetColorKey(iconSurf.get(), SDL_SRCCOLORKEY, 0);
+		SDL_WM_SetIcon(iconSurf.get(), NULL);
 	}
 
 	inputEventGenerator_.getGrabInput().attach(*this);
 	renderSettings.getPointerHideDelay().attach(*this);
 	renderSettings.getFullScreen().attach(*this);
-	eventDistributor.registerEventListener(
-		OPENMSX_POINTER_TIMER_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_MOUSE_MOTION_EVENT, *this);
 	eventDistributor.registerEventListener(
@@ -102,6 +157,10 @@ void VisibleSurface::createSurface(unsigned width, unsigned height, int flags)
 	// on SDL-GP2X we need to re-hide the mouse after SDL_SetVideoMode()
 	SDL_ShowCursor(SDL_DISABLE);
 
+#if PLATFORM_MAEMO5
+	setMaemo5WMHints(flags & SDL_FULLSCREEN);
+#endif
+
 #ifdef _WIN32
 	// find our current location...
 	HWND handle = GetActiveWindow();
@@ -119,9 +178,6 @@ void VisibleSurface::createSurface(unsigned width, unsigned height, int flags)
 
 VisibleSurface::~VisibleSurface()
 {
-	prepareDelete();
-	eventDistributor.unregisterEventListener(
-		OPENMSX_POINTER_TIMER_EVENT, *this);
 	eventDistributor.unregisterEventListener(
 		OPENMSX_MOUSE_MOTION_EVENT, *this);
 	eventDistributor.unregisterEventListener(
@@ -197,9 +253,9 @@ bool VisibleSurface::signalEvent(shared_ptr<const Event> event)
 
 void VisibleSurface::updateCursor()
 {
-	cancel();
+	alarm->cancel();
 	if (renderSettings.getFullScreen().getValue() ||
-			inputEventGenerator.getGrabInput().getValue()) {
+	    inputEventGenerator.getGrabInput().getValue()) {
 		// always hide cursor in fullscreen or grabinput mode
 		SDL_ShowCursor(SDL_DISABLE);
 		return;
@@ -210,18 +266,9 @@ void VisibleSurface::updateCursor()
 	} else {
 		SDL_ShowCursor(SDL_ENABLE);
 		if (delay > 0.0) {
-			schedule(int(delay * 1e6)); // delay in s, schedule in us
+			alarm->schedule(int(delay * 1e6)); // delay in s, schedule in us
 		}
 	}
 }
 
-bool VisibleSurface::alarm()
-{
-	// this runs in a different thread,
-	// so we can't directly do our tricks here
-	eventDistributor.distributeEvent(
-		new SimpleEvent<OPENMSX_POINTER_TIMER_EVENT>());
-	return false; // don't repeat alarm
-}
-
 } // namespace openmsx
