@@ -1,8 +1,12 @@
-// $Id: ZMBVEncoder.cc 9240 2009-02-24 22:39:35Z Vampier $
+// $Id: ZMBVEncoder.cc 11123 2010-01-10 16:10:11Z mthuurne $
 
 // Code based on DOSBox-0.65
 
 #include "ZMBVEncoder.hh"
+#include "FrameSource.hh"
+#include "PixelOperations.hh"
+#include "unreachable.hh"
+#include "openmsx.hh"
 #include "build-info.hh"
 #include <algorithm>
 #include <cassert>
@@ -61,15 +65,30 @@ struct KeyframeHeader {
 const char* ZMBVEncoder::CODEC_4CC = "ZMBV";
 
 
-static inline short pixelBEtoLE(short pixel)
+static inline void writePixel(
+	const PixelOperations<unsigned short>& pixelOps,
+	unsigned short pixel, unsigned short& dest
+	)
 {
-	return (pixel >> 8) | (pixel << 8);
+	unsigned r = pixelOps.red256(pixel);
+	unsigned g = pixelOps.green256(pixel);
+	unsigned b = pixelOps.blue256(pixel);
+	unsigned short rgb16 =
+		((r & 0xF8) << (11 - 3)) | ((g & 0xFC) << (5 - 2)) | (b >> 3);
+	dest = OPENMSX_BIGENDIAN ? (rgb16 << 8) | (rgb16 >> 8) : rgb16;
 }
 
-static inline unsigned pixelBEtoLE(unsigned pixel)
+static inline void writePixel(
+	const PixelOperations<unsigned>& pixelOps,
+	unsigned pixel, unsigned& dest
+	)
 {
-	return ((pixel << 24)             ) | ((pixel <<  8) & 0x00FF0000)
-	     | ((pixel >>  8) & 0x0000FF00) | ((pixel >> 24)             );
+	unsigned r = pixelOps.red256(pixel);
+	unsigned g = pixelOps.green256(pixel);
+	unsigned b = pixelOps.blue256(pixel);
+	dest = OPENMSX_BIGENDIAN
+		? (r <<  8) | (g << 16) | (b << 24)
+		: (r << 16) | (g <<  8) |  b;
 }
 
 static void createVectorTable()
@@ -158,24 +177,25 @@ ZMBVEncoder::~ZMBVEncoder()
 void ZMBVEncoder::setupBuffers(unsigned bpp)
 {
 	switch (bpp) {
+#if HAVE_16BPP
 	case 15:
-		format = ZMBV_FORMAT_15BPP;
-		pixelsize = 2;
-		break;
 	case 16:
 		format = ZMBV_FORMAT_16BPP;
-		pixelsize = 2;
+		pixelSize = 2;
 		break;
+#endif
+#if HAVE_32BPP
 	case 32:
 		format = ZMBV_FORMAT_32BPP;
-		pixelsize = 4;
+		pixelSize = 4;
 		break;
+#endif
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 
 	pitch = width + 2 * MAX_VECTOR;
-	unsigned bufsize = (height + 2 * MAX_VECTOR) * pitch * pixelsize + 2048;
+	unsigned bufsize = (height + 2 * MAX_VECTOR) * pitch * pixelSize + 2048;
 
 	oldframe = new unsigned char[bufsize];
 	newframe = new unsigned char[bufsize];
@@ -200,7 +220,7 @@ void ZMBVEncoder::setupBuffers(unsigned bpp)
 
 unsigned ZMBVEncoder::neededSize()
 {
-	unsigned f = pixelsize;
+	unsigned f = pixelSize;
 	f = f * width * height + 2 * (1 + (width / 8)) * (1 + (height / 8)) + 1024;
 	return f + f / 1000;
 }
@@ -238,17 +258,15 @@ unsigned ZMBVEncoder::compareBlock(int vx, int vy, unsigned offset)
 }
 
 template<class P>
-void ZMBVEncoder::addXorBlock(int vx, int vy, unsigned offset)
+void ZMBVEncoder::addXorBlock(
+	const PixelOperations<P>& pixelOps, int vx, int vy, unsigned offset)
 {
 	P* pold = &(reinterpret_cast<P*>(oldframe))[offset + (vy * pitch) + vx];
 	P* pnew = &(reinterpret_cast<P*>(newframe))[offset];
 	for (unsigned y = 0; y < BLOCK_HEIGHT; ++y) {
 		for (unsigned x = 0; x < BLOCK_WIDTH; ++x) {
 			P pxor = pnew[x] ^ pold[x];
-			if (OPENMSX_BIGENDIAN) {
-				pxor = pixelBEtoLE(pxor);
-			}
-			*reinterpret_cast<P*>(&work[workUsed]) = pxor;
+			writePixel(pixelOps, pxor, *reinterpret_cast<P*>(&work[workUsed]));
 			workUsed += sizeof(P);
 		}
 		pold += pitch;
@@ -257,8 +275,9 @@ void ZMBVEncoder::addXorBlock(int vx, int vy, unsigned offset)
 }
 
 template<class P>
-void ZMBVEncoder::addXorFrame()
+void ZMBVEncoder::addXorFrame(const SDL_PixelFormat& pixelFormat)
 {
+	PixelOperations<P> pixelOps(pixelFormat);
 	signed char* vectors = reinterpret_cast<signed char*>(&work[workUsed]);
 
 	// Align the following xor data on 4 byte boundary
@@ -293,23 +312,60 @@ void ZMBVEncoder::addXorFrame()
 		vectors[b * 2 + 1] = (bestvy << 1);
 		if (bestchange) {
 			vectors[b * 2 + 0] |= 1;
-			addXorBlock<P>(bestvx, bestvy, offset);
+			addXorBlock<P>(pixelOps, bestvx, bestvy, offset);
 		}
 	}
 }
 
 template<class P>
-void ZMBVEncoder::lineBEtoLE(unsigned char* input, unsigned width)
+void ZMBVEncoder::addFullFrame(const SDL_PixelFormat& pixelFormat)
 {
-	P* pixelsIn = reinterpret_cast<P*>(input);
-	P* pixelsOut = reinterpret_cast<P*>(&work[workUsed]);
-	for (unsigned i = 0; i < width; i++) {
-		pixelsOut[i] = pixelBEtoLE(pixelsIn[i]);
+	PixelOperations<P> pixelOps(pixelFormat);
+	unsigned char* readFrame =
+		newframe + pixelSize * (MAX_VECTOR + MAX_VECTOR * pitch);
+	for (unsigned y = 0; y < height; ++y) {
+		P* pixelsIn = reinterpret_cast<P*>(readFrame);
+		P* pixelsOut = reinterpret_cast<P*>(&work[workUsed]);
+		for (unsigned x = 0; x < width; ++x) {
+			writePixel(pixelOps, pixelsIn[x], pixelsOut[x]);
+		}
+		readFrame += pitch * sizeof(P);
+		workUsed += width * sizeof(P);
 	}
 }
 
-void ZMBVEncoder::compressFrame(bool keyFrame, const void** lineData,
-                               void*& buffer, unsigned& written)
+const void* ZMBVEncoder::getScaledLine(FrameSource* frame, unsigned y)
+{
+#if HAVE_32BPP
+	if (pixelSize == 4) { // 32bpp
+		switch (height) {
+		case 240:
+			return frame->getLinePtr320_240<unsigned int>(y);
+		case 480:
+			return frame->getLinePtr640_480<unsigned int>(y);
+		default:
+			UNREACHABLE;
+		}
+	}
+#endif
+#if HAVE_16BPP
+	if (pixelSize == 2) { // 15bpp or 16bpp
+		switch (height) {
+		case 240:
+			return frame->getLinePtr320_240<unsigned short>(y);
+		case 480:
+			return frame->getLinePtr640_480<unsigned short>(y);
+		default:
+			UNREACHABLE;
+		}
+	}
+#endif
+	UNREACHABLE;
+	return 0; // avoid warning
+}
+
+void ZMBVEncoder::compressFrame(bool keyFrame, FrameSource* frame,
+                                void*& buffer, unsigned& written)
 {
 	std::swap(newframe, oldframe); // replace oldframe with newframe
 
@@ -334,48 +390,50 @@ void ZMBVEncoder::compressFrame(bool keyFrame, const void** lineData,
 	}
 
 	// copy lines (to add black border)
-	unsigned linePitch = pitch * pixelsize;
-	unsigned lineWidth = width * pixelsize;
+	unsigned linePitch = pitch * pixelSize;
+	unsigned lineWidth = width * pixelSize;
 	unsigned char* dest = newframe +
-	                      pixelsize * (MAX_VECTOR + MAX_VECTOR * pitch);
+	                      pixelSize * (MAX_VECTOR + MAX_VECTOR * pitch);
 	for (unsigned i = 0; i < height; ++i) {
-		memcpy(dest, lineData[i], lineWidth);
+		memcpy(dest, getScaledLine(frame, i), lineWidth);
 		dest += linePitch;
 	}
 
-	// compress
+	// Add the frame data.
 	if (keyFrame) {
-		// Add the full frame data
-		unsigned char* readFrame =
-			newframe + pixelsize * (MAX_VECTOR + MAX_VECTOR * pitch);
-		for (unsigned i = 0; i < height; ++i) {
-			if (OPENMSX_BIGENDIAN) {
-				switch (pixelsize) {
-				case 2:
-					lineBEtoLE<short>(readFrame, width);
-					break;
-				case 4:
-					lineBEtoLE<unsigned>(readFrame, width);
-					break;
-				}
-			} else {
-				memcpy(&work[workUsed], readFrame, lineWidth);
-			}
-			readFrame += linePitch;
-			workUsed += lineWidth;
+		// Key frame: full frame data.
+		switch (pixelSize) {
+#if HAVE_16BPP
+		case 2:
+			addFullFrame<unsigned short>(frame->getSDLPixelFormat());
+			break;
+#endif
+#if HAVE_32BPP
+		case 4:
+			addFullFrame<unsigned int>(frame->getSDLPixelFormat());
+			break;
+#endif
+		default:
+			UNREACHABLE;
 		}
 	} else {
-		// Add the delta frame data
-		switch (pixelsize) {
+		// Non-key frame: delta frame data.
+		switch (pixelSize) {
+#if HAVE_16BPP
 		case 2:
-			addXorFrame<short>();
+			addXorFrame<unsigned short>(frame->getSDLPixelFormat());
 			break;
+#endif
+#if HAVE_32BPP
 		case 4:
-			addXorFrame<unsigned>();
+			addXorFrame<unsigned int>(frame->getSDLPixelFormat());
 			break;
+#endif
+		default:
+			UNREACHABLE;
 		}
 	}
-	// Create the actual frame with compression
+	// Compress the frame data with zlib.
 	zstream.next_in = static_cast<Bytef*>(work);
 	zstream.avail_in = workUsed;
 	zstream.total_in = 0;
