@@ -1,4 +1,4 @@
-// $Id: V9990.cc 9286 2009-02-28 18:15:49Z mfeingol $
+// $Id: V9990.cc 11148 2010-01-13 16:14:45Z m9710797 $
 
 #include "V9990.hh"
 #include "Display.hh"
@@ -8,7 +8,9 @@
 #include "V9990Renderer.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "serialize.hh"
+#include "unreachable.hh"
 #include <cassert>
 #include <cstring>
 
@@ -35,8 +37,13 @@ private:
 };
 
 
-enum RegisterAccess { NO_ACCESS, RD_ONLY, WR_ONLY, RD_WR };
-static const RegisterAccess regAccess[64] = {
+static const byte ALLOW_READ  = 1;
+static const byte ALLOW_WRITE = 2;
+static const byte NO_ACCESS = 0;
+static const byte RD_ONLY   = ALLOW_READ;
+static const byte WR_ONLY   = ALLOW_WRITE;
+static const byte RD_WR     = ALLOW_READ | ALLOW_WRITE;
+static const byte regAccess[64] = {
 	WR_ONLY, WR_ONLY, WR_ONLY,          // VRAM Write Address
 	WR_ONLY, WR_ONLY, WR_ONLY,          // VRAM Read Address
 	RD_WR, RD_WR,                       // Screen Mode
@@ -74,6 +81,7 @@ V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config)
 	, v9990RegDebug(new V9990RegDebug(*this))
 	, v9990PalDebug(new V9990PalDebug(*this))
 	, irq(motherBoard, getName() + ".IRQ")
+	, display(motherBoard.getReactor().getDisplay())
 	, frameStartTime(Schedulable::getCurrentTime())
 	, hScanSyncTime(Schedulable::getCurrentTime())
 	, pendingIRQs(0)
@@ -95,8 +103,8 @@ V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config)
 	vram.reset(new V9990VRAM(*this, time));
 
 	// create Command Engine
-	cmdEngine.reset(new V9990CmdEngine(*this, time,
-		getMotherBoard().getDisplay().getRenderSettings()));
+	cmdEngine.reset(new V9990CmdEngine(
+		*this, time, display.getRenderSettings()));
 	vram->setCmdEngine(*cmdEngine);
 
 	// Start with NTSC timing
@@ -109,19 +117,25 @@ V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config)
 	displayEnabled = false; // avoid UMR (used by createRenderer())
 	createRenderer(time);
 
-	reset(time);
-	getMotherBoard().getDisplay().attach(*this);
+	powerUp(time);
+	display.attach(*this);
 }
 
 V9990::~V9990()
 {
-	getMotherBoard().getDisplay().detach(*this);
+	display.detach(*this);
 }
 
 // -------------------------------------------------------------------------
 // MSXDevice
 // -------------------------------------------------------------------------
 
+void V9990::powerUp(EmuTime::param time)
+{
+	vram->clear();
+	reset(time);
+}
+
 void V9990::reset(EmuTime::param time)
 {
 	removeSyncPoint(V9990_VSYNC);
@@ -134,6 +148,7 @@ void V9990::reset(EmuTime::param time)
 	memset(regs, 0, sizeof(regs));
 	status = 0;
 	regSelect = 0xFF; // TODO check value for power-on and reset
+	systemReset = false; // verified on real MSX
 	calcDisplayMode();
 
 	isDisplayArea = false;
@@ -177,6 +192,8 @@ byte V9990::readIO(word port, EmuTime::param time)
 	default:
 		result = peekIO(port, time);
 	}
+	// TODO verify this, especially REGISTER_DATA
+	if (systemReset) return result; // no side-effects
 
 	// execute side-effects
 	switch (port) {
@@ -215,6 +232,7 @@ byte V9990::peekIO(word port, EmuTime::param time) const
 	byte result;
 	switch (port & 0x0F) {
 	case VRAM_DATA: {
+		// TODO in 'systemReset' mode, this seems to hang the MSX
 		unsigned addr = getVRAMAddr(VRAM_READ_ADDRESS_0);
 		result = vram->readVRAMCPU(addr, time);
 		break;
@@ -283,6 +301,12 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 	switch (port) {
 		case VRAM_DATA: {
 			// write VRAM
+			if (systemReset) {
+				// TODO writes in systemReset mode seem to have
+				//  'some' effect but it's not immediately clear
+				//  what the exact behaviour is
+				return;
+			}
 			unsigned addr = getVRAMAddr(VRAM_WRITE_ADDRESS_0);
 			vram->writeVRAMCPU(addr, val, time);
 			if (!(regs[VRAM_WRITE_ADDRESS_2] & 0x80)) {
@@ -291,6 +315,13 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 			break;
 		}
 		case PALETTE_DATA: {
+			if (systemReset) {
+				// Equivalent to writing 0 and keeping palPtr = 0
+				// The above interpretation makes it similar to
+				// writes to REGISTER_DATA, REGISTER_SELECT.
+				writePaletteRegister(0, 0, time);
+				return;
+			}
 			byte& palPtr = regs[PALETTE_POINTER];
 			writePaletteRegister(palPtr, val, time);
 			switch (palPtr & 3) {
@@ -302,12 +333,24 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 			break;
 		}
 		case COMMAND_DATA:
+			// systemReset state doesn't matter:
+			//   command below has no effect in systemReset mode
 			//assert(cmdEngine != NULL);
 			cmdEngine->setCmdData(val, time);
 			break;
 
 		case REGISTER_DATA: {
 			// write register
+			if (systemReset) {
+				// In systemReset mode, write has no effect,
+				// but 'regSelect' is increased.
+				// I don't know if write is ignored or a write
+				// with val=0 is executed. Though both have the
+				// same effect and the latter is more in line
+				// with writes to PALETTE_DATA and
+				// REGISTER_SELECT.
+				val = 0;
+			}
 			writeRegister(regSelect & 0x3F, val, time);
 			if (!(regSelect & 0x80)) {
 				regSelect = ( regSelect      & 0xC0) |
@@ -316,6 +359,14 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 			break;
 		}
 		case REGISTER_SELECT:
+			if (systemReset) {
+				// Tested on real MSX. Also when no write is done
+				// to this port, regSelect is not RESET when
+				// entering/leaving systemReset mode.
+				// This behavior is similar to PALETTE_DATA and
+				// REGISTER_DATA.
+				val = 0;
+			}
 			regSelect = val;
 			break;
 
@@ -324,6 +375,8 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 			break;
 
 		case INTERRUPT_FLAG:
+			// systemReset state doesn't matter:
+			//   stuff below has no effect in systemReset mode
 			pendingIRQs &= ~val;
 			if (!(pendingIRQs & regs[INTERRUPT_0])) {
 				irq.reset();
@@ -331,13 +384,28 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 			scheduleHscan(time);
 			break;
 
-		case SYSTEM_CONTROL:
+		case SYSTEM_CONTROL: {
 			// TODO investigate: does switching overscan mode
 			//      happen at next line or next frame
 			status = (status & 0xFB) | ((val & 1) << 2);
 			syncAtNextLine(V9990_SET_MODE, time);
-		break;
 
+			bool newSystemReset = (val & 2) != 0;
+			if (newSystemReset != systemReset) {
+				systemReset = newSystemReset;
+				if (systemReset) {
+					// Enter systemReset mode
+					//   Verified on real MSX: palette data
+					//   and VRAM content are NOT reset.
+					for (int i = 0; i < 64; ++i) {
+						writeRegister(i, 0, time);
+					}
+					// TODO verify IRQ behaviour
+					writeIO(INTERRUPT_FLAG, 0xFF, time);
+				}
+			}
+			break;
+		}
 		case KANJI_ROM_0:
 		case KANJI_ROM_1:
 		case KANJI_ROM_2:
@@ -396,7 +464,7 @@ void V9990::executeUntil(EmuTime::param time, int userData)
 		break;
 
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -487,10 +555,11 @@ inline void V9990::setVRAMAddr(RegisterId base, unsigned addr)
 byte V9990::readRegister(byte reg, EmuTime::param time) const
 {
 	// TODO sync(time) (if needed at all)
+	if (systemReset) return 255; // verified on real MSX
 
 	assert(reg < 64);
 	byte result;
-	if (regAccess[reg] != NO_ACCESS && regAccess[reg] != WR_ONLY) {
+	if (regAccess[reg] & ALLOW_READ) {
 		if (reg < CMD_PARAM_BORDER_X_0) {
 			result = regs[reg];
 		} else {
@@ -532,7 +601,7 @@ void V9990::writeRegister(byte reg, byte val, EmuTime::param time)
 	//	                        " val=0x" << std::hex << int(val));
 
 	assert(reg < 64);
-	if ((regAccess[reg] == NO_ACCESS) || (regAccess[reg] == RD_ONLY)) {
+	if (!(regAccess[reg] & ALLOW_WRITE)) {
 		// register not writable
 		return;
 	}
@@ -624,7 +693,6 @@ void V9990::getPalette(int index, byte& r, byte& g, byte& b) const
 void V9990::createRenderer(EmuTime::param time)
 {
 	assert(!renderer.get());
-	Display& display = getMotherBoard().getDisplay();
 	renderer.reset(RendererFactory::createV9990Renderer(*this, display));
 	renderer->reset(time);
 }
@@ -682,7 +750,7 @@ void V9990::setHorizontalTiming()
 	case B5: case B6:
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -702,7 +770,7 @@ void V9990::setVerticalTiming()
 	case B5: case B6:
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
@@ -722,11 +790,11 @@ V9990ColorMode V9990::getColorMode(byte pal_ctrl) const
 					case 0x40: mode = BD8; break;
 					case 0x80: mode = BYJK; break;
 					case 0xC0: mode = BYUV; break;
-					default: assert(false); break;
+					default: UNREACHABLE;
 				}
 				break;
 			case 0x03: mode = BD16; break;
-			default: assert(false); break;
+			default: UNREACHABLE;
 		}
 	}
 
@@ -757,7 +825,7 @@ void V9990::calcDisplayMode()
 				case 0x10: mode = B2; break;
 				case 0x20: mode = B4; break;
 				case 0x30: mode = INVALID_DISPLAY_MODE; break;
-				default: assert(false);
+				default: UNREACHABLE;
 				}
 			} else { // XTAL1 timing
 				switch(regs[SCREEN_MODE_0] & 0x30) {
@@ -820,8 +888,10 @@ static enum_string<V9990DisplayMode> displayModeInfo[] = {
 };
 SERIALIZE_ENUM(V9990DisplayMode, displayModeInfo);
 
+// version 1:  initial version
+// version 2:  added systemReset
 template<typename Archive>
-void V9990::serialize(Archive& ar, unsigned /*version*/)
+void V9990::serialize(Archive& ar, unsigned version)
 {
 	ar.template serializeBase<MSXDevice>(*this);
 	ar.template serializeBase<Schedulable>(*this);
@@ -844,6 +914,12 @@ void V9990::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("scrollAYHigh", scrollAYHigh);
 	ar.serialize("scrollBYHigh", scrollBYHigh);
 
+	if (ar.versionBelow(version, 2)) {
+		systemReset = false;
+	} else {
+		ar.serialize("systemReset", systemReset);
+	}
+
 	if (ar.isLoader()) {
 		// TODO This uses 'mode' to calculate 'horTiming' and
 		//      'verTiming'. Are these always in sync? Or can for
