@@ -1,4 +1,4 @@
-// $Id: V9990BitmapConverter.cc 8462 2009-01-10 19:52:00Z m9710797 $
+// $Id: V9990BitmapConverter.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "V9990BitmapConverter.hh"
 #include "V9990VRAM.hh"
@@ -7,6 +7,7 @@
 #include "Math.hh"
 #include "static_assert.hh"
 #include "type_traits.hh"
+#include "unreachable.hh"
 #include "build-info.hh"
 #include <cassert>
 
@@ -20,12 +21,12 @@ V9990BitmapConverter<Pixel>::V9990BitmapConverter(
 	, palette64(palette64_), palette256(palette256_), palette32768(palette32768_)
 {
 	// make sure function pointers have valid values
-	setColorMode(PP);
+	setColorMode(PP, B0);
 }
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBYUV(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
 	for (int p = 0; p < nrPixels; p += 4) {
 		byte data[4];
@@ -48,8 +49,10 @@ void V9990BitmapConverter<Pixel>::rasterBYUV(
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBYUVP(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
+	// TODO this mode cannot be shown in B4 and higher resolution modes
+	//      (So the dual palette for B4 modes is not an issue here.)
 	for (int p = 0; p < nrPixels; p += 4) {
 		byte data[4];
 		for (int i = 0; i < 4; ++i) {
@@ -75,7 +78,7 @@ void V9990BitmapConverter<Pixel>::rasterBYUVP(
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBYJK(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
 	for (int p = 0; p < nrPixels; p += 4) {
 		byte data[4];
@@ -98,8 +101,10 @@ void V9990BitmapConverter<Pixel>::rasterBYJK(
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBYJKP(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
+	// TODO this mode cannot be shown in B4 and higher resolution modes
+	//      (So the dual palette for B4 modes is not an issue here.)
 	for (int p = 0; p < nrPixels; p += 4) {
 		byte data[4];
 		for (int i = 0; i < 4; ++i) {
@@ -125,7 +130,7 @@ void V9990BitmapConverter<Pixel>::rasterBYJKP(
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBD16(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
 	for (int p = 0; p < nrPixels; ++p) {
 		byte low  = vram.readVRAMBx(address++);
@@ -136,7 +141,7 @@ void V9990BitmapConverter<Pixel>::rasterBD16(
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBD8(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
 	for (int p = 0; p < nrPixels; ++p) {
 		*pixelPtr++ = palette256[vram.readVRAMBx(address++)];
@@ -145,7 +150,7 @@ void V9990BitmapConverter<Pixel>::rasterBD8(
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBP6(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
 	for (int p = 0; p < nrPixels; ++p) {
 		*pixelPtr++ = palette64[vram.readVRAMBx(address++) & 0x3F];
@@ -154,25 +159,63 @@ void V9990BitmapConverter<Pixel>::rasterBP6(
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBP4(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
+	byte offset = (vdp.getPaletteOffset() & 0xC) << 2;
+	const Pixel* pal = &palette64[offset];
 	for (int p = 0; p < nrPixels; p += 2) {
 		byte data = vram.readVRAMBx(address++);
-		*pixelPtr++ = palette64[data >> 4];
-		*pixelPtr++ = palette64[data & 0x0F];
+		*pixelPtr++ = pal[data >> 4];
+		*pixelPtr++ = pal[data & 0x0F];
+	}
+}
+template <class Pixel>
+void V9990BitmapConverter<Pixel>::rasterBP4HiRes(
+	Pixel* pixelPtr, unsigned address, int nrPixels)
+{
+	// Verified on real HW:
+	//   Bit PLT05 in palette offset is ignored, instead for even pixels
+	//   bit 'PLT05' is '0', for odd pixels it's '1'.
+	byte offset = (vdp.getPaletteOffset() & 0x4) << 2;
+	const Pixel* pal1 = &palette64[offset |  0];
+	const Pixel* pal2 = &palette64[offset | 32];
+	for (int p = 0; p < nrPixels; p += 2) {
+		byte data = vram.readVRAMBx(address++);
+		*pixelPtr++ = pal1[data >> 4  ];
+		*pixelPtr++ = pal2[data & 0x0F];
 	}
 }
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBP2(
-	Pixel* pixelPtr, unsigned address, int nrPixels)
+	Pixel* __restrict pixelPtr, unsigned address, int nrPixels)
 {
+	byte offset = vdp.getPaletteOffset() << 2;
+	const Pixel* pal = &palette64[offset];
 	for (int p = 0; p < nrPixels; p += 4) {
 		byte data = vram.readVRAMBx(address++);
-		*pixelPtr++ = palette64[(data & 0xC0) >> 6];
-		*pixelPtr++ = palette64[(data & 0x30) >> 4];
-		*pixelPtr++ = palette64[(data & 0x0C) >> 2];
-		*pixelPtr++ = palette64[(data & 0x03)];
+		*pixelPtr++ = pal[(data & 0xC0) >> 6];
+		*pixelPtr++ = pal[(data & 0x30) >> 4];
+		*pixelPtr++ = pal[(data & 0x0C) >> 2];
+		*pixelPtr++ = pal[(data & 0x03) >> 0];
+	}
+}
+template <class Pixel>
+void V9990BitmapConverter<Pixel>::rasterBP2HiRes(
+	Pixel* pixelPtr, unsigned address, int nrPixels)
+{
+	// Verified on real HW:
+	//   Bit PLT05 in palette offset is ignored, instead for even pixels
+	//   bit 'PLT05' is '0', for odd pixels it's '1'.
+	byte offset = (vdp.getPaletteOffset() & 0x7) << 2;
+	const Pixel* pal1 = &palette64[offset |  0];
+	const Pixel* pal2 = &palette64[offset | 32];
+	for (int p = 0; p < nrPixels; p += 4) {
+		byte data = vram.readVRAMBx(address++);
+		*pixelPtr++ = pal1[(data & 0xC0) >> 6];
+		*pixelPtr++ = pal2[(data & 0x30) >> 4];
+		*pixelPtr++ = pal1[(data & 0x0C) >> 2];
+		*pixelPtr++ = pal2[(data & 0x03) >> 0];
 	}
 }
 
@@ -180,14 +223,19 @@ template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterP(
 	Pixel* /*pixelPtr*/, unsigned /*address*/, int /*nrPixels*/)
 {
-	assert(false);
+	UNREACHABLE;
 }
 
-
-template <class Pixel>
-void V9990BitmapConverter<Pixel>::setColorMode(V9990ColorMode mode)
+static bool isHighRes(V9990DisplayMode display)
 {
-	switch (mode) {
+	return (display == B4) || (display == B5) ||
+	       (display == B6) || (display == B7);
+}
+template <class Pixel>
+void V9990BitmapConverter<Pixel>::setColorMode(V9990ColorMode color,
+                                               V9990DisplayMode display)
+{
+	switch (color) {
 	case PP:    rasterMethod = &V9990BitmapConverter::rasterP;     break;
 	case BYUV:  rasterMethod = &V9990BitmapConverter::rasterBYUV;  break;
 	case BYUVP: rasterMethod = &V9990BitmapConverter::rasterBYUVP; break;
@@ -196,16 +244,20 @@ void V9990BitmapConverter<Pixel>::setColorMode(V9990ColorMode mode)
 	case BD16:  rasterMethod = &V9990BitmapConverter::rasterBD16;  break;
 	case BD8:   rasterMethod = &V9990BitmapConverter::rasterBD8;   break;
 	case BP6:   rasterMethod = &V9990BitmapConverter::rasterBP6;   break;
-	case BP4:   rasterMethod = &V9990BitmapConverter::rasterBP4;   break;
-	case BP2:   rasterMethod = &V9990BitmapConverter::rasterBP2;   break;
-	default:    assert (false);
+	case BP4:   rasterMethod = isHighRes(display) ?
+	                           &V9990BitmapConverter::rasterBP4HiRes :
+	                           &V9990BitmapConverter::rasterBP4;   break;
+	case BP2:   rasterMethod = isHighRes(display) ?
+	                           &V9990BitmapConverter::rasterBP2HiRes :
+	                           &V9990BitmapConverter::rasterBP2;   break;
+	default:    UNREACHABLE;
 	}
 }
 
 
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::drawCursor(
-	Pixel* buffer, int displayY, unsigned attrAddr, unsigned patAddr)
+	Pixel* __restrict buffer, int displayY, unsigned attrAddr, unsigned patAddr)
 {
 	int cursorY = vram.readVRAMBx(attrAddr + 0) +
 	             (vram.readVRAMBx(attrAddr + 2) & 1) * 256;
@@ -230,6 +282,7 @@ void V9990BitmapConverter<Pixel>::drawCursor(
 	unsigned x = vram.readVRAMBx(attrAddr + 4) + (attr & 3) * 256;
 
 	// TODO EOR colors
+	// TODO investigate dual palette in B4 and higher modes
 	Pixel color = palette64[vdp.getSpritePaletteOffset() + (attr >> 6)];
 	for (int i = 0; i < 32; ++i) {
 		if (pattern & 0x80000000) {
@@ -268,9 +321,8 @@ template class V9990BitmapConverter<word>;
 template class V9990BitmapConverter<unsigned>;
 #endif
 
-#ifdef COMPONENT_GL
-#ifdef _MSC_VER
-
+#if COMPONENT_GL
+#if defined(_MSC_VER)
 // The template stuff below fails to compile on VC++, it triggers this error
 //   http://msdn.microsoft.com/en-us/library/9045w50z.aspx
 // It's not clear whether this is a limitation in VC++ or a C++ extension
@@ -279,12 +331,11 @@ template class V9990BitmapConverter<unsigned>;
 // so the stuff below is not required (it would only instantiate a dummy class
 // when these types are the same).
 STATIC_ASSERT((is_same_type<unsigned, GLuint>::value));
-
-#else
-
+#elif HAVE_32BPP
 template <> class V9990BitmapConverter<GLUtil::NoExpansion> {};
 template class V9990BitmapConverter<GLUtil::ExpandGL>;
-
+#else
+template class V9990BitmapConverter<GLuint>;
 #endif
 #endif // COMPONENT_GL
 
