@@ -1,4 +1,4 @@
-// $Id: V9990CmdEngine.cc 9300 2009-03-01 01:03:07Z mfeingol $
+// $Id: V9990CmdEngine.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "V9990CmdEngine.hh"
 #include "V9990.hh"
@@ -9,6 +9,7 @@
 #include "EnumSetting.hh"
 #include "serialize.hh"
 #include "likely.hh"
+#include "unreachable.hh"
 #include <iostream>
 
 namespace openmsx {
@@ -832,7 +833,7 @@ void V9990CmdEngine::setCurrentCommand()
 	} else { // Bx
 		switch (vdp.getColorMode()) {
 			default:
-				assert(false);
+				UNREACHABLE;
 			case BP2:
 				cmdMode = CMD_BPP2;
 				break;
