@@ -1,4 +1,4 @@
-// $Id: V9990P1Converter.cc 8462 2009-01-10 19:52:00Z m9710797 $
+// $Id: V9990P1Converter.cc 10421 2009-08-27 13:34:27Z mthuurne $
 
 #include "V9990P1Converter.hh"
 #include "V9990.hh"
@@ -93,7 +93,7 @@ void V9990P1Converter<Pixel>::renderPattern(
 
 template <class Pixel>
 void V9990P1Converter<Pixel>::renderPattern2(
-	Pixel* buffer, unsigned width, unsigned x, unsigned y,
+	Pixel* __restrict buffer, unsigned width, unsigned x, unsigned y,
 	unsigned nameTable, unsigned patternTable, byte pal)
 {
 	x &= 511;
@@ -185,7 +185,7 @@ void V9990P1Converter<Pixel>::renderPattern2(
 
 template <class Pixel>
 void V9990P1Converter<Pixel>::determineVisibleSprites(
-	int* visibleSprites, unsigned displayY)
+	int* __restrict visibleSprites, unsigned displayY)
 {
 	static const unsigned spriteTable = 0x3FE00;
 
@@ -210,8 +210,8 @@ void V9990P1Converter<Pixel>::determineVisibleSprites(
 
 template <class Pixel>
 void V9990P1Converter<Pixel>::renderSprites(
-	Pixel* buffer, int displayX, int displayEnd, unsigned displayY,
-	int* visibleSprites, bool front)
+	Pixel* __restrict buffer, int displayX, int displayEnd, unsigned displayY,
+	int* __restrict visibleSprites, bool front)
 {
 	static const unsigned spriteTable = 0x3FE00;
 	unsigned spritePatternTable = vdp.getSpritePatternAddress(P1);
@@ -255,13 +255,15 @@ template class V9990P1Converter<word>;
 template class V9990P1Converter<unsigned>;
 #endif
 
-#ifdef COMPONENT_GL
-#ifdef _MSC_VER
+#if COMPONENT_GL
+#if defined(_MSC_VER)
 // see comment in V9990BitmapConverter
 STATIC_ASSERT((is_same_type<unsigned, GLuint>::value));
-#else
+#elif HAVE_32BPP
 template <> class V9990P1Converter<GLUtil::NoExpansion> {};
 template class V9990P1Converter<GLUtil::ExpandGL>;
+#else
+template class V9990P1Converter<GLuint>;
 #endif
 #endif // COMPONENT_GL
 
