@@ -1,4 +1,4 @@
-// $Id: V9990P2Converter.cc 8462 2009-01-10 19:52:00Z m9710797 $
+// $Id: V9990P2Converter.cc 10421 2009-08-27 13:34:27Z mthuurne $
 
 #include "V9990P2Converter.hh"
 #include "V9990VRAM.hh"
@@ -58,7 +58,7 @@ void V9990P2Converter<Pixel>::convertLine(
 
 template <class Pixel>
 void V9990P2Converter<Pixel>::renderPattern(
-	Pixel* buffer, unsigned width, unsigned x, unsigned y, byte pal)
+	Pixel* __restrict buffer, unsigned width, unsigned x, unsigned y, byte pal)
 {
 	static const unsigned patternTable = 0x00000;
 	static const unsigned nameTable    = 0x7C000;
@@ -152,7 +152,7 @@ void V9990P2Converter<Pixel>::renderPattern(
 
 template <class Pixel>
 void V9990P2Converter<Pixel>::determineVisibleSprites(
-	int* visibleSprites, int displayY)
+	int* __restrict visibleSprites, int displayY)
 {
 	static const unsigned spriteTable = 0x3FE00;
 
@@ -178,8 +178,8 @@ void V9990P2Converter<Pixel>::determineVisibleSprites(
 
 template <class Pixel>
 void V9990P2Converter<Pixel>::renderSprites(
-	Pixel* buffer, int displayX, int displayEnd, unsigned displayY,
-	int* visibleSprites, bool front)
+	Pixel* __restrict buffer, int displayX, int displayEnd, unsigned displayY,
+	int* __restrict visibleSprites, bool front)
 {
 	static const unsigned spriteTable = 0x3FE00;
 	unsigned spritePatternTable = vdp.getSpritePatternAddress(P1);
@@ -223,13 +223,15 @@ template class V9990P2Converter<word>;
 template class V9990P2Converter<unsigned>;
 #endif
 
-#ifdef COMPONENT_GL
-#ifdef _MSC_VER
+#if COMPONENT_GL
+#if defined(_MSC_VER)
 // see comment in V9990BitmapConverter
 STATIC_ASSERT((is_same_type<unsigned, GLuint>::value));
-#else
+#elif HAVE_32BPP
 template <> class V9990P2Converter<GLUtil::NoExpansion> {};
 template class V9990P2Converter<GLUtil::ExpandGL>;
+#else
+template class V9990P2Converter<GLuint>;
 #endif
 #endif // COMPONENT_GL
 
