@@ -1,4 +1,4 @@
-// $Id: V9990PixelRenderer.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: V9990PixelRenderer.cc 10580 2009-09-14 20:37:31Z m9710797 $
 
 #include "V9990PixelRenderer.hh"
 #include "V9990.hh"
@@ -12,20 +12,23 @@
 #include "Timer.hh"
 #include "EventDistributor.hh"
 #include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "RenderSettings.hh"
 #include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
 #include "EnumSetting.hh"
+#include "unreachable.hh"
 
 namespace openmsx {
 
 V9990PixelRenderer::V9990PixelRenderer(V9990& vdp_)
 	: vdp(vdp_)
-	, eventDistributor(vdp.getMotherBoard().getEventDistributor())
+	, eventDistributor(vdp.getMotherBoard().getReactor().getEventDistributor())
 	, realTime(vdp.getMotherBoard().getRealTime())
-	, renderSettings(vdp.getMotherBoard().getDisplay().getRenderSettings())
-	, rasterizer(vdp.getMotherBoard().getDisplay().getVideoSystem().
-	                createV9990Rasterizer(vdp))
+	, renderSettings(vdp.getMotherBoard().getReactor().getDisplay().
+	                    getRenderSettings())
+	, rasterizer(vdp.getMotherBoard().getReactor().getDisplay().
+	                getVideoSystem().createV9990Rasterizer(vdp))
 {
 	frameSkipCounter = 999; // force drawing of frame;
 	finishFrameDuration = 0;
@@ -160,7 +163,7 @@ void V9990PixelRenderer::renderUntil(EmuTime::param time)
 		             V9990DisplayTiming::UC_TICKS_PER_LINE;
 		break;
 	default:
-		assert(false);
+		UNREACHABLE;
 		toX = toY = 0; // avoid warning
 	}
 
@@ -314,7 +317,7 @@ void V9990PixelRenderer::update(const Setting& setting)
 		// Force drawing of frame
 		frameSkipCounter = 999;
 	} else {
-		assert(false);
+		UNREACHABLE;
 	}
 }
 
