@@ -1,4 +1,4 @@
-// $Id: V9990Renderer.cc 4647 2005-08-27 22:06:01Z m9710797 $
+// $Id: V9990Renderer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "V9990Renderer.hh"
 
@@ -15,15 +15,15 @@ V9990Renderer::~V9990Renderer()
 {
 }
 
-void V9990Renderer::reset(const EmuTime& /*time*/)
+void V9990Renderer::reset(EmuTime::param /*time*/)
 {
 }
 
-void V9990Renderer::frameEnd(const EmuTime& /*time*/)
+void V9990Renderer::frameEnd(EmuTime::param /*time*/)
 {
 }
 
-void V9990Renderer::renderUntil(const EmuTime& /*time*/)
+void V9990Renderer::renderUntil(EmuTime::param /*time*/)
 {
 }
 
