@@ -1,4 +1,4 @@
-// $Id: V9990SDLRasterizer.cc 8352 2008-11-12 18:39:08Z m9710797 $
+// $Id: V9990SDLRasterizer.cc 10849 2009-11-30 15:47:53Z m9710797 $
 
 #include "V9990SDLRasterizer.hh"
 #include "V9990.hh"
@@ -9,11 +9,13 @@
 #include "V9990P2Converter.hh"
 #include "FloatSetting.hh"
 #include "StringSetting.hh"
+#include "MSXMotherBoard.hh"
 #include "Display.hh"
 #include "VisibleSurface.hh"
 #include "RenderSettings.hh"
 #include "MemoryOps.hh"
 #include "build-info.hh"
+#include "components.hh"
 #include <algorithm>
 
 using std::min;
@@ -29,6 +31,8 @@ V9990SDLRasterizer<Pixel>::V9990SDLRasterizer(
 	: vdp(vdp_), vram(vdp.getVRAM())
 	, screen(screen_)
 	, renderSettings(display.getRenderSettings())
+	, displayMode(P1) // dummy value
+	, colorMode(PP)   //   avoid UMR
 	, postProcessor(postProcessor_)
 	, bitmapConverter(new V9990BitmapConverter<Pixel>(
 	                           vdp, palette64, palette256, palette32768))
@@ -60,7 +64,8 @@ V9990SDLRasterizer<Pixel>::~V9990SDLRasterizer()
 template <class Pixel>
 bool V9990SDLRasterizer<Pixel>::isActive()
 {
-	return postProcessor->getZ() != Layer::Z_MSX_PASSIVE;
+	return (postProcessor->getZ() != Layer::Z_MSX_PASSIVE) &&
+	       vdp.getMotherBoard().isActive();
 }
 
 template <class Pixel>
@@ -106,13 +111,14 @@ template <class Pixel>
 void V9990SDLRasterizer<Pixel>::setDisplayMode(V9990DisplayMode mode)
 {
 	displayMode = mode;
+	bitmapConverter->setColorMode(colorMode, displayMode);
 }
 
 template <class Pixel>
 void V9990SDLRasterizer<Pixel>::setColorMode(V9990ColorMode mode)
 {
 	colorMode = mode;
-	bitmapConverter->setColorMode(mode);
+	bitmapConverter->setColorMode(colorMode, displayMode);
 }
 
 template <class Pixel>
@@ -145,7 +151,7 @@ void V9990SDLRasterizer<Pixel>::drawBorder(
 	unsigned lineWidth = vdp.getLineWidth();
 	MemoryOps::MemSet<Pixel, MemoryOps::NO_STREAMING> memset;
 	for (int y = startY; y < endY; ++y) {
-		memset(workFrame->getLinePtr<Pixel>(y) + startX,
+		memset(workFrame->getLinePtrDirect<Pixel>(y) + startX,
 		       endX - startX, bgColor);
 		workFrame->setLineWidth(y, lineWidth);
 	}
@@ -213,7 +219,7 @@ void V9990SDLRasterizer<Pixel>::drawP1Mode(
 	int displayWidth, int displayHeight)
 {
 	while (displayHeight--) {
-		Pixel* pixelPtr = workFrame->getLinePtr<Pixel>(fromY) + fromX;
+		Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(fromY) + fromX;
 		p1Converter->convertLine(pixelPtr, displayX, displayWidth,
 		                         displayY, displayYA, displayYB);
 		workFrame->setLineWidth(fromY, 320);
@@ -230,7 +236,7 @@ void V9990SDLRasterizer<Pixel>::drawP2Mode(
 	int displayWidth, int displayHeight)
 {
 	while (displayHeight--) {
-		Pixel* pixelPtr = workFrame->getLinePtr<Pixel>(fromY) + fromX;
+		Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(fromY) + fromX;
 		p2Converter->convertLine(pixelPtr, displayX, displayWidth,
 		                         displayY, displayYA);
 		workFrame->setLineWidth(fromY, 640);
@@ -263,7 +269,7 @@ void V9990SDLRasterizer<Pixel>::drawBxMode(
 	while (displayHeight--) {
 		unsigned y = scrollYBase + ((displayYA + scrollY) & rollMask);
 		unsigned address = vdp.XYtoVRAM(&x, y, colorMode);
-		Pixel* pixelPtr = workFrame->getLinePtr<Pixel>(fromY) + fromX;
+		Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(fromY) + fromX;
 		bitmapConverter->convertLine(pixelPtr, address, displayWidth,
 		                             displayY);
 		workFrame->setLineWidth(fromY, vdp.getLineWidth());
@@ -349,7 +355,7 @@ void V9990SDLRasterizer<Pixel>::update(const Setting& setting)
 #if HAVE_16BPP
 template class V9990SDLRasterizer<word>;
 #endif
-#if HAVE_32BPP
+#if HAVE_32BPP || COMPONENT_GL
 template class V9990SDLRasterizer<unsigned>;
 #endif
 
